/*******************************************************************************
*
* Project             GATT Sample Application
* File                main.cpp
* Description         main application file
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

// _____________________________________________________________________________
//
#undef  CLX_MODULE_ID
#define CLX_MODULE_ID  20123
// _____________________________________________________________________________
//

#include "ClxBsp.h"
#include "ClarinoxBlue.h"

#include "Gap.Api.h"
#include "Gap.Ble.Api.h"
#include "Gatt.Ble.Common.Api.h"
#include "Gatt.Ble.Service.h"
#include "Gatt.Ble.Server.Api.h"
#include "Gatt.Ble.Client.Api.h"

#include "GattApp.h"
#include "GattService.h"

#include "string.h"
#include "stdio.h"

extern void menuFunction(ClxStack stack);
extern void printMemory( u1* data, u4 length, u4 lineLength, u4 columnLength, u4 flags );
ClxCharacteristicList cl;
static s1 handle_in[128];
extern ClxBleBdAddress  bleAddress;
extern u2               testServiceHandleBase;

#if defined (CLX_UCOS_III) | (CLX_THREADX)
extern ClxResult CLX_CALLBACK cameraThread( void* data );
extern bool             cameraRunning;
extern ClxSemaphore     cameraSemaphore;
#endif //defined (CLX_UCOS_III) | (CLX_THREADX)

s1* deviceName = "ClxBleTestApp";                            /* Local Bluetooth Device name                            */

/*
Synchronisation for termination
*/
ClxSemaphore terminationSemaphore;


/*
The content of the advertising data must be conserved during the API operation
*/
static u1 advertisingData[32]; 


#ifdef __cplusplus
extern "C" {
#endif
 extern void clxInitBsp(void );
#ifdef __cplusplus
}
#endif

extern boolean stackMessageHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode);
static ClxHandle gattHandle    = 0;
static s1 message[128];
static ClxSize readSize;

/*------------------------------- Device discovery procedure -------------------------------*/
/**
Start the inquiry (device discovery) process
*/
void discoverDevices(ClxStack stack, boolean liacMode)
{
	s1 ch = 0;

	struct ClxGapInquiryFilter filter;

	filter.filterMask = ClxNoInquiryFilter;
    /*
    Start inquiry in one of the inquiry modes 
    (most application use general inquiry mode, only special applications require limited inquiry)
    */
	ClxError err;
	if (liacMode)
	{
		err = clxGapStartLimitedInquiry(stack, &filter, TRUE);
	}
	else 
	{
		err = clxGapStartInquiry(stack, &filter, TRUE);
	}
    /*
    Wait until enough number of devices discovered. While the main application
    is waiting here, each discovered device will cause an application callback 
    to be triggered with the CLX_GAP_DEVICE_DISCOVERED_INDICATION event id.
    */

	do
    {
        ch = ClxConsoleUIEngine::messageBox("Enter c to stop searching", "cC", 2);        
    }
    while ((ch != 'c') && (ch != 'C'));
	ClxConsoleUIEngine::text ("\nDevice discovery stopped . . .\n");

	
    /*
    Stop inquiry so that the Bluetooth radio is fully available for data transmission
    */
	err = clxGapStopInquiry(stack, TRUE);

	ClxDeviceDetail list[20];
	u4 listSize = 20;
    /*
    Present the list of devices discovered during the inquiry operation to the user
    */
	err = clxGapGetListOfDiscoveredDevices(stack, list, 20, &listSize, TRUE);

	if (listSize)
	{
		const s1 returnText[] = {"Return to previous menu"};
		s1* menuItems = new s1[listSize*64 + sizeof(returnText)];
		menuItems[0] = '\0';
		
		u4 index = 0;
        for (u4 i = 0; i < listSize; i++) 
        { 
            /* 
            Add the name and the RSSI value of the device to the UI menu to select a device
            */
            strcpy(menuItems + index, list[i].deviceName);
            s1 rssi[16];
            sprintf(rssi, " (RSSI: %ddBm)", list[i].rssi);
            strcat(menuItems + index, rssi);
            index += (strlen(menuItems + index) + 1);
        }
        
        strcpy (menuItems + index, returnText);

        u4 s = ClxConsoleUIEngine::showMenu("\nSelect a device to pair", menuItems, listSize + 1);

        delete[] menuItems;

		// Optionally discover all the services provided by the selected device 
        // discoverServices(stack, list[s-1].deviceId);

        /*
        Pair with the selected device; this will potentially cause some callback functions
        depending on the authentication mode (i.e. pincode entry or passcode verification etc)
        */
        ClxConsoleUIEngine::text ("Pairing with the remote device . . .\n");

        ClxSecurityRequirement securityRequirement = ClxMediumSecurity;
        err = clxGapInitiateBonding(stack, list[s - 1].deviceId, securityRequirement, TRUE);
		if (err != CLX_SUCCESS)
		{
 			ClxConsoleUIEngine::text ("Pairing complete with the result : %s\n", clxGetErrorCodeText(err));
		}

    }
}


/*******************************************************************************************************************************
*                                                User Exception Handler
*
* Trap the stack exceptions.
*
*******************************************************************************************************************************/
void userExceptionHandler( s4 moduleId, s4 lineNumber )
{
    printf("ClarinoxBlue raised an exception in module: %u, line: %u\n", moduleId, lineNumber);
}

boolean initializeClarinoxBlue(ClxStack* stack)
{
    boolean ret = TRUE;

    struct GenericConfigParameters
    {
        ClxConfigInteger supportSecureSimplePairing;
        ClxConfigInteger enableLinkLevelAuthentication;
        ClxConfigInteger ioCapabilities;
        ClxConfigInteger linkRequestTimeout;
        ClxConfigInteger majorClassOfDevice;
        ClxConfigInteger minorClassOfDevice;
        ClxConfigInteger serviceClasses;

        ClxConfigString localDeviceName;
    };

    GenericConfigParameters genericConfigParameters;

    ClxConfigList* configList = initializeClarinoxBlueBspConfigParameters();

    clxConfigInitIntegerParam(&genericConfigParameters.supportSecureSimplePairing,      "SupportSecureSimplePairing",       TRUE,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.enableLinkLevelAuthentication,   "EnableLinkLevelAuthentication",    FALSE,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.ioCapabilities,                  "IoCapabilities",                   IO_DISPLAY_YES_NO,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.linkRequestTimeout,              "LinkRequestTimeout",               16000,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.majorClassOfDevice,              "MajorClassOfDevice",               MJ_AUDIO_VIDEO,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.minorClassOfDevice,              "MinorClassOfDevice",               AVMI_CAR_AUDIO,   configList); 
    clxConfigInitStringParam(&genericConfigParameters.localDeviceName,                  "LocalDeviceName",                  deviceName, configList); 

    clxConfigInitIntegerParam(&genericConfigParameters.serviceClasses,                  "ServiceClasses",                   ( ST_LIMITED_DISCOVERABLE_MODE |
                                                                                                                              ST_POSITIONING               |
                                                                                                                              ST_NETWORKING                |
                                                                                                                              ST_RENDERING                 |
                                                                                                                              ST_CAPTURING                 |
                                                                                                                              ST_OBJECT_TRANSFER           |
                                                                                                                              ST_AUDIO                     |
                                                                                                                              ST_TELEPHONY                 |
                                                                                                                              ST_INFORMATION ),   configList); 

    if (clxInitClarinoxBlue(configList,                    /* Initialize the Bluetooth stack                         */
        FALSE,
        stackMessageHandler,
        userExceptionHandler,
        stack, TRUE) != CLX_SUCCESS)
    {
        ClxConsoleUIEngine::text("Initialization of ClarinoxBlue failed\n");
        ret = FALSE;
    }
    else
    {
        ClxConsoleUIEngine::text("ClarinoxBlue version %s.%u \n",
                                    clxGetClarinoxBlueVersion(),
                                    clxGetClarinoxBlueRevision());
    }

    releaseClarinoxBlueBspConfigParameters(configList);

    return ret;
}

/*
Main function initializes the ClarinoxBlue stack
*/
#if defined ( CLX_UCOS_III ) || defined ( CLX_THREADX ) || defined ( CLX_NUCLEUS ) || defined (CLX_FREE_RTOS)
int mainF(void)
#else
int main(void)
#endif
{
    clxInitBsp();
    
	ClxStack stack = NULL; 
	/*
    Initialize user interface engine and start a menu for possible operations
    */
	ClxConsoleUIEngine::init(128);   // input keyboard buffer size
    ClxConsoleUIEngine::text("\nMake sure to initialize the Bluetooth stack first.\n");

    static bool running = TRUE;
    while(running)
    {
        const s1* menu = "Initialize Bluetooth Stack\0"
                         "Terminate Bluetooth Stack\0"
						 "Disconnect from a paired device\0"
                         "Start Advertising\0"
                         "Stop Advertising\0"
                         "Start Scanning\0"
                         "Stop Scanning\0"
                         "Discover Primary Services\0"
                         "Discover All Characteristics of a Service\0"
                         "Read Characteristic Value\0"
                         "Write Characteristic Value\0"
                         "Delete the oldest pairing information\0"
                         "Delete all paired devices\0"
                         ;

        u4 index = ClxConsoleUIEngine::showMenu("Enter your selection:", menu, 13); 
        /*
        Handle menu selection
        */
		if (index == 0)
		{
        }
        else if ((stack == 0) && (index == 1))
        {
            /* 
            Initialize the Bluetooth stack with the parameters; params, event call-back function; stackMessageHandler
            and the exception handler; userExceptionHandler 
            */
            if (!initializeClarinoxBlue(&stack))
            {
                return 0;
            }
            
            terminationSemaphore = clxCreateSemaphore(0);

        }
        else if ((index == 2) && (stack != 0))
        {

            //deleteGattHandle();
           /* clxTerminateClarinoxBlue(stack);
            clxAcquireSemaphore(terminationSemaphore);
            clxDeleteSemaphore(terminationSemaphore);*/
            /*
            Destroy the stack, after this call is completed, stack can be initialized again
            */
          /*  clxDestroyClarinoxBlue(stack);
            stack = 0;
            ClxConsoleUIEngine::destroy();
            ClxConsoleUIEngine::init(32);
            ClxConsoleUIEngine::text("\nBluetooth stack is terminated, make sure to initialize the Bluetooth stack\n");*/
            
        }
        else if ((index == 3) && (stack != 0))
        {
            clxGapDisconnectFromPeripheral( stack, &bleAddress, TRUE);
        }
        else if ((index == 4) && (stack != 0))
        {
            ClxBleBdAddress directAddress;
            directAddress.addressType = ClxBleAddressType_Public;
            memset( directAddress.value, 0, 6);
            s1 shortName[] = "Clx Ble Device";
            s1 longName[] = "This is a huge name for this bluetooth device. This will cause a long attribute read. This is used to test blob read with Bluetooth Low Energy Generic Access Service000011119912";
            advertisingData[0] = strlen(shortName) + 1;
            advertisingData[1] = 9;
            memcpy(&advertisingData[2], shortName, strlen(shortName));
            clxGapBleStartAdvertising(stack,
                                        3000,
                                        4000,
                                        ClxBleAdvertisingType_ConnectableUndirected,
                                        ClxBleAddressType_Public,
                                        &directAddress,
                                        ClxBleAdvertisingFilterPolicy_ScanConnectionAnyone,
                                        advertisingData,
                                        strlen(shortName) + 2,
                                        FALSE );
            gattHandle = createGattHandle(stack, longName, ClxGattGapAppearance_GenericDisplay, true);
        }
        else if ((index == 5) && (stack != 0))
        {
            clxGapBleStopAdvertising(stack, FALSE);
        }
        else if ((index == 6) && (stack != 0))
        {
            gattHandle = createGattHandle(stack, 0, 0, false);
            clxGapBleStartScan(stack, BleScanType_Passive, 100, 80, FALSE, FALSE);
           // gattHandle = createGattHandle(stack, "", ClxGattGapAppearance_GenericDisplay, false);
            //menuFunction(stack);
            //discoverDevices(stack, FALSE);
        }
        else if ((index == 7) && (stack != 0))
        {
            clxGapBleStopScan(stack, FALSE);
        }
        else if((index==8) && (stack != 0))
        {
            ClxGattUuid u;
            ClxGattUuid id;
            clxInitGattUuid2(&id, 0x0000);
            ClxServiceList sl;
            sl.list = new ClxServiceDetail[100]; //MAX_NUM_SERVICES = 20
            clxGattClientGetListOfServices(gattHandle, id, &sl, false);            
        }
       else if((index==9) && (stack != 0))
        {
            ClxGattUuid u;
            ClxGattUuid id;
            clxInitGattUuid2(&id, 0x0000);
            ClxServiceDetail sd;
            ClxConsoleUIEngine::inputBox ("Enter the first handle of the Service (hex) to search: ", handle_in, 128);
            //u4 handleIndex_ = atoi(handle_in);
            u4 handleIndex_;
            sscanf(handle_in, "%x", &handleIndex_);

            sd.firstHandle = (u2)handleIndex_;
            ClxConsoleUIEngine::inputBox ("Enter the last handle of the Service (hex) to search: ", handle_in, 128);
            //handleIndex_ = atoi(handle_in);
            sscanf(handle_in, "%x", &handleIndex_);

            sd.lastHandle = (u2)handleIndex_;
            
            cl.list = new ClxCharacteristicDetail[100];
            cl.numberOfCharacteristics = 0;           

            clxGattClientDiscoverCharacteristics(gattHandle, id, &sd, &cl, false);     
        }
        else if((index==10) && (stack != 0))
        {
            ClxGattUuid id;
            clxInitGattUuid2(&id, 0x0000);
            ClxConsoleUIEngine::inputBox ("Enter number of characteristics to read: ", handle_in, 128);
            u4 numToRead = atoi(handle_in);           
            
            
            ClxCharacteristicList cl;
            cl.list = new ClxCharacteristicDetail[numToRead];
            
            cl.numberOfCharacteristics = 0;

            for(u4 i = 0; i<numToRead; i++)
            {
                ClxConsoleUIEngine::inputBox ("Enter the characteristic handle index (hex): ", handle_in, 128);
                //u4 handleIndex_ = atoi(handle_in);
                u4 handleIndex_;
                sscanf(handle_in, "%x", &handleIndex_);
                cl.list[i].valueHandle = handleIndex_;
                cl.numberOfCharacteristics++;
            }
            
           
            u1* buf = new u1[400]; 
            u4 length = 0;

            
            clxGattClientReadCharacteristicValue(gattHandle,id,&cl,buf,400, &length,false);
        }
        else if((index==11) && (stack != 0))
        {
             //if no write response is required, optionsFlag = 0x1
            ClxCharacteristicDetail cd ;           
            ClxConsoleUIEngine::inputBox ("Enter handle of the characteristic (hex) to write to: ", handle_in, 128);
            // u4 numToRead = atoi(handle_in);
            u4 numToRead;
            sscanf(handle_in, "%x", &numToRead);
            cd.valueHandle = numToRead;
            s1 message[128];
            ClxConsoleUIEngine::inputBox ("enter the text value for the characteristic - beware: NO SIZE CHECK:", message, 128);


            clxGattClientWriteCharacteristicValue(gattHandle,&cd,0,(u1*)message,strlen(message),false); 
        }
        else if ((index == 12) && (stack != 0))
        {
            ClxError err = clxGapDeletePairedDeviceInfo(stack, 0, TRUE);
            if (err == CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("\nOldest pairing information deleted successfully.\n");
            }
            else
            {
                ClxConsoleUIEngine::text("Deleting oldest pairing information completed with error code %u\n", err);
            }
        }
        else if ((index == 13) && (stack != 0))
        {
            ClxError err = clxGapDeleteAllPairedDevicesInfo(stack, 1);
            if (err == CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("\nAll paired devices deleted successfully.\n");
            }
            else
            {
                ClxConsoleUIEngine::text("Deleting paired devices completed with error code %u\n", err);
            }
        }
        
    }
    return 0;
}

#if defined ( CLX_UCOS_III ) || defined ( CLX_THREADX ) || defined ( CLX_NUCLEUS ) || defined (CLX_FREE_RTOS)
extern "C" {
  int clarinoxMain(void)
 {
    mainF();
    return 0;
 }
}
#endif
