/*******************************************************************************
*
* Project             GattBle
* File                GattService.cpp
* Description         Declares attribute database for Test GATT Service
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "ClxCommon.h"
#include "ClarinoxBlue.h"
#include "Gatt.Ble.Common.Api.h"
#include "Gatt.Ble.Service.h"
#include <new>

// #define BLE_DEVICE_INFO_SERVICE

#include "GattService.h"



class ClxBleGattTestDatabase : public ClxBleGattDatabase
{
public:
    ClxBlePrimaryServiceAttribute                       primaryService;

    ClxBleCharacteristicDeclarationAttribute            deviceName_CharacteristicDeclarationAttribute;
    ClxBleCharacteristicValueAttribute                  deviceName_CharacteristicValueAttribute;
    u4                                                  deviceName_Storage[(CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_SIZE + 3)/4];

    ClxBleCharacteristicDeclarationAttribute            maxLength_CharacteristicDeclarationAttribute;
    ClxBleCharacteristicValueAttribute                  maxLength_CharacteristicValueAttribute;
    u4                                                  maxLength_Storage[(CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_SIZE + 3)/4];

    ClxBleCharacteristicDeclarationAttribute            clientLength_CharacteristicDeclarationAttribute;
    ClxBleCharacteristicValueAttribute                  clientLength_CharacteristicValueAttribute;
    u4                                                  clientLength_Storage[(CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_SIZE + 3)/4];

    ClxBleCharacteristicDeclarationAttribute            sendData_CharacteristicDeclarationAttribute;
    ClxBleCharacteristicValueAttribute                  sendData_CharacteristicValueAttribute;
    ClxBleCharacteristicClientConfigurationAttribute    sendData_CharacteristicClientConfigurationAttribute;
    u4                                                  sendData_Storage[(CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_SIZE + 3)/4];

    ClxBleCharacteristicDeclarationAttribute            rxData_CharacteristicDeclarationAttribute;
    ClxBleCharacteristicValueAttribute                  rxData_CharacteristicValueAttribute;
    u4                                                  rxData_Storage[(CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_SIZE + 3)/4];

#ifdef BLE_DEVICE_INFO_SERVICE
    ClxBleCharacteristicDeclarationAttribute            deviceInformationMessage_CharacteristicDeclarationAttribute[5];
    ClxBleFixedLengthCharacteristicValueAttribute       deviceInformationMessage_CharacteristicValueAttribute[5];
    u4                                                  deviceInformationMessage_Storage[(CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_SIZE+3)/4][5];

    ClxBleAttribute*                                    list[22];
#else
    ClxBleAttribute*                                    list[12];
#endif
public:
    ClxBleGattTestDatabase()
    {
        ClxBleGattDatabase::attrList = list;
        ClxBleGattDatabase::numberOfAttributes = sizeof(list)/sizeof(ClxBleAttribute*);

        list[0] = (ClxBleAttribute*)&primaryService;

        list[1] = (ClxBleAttribute*)&deviceName_CharacteristicDeclarationAttribute;
        list[2] = (ClxBleAttribute*)&deviceName_CharacteristicValueAttribute;

        list[3] = (ClxBleAttribute*)&maxLength_CharacteristicDeclarationAttribute;
        list[4] = (ClxBleAttribute*)&maxLength_CharacteristicValueAttribute;

        list[5] = (ClxBleAttribute*)&clientLength_CharacteristicDeclarationAttribute;
        list[6] = (ClxBleAttribute*)&clientLength_CharacteristicValueAttribute;

        list[7] = (ClxBleAttribute*)&sendData_CharacteristicDeclarationAttribute;
        list[8] = (ClxBleAttribute*)&sendData_CharacteristicValueAttribute;
        list[9] = (ClxBleAttribute*)&sendData_CharacteristicClientConfigurationAttribute;

        list[10] = (ClxBleAttribute*)&rxData_CharacteristicDeclarationAttribute;
        list[11] = (ClxBleAttribute*)&rxData_CharacteristicValueAttribute;

#ifdef BLE_DEVICE_INFO_SERVICE
        list[12] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicDeclarationAttribute[0];
        list[13] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicValueAttribute[0];

        list[14] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicDeclarationAttribute[1];
        list[15] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicValueAttribute[1];

        list[16] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicDeclarationAttribute[2];
        list[17] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicValueAttribute[2];

        list[18] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicDeclarationAttribute[3];
        list[19] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicValueAttribute[3];

        list[20] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicDeclarationAttribute[4];
        list[21] = (ClxBleAttribute*)&deviceInformationMessage_CharacteristicValueAttribute[4];
#endif
    }

    ~ClxBleGattTestDatabase()
    {}
};


static ClxBleGattDatabase* initTestGattServiceDatabase(const ClxConfigList*)
{
    void* tmp = clxPoolsetAlloc(0, 0, sizeof(ClxBleGattTestDatabase)); //new ClxBleGattTestDatabase;
    ClxBleGattTestDatabase* ret = new (tmp)ClxBleGattTestDatabase;

    /* Primary Service: */
    {
        ClxGattUuid uuid;
        if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_PRIMARY_SERVICE_UUID))
        {
            return NULL;
        }

        clxInitBlePrimaryServiceAttribute(&ret->primaryService, &uuid, (sizeof(ret->list) / sizeof(ClxBleAttribute*) - 1));
    }

    /* Device Name */
    {
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_UUID))
            {
                return NULL;
            }
       
            clxInitBleCharacteristicDeclarationAttribute(&ret->deviceName_CharacteristicDeclarationAttribute, &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_UUID))
            {
                return NULL;
            } 

            clxInitBleCharacteristicValueAttribute(&ret->deviceName_CharacteristicValueAttribute, &uuid, ClxBleAttributeValueType_FixedLength, (u1*)ret->deviceName_Storage, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_PERMISSIONS, NULL, NULL);
        }
        /* </Value> */
    }

    /* Max Length (our maximum length) */
    {
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_UUID))
            {
                return NULL;
            }
       
            clxInitBleCharacteristicDeclarationAttribute(&ret->maxLength_CharacteristicDeclarationAttribute, &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_UUID))
            {
                return NULL;
            } 

            clxInitBleCharacteristicValueAttribute(&ret->maxLength_CharacteristicValueAttribute, &uuid, ClxBleAttributeValueType_FixedLength, (u1*)ret->maxLength_Storage, CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_PERMISSIONS, NULL, NULL);
        }
        /* </Value> */
    }

    /* Client Length (their maximum length) */
    {
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_UUID))
            {
                return NULL;
            }
       
            clxInitBleCharacteristicDeclarationAttribute(&ret->clientLength_CharacteristicDeclarationAttribute, &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_UUID))
            {
                return NULL;
            } 

            clxInitBleCharacteristicValueAttribute(&ret->clientLength_CharacteristicValueAttribute, &uuid, ClxBleAttributeValueType_FixedLength, (u1*)ret->clientLength_Storage, CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_PERMISSIONS, NULL, NULL);
        }
        /* </Value> */
    }

    /* Send data - we transmit on this */
    {
        /* <Declaration> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_UUID))
            {
                return NULL;
            }

            clxInitBleCharacteristicDeclarationAttribute(&ret->sendData_CharacteristicDeclarationAttribute, &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_UUID))
            {
                return NULL;
            }

            clxInitBleCharacteristicValueAttribute(&ret->sendData_CharacteristicValueAttribute, &uuid, ClxBleAttributeValueType_VariableLength, (u1*)ret->sendData_Storage, CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_PERMISSIONS, NULL, NULL);
        }
        /* </Value> */

        /* <ClientConfiguration> */
        clxInitBleCharacteristicClientConfigurationAttribute(&ret->sendData_CharacteristicClientConfigurationAttribute);
        /* <C/lientConfiguration> */
    }
    /* </Characteristic> */

    /* Rx Data - they send us data on this */
    {
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_UUID))
            {
                return NULL;
            }
       
            clxInitBleCharacteristicDeclarationAttribute(&ret->rxData_CharacteristicDeclarationAttribute, &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_UUID))
            {
                return NULL;
            } 

            clxInitBleCharacteristicValueAttribute(&ret->rxData_CharacteristicValueAttribute, &uuid, ClxBleAttributeValueType_VariableLength, (u1*)ret->rxData_Storage, CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_PERMISSIONS, NULL, NULL);
        }
        /* </Value> */
    }


#ifdef BLE_DEVICE_INFO_SERVICE
//TODO -- DeviceInformationMessages are currently not supported
    /* <Characteristic name="DeviceInformationMessage1"> */
    {
        /* <Declaration> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE1_UUID))
            {
                return NULL;
            }

            clxInitBleCharacteristicDeclarationAttribute(&ret->deviceInformationMessage_CharacteristicDeclarationAttribute[0], &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE1_UUID))
            {
                return NULL;
            }

            clxInitBleFixedLengthCharacteristicValueAttribute(&ret->deviceInformationMessage_CharacteristicValueAttribute[0], &uuid, (u1*)ret->deviceInformationMessage_Storage[0], CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_PERMISSIONS);
        }
        /* </Value> */
    }
    /* </Characteristic> */


    /* <Characteristic name="DeviceInformationMessage2"> */
    {
        /* <Declaration> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE2_UUID))
            {
                return NULL;
            }

            clxInitBleCharacteristicDeclarationAttribute(&ret->deviceInformationMessage_CharacteristicDeclarationAttribute[1], &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE2_UUID))
            {
                return NULL;
            }

            clxInitBleFixedLengthCharacteristicValueAttribute(&ret->deviceInformationMessage_CharacteristicValueAttribute[1], &uuid, (u1*)ret->deviceInformationMessage_Storage[1], CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_PERMISSIONS);
        }
        /* </Value> */
    }
    /* </Characteristic> */


    /* <Characteristic name="DeviceInformationMessage3"> */
    {
        /* <Declaration> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE3_UUID))
            {
                return NULL;
            }

            clxInitBleCharacteristicDeclarationAttribute(&ret->deviceInformationMessage_CharacteristicDeclarationAttribute[2], &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE3_UUID))
            {
                return NULL;
            }

            clxInitBleFixedLengthCharacteristicValueAttribute(&ret->deviceInformationMessage_CharacteristicValueAttribute[2], &uuid, (u1*)ret->deviceInformationMessage_Storage[2], CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_PERMISSIONS);
        }
        /* </Value> */
    }
    /* </Characteristic> */


    /* <Characteristic name="DeviceInformationMessage4"> */
    {
        /* <Declaration> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE4_UUID))
            {
                return NULL;
            }

            clxInitBleCharacteristicDeclarationAttribute(&ret->deviceInformationMessage_CharacteristicDeclarationAttribute[3], &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE4_UUID))
            {
                return NULL;
            }

            clxInitBleFixedLengthCharacteristicValueAttribute(&ret->deviceInformationMessage_CharacteristicValueAttribute[3], &uuid, (u1*)ret->deviceInformationMessage_Storage[3], CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_PERMISSIONS);
        }
        /* </Value> */
    }
    /* </Characteristic> */


    /* <Characteristic name="DeviceInformationMessage5"> */
    {
        /* <Declaration> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE5_UUID))
            {
                return NULL;
            }

            clxInitBleCharacteristicDeclarationAttribute(&ret->deviceInformationMessage_CharacteristicDeclarationAttribute[4], &uuid);
        }
        /* </Declaration> */

        /* <Value> */
        {
            ClxGattUuid uuid;
            if(!clxInitGattUuid16FromAscii(&uuid, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE5_UUID))
            {
                return NULL;
            }

            clxInitBleFixedLengthCharacteristicValueAttribute(&ret->deviceInformationMessage_CharacteristicValueAttribute[4], &uuid, (u1*)ret->deviceInformationMessage_Storage[4], CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_SIZE, CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_PERMISSIONS);
        }
        /* </Value> */
    }
    /* </Characteristic> */
#endif

    return ret;
}

static void destroyTestGattServiceDatabase(ClxBleGattDatabase* database)
{
    delete static_cast<ClxBleGattTestDatabase*>(database);
}



#ifdef __cplusplus
extern "C" {
#endif


static ClxBleGattServiceInterface clxBleGattTestServiceInterface =
{
    initTestGattServiceDatabase,
    destroyTestGattServiceDatabase
};

const ClxBleGattServiceInterface* clxGetBleGattTestServiceInterface()
{
    return &clxBleGattTestServiceInterface;
}

#ifdef __cplusplus
}
#endif

