#ifndef __Gatt_Service_Api_h__
#define __Gatt_Service_Api_h__

/*******************************************************************************
*
* Project             GattBle
* File                GattService.h
* Description         Declares attribute database for Test GATT Service
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

#include "Gatt.Ble.Service.h"

#define CLX_GATT_SERVICE_PRIMARY_SERVICE_UUID "50d8c1f0-9464-11e3-baa8-0800200c9a66"

// Our Device Name / URL
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_UUID         "80bed780-c590-11e3-9c1a-0800200c9a66"
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_HANDLE_INDEX (2)
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_SIZE         (10)
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_NAME_PERMISSIONS  CLX_GATT_ATTRIBUTE_PERMISSION_READABLE

// Longest packet we can handle
#define CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_UUID         "0e4abb90-ead1-11e3-ac10-0800200c9a66"
#define CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_HANDLE_INDEX (4)
#define CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_SIZE         (2)
#define CLX_GATT_SERVICE_CHARACTERISTIC_MAX_LENGTH_PERMISSIONS  CLX_GATT_ATTRIBUTE_PERMISSION_READABLE

// Longest packet the client can handle
#define CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_UUID         "38c18d40-ead1-11e3-ac10-0800200c9a66"
#define CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_HANDLE_INDEX (6)
#define CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_SIZE         (2)
#define CLX_GATT_SERVICE_CHARACTERISTIC_CLIENT_LENGTH_PERMISSIONS  CLX_GATT_ATTRIBUTE_PERMISSION_WRITABLE

// We transmit packets on this characteristic
#define CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_UUID         "fffb4000-ead0-11e3-ac10-0800200c9a66"
#define CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_HANDLE_INDEX (8)
// TODO - make this 100 bytes when the stack can handle it
#define CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_SIZE         (100)
#define CLX_GATT_SERVICE_CHARACTERISTIC_SEND_DATA_PERMISSIONS  CLX_GATT_ATTRIBUTE_PERMISSION_READABLE | CLX_GATT_ATTRIBUTE_PERMISSION_NOTIFIABLE

// They send us packets on this characteristic
#define CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_UUID         "08c05bb0-ead3-11e3-ac10-0800200c9a66"
#define CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_HANDLE_INDEX (11)
// TODO - make this 100 bytes when the stack can handle it
#define CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_SIZE         (100)
#define CLX_GATT_SERVICE_CHARACTERISTIC_RX_DATA_PERMISSIONS  CLX_GATT_ATTRIBUTE_PERMISSION_WRITABLE


#ifdef BLE_DEVICE_INFO_SERVICE
// TODO - not currently supported
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_SIZE            20 // Bytes  
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE_PERMISSIONS     CLX_GATT_ATTRIBUTE_PERMISSION_READABLE | CLX_GATT_ATTRIBUTE_PERMISSION_WRITABLE

#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE1_UUID            "d246cbbd-cd4c-4652-8bfc-98080136987f"
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE1_HANDLE_INDEX    13

#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE2_UUID            "a9151df3-fbbb-4149-a211-8a42c1beca81"
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE2_HANDLE_INDEX    15

#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE3_UUID            "eb407542-109b-44b8-a48b-78f5334c3dd1"
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE3_HANDLE_INDEX    17

#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE4_UUID            "74b09aa8-a0e9-44ec-8378-5384bda01d87"
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE4_HANDLE_INDEX    19

#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE5_UUID            "d169df95-b40d-4e35-beae-b52581d47394"
#define CLX_GATT_SERVICE_CHARACTERISTIC_DEVICE_INFORMATION_MESSAGE5_HANDLE_INDEX    21
#endif


#ifdef __cplusplus
extern "C" {
#endif

const ClxBleGattServiceInterface* clxGetBleGattTestServiceInterface();


#ifdef __cplusplus
}
#endif

#endif  // __Gatt_Service_Api_h__

