/******************************************************************************
*
* Project             Bluetooth Application
* File                ServiceDiscovery.cpp
* Description         Service Discovery application
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

// _____________________________________________________________________________
//
#undef  CLX_MODULE_ID
#define CLX_MODULE_ID  20024
// _____________________________________________________________________________
//

#include "ClxCommon.h"
#include "ClarinoxBlue.h"
#include "Sdap.Api.h"

const s1* servicesList[] = {"Headset Profile (HSP) Headset role",
                            "Headset Profile (HSP) AudioGateway role",
                            "Hands-Free Profile (HFP) Hands-Free role",
                            "Hands-Free Profile (HFP) AudioGateway role",
                            "Serial Port Profile (SPP)",
                            "Phonebook Access Profile (PBAP) Client role",
                            "Phonebook Access Profile (PBAP) Server role",
                            "Object Push Profile (OPP)",
                            "Message Notification Server (MNS) of Message Access Profile (MAP)",
                            "Message Access Server (MAS) of Message Access Profile (MAP)",
                            "SIM Access Profile (SAP) Server role",
                            "File Transfer Profile (FTP) Server Role",
                            "Audio-Video Remote Control Profile (AVRCP) Target role",
                            "Audio-Video Remote Control Profile (AVRCP) Controller role",
                            "Audio Advanced Distribution Profile (A2DP) Sink role",
                            "Audio Advanced Distribution Profile (A2DP) Source role",
                            "Dialup Networking Profile (DUN)",
                            "Personal Area Networking Profile (PAN)",
                            "Human Interface Device Profile (HID)",
                            "Video Distribution Profile (VDP) Sink role",
                            "Video Distribution Profile (VDP) Source role",
                            "Health Device Profile (HDP) Sink role",
                            "Health Device Profile (HDP) Source role",
                            "Synchronization Profile (SYNC)",
                            "DeviceID Profile (DI)"};

/*******************************************************************************************************************************
*                                                       SdapIndicationHandler
*
* This call-back function is registered for the SDP protocol, any events raised by SDP protocol causes this call-back function
* executed with the associated event and parameters.
*
*******************************************************************************************************************************/
boolean SdapIndicationHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode)
{
    return false;
}

/*******************************************************************************************************************************
*                                                       discoverServices
*
* Create the SDP handle, and send a SDP request to the remote device, and inform the user of the available services
* in the remote device and close the SDP handled.
*
*******************************************************************************************************************************/
void discoverServices(ClxStack stack, ClxDeviceId deviceId)
{
    u4 read = 0;
    ClxError ret = CLX_FAIL;

    ClxBlueServiceInfo services[30];

    ClxHandle sdap = clxSdapCreate(stack, SdapIndicationHandler);

    ClxConsoleUIEngine::text ("\nDiscovering Services . . .\n");

    ret = clxSdapDiscoverRemoteServices(sdap, deviceId, services, 20, &read, CLX_SDAP_FILTER_ALL, TRUE);

    if (ret != CLX_SUCCESS)
    {
        ClxConsoleUIEngine::text ("\nError : %s\n", clxGetErrorCodeText(ret));
    }
    else if (read == 0)
    {
        ClxConsoleUIEngine::text ("\nNo service discovered\n", clxGetErrorCodeText(ret));
    }
    else
    {
        ClxConsoleUIEngine::text ("\nThe following services are supported:\n\n", clxGetErrorCodeText(ret));

        for (u4 i = 0; i < read; i++)
        {
            ClxConsoleUIEngine::text ("%u. %s\n", i+1, servicesList[(u1)services[i].type]);
        }
    }
    ClxConsoleUIEngine::text ("\n");

    clxCloseHandle(sdap);
}

