/*******************************************************************************
*
* Project             SPP Sample Application
* File                main.cpp
* Description         main application file
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

// _____________________________________________________________________________
//
#undef  CLX_MODULE_ID
#define CLX_MODULE_ID  20123
// _____________________________________________________________________________
//

#include "ClxBsp.h"
#include "ClarinoxBlue.h"

#include "ClxList.h"

#include "Gap.Api.h"
#include "Spp.Api.h"
#include "SppApp.h"

#include "string.h"
#include "stdio.h"

extern void menuFunction(ClxStack stack, bool waitForConnection);

#if defined (CLX_UCOS_III) | (CLX_THREADX)
extern ClxResult CLX_CALLBACK cameraThread( void* data );
extern bool             cameraRunning;
extern ClxSemaphore     cameraSemaphore;
#endif //defined (CLX_UCOS_III) | (CLX_THREADX)

s1* deviceName = "ClxSpp_Windows";                        /* Local Bluetooth Device name                            */ 

/*
Synchronisation for connection and termination
*/
ClxSemaphore sendSemaphore;
ClxSemaphore terminationSemaphore;


/*
Time measurement variables for diagnostic purposes 
*/
static u4 startTime;
static u4 endTime;


#ifdef __cplusplus
extern "C" {
#endif
 extern void clxInitBsp(void );
#ifdef __cplusplus
}
#endif

extern boolean stackMessageHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode);
static ClxHandle sppHandle    = 0;
/*------------------------------- Device discovery procedure -------------------------------*/
/**
Starts the inquiry (device discovery) process
*/
void discoverDevices(ClxStack stack, boolean liacMode)
{
	s1 ch = 0;

	struct ClxGapInquiryFilter filter;

	filter.filterMask = ClxNoInquiryFilter;
    /*
    Start inquiry in one of the inquiry modes 
    (most application use general inquiry mode, only special applications require limited inquiry)
    */
	ClxError err;
	if (liacMode)
	{
		err = clxGapStartLimitedInquiry(stack, &filter, TRUE);
	}
	else 
	{
		err = clxGapStartInquiry(stack, &filter, TRUE);
	}
	startTime = clxTickTime();
    /*
    Wait until enough number of devices discovered. While the main application
    is waiting here, each discovered device will cause an application callback 
    to be triggered with the CLX_GAP_DEVICE_DISCOVERED_INDICATION event id.
    */
	ClxConsoleUIEngine::text ("\nDevice discovery started at %u\n ", startTime);
	do
    {
        ch = ClxConsoleUIEngine::messageBox("Enter c to stop searching", "cC", 2);        
    }
    while ((ch != 'c') && (ch != 'C'));
	ClxConsoleUIEngine::text ("\nDevice discovery stopped . . .\n");
	endTime = clxTickTime();
	ClxConsoleUIEngine::text ("\nDevice discovery ended at %u . .\n", endTime);

	ClxConsoleUIEngine::text("\nTime elapsed in discovering: %u msec\n", (endTime - startTime));
	
    /*
    Stop inquiry so that the Bluetooth radio is fully available for data transmission
    */
	err = clxGapStopInquiry(stack, TRUE);

	ClxDeviceDetail list[20];
	u4 listSize = 20;
    /*
    Present the list of devices discovered during the inquiry operation to the user
    */
	err = clxGapGetListOfDiscoveredDevices(stack, list, 20, &listSize, TRUE);

	if (listSize)
	{
		const s1 returnText[] = {"Return to previous menu"};
		s1* menuItems = new s1[listSize*64 + sizeof(returnText)];
		menuItems[0] = '\0';
		
		u4 index = 0;
        for (u4 i = 0; i < listSize; i++) 
        { 
            /* 
            Add the name and the RSSI value of the device to the UI menu to select a device
            */
            strcpy(menuItems + index, list[i].deviceName);
            s1 rssi[16];
            sprintf(rssi, " (RSSI: %ddBm)", list[i].rssi);
            strcat(menuItems + index, rssi);
            index += (strlen(menuItems + index) + 1);
        }
        
        strcpy (menuItems + index, returnText);

        u4 s = ClxConsoleUIEngine::showMenu("\nSelect a device to pair", menuItems, listSize + 1);

        delete[] menuItems;

		// Optionally discover all the services provided by the selected device 
        // discoverServices(stack, list[s-1].deviceId);

        /*
        Pair with teh selected device; this will potentially cause some callback functions
        depending on the authentication mode (i.e. pincode entry or passcode verification etc)
        */
        ClxConsoleUIEngine::text ("Pairing with the remote device . . .\n");
        startTime = clxTickTime();
        ClxSecurityRequirement securityRequirement = ClxMediumSecurity;
        err = clxGapInitiateBonding(stack, list[s - 1].deviceId, securityRequirement, TRUE);
		if (err != CLX_SUCCESS)
		{
 			ClxConsoleUIEngine::text ("Pairing complete with the result : %s\n", clxGetErrorCodeText(err));
		}
        endTime = clxTickTime();
        ClxConsoleUIEngine::text("Time elapsed in pairing: %u msec\n", endTime - startTime);
        /*
        Connect to the paired device
        */
        //connect(stack,connectedDeviceId, sppHandle, FALSE);
        //menuFunction();
    }
}


/*******************************************************************************************************************************
*                                                User Exception Handler
*
* Trap the stack exceptions.
*
*******************************************************************************************************************************/
void userExceptionHandler( s4 moduleId, s4 lineNumber )
{
    printf("ClarinoxBlue raised an exception in module: %u, line: %u\n", moduleId, lineNumber);
}

boolean initializeClarinoxBlue(ClxStack* stack)
{
    boolean ret = TRUE;

    struct GenericConfigParameters
    {
        ClxConfigInteger supportSecureSimplePairing;
        ClxConfigInteger enableLinkLevelAuthentication;
        ClxConfigInteger ioCapabilities;
        ClxConfigInteger linkRequestTimeout;
        ClxConfigInteger majorClassOfDevice;
        ClxConfigInteger minorClassOfDevice;
        ClxConfigInteger serviceClasses;

        ClxConfigString localDeviceName;
    };

    GenericConfigParameters genericConfigParameters;

    ClxConfigList* configList = initializeClarinoxBlueBspConfigParameters();

    clxConfigInitIntegerParam(&genericConfigParameters.supportSecureSimplePairing,      "SupportSecureSimplePairing",       TRUE,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.enableLinkLevelAuthentication,   "EnableLinkLevelAuthentication",    FALSE,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.ioCapabilities,                  "IoCapabilities",                   IO_DISPLAY_YES_NO,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.linkRequestTimeout,              "LinkRequestTimeout",               16000,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.majorClassOfDevice,              "MajorClassOfDevice",               MJ_AUDIO_VIDEO,   configList); 
    clxConfigInitIntegerParam(&genericConfigParameters.minorClassOfDevice,              "MinorClassOfDevice",               AVMI_CAR_AUDIO,   configList); 
    clxConfigInitStringParam(&genericConfigParameters.localDeviceName,                  "LocalDeviceName",                  deviceName, configList); 

    clxConfigInitIntegerParam(&genericConfigParameters.serviceClasses,                  "ServiceClasses",                   ( ST_LIMITED_DISCOVERABLE_MODE |
                                                                                                                              ST_POSITIONING               |
                                                                                                                              ST_NETWORKING                |
                                                                                                                              ST_RENDERING                 |
                                                                                                                              ST_CAPTURING                 |
                                                                                                                              ST_OBJECT_TRANSFER           |
                                                                                                                              ST_AUDIO                     |
                                                                                                                              ST_TELEPHONY                 |
                                                                                                                              ST_INFORMATION ),   configList); 

    if (clxInitClarinoxBlue(configList,                    /* Initialize the Bluetooth stack                         */
        FALSE,
        stackMessageHandler,
        userExceptionHandler,
        stack, TRUE) != CLX_SUCCESS)
    {
        ClxConsoleUIEngine::text("Initialization of ClarinoxBlue failed\n");
        ret = FALSE;
    }
    else
    {
        ClxConsoleUIEngine::text("ClarinoxBlue version %s.%u \n",
                                    clxGetClarinoxBlueVersion(),
                                    clxGetClarinoxBlueRevision());
    }

    releaseClarinoxBlueBspConfigParameters(configList);

    return ret;
}


/*
Main function initializes the ClarinoxBlue stack
*/
#if defined ( CLX_UCOS_III ) || defined ( CLX_THREADX ) || defined ( CLX_NUCLEUS ) || defined (CLX_FREE_RTOS) || defined (CLX_MQX)
int mainF(void)
#else
int main(void)
#endif
{

    ClxStack stack = NULL;                                          /* A stack object must be created before any other        */
                                                                    /* profiles are created                                   */

    clxInitBsp();                                                   /* Board support package initialization                   */

	/*
    Initialize user interface engine and start a menu for possible operations
    */
	ClxConsoleUIEngine::init(32);   
    ClxConsoleUIEngine::text("\nMake sure to initialize the Bluetooth stack first.\n");

    static bool running = TRUE;
    while(running)
    {
        const s1* menu = "Initialize Bluetooth Stack\0"
                         "Terminate Bluetooth Stack\0"
						 "Connect to a paired device\0"
                         "Wait for an incoming connection request\0"
                         "Search for devices in proximity\0"
                         "Make this device discoverable\0"
                         "Make this device non-discoverable\0"
                         "Go to connected device menu\0"
                         "Delete the oldest pairing information\0"
                         "Delete all paired devices\0"
                         "Display Camera\0";

        u4 index = ClxConsoleUIEngine::showMenu("Enter your selection:", menu, 11); 
        /*
        Handle menu selection
        */
		if (index == 0)
		{
        }
        else if ((stack == 0) && (index == 1))
        {
            /* 
            Initialize the Bluetooth stack with the parameters; params, event call-back function; stackMessageHandler
            and the exception handler; userExceptionHandler 
            */
            if (!initializeClarinoxBlue(&stack))
            {
                return 0; 
            } 
          
            sendSemaphore = clxCreateSemaphore(0);
            terminationSemaphore = clxCreateSemaphore(0);
        }
        else if ((index == 2) && (stack != 0))
        {
            deleteSppHandle();
            clxTerminateClarinoxBlue(stack);
            clxAcquireSemaphore(terminationSemaphore);
            clxDeleteSemaphore(terminationSemaphore);
            clxDeleteSemaphore(sendSemaphore);
            /*
            Destroy the stack, after this call is completed, stack can be initialized again
            */
            clxDestroyClarinoxBlue(stack);
            stack = 0;
            ClxConsoleUIEngine::destroy();
            ClxConsoleUIEngine::init(32);
            ClxConsoleUIEngine::text("\nBluetooth stack is terminated, make sure to initialize the Bluetooth stack\n");
        }
        else if ((index == 3) && (stack != 0))
        {
            sppHandle = createSppHandle(stack, false);
            connect(stack, 1, sppHandle, false);
            menuFunction(stack,false);          
        }
        else if ((index == 4) && (stack != 0))
        {
            //deleteSppHandle();
            sppHandle  = createSppHandle(stack, true);
			clxGapSetDiscoverability(stack, TRUE, TRUE);
            clxGapSetConnectability(stack, TRUE, TRUE);
            ClxConsoleUIEngine::text("Waiting for incoming connection request...\n");    
            menuFunction(stack,true);  
        }
        else if ((index == 5) && (stack != 0))
        {
            //deleteSppHandle();
            sppHandle  = createSppHandle(stack, false);
            discoverDevices(stack, FALSE);
        }
        else if ((index == 6) && (stack != 0))
        {
            clxGapSetDiscoverability(stack, TRUE, TRUE);
            clxGapSetConnectability(stack, TRUE, TRUE);
            ClxConsoleUIEngine::text("Device put in Discoverable mode.\n");
        }
        else if ((index == 7) && (stack != 0))
        {
            clxGapSetDiscoverability(stack, FALSE, TRUE);
            ClxConsoleUIEngine::text("Device put in Non-Discoverable mode.\n");
        }
        else if ((index == 8) && (stack != 0))
        {
           menuFunction(stack,false);
        }
        else if ((index == 9) && (stack != 0))
        {
            ClxError err = clxGapDeletePairedDeviceInfo(stack, 0, TRUE);
            if (err == CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("\nOldest pairing information deleted successfully.\n");
            }
            else
            {
                ClxConsoleUIEngine::text("Deleting oldest pairing information completed with error code %u\n", err);
            }
        }
        else if ((index == 10) && (stack != 0))
        {
            ClxError err = clxGapDeleteAllPairedDevicesInfo(stack, 1);
            if (err == CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("\nAll paired devices deleted successfully.\n");
            }
            else
            {
                ClxConsoleUIEngine::text("Deleting paired devices completed with error code %u\n", err);
            }
        }
        else if ((index == 11))
        {
#if defined ( CLX_KOALA_CAMERA )         
            clxBeginThread( cameraThread, NULL, "cameraThread", 400, clarinoxBluePlatformTaskPriorityTable, ClxThreadPriority_Lowest );
            s1 inputBuffer[4];
            
            ClxConsoleUIEngine::inputBox ("press any key to stop camera", inputBuffer, 2);

            cameraRunning = FALSE;
 	    clxReleaseSemaphore( cameraSemaphore );
#endif // defined ( CLX_KOALA_CAMERA )
        }
    }
    return 0;
}

#if defined ( CLX_UCOS_III ) || defined ( CLX_THREADX ) || defined ( CLX_NUCLEUS ) || defined (CLX_FREE_RTOS) || defined (CLX_MQX)
extern "C" {
  int clarinoxMain(void)
 {
    mainF();
    return 0;
 }
}
#endif
