/*******************************************************************************
*
* Project             GATT Sample Application
* File                main.cpp
* Description         main application file
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

#ifdef CLX_TEMPERATURE_SENSOR_ENABLED

#include "ClxBsp.h"

#include "string.h"
#include "stdio.h"

//Libraries for temperature sensing module 
#include <stdio.h>
#include "stm32f4xx_adc.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include <math.h>
#include <stdlib.h>


extern "C" 
{


static s4 adc_convert()
{
    ADC_SoftwareStartConv(ADC1);//Start the conversion
    while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));//Processing the conversion
    return ADC_GetConversionValue(ADC1); //Return the converted data
}

float clxMeasureTemperature()
{
    float temp = 0;                                                         //resetting temperature

    //Filter to combat high frequency noise
    for(u4 i = 0; i < 100; i++)                                            //check iteratios for optimisation
    {
        s4 ConvertedValue = adc_convert();                                 //Read the ADC converted value
        temp = ((float)ConvertedValue/4095.0*3*100)+temp;             //Conversion for LM35
    }
    
    temp = temp / 100;    

    return temp;
}

void clxSetLEDs(s4 temp, s4 trigger)
{
    //Output conditions
    if (temp >= trigger)
    {
        GPIO_SetBits(GPIOI,GPIO_Pin_7);                                 //Turn on Red LED
        GPIO_ResetBits(GPIOF,GPIO_Pin_1);                               //Turn off Green LED
    }
    else if(temp < trigger)
    {
        GPIO_SetBits(GPIOF,GPIO_Pin_1);                                 //Turn on Green LED
        GPIO_ResetBits(GPIOI,GPIO_Pin_7);                                //Turn off Red LED
    }    
}

void clxConfigureSensorAndGpio()
{
    ADC_InitTypeDef ADC_init_structure; //Structure for adc confguration
    GPIO_InitTypeDef GPIO_initStructre; //Structure for Analog input pin
    GPIO_InitTypeDef GPIO_initStructre1; //Structure for Digital output pin
    GPIO_InitTypeDef GPIO_initStructre2; //Structure for Digital output pin

    //Clock configuration
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1,ENABLE);//The ADC1 is connected the APB2 peripheral bus thus we will use its clock source
    RCC_AHB1PeriphClockCmd(RCC_AHB1ENR_GPIOAEN,ENABLE);//Clock for the ADC port
    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOI,ENABLE);//Clock for GPIO pins
    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF,ENABLE);//Clock for GPIO pins

    //Analog pin configuration
    GPIO_initStructre.GPIO_Pin = GPIO_Pin_6;//The channel 6 is connected to PA6
    GPIO_initStructre.GPIO_Mode = GPIO_Mode_AN; //The PA6 pin is configured in analog mode
    GPIO_initStructre.GPIO_PuPd = GPIO_PuPd_NOPULL; //No pull up or pull down
    GPIO_Init(GPIOA,&GPIO_initStructre);//Affecting the port with the initialization structure configuration

    //Digital pin configuration
    GPIO_initStructre1.GPIO_Pin = GPIO_Pin_7;      //| GPIO_Pin_6;//Configuringg Pins for digital outputs- LEDs
    GPIO_initStructre1.GPIO_Mode = GPIO_Mode_OUT; //Output mode
    GPIO_initStructre1.GPIO_PuPd = GPIO_PuPd_NOPULL; //No pull up or pull down
    GPIO_initStructre1.GPIO_OType =GPIO_OType_PP;//
    GPIO_initStructre1.GPIO_Speed=GPIO_Speed_25MHz;//
    GPIO_Init(GPIOI,&GPIO_initStructre1);//Affecting the port with the initialization structure configuration

    //Digital pin configuration
    GPIO_initStructre2.GPIO_Pin = GPIO_Pin_1;      //| GPIO_Pin_6;//Configuringg Pins for digital outputs- LEDs
    GPIO_initStructre2.GPIO_Mode = GPIO_Mode_OUT; //Output mode
    GPIO_initStructre2.GPIO_PuPd = GPIO_PuPd_NOPULL; //No pull up or pull down
    GPIO_initStructre2.GPIO_OType =GPIO_OType_PP;//
    GPIO_initStructre2.GPIO_Speed=GPIO_Speed_25MHz;//
    GPIO_Init(GPIOF,&GPIO_initStructre2);//Affecting the port with the initialization structure configuration

    //ADC structure configuration
    ADC_DeInit();
    ADC_init_structure.ADC_DataAlign = ADC_DataAlign_Right;//data converted will be shifted to right
    ADC_init_structure.ADC_Resolution = ADC_Resolution_12b;//Input voltage is converted into a 12bit number giving a maximum value of 4096
    ADC_init_structure.ADC_ContinuousConvMode = ENABLE; //the conversion is continuous, the input data is converted more than once
    ADC_init_structure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T1_CC1;// conversion is synchronous with TIM1 and CC1 (actually I'm not sure about this one :/)
    ADC_init_structure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;//no trigger for conversion
    ADC_init_structure.ADC_NbrOfConversion = 1;
    ADC_init_structure.ADC_ScanConvMode = DISABLE;//The scan is configured in one channel
    ADC_Init(ADC1,&ADC_init_structure);//Initialize ADC with the previous configuration
    //Enable ADC conversion
    ADC_Cmd(ADC1,ENABLE);
    //Select the channel to be read from
    ADC_RegularChannelConfig(ADC1,ADC_Channel_6,1,ADC_SampleTime_144Cycles);
}

}
    
#endif

