/******************************************************************************
*
* Project             Wlan Sample Application
* File                WlanP2P_GO.cpp
* Description         Wlan P2P Group Owner (GO) role application file
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/
  
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "ClxBsp.h"
#include "ConsoleUIEngine.h"
#include "ClarinoxWlan.h"
#include "Wlan.Api.h"
#include "Wlan.Config.h"
#include "Wlan.WifiDirect.Api.h"
#include "Wlan.WifiDisplay.Api.h"
#include "Wlan.MirrorLink.Api.h"
#include "ClxCommInterface.h"
#include "ClxSocketsInterface.h"
#include "MacAddress.h"
#include "WiFiApp.h"

extern "C" void clxInitBsp(void);
extern "C" void* getWl1271PhysicalInterface(); 
extern "C" bool clxEnableWl1271WlanHardware();
extern "C" bool clxDisableWl1271WlanHardware();
extern "C" ClxWlanNetworkInterface* clxGetWlanTcpIpInterface();


extern InterfaceState interfaceState;


void show_P2P_GO_Menu(ClxStack stack)
{
    ClxError ret;
    while (1)
    {  
        const s1* menu = "Start Interface as a P2P Group Owner\0"
            "Cancel ongoing connection request\0"
            "Get the number of joined peer\0"
            "Get the P2P Interface address from the Device address\0"
            "Stop Interface\0"
            "Return to previous menu";

        u4 s = ClxConsoleUIEngine::showMenu("Wlan P2P GO Menu", menu, 6);

        if (s == 1)
        {
            struct GoConfig
            {
                ClxConfigList    parentList;
                struct
                {
                    ClxWlanConfigRate rates[21];
                    ClxConfigArray    ratesArray;
                }supportedRates;  
                ClxConfigString  freqBand;
                ClxConfigInteger maxTxPower;
                ClxConfigInteger beaconInterval;
                ClxConfigInteger dtimInterval;
                ClxConfigInteger maxNumStations;
                ClxConfigInteger channel;
                ClxConfigInteger clientInactivityTimeout;
                ClxConfigInteger frameCaptureMask;
                ClxConfigInteger wifiDisplayEnabled;
                ClxConfigInteger mirrorLinkEnabled;         
                /*
                  Driver specific parameters:
                  */
                ClxConfigString  sdioDriverName;
                ClxConfigString  firwareFileName;
                ClxConfigString  nvsFileName;
                ClxConfigData    macAddress;
                ClxWlanP2PDevice p2pDeviceInfo;
                ClxConfigInteger goCapabilities;
            };

            GoConfig goConfig;

            clxConfigInitParamsList(&goConfig.parentList, NULL, NULL);


    /* All of the following parameters are MANDATORY to be set by the application: */

#if defined(IMX6_SABRE_LITE)
            clxConfigInitStringParam(&goConfig.sdioDriverName, "SdioDriverName", "SDCardDev", &goConfig.parentList);    
#elif defined(IMX6_SABRE_AUTO)
            clxConfigInitStringParam(&goConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &goConfig.parentList);
#elif defined(IMELCO)
            clxConfigInitStringParam(&goConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &goConfig.parentList);         
#elif defined(DM3730_BSP)     
            clxConfigInitStringParam(&goConfig.sdioDriverName, "SdioDriverName", "SDCardDev1", &goConfig.parentList);
#elif defined(PROPAK6_BSP)
            clxConfigInitStringParam(&goConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &goConfig.parentList);
#elif defined(CLARINOX_KOALA_V20)
            clxConfigInitStringParam(&goConfig.sdioDriverName, "SdioDriverName", "SDCardDev0", &goConfig.parentList);
#else
#error Select your board
#endif

            u1 macAddr[6] = { WLAN_MAC_ADDRESS };

            clxConfigInitStringParam (&goConfig.firwareFileName,         "FirmwareFileName",  "wl18xx-fw-2.bin",       &goConfig.parentList);
            clxConfigInitStringParam (&goConfig.nvsFileName,             "NvsFileName",       "wl1271-nvs-sample.nvs", &goConfig.parentList);
            clxConfigInitDataParam   (&goConfig.macAddress,              "MacAddress",         macAddr, 6,              &goConfig.parentList);
            clxConfigInitIntegerParam(&goConfig.maxTxPower,              "MaxTxPower",         25,                      &goConfig.parentList);

            clxConfigInitStringParam (&goConfig.freqBand,                "FreqBand",           "2.4GHZ",          &goConfig.parentList);
            clxConfigInitIntegerParam(&goConfig.channel,                 "Channel",            6,                 &goConfig.parentList);    
            clxConfigInitIntegerParam(&goConfig.beaconInterval,          "BeaconInterval",     100,               &goConfig.parentList);
            clxConfigInitIntegerParam(&goConfig.dtimInterval,            "DtimInterval",       2,                 &goConfig.parentList);
            clxConfigInitIntegerParam(&goConfig.maxNumStations,          "MaxNumberOfClients", 8,                 &goConfig.parentList);

            clxConfigInitIntegerParam(&goConfig.frameCaptureMask,      "FrameCaptureMask", 0xFFFFFFFF & (~CLX_WLAN_FRAME_TYPE_BEACON),  &goConfig.parentList);

            clxConfigInitIntegerParam(&goConfig.clientInactivityTimeout, "ClientInactivityTimeout", 300 /* Seconds */, &goConfig.parentList);

            clxConfigInitIntegerParam(&goConfig.wifiDisplayEnabled, "WIfiDisplayEnabled", 1 /* Enabled */, &goConfig.parentList);
            clxConfigInitIntegerParam(&goConfig.mirrorLinkEnabled, "MirrorLinkEnabled", 1 /* Enabled */, &goConfig.parentList);

            /*
              Non-HT (IEEE802.11abg) rates.
              According to WIFI Direct specification, we must not use non-OFDM (IEEE802.11b) rates. We must only use IEEE802.11g
              rates. Only 6Mbps, and 12Mbps are set as basic rates. These are the rates which must be supported by all
              P2P peers joining the P2P group. In station mode, the basicRate flag
              is ignored.
              */
            clxConfigInitRateParam(&goConfig.supportedRates.rates[0],  NULL, ClxWlanNonHTRate_6Mbps,  0, TRUE,  NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[1],  NULL, ClxWlanNonHTRate_9Mbps,  0, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[2],  NULL, ClxWlanNonHTRate_12Mbps, 0, TRUE,  NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[3],  NULL, ClxWlanNonHTRate_18Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[4],  NULL, ClxWlanNonHTRate_24Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[5],  NULL, ClxWlanNonHTRate_33Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[6],  NULL, ClxWlanNonHTRate_36Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[7], NULL, ClxWlanNonHTRate_48Mbps, 0, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[8], NULL, ClxWlanNonHTRate_54Mbps, 0, FALSE, NULL);

            /* HT (IEEE802.11n) rates (MCS0 to MCS7 are supported by WL1271): */
            clxConfigInitRateParam(&goConfig.supportedRates.rates[9], NULL, ClxWlanNonHTRate_NULL, 0, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[10], NULL, ClxWlanNonHTRate_NULL, 1, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[11], NULL, ClxWlanNonHTRate_NULL, 2, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[12], NULL, ClxWlanNonHTRate_NULL, 3, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[13], NULL, ClxWlanNonHTRate_NULL, 4, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[14], NULL, ClxWlanNonHTRate_NULL, 5, FALSE, NULL);
            clxConfigInitRateParam(&goConfig.supportedRates.rates[15], NULL, ClxWlanNonHTRate_NULL, 6, FALSE, NULL);

            clxConfigInitParamsArray(&goConfig.supportedRates.ratesArray,
                    "SupportedRates",
                    (ClxConfigParam*)goConfig.supportedRates.rates,
                    16,
                    &goConfig.parentList);

            clxWlanConfigInitP2PDevice(&goConfig.p2pDeviceInfo, 
                    "LocalDeviceP2PInfo",
                    "ClxP2PDevice",         /* The name of the device shown when it's discovered by other P2P devices */
                    ClxWps_Computer_PC,
                    0,
                    (u4)ClxWpsSupportedMethod_Display | (u4)ClxWpsSupportedMethod_PushButton | (u4)ClxWpsSupportedMethod_Keypad,    /* The Push button method is mandatory with P2P and must be present */
                    0,  /* We do not support any optional P2P features at this moment */
                    &goConfig.parentList);


            clxConfigInitIntegerParam(&goConfig.goCapabilities, 
                    "LocalP2PGroupCapabilities",
                    (u4)ClxWlanP2PGroupCabailityBitmap_IntraBSSDistribution, 
                    &goConfig.parentList);              
            if(interfaceState == Stopped)
            {
                if (!clxEnableWl1271WlanHardware())
                {
                    ClxConsoleUIEngine::text("Enabling WLAN hardware failed\n");
                    break;
                }
            }

            ret = clxWlanStartInterface(wlanHandle, ClxIfTypeP2P_GO, clxGetWlanTcpIpInterface(), &goConfig.parentList, TRUE);

            if (ret == CLX_SUCCESS)
            {
                /* We need to enable the WLAN device so It will start sending beacons: */
                ret = clxWlanStartRole(wlanHandle, TRUE);
            }

            if (ret != CLX_SUCCESS)
            {
                ClxConsoleUIEngine::text("Starting the P2P GO failed with error %s\n", 
                        clxGetWlanErrorCodeText(ret)); 
            }
            else
            {
                ClxWlanP2PDevice localDevice;
                clxWlanP2PGetLocalDevice(wlanHandle, &localDevice, TRUE);
                
                ClxConsoleUIEngine::text("P2P GO started with the name %s, and GroupID %02X:%02X:%02X:%02X:%02X:%02X\n",
                    localDevice.deviceName,  
                    localDevice.deviceAddress[0],
                    localDevice.deviceAddress[1],
                    localDevice.deviceAddress[2],
                    localDevice.deviceAddress[3],
                    localDevice.deviceAddress[4],
                    localDevice.deviceAddress[5]);

                interfaceState = Started;
             
                /* We set our Wifi Display and MirrorLink sub-elements: */
                struct WifiDisplayMirrorLinkLocalInfo
                {
                    ClxConfigList         parentList; 
                    ClxWlanWfdSubElement  associatedBSSID;
                    ClxWlanWfdSubElement  deviceInfo;                
                    ClxWlanWfdSubElement  sessionInfo;

                    ClxWlanCccSubElement  cccSubElemenet1;
                };

                WifiDisplayMirrorLinkLocalInfo wifiDisplayMirrorLinkLocalInfo;

                clxConfigInitParamsList(&wifiDisplayMirrorLinkLocalInfo.parentList, NULL, NULL);

                /*
                  Sample data:
                  */
                u1 deviceInfo[6] = {0x00, 0x10, 0x1C, 0x44, 0x00, 0x28};
                u1 sessionInfo[] = {0xFF, 0xFE, 0xFD, 0xFC, 0xFB};

                clxWlanConfigWfdSubElement(&wifiDisplayMirrorLinkLocalInfo.deviceInfo,
                        "Local_WFD_SubElement",
                        CLX_WLAN_WFD_SUBELEMENT_DEVICE_INFO,
                        deviceInfo,
                        sizeof(deviceInfo),
                        &wifiDisplayMirrorLinkLocalInfo.parentList);

                 clxWlanConfigWfdSubElement(&wifiDisplayMirrorLinkLocalInfo.associatedBSSID,
                        "Local_WFD_SubElement",
                        CLX_WLAN_WFD_SUBELEMENT_ASSOCIATED_BSSID,
                        macAddr,
                        6,
                        &wifiDisplayMirrorLinkLocalInfo.parentList);
                 
                clxWlanConfigWfdSubElement(&wifiDisplayMirrorLinkLocalInfo.sessionInfo,
                        "Local_WFD_SubElement",
                        CLX_WLAN_WFD_SUBELEMENT_SESSION_INFO,
                        sessionInfo,
                        sizeof(sessionInfo),
                        &wifiDisplayMirrorLinkLocalInfo.parentList);


                u1 cccSubelement1Payload[] = {0x12, 0x34, 0x43, 0x21, 0x00};
                
                clxWlanConfigCccSubElement(&wifiDisplayMirrorLinkLocalInfo.cccSubElemenet1,
                        "Local_CCC_SubElement",
                        3,
                        cccSubelement1Payload,
                        sizeof(cccSubelement1Payload),
                        &wifiDisplayMirrorLinkLocalInfo.parentList);
                
                    
                ret = clxWlanSetParametersValue(wlanHandle,  &wifiDisplayMirrorLinkLocalInfo.parentList, TRUE);
                if (ret != CLX_SUCCESS)
                {
                    ClxConsoleUIEngine::text("Setting WIFI Display sub-elements failed with error %s\n", 
                            clxGetWlanErrorCodeText(ret));                    
                }              
            }          
        }
        else if (s == 2)
        {
            ret = clxWlanP2PCancelConnectionRequest(wlanHandle, TRUE);
            ClxConsoleUIEngine::text("Canceling the ongoing connection request is complete with result %s\n", 
                    clxGetWlanErrorCodeText(ret));           
        }
        else if (s == 3)
        {
            ClxWlanP2PGroup group;
            ClxWlanP2PDevice device;

            clxWlanP2PGetCurrentGroup(wlanHandle, &group, &device, TRUE);
            ClxConsoleUIEngine::text("There are %u peer devices joined the group\n", group.numberOfClients);  
		}
        else if (s == 4)
        {
            struct Config
            {
                ClxConfigList parentList;
                ClxConfigData address;
                u1 addressBuffer[6];               
            };

            Config config;    
            s1 deviceAddressStr[32];
            ClxConsoleUIEngine::inputBox("\nEnter the device address : ", deviceAddressStr, sizeof(deviceAddressStr));

            if (!clxWlanParseMacAddressString(deviceAddressStr, config.addressBuffer))
            {
                ClxConsoleUIEngine::text ("\nPlease enter a MAC address in the format XX:XX:XX:XX:XX:XX\n");
                continue;
            }

            clxConfigInitParamsList(&config.parentList, NULL, NULL);
            clxConfigInitDataParam (&config.address, "P2PInterfaceAddress", config.addressBuffer, sizeof(config.addressBuffer), &config.parentList);

            ret = clxWlanGetParametersValue(wlanHandle, &config.parentList, TRUE);
            if ((ret == CLX_SUCCESS) && (config.address.paramInfo.processed == TRUE))
            {
                ClxConsoleUIEngine::text("The P2P interface address is %02X:%02X:%02X:%02X:%02X:%02X\n", 
                        config.addressBuffer[0],
                        config.addressBuffer[1],
                        config.addressBuffer[2],
                        config.addressBuffer[3],
                        config.addressBuffer[4],
                        config.addressBuffer[5]);                    
            }
            else
            {
                ClxConsoleUIEngine::text("The P2P interface address could not be retrieved");
            }
        }
        else if (s == 5)
        {
            ret = clxWlanStopInterface(wlanHandle, TRUE);
            if(ret == CLX_SUCCESS)
            {
                interfaceState = Stopped;
            }
            ClxConsoleUIEngine::text ("\nStopping interface is complete with result %s", clxGetWlanErrorCodeText(ret));
            clxDisableWl1271WlanHardware();         
        }
        else if (s == 6)
        {
            return;
        }
    }
}


