/******************************************************************************
*
* Project             Wlan Sample Application
* File                mainWiFi.cpp
* Description         Wlan Sample application main file
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

#include <stdio.h>
#include "ClxBsp.h"
#include "ConsoleUIEngine.h"
#include "ClarinoxWlan.h"
#include "ClarinoxWlanConst.h"
#include "Wlan.Api.h"
#include "Wlan.WifiDirect.Api.h"
#include "Wlan.Config.h"
#include "time.h"
#include "string.h"
#include "WiFiApp.h"

ClxSemaphore terminationCompleteSemaphore;

ClxHandle wlanHandle;
ClxStInfo jointStList[MAX_JOINT_ST];
u1 jointStCount = 0;

extern void show_P2P_GO_Menu(ClxStack stack);

extern "C" void Init_LWIP(void);


extern "C" void showLocalIP(const unsigned char* ip)
{
  ClxConsoleUIEngine::text("\nLocal IP Address is %u.%u.%u.%u\n", ip[3], ip[2], ip[1], ip[0]);
}


/*******************************************************************************************************************************
*                                                       Main
*
* Initialize BSP and UI interface, and provide the main UI menu.
*
*******************************************************************************************************************************/


int mainWiFi()
{
    ClxStack stack = NULL;                                          /* A stack object must be created before any other        */
    clxInitBsp();                                                   /* Board support package initialization                   */

    Init_LWIP();                                                    /* Initialize the LwIP stack */
     
    ClxConsoleUIEngine::init(MAX_UI_INPUT_SIZE);                    /* Initializing the Console UI engine                     */

    terminationCompleteSemaphore = clxCreateSemaphore(0);           /* Termination semaphore                                  */
    
    while(1)
    {
        const s1* menu = "Enable Wifi Stack\0"
                         "Wlan Station(STA) Test\0"
                         "Wlan AccessPoint(AP) Test\0"
                         "Wlan P2P GO Test\0"
                         "Get statistics\0"
                         "Disable Wifi Stack\0"
                         "Exit";
                            
        u4 s = ClxConsoleUIEngine::showMenu("Please select how to proceed:", menu, 7);
        if (s == 1)
        {
            if(stack == 0 )
            { 
                /*
                Initialize the ClarinoxWLAN stack and bind it to a physical WLAN interface. If successful, one or more
                Virtual Interfaces can be started on top of this physical interface:
                */
                if(clxInitClarinoxWlan(getWl18xxPhysicalInterface(),        /* Returns a pointer to the WL18xx physical interface object                                                */
                    FALSE,                                                  /* Start a new thread for indication call-back function                                                     */
                    wlanIndicationHandler,                                  /* Indication handler function which is called when there is any indication from WLAN stack                 */
                    NULL,                                                   /* No exception handler function is provided. The default exception handler function will be used           */
                    &stack,                                                 /* On a successful return, will contain the stack object                                                    */
                    TRUE)                                                   /* Do not return until the initialization of ClarinoxWLAN is complete either in error or with success       */
                  != CLX_SUCCESS)
                {
                    printf("Initialization of ClarinoxWLAN failed\n");
                    return 0; 
                }

                printf("ClarinoxWLAN version      %s.%u \n", clxGetClarinoxWlanVersion(), clxGetClarinoxWlanRevision());
                //printf("ClarinoxSoftFrame version %s.%u \n", clxGetClarinoxSoftFrameVersion(), clxGetClarinoxSoftFrameRevision());
                //printf("ClarinoxTasks version     %s    \n", clxGetClarinoxTasksVersion());
                
                /* 
                Create a single virtual interface and get a handle to it. 
                The handle will to start, stop and control the virtual interface: 
                */
                ClxError ret = clxWlanCreateInterface(stack,        /* The WLAN stack object, as returned by clxInitClarinoxWlan()                 */
                    "TI-Wl18xx",                                    /* Name of WL18xx virtual interface (fixed)                                    */
                    &wlanHandle);                                   /* On a successful return, will contain a handle to the virtual interface      */

                if (ret != CLX_SUCCESS)
                {
                    ClxConsoleUIEngine::text("Creation of WLAN virtual interface failed with error %u\n", ret);
                    return 0;
                }
            }
            else
            {
                ClxConsoleUIEngine::text("Wifi Stack is already enabled\n");
            }
        }
        else if (s == 2)
        {
            if(stack != 0 )
            { 
                showStationMenu( stack );
            }
            else
            {
                ClxConsoleUIEngine::text("Wifi Stack  does not exist!\n");
            }
        }
        else if (s == 3)
        {
            if(stack != 0 )
            { 
                showAccessPointMenu( stack );
            }
            else
            {
                ClxConsoleUIEngine::text("Wifi Stack does not exist!\n");
            }
        }
        else if (s == 4)
        {
            if(stack != 0 )
            { 
                ClxConsoleUIEngine::text("P2P_GO_Menu does not exist!\n");
                //show_P2P_GO_Menu( stack );
            }
            else
            {
                ClxConsoleUIEngine::text("Wifi Stack does not exist!\n");
            }         
        }
        else if (s == 5)
        {
            struct Stat
            {
                ClxConfigList     parentList;
                ClxWlanConfigRate maxTxRate;
                ClxWlanConfigRate maxLinkTxRate;
                ClxWlanConfigRate currentTxRate;
            };

            Stat stat;
            clxConfigInitParamsList(&stat.parentList, NULL, NULL);

            clxConfigInitRateParam(&stat.maxTxRate,     "Stat.MaxTxRate",     ClxWlanNonHTRate_NULL , 0, FALSE, &stat.parentList);
            clxConfigInitRateParam(&stat.maxLinkTxRate, "Stat.MaxLinkTxRate", ClxWlanNonHTRate_NULL , 0, FALSE, &stat.parentList);
            clxConfigInitRateParam(&stat.currentTxRate, "Stat.CurrentTxRate", ClxWlanNonHTRate_NULL , 0, FALSE, &stat.parentList);

            ClxResult ret = clxWlanGetParametersValue(wlanHandle, &stat.parentList, TRUE);

            if (ret == CLX_SUCCESS)
            {
                if (stat.maxTxRate.paramInfo.processed)
                {
                    if (stat.maxTxRate.nonHTValue == ClxWlanNonHTRate_NULL)
                    {
                        ClxConsoleUIEngine::text("Stat.MaxTxRate = MCS %u\n", (u4)stat.maxTxRate.mcsIndex);
                    }
                    else
                    {
                        ClxConsoleUIEngine::text("Stat.MaxTxRate = %u Mbps\n", (u4)stat.maxTxRate.nonHTValue/2);
                    }
                }
                if (stat.maxLinkTxRate.paramInfo.processed)
                {
                    if (stat.maxLinkTxRate.nonHTValue == ClxWlanNonHTRate_NULL)
                    {
                        ClxConsoleUIEngine::text("Stat.MaxLinkTxRate = MCS %u\n", (u4)stat.maxLinkTxRate.mcsIndex);
                    }
                    else
                    {
                        ClxConsoleUIEngine::text("Stat.MaxLinkTxRate = %u Mbps\n", (u4)stat.maxLinkTxRate.nonHTValue/2);
                    }
                }
                if (stat.currentTxRate.paramInfo.processed)
                {
                    if (stat.currentTxRate.nonHTValue == ClxWlanNonHTRate_NULL)
                    {
                        ClxConsoleUIEngine::text("Stat.CurrentTxRate = MCS %u\n", (u4)stat.currentTxRate.mcsIndex);
                    }
                    else
                    {
                        ClxConsoleUIEngine::text("Stat.CurrentTxRate = %u Mbps\n", (u4)stat.currentTxRate.nonHTValue/2);
                    }
                }
            }
        }
        else if (s == 6)
        {
            if(stack != 0 )
            {
                if(wlanHandle)
                {
                    if(interfaceState != Stopped)
                    {
                        /* 
                        Before disabling the stack, we have to stop any virtual interface which has already
                        been started. In this code, we only have one virtual interface:
                        */
                        ClxResult ret = clxWlanStopInterface(wlanHandle, TRUE);
                        if(ret == CLX_SUCCESS)
                        {
                            interfaceState = Stopped;
                        }
                        else
                        {
                            ClxConsoleUIEngine::text ("\nStopping interface failed with result %s", 
                                                      clxGetWlanErrorCodeText(ret));
                        }
                    }
                    else
                    {
                        ClxConsoleUIEngine::text ("\nInterface has been already stopped");
                    }

                    clxDisableWl18xxWlanHardware();
                    clxCloseHandle(wlanHandle);
                    
                    for(int i=0; i<jointStCount;i++)                /* Empty the joint STAs list                              */
                    {
                        memset(jointStList[i].macAddr, 0, CLX_MAC_ADDR_LEN);
                    }
                    jointStCount = 0;    
                }

                /* 
                Disabling the WLAN stack consists of two phases: Termination of the stack internal tasks and state machines, and then
                Destroying the stack object and all memory and resources associated to it. We call clxTerminateClarinoxWlan() first
                to terminate the WLAN stack. the call is always asynchronous (it returns immediately). When the termination procedure
                is complete, the indication CLX_WLAN_STACK_TERMINATION_COMPLETE_INDICATION is received. Upon reception of this indication,
                we can destroy the stack object by calling clxDestroyClarinoxWlan(). But, we cannot call this function in the context of the
                indication thread. So, we use a semaphore to wake up and destroy the stack in the context of the main thread:
                */
                clxTerminateClarinoxWlan(stack);

                /* Wait for the CLX_WLAN_STACK_TERMINATION_COMPLETE_INDICATION */
                ClxConsoleUIEngine::text("\nWaiting for stack termination, to complete . . .");
                clxAcquireSemaphore(terminationCompleteSemaphore);

                /* CLX_WLAN_STACK_TERMINATION_COMPLETE_INDICATION received */
                clxDestroyClarinoxWlan(stack);

                /* If clxDestroyClarinoxWlan returns it means that stack has been destroyed */
                ClxConsoleUIEngine::text("\nStack destroyed completely");
                stack = 0;
            }
            else
            {
                ClxConsoleUIEngine::text("Wifi Stack does not exist!\n");
            }
        }
        else if (s == 7)
        {
            break;
        }
    }

    ClxConsoleUIEngine::destroy();
    return 0;
}

/*******************************************************************************************************************************
*                                                     wlanIndicationHandler
*
* This call-back function is registered for wlan stack, any events raised inside stack, causes this 
* call-back function executed with the associated event and parameters. Executed from the stack thread context
*
* \param stack Local device stack handle
* \param serviceHandle profile/service handle
* \param messageID indication id
* \param params void pointer to the indication parameters
* \param errorCode contains the error code
*
*******************************************************************************************************************************/
boolean wlanIndicationHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode)
{  
    switch (messageID)
    {
    case CLX_WLAN_FATAL_HARDWARE_ERROR_INDICATION:
        ClxConsoleUIEngine::text("\nWlan Fatal hardware error received\n"); 
        break;

    case CLX_WLAN_STACK_INITIALIZATION_COMPLETE_INDICATION:
        ClxConsoleUIEngine::text("\nWlan stack has been initialized successfully\n"); 
        break;

    case CLX_WLAN_STACK_TERMINATION_COMPLETE_INDICATION:
        ClxConsoleUIEngine::text("\nWlan stack has been terminated successfully\n"); 
        clxReleaseSemaphore (terminationCompleteSemaphore);
        break;

    case CLX_WLAN_SCAN_COMPLETE:
        ClxConsoleUIEngine::text ("\nScan is complete with the result : %s\n", clxGetWlanErrorCodeText(errorCode));
        break;
        
    case CLX_WLAN_NEW_STATION_JOINED_INDICATION:
        {
            struct ClientDetails
            {
                ClxConfigList     parentList;
                u1                addrBuf[6];
                ClxConfigData     macAddress;
                ClxWlanConfigRate maxRate;
            };


            /*
            These values are recommended by the driver:
            */
            //clxConfigInitIntegerParam(&clientDetails.macAddress, "MacAddress", 2352, &stationConfig.parentList);  
            //clxConfigInitIntegerParam(&stationConfig.rtsThreshold, "RTSThreshold", 2353, &stationConfig.parentList);


            ClxWlanStationJoined* arg = (ClxWlanStationJoined*)params;
            ClxConsoleUIEngine::text("\nStation Joined...");
            ClxConsoleUIEngine::text("\nMac Address: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->macAddress[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }

            /* Add the joint STA to the list of STAs */
            memcpy( jointStList[jointStCount].macAddr, arg->macAddress, CLX_MAC_ADDR_LEN);
            jointStCount++;

            ClxConsoleUIEngine::text("\nbssID: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->bssID[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }
            ClxConsoleUIEngine::text("\n");
            break;
        }

    case CLX_WLAN_STATION_DISCONNECTED_INDICATION:
        {
            ClxWlanStationDisconnected* arg = (ClxWlanStationDisconnected*)params;
            ClxConsoleUIEngine::text("\nStation Disconnected from...");
            ClxConsoleUIEngine::text("\nMac Address: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->macAddress[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }

            /* Find the position of disconnected STA in the list of STAs */
            u1 disconnectedStPosition = 0;
            for (u4 i=0; i<MAX_JOINT_ST; i++)
            {
                if(memcmp(jointStList[i].macAddr, arg->macAddress, CLX_MAC_ADDR_LEN) == 0)
                {
                    disconnectedStPosition = i;
                    /* Delete the disconnected STA from the list of STAs and resort the STAs*/
                    for (u4 i=disconnectedStPosition; i < jointStCount; i++)
                    {
                        memcpy( jointStList[disconnectedStPosition].macAddr, jointStList[disconnectedStPosition+1].macAddr, CLX_MAC_ADDR_LEN);
                        disconnectedStPosition++;
                    }
                    jointStCount--;
                    break;
                }
            }

            ClxConsoleUIEngine::text("\nbssID: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->bssID[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }
            ClxConsoleUIEngine::text("\nReason for disconnection: %s\n", clxGetWlanErrorCodeText(arg->reason));
            ClxConsoleUIEngine::text("\n");
            break;
        }

    case CLX_WLAN_STATION_AUTHENTICATION_INITIATED_INDICATION:
        {
            ClxWlanStationAuthInitiated* arg = (ClxWlanStationAuthInitiated*)params;
            ClxConsoleUIEngine::text("\nStation initiated the authentication");
            ClxConsoleUIEngine::text("\nMac Address: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->macAddress[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }            
        }
        break;

    case CLX_WLAN_STATION_ASSOCIATION_INITIATED_INDICATION:
        {
            ClxWlanStationAuthInitiated* arg = (ClxWlanStationAuthInitiated*)params;
            ClxConsoleUIEngine::text("\nStation initiated the association.");

            ClxConsoleUIEngine::text("\nMac Address: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->macAddress[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }
        }
        break;

    case CLX_WLAN_STATION_AUTHENTICATION_COMPLETED_INDICATION:
        {
            ClxWlanStationAuthAssociationCompleted* arg = (ClxWlanStationAuthAssociationCompleted*)params;
            ClxConsoleUIEngine::text("\nStation completed the authentication with the result: %s", clxGetWlanErrorCodeText(arg->statusCode));
            ClxConsoleUIEngine::text("\nMac Address: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->macAddress[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }          

        }
        break;

    case CLX_WLAN_STATION_ASSOCIATION_COMPLETED_INDICATION:
        {
            ClxWlanStationAuthAssociationCompleted* arg = (ClxWlanStationAuthAssociationCompleted*)params;
            ClxConsoleUIEngine::text("\nStation completed the association with the result: %s", clxGetWlanErrorCodeText(arg->statusCode));
            ClxConsoleUIEngine::text("\nMac Address: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->macAddress[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }
        }
        break;
    case CLX_WLAN_DATA_LINK_SUSPENDED_DUE_TO_LINK_LOSS_INDICATION:
        ClxConsoleUIEngine::text("\nBeacons have been lost\n");
        break;

    case CLX_WLAN_AP_DISCONNECTED_INDICATION:
        {
            ClxWlanApDisconnected* arg = (ClxWlanApDisconnected*)params;
            ClxConsoleUIEngine::text("\nAp disconnected with the reason: %s\n", clxGetWlanErrorCodeText(arg->reason));

            ClxConsoleUIEngine::text("\nMac Address: ");
            for(int i=0; i<6; i++)
            {
                ClxConsoleUIEngine::text ("%X", arg->macAddress[i]);
                if(i<5)
                {
                    ClxConsoleUIEngine::text (":");
                }
            }
            ClxConsoleUIEngine::text("\n");
        }
        break;
#if 0        
    case CLX_WLAN_P2P_CONNECTION_REQUEST_INDICATION:
        {

            ClxWlanP2PConnectionRequestIndication* arg = (ClxWlanP2PConnectionRequestIndication*)params;

            ClxConsoleUIEngine::text("\nA connection request received from %s. Accept ?\n", arg->remoteDevice.deviceName);
            s1 ch = ClxConsoleUIEngine::messageBox("", "yYnN", 4); 
            if ((ch == 'y') || (ch == 'Y'))
            {
                /*
                In the context of the indication handler, we should not call a command in blocking mode.
                We call this in non-blocking mode:
                */
                clxWlanP2PAcceptConnectionRequest(serviceHandle, 0 /*Ignored*/, NULL /*Ignored*/, FALSE);
            }
            else
            {
                clxWlanP2PRejectConnectionRequest(serviceHandle, FALSE);
            }
        }
        break;

    case CLX_WLAN_P2P_CONNECTION_REQUEST_DISPLAY_METHOD_INDICATION:
        {
            ClxWlanP2PConnectionRequestIndication* arg = (ClxWlanP2PConnectionRequestIndication*)params;

            ClxConsoleUIEngine::text("\nA connection request received from %s. Accept ?\n", arg->remoteDevice.deviceName);
            s1 ch = ClxConsoleUIEngine::messageBox("", "yYnN", 4); 
            if ((ch == 'y') || (ch == 'Y'))
            {
                 ClxConsoleUIEngine::text ("\nEnter the PIN code %s in the remote device.\n", arg->pincode);
                /*
                In the context of the indication handler, we should not call a command in blocking mode.
                We call this in non-blocking mode:
                */
                clxWlanP2PAcceptConnectionRequest(serviceHandle, 0 /*Ignored*/, NULL /*Ignored*/, FALSE);
            }
            else
            {
                clxWlanP2PRejectConnectionRequest(serviceHandle, FALSE);
            }
        }
        break;
        
    case CLX_WLAN_P2P_CONNECTION_REQUEST_KEYPAD_METHOD_INDICATION:
        {
            ClxWlanP2PConnectionRequestIndication* arg = (ClxWlanP2PConnectionRequestIndication*)params;

            ClxConsoleUIEngine::text("\nA connection request received from %s. Accept ?\n", arg->remoteDevice.deviceName);
            s1 ch = ClxConsoleUIEngine::messageBox("", "yYnN", 4); 
            if ((ch == 'y') || (ch == 'Y'))
            {
                s1 pincode[9];
                ClxConsoleUIEngine::inputBox ("\nEnter the PIN code (max 8 characters): ", pincode, sizeof(pincode));

                /*
                  In the context of the indication handler, we should not call a command in blocking mode.
                  We call this in non-blocking mode:
                  */
                clxWlanP2PAcceptConnectionRequest(serviceHandle, 0 /*Ignored*/, pincode, FALSE);
            }
            else
            {
                clxWlanP2PRejectConnectionRequest(serviceHandle, FALSE);
            }
        }
        break;
        
    case CLX_WLAN_P2P_ACCEPT_CONNECTION_REQUEST_COMPLETE:
        if (errorCode != CLX_SUCCESS)
        {
            ClxConsoleUIEngine::text ("\nAccepting the remote connection request failed with error %s\n", clxGetWlanErrorCodeText(errorCode));
        }
        else
        {
            ClxConsoleUIEngine::text("\nConnection procedure is in progress. The procedure may be cancelled at any time . . .\n");
        }
        break;

    case CLX_WLAN_P2P_REJECT_CONNECTION_REQUEST_COMPLETE:
        if (errorCode != CLX_SUCCESS)
        {
            ClxConsoleUIEngine::text ("\nRejecting the remote connection request failed with error %s\n", clxGetWlanErrorCodeText(errorCode));
        }
        break;

    case CLX_WLAN_P2P_PEER_DEVICE_JOINED_GROUP_INDICATION:
        {
            ClxWlanP2PDevice* arg = (ClxWlanP2PDevice*)params;
            ClxConsoleUIEngine::text("\n%s (Device Address %02X:%02X:%02X:%02X:%02X:%02X) has joined the P2P group\n", arg->deviceName,
                   arg->deviceAddress[0], arg->deviceAddress[1], arg->deviceAddress[2],
                   arg->deviceAddress[3], arg->deviceAddress[4], arg->deviceAddress[5]);
        }
        break;


     case CLX_WLAN_P2P_PEER_DEVICE_DISCONNECTED_FROM_GROUP_INDICATION:
        {
            ClxWlanP2PDevice* arg = (ClxWlanP2PDevice*)params;
            ClxConsoleUIEngine::text("\n%s (Device Address %02X:%02X:%02X:%02X:%02X:%02X) has left the P2P group\n", arg->deviceName,
                   arg->deviceAddress[0], arg->deviceAddress[1], arg->deviceAddress[2],
                   arg->deviceAddress[3], arg->deviceAddress[4], arg->deviceAddress[5]);
        }
        break;
 #endif       
    }
    return false;
}


/*******************************************************************************************************************************
*                                                     printCapabilties
*******************************************************************************************************************************/
void printCapabilties(u4 capabilities)
{
    /* 
    General capabilities of a WLAN station. Can be bit-wise-combined into a single variable: 
    */
    ClxConsoleUIEngine::text("\nGeneral Capabilities:\n");

    if (capabilities & CLX_WLAN_CAPABILITY_SHORT_PREAMBLE_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_SHORT_PREAMBLE_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_QOS_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_QOS_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_SHORT_SLOT_TIME_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_SHORT_SLOT_TIME_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_CHANNEL_AGILITY_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_CHANNEL_AGILITY_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_DSSS_OFDM_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_DSSS_OFDM_SUPPORTED\n");
    }
    
    /* 
    IEEE802.11n HT (High Throughput) capabilities : 
    */
    ClxConsoleUIEngine::text("\nHT (High Throughput) capabilities:\n");

    if (capabilities & CLX_WLAN_CAPABILITY_HT_40MHZ_CHANNEL_WIDTH_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_HT_40MHZ_CHANNEL_WIDTH_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_HT_SGI_FOR_20MHZ_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_HT_SGI_FOR_20MHZ_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_HT_SGI_FOR_40MHZ_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_HT_SGI_FOR_40MHZ_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_HT_GREENFIELD_MODE_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_HT_GREENFIELD_MODE_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_HT_40MHZ_INTOLERANT)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_HT_40MHZ_INTOLERANT\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_HT_STATIC_POWER_SAVING_MODE_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_HT_STATIC_POWER_SAVING_MODE_SUPPORTED\n");
    }
    if (capabilities & CLX_WLAN_CAPABILITY_HT_DYNAMIC_POWER_SAVING_MODE_SUPPORTED)
    {
        ClxConsoleUIEngine::text("CLX_WLAN_CAPABILITY_HT_DYNAMIC_POWER_SAVING_MODE_SUPPORTED\n");
    }    
}
extern "C" {
  int clarinoxMain(void)
 {
    mainWiFi();
    return 0;
 }
}

