/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                File.cpp
* Description         BSP Support for File contents
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

// _____________________________________________________________________________
//
#undef  CLX_MODULE_ID
#define CLX_MODULE_ID  1139
// _____________________________________________________________________________
//

#include <stdlib.h>
#include "ClxBsp.h"
#include "ClxUserFile.h"
#include "Wl1835/Wl18xx-fw-2-Rev8-8-0-0-13.h"


#ifdef __cplusplus
extern "C" {
#endif

 /*
These below tables are used as the fixed values if RAM_FILE_SUPPORTED, and CANNOT be changed
If FLASH_FILE_SUPPORTED, then these values are used as the default values, 
and they will be changed through out the execution
*/

// The following list specifies the system-defined debugger features:
// Bit 0   : Stack Tracing (function calls)
// Bit 1   : Warnings
// Bit 2   : Logging messages
// Bit 3   : Errors and Fatal Errors
// Bit 4   : Memory Analysis messages (allocations and deallocations)
// Bit 5   : Bluetooth Protocol Monitor messages
// Bit 6   : RFID Protocol Monitor messages
// Bit 7   : GPRS Protocol Monitor messages
// Bit 8   : Inter module Messaging
// Bit 9   : Virtual Console (Both input and output) messages
// Bit 10  : Co-operative Tasking Monitoring
// Bit 11  : SDIO/WiFi Protocol Monitor messages
// Bit 12  : General purpose logger messages
  
static const s1 debugConfig[] =
"DEBUG_MEDIUM = UART\n"
"DEBUG_LOCAL_IP = 0.0.0.0\n"
"DEBUG_LOCAL_PORT = 4002\n"
"DEBUG_REMOTE_IP = 192.168.0.10\n"
"DEBUG_REMOTE_PORT = 14321\n"
"DEBUG_PORT_NAME = DEBUG_PORT\n"
"DEBUG_BAUD_RATE = 921600\n"
"DEBUG_DATA_BITS = 8\n"
"DEBUG_STOP_BITS = 0\n"
"TE_MEDIUM = UART\n"
"TE_PORT_NAME = DEBUG_PORT\n"
"TE_BAUD_RATE = 921600\n"
"TE_DATA_BITS = 8\n"
"TE_STOP_BITS = 0\n"
"DEBUG_TIMEOUT = 2000\n"
"DEBUG_BUFFER_SIZE = 3588\n"
//"DEBUG_FLAG =   0100000111000\n\n";
"DEBUG_FLAG =   0001000001000\n\n";
//"DEBUG_FLAG = 1111111111111\n\n";



static const s1 bluetoothConfig[] =
"Role = Master\n"
"TransportType = TT_UART_H4\n"
"ComPortName = BLUETOOTH_PORT\n"
"ComPortSpeed = 115200\n"
"TraceMode = TraceOn\n"
"ExtendedInquiryResponse = TRUE\n"
"DeviceID = TRUE\n"
"PageScanInterval = 640\n"
"PageScanWindow = 632\n"
"InquiryScanInterval = 2048\n"
"InquiryScanWindow = 2040\n"
"MaxInquiryPeriod = 8\n"
"MinInquiryPeriod = 7\n"
"MaxInquiryTimeUnits = 6\n"
"MaxDevicesToReport = 8\n\n";


const VirtualFile fileList[] = { { StorageType_RAM, "SoftFrame.cfg",        {sizeof(debugConfig),               debugConfig                       } },
							     { StorageType_RAM, "Wl18xx-fw-2-Rev8-8-0-0-13.bin", {sizeof(Wl18xx_Firmware),  (const s1*)Wl18xx_Firmware        } },
                                 { StorageType_RAM, NULL,                   {0, NULL                                                              } } };


extern const VirtualFile* clxVirtualFileSystemMap;

void initROMFileSystemMap()
{
  clxVirtualFileSystemMap = fileList;
}

#ifdef __cplusplus
}
#endif


