#ifndef _ClxUnalignedAccess_h_
#define _ClxUnalignedAccess_h_

/*******************************************************************************
*
* Project           ClarinoxBlue
* File              ClxUnalignedAccess.h
* Description       Unaligned Memory Write and Read functions
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


/**
reads a 32bit value from an unaligned memory address.

\param[ in ] buffer a pointer to a buffer which contains a 32bit number. The address of the buffer may
not be 4-byte aligned.

\return The 32bit value as an u4 variable.
*/
extern  u4               readFromUnaligned       ( const void* buffer );

/**
reads a 16bit value from an unaligned memory address.

\param[ in ] buffer a pointer to a buffer which contains a 16bit number. The address of the buffer may
not be 4-byte aligned.

\return The 16bit value as an u2 variable.
*/
extern  u2               readFromUnaligned16     ( const void* buffer );

/**
writes a 32bit value to an unaligned memory address.

\param[ in ] value a 32bit value which will be written in \e buffer.
\param[ out ] buffer a pointer to a buffer which on return will contain \e value. The address of the buffer may
not be 4-byte aligned.
*/
extern  void             writeToUnaligned        ( u4 value, void* buffer );

/**
writes a 16bit value to an unaligned memory address.

\param[ in ] value a 16bit value which will be written in \e buffer.
\param[ out ] buffer a pointer to a buffer which on return will contain \e value. The address of the buffer may not be
4-byte aligned.
*/
extern  void             writeToUnaligned16      ( u2 value, void* buffer );


#endif // _ClxUnalignedAccess_h_
