#ifndef __ClxUserFile_h__
#define __ClxUserFile_h__

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxUserFile.h
* Description         File System interface for BSP
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/
#include "ClxTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
StorageType is used for defining the user file storage type. Either Flash or RAM file is used, the RAM file is a read-only storage.
This is only valid if a file system is not found on the target platform (CLX_FILE_SYSTEM is not defined).
*/
enum StorageType
{
    StorageType_RAM = 0,
    StorageType_Flash
};

/**
VirtualFileStruct defines a structure where user can declare all the files used in the application. 
This is only valid if a file system is not found on the target platform (CLX_FILE_SYSTEM is not defined). Below is an example use of the structure;
\code
 const VirtualFile fileList[] = { { StorageType_Flash, "SoftFrame.cfg",    {sizeof(debugConfig),     debugConfig}     },
                                  { StorageType_Flash, "ClarinoxBlue.cfg", {sizeof(bluetoothConfig), bluetoothConfig} },
                                  { StorageType_Flash, "licence.txt",      {sizeof(licence),         licence}         } };
\endcode

*/
typedef struct VirtualFileStruct
{
    StorageType type;
    const s1*   fileName;
#if defined( READ_ONLY_FILE_SUPPORTED )
    struct
    {
        u4            fileLen;
        const s1*    fileBuffer;
    } ram;
#elif defined( FLASH_FILE_SUPPORTED )
    struct
    {
        u4          blockIndex;
    } flash;
#else
#   error "define only one of the following macros: READ_ONLY_FILE_SUPPORTED or FLASH_FILE_SUPPORTED"
#endif
} VirtualFile;

#ifdef __cplusplus
}
#endif


#endif // __ClxUserFile_h__
