#ifndef __Console_Bsp_h__
#define __Console_Bsp_h__

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                Console.Bsp.h
* Description         Console IO interface for BSP
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif

/**
External print function prototype is used by an application to redirect printf/cout function calls.
If this function pointer is assigned to a user "printf" function, then the debug prints will be redirected here.
Otherwise platform specific default "printf" function will be used.
Print a \p formatted printable string to the user defined printf function.
\param[ in ] formattedOutput is the null-terminated string containing the formatted printable string.
*/
extern void (*userPrintfFunction) ( char* formattedOutput );

/**
External getchar function prototype is used by an application to redirect character input function calls.
If this function pointer is assigned to a user "getchar" function, then the console input will use this function.
If this pointer is not assigned by the user, platform specific default "getchar" function will be used.

\return The character read.
*/
extern char (*userGetCharFunction) ( void );

/**
Called by ClarinoxSoftFrame when a string of characters needs to be provided by the user. If this function pointer is not assigned to a
user-defined function, scanf will be used.
*/
extern int (*userGetInputFunction) ( char* buffer, unsigned int bufferSize );


#ifdef __cplusplus
}
#endif


#endif // __Console_Bsp_h__
