#ifndef __DHCP_BSP__
#define __DHCP_BSP__

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                Dhcp.Bsp.h
* Description         Declares DCHP interface for BSP
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif
  
  
#define CLX_DHCP_SERVER_PORT     67  // Standard No 67
#define CLX_DHCP_CLIENT_PORT     68  // Standard No 68

/* Default IP Address of this DHCP Server    */
#define CLX_DHCP_SERVER_IP_A    192
#define CLX_DHCP_SERVER_IP_B    168
#define CLX_DHCP_SERVER_IP_C      0
#define CLX_DHCP_SERVER_IP_D      6

/* Subnet Mask                               */
#define CLX_SUBNET_MASK_A       255
#define CLX_SUBNET_MASK_B       255
#define CLX_SUBNET_MASK_C       255
#define CLX_SUBNET_MASK_D         0

/* DNS Server IP address                     */
#define CLX_DNS_IP_A            192
#define CLX_DNS_IP_B            168
#define CLX_DNS_IP_C              0
#define CLX_DNS_IP_D              2

/* Start IP address of the host IP addresses */
#define CLX_DHCP_START_IP_A     192
#define CLX_DHCP_START_IP_B     168
#define CLX_DHCP_START_IP_C       0
#define CLX_DHCP_START_IP_D      10

/* Number of host IP address                 */
#define CLX_NUM_IP_ADDRESS       10

#define CLX_IP_LEASE_TIME     86400  //(1 day)  
  
 
namespace ClxDhcp
{

  enum  ClxBootpMessageType
  {
          BOOTREQUEST = 0x01,
          BOOTREPLY   = 0x02,
  };

  enum ClxIPAssignType
  {
      NEW_IP      =  0,
      EXISTING_IP =  1,
      NO_IP       =  255,
  };

  enum  ClxDhcpMessageType
  {
      CLX_DHCP_DISCOVER   = 0x01,
      CLX_DHCP_OFFER      = 0x02,
      CLX_DHCP_REQUEST    = 0x03,
      CLX_DHCP_DECLINE    = 0x04,
      CLX_DHCP_ACK        = 0x05,
      CLX_DHCP_NAK        = 0x06,
      CLX_DHCP_RELEASE    = 0x07,
      CLX_DHCP_INFORM     = 0x08,
      CLX_DHCP_INFORM_ACK = 0x40,
  };

  enum ClxDhcpOptionType
  {
      // RFC 1497 vendor extensions
      PAD                       = 0,
      SUBNET_MASK               = 1,
      TIME_OFFSET               = 2,
      ROUTER                    = 3,
      TIMER_SERVER              = 4,
      NAME_SERVER               = 5,
      DOMAIN_NAME_SERVER        = 6,
      LOG_SERVER                = 7,
      COOKIE_SERVER             = 8,
      LPR_SERVER                = 9,
      IMPRESS_SERVER            = 10,
      RESOURCE_LOCATION_SERVER  = 11,
      HOST_NAME                 = 12,
      BOOT_FILE_SIZE            = 13,
      MERIT_DUMP_FILE           = 14,
      DOMAIN_NAME               = 15,
      SWAP_SERVER               = 16,
      ROOT_PATH                 = 17,
      EXTENSIONS_PATH           = 18,
      END                       = 255,

      //DHCP extensions
      REQUESTED_IP_ADDRESS      = 50,
      IP_ADDRESS_LEASE_TIME     = 51,
      OPTION_OVERLOAD           = 52,
      DHCP_MESSAGE_TYPE         = 53,
      SERVER_IDENTIFIER         = 54,
      PARAMETER_REQUEST_LIST    = 55,
      MESSAGE                   = 56,
      MAXIMUM_DHCP_MESSAGE_SIZE = 57,
      RENEWAL_T1_TIME_VALUE     = 58,
      REBINDING_T2_TIME_VALUE   = 59,
      VENDOR_CLASS_IDENTIFIER   = 60,
      CLIENT_IDENTIFIER         = 61,
      TFTP_SERVER_NAME          = 62,
      BOOTFILE_NAME             = 63,
  };

}

struct ClxHost{
    u4 hostIPAddress;
    u1 MACaddress[6];
};


class ClxDhcpOptionData
{
    private:
    u4 ourIP;                   // ip address of this server.
  
    public:
    u1 dhcpMessageType;
    u2 flags;
    u4 yourIP;                  //yiaddr
    u4 subnetMask;
    u4 dhcpServerIP;            //siaddr
    u4 dnsIP;
    u4 router;
    u4 renewalTime;
    u4 rebindingTime;
    u4 ipLeaseTime;
    u4 requestedIP;
    u4 transactionID;
    u4 gateWayIP;               //giaddr
    u4 clientIP;                //ciaddr
    u1 clientHardwareAddress[6];    
    ClxDhcp::ClxDhcpMessageType dhcp_state;
    ClxHost dhcpHosts[CLX_NUM_IP_ADDRESS];
    s1 domainName[64];
    
    public:
    ClxDhcpOptionData()
    {
        dhcpMessageType = 0;
        yourIP          = 0;
        subnetMask      = 0;
        dhcpServerIP    = 0;
        dnsIP           = 0;
        router          = 0;
        renewalTime     = 0;
        rebindingTime   = 0;
        ipLeaseTime     = 0;
        requestedIP     = 0;
        transactionID   = 0;
        gateWayIP       = 0;
        clientIP        = 0;
        flags           = 0;
        memset(clientHardwareAddress,0x00,6);
        dhcp_state = ClxDhcp::CLX_DHCP_DISCOVER;
        
        // set our IP address to the default value of 192.168.0.6
        ourIP = (((u4)CLX_DHCP_SERVER_IP_A << 24) | ((u4)CLX_DHCP_SERVER_IP_B << 16) | ((u4)CLX_DHCP_SERVER_IP_C << 8) | (u4)CLX_DHCP_SERVER_IP_D);
        
        // initialize pool of host addresses to zero
        for(u1 i1 = 0;i1 < CLX_NUM_IP_ADDRESS;i1++)
        {
            dhcpHosts[i1].hostIPAddress = (((u4)CLX_DHCP_START_IP_A << 24) | ((u4)CLX_DHCP_START_IP_B << 16) | ((u4)CLX_DHCP_START_IP_C << 8) | (u4)CLX_DHCP_START_IP_D) + i1;
            memset(dhcpHosts[i1].MACaddress,0x00,6);
        }
        
    }
    
    void setOurIP(u1 a, u1 b, u1 c, u1 d)
    {
      // set our IP address to the value specified by the user
      ourIP = (((u4)a << 24) | ((u4)b << 16) | ((u4)c << 8) | (u4)d);
    }
    
    u4 getOurIP()
    {
      return ourIP;
    }
    
};


typedef struct ClxDhcpDataBspInterfaceStruct
{
    ClxResult (*init) (struct ClxDhcpDataBspInterfaceStruct* this_, ClxDhcpOptionData& reqDhcpOptions);

    void (*destroy) (struct ClxDhcpDataBspInterfaceStruct* this_);

    u4 (*newRequest) (struct ClxDhcpDataBspInterfaceStruct* this_);
        

    void (*dataRequired) (struct ClxDhcpDataBspInterfaceStruct* this_);

    u4 (*dataReceived) (struct ClxDhcpDataBspInterfaceStruct* this_);

    void (*requestComplete) (struct ClxDhcpDataBspInterfaceStruct* this_);

} ClxDhcpDataBspInterface;

extern ClxResult clxDhcpServerInit(ClxDhcpDataBspInterface* dhcpDataInterface, const struct ClxNetworkServiceAddress* localAddr);

#ifdef __cplusplus
}
#endif

#endif // __DHCP_BSP__

