#ifndef __HttpServer_Bsp__
#define __HttpServer_Bsp__

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                HttpServer.Bsp.h
* Description         Declares HTTP server interface to the file system and/or
*                     HTTP request handlers
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif

#define CLX_HTTP_RESPONSE_STATUS_SUCCESS                                    200
#define CLX_HTTP_RESPONSE_STATUS_BAD_REQUEST                                400
#define CLX_HTTP_RESPONSE_STATUS_UNAUTHORIZED                               401
#define CLX_HTTP_RESPONSE_STATUS_FORBIDDEN                                  403
#define CLX_HTTP_RESPONSE_STATUS_NOT_FOUND                                  404
#define CLX_HTTP_RESPONSE_STATUS_METHOD_NOT_ALLOWED                         405
#define CLX_HTTP_RESPONSE_STATUS_LENGTH_REQUIRED                            411
#define CLX_HTTP_RESPONSE_STATUS_REQUEST_ENTITY_TOO_LARGE                   413
#define CLX_HTTP_RESPONSE_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE            416
#define CLX_HTTP_RESPONSE_STATUS_INTERNAL_ERROR                             500


/**
ClxHttpRequestMethod is an enumerator type which contains all standard request methods used in Http protocol.
*/
enum ClxHttpRequestMethod 
{
    ClxHttpRequestMethod_Get, 
    ClxHttpRequestMethod_Post, 
    ClxHttpRequestMethod_Head, 
    ClxHttpRequestMethod_Put, 
    ClxHttpRequestMethod_Connect, 
    ClxHttpRequestMethod_Delete, 
    ClxHttpRequestMethod_Trace, 
    ClxHttpRequestMethod_Options,
    ClxHttpRequestMethod_Unknown, 
    ClxHttpRequestMethod_NotRequest
}; 

/**
ClxHttpVersion is an enumerator type which contains the standard Http protocol versions.
*/
enum ClxHttpVersion 
{
    ClxHttpVersion_10, 
    ClxHttpVersion_11, 
    ClxHttpVersion_Unknown
};


enum ClxHttpRequiredDataType
{
    ClxHttpRequiredDataType_Header = 0,
    ClxHttpRequiredDataType_Payload,
    ClxHttpRequiredDataType_Any,
};

struct ClxWebDataBspInterfaceBuffer
{
    const u1*   data;
    size_t      dataLength;
    u4          flags;
};


typedef u1* _Clx32bitAligned_ ClxHttpServerRequestHandle;
typedef u4 ClxHttpResponseStatus;
typedef void* ClxHttpHeaderFieldsHandle;
typedef void* ClxHttpServerConnection;

typedef struct ClxWebDataBspInterfaceStruct
{
    ClxResult (*init) (struct ClxWebDataBspInterfaceStruct* this_,
                       u2* maxNumberOfRequests,
                       u2* requestHandleBufferSize);

    void (*destroy) (struct ClxWebDataBspInterfaceStruct* this_);

    ClxHttpResponseStatus (*newRequest) (struct ClxWebDataBspInterfaceStruct* this_,
                                         ClxHttpServerConnection connection,
                                         ClxHttpServerRequestHandle requestHandle,
                                         enum ClxHttpRequestMethod method,
                                         ClxHttpVersion requestVersion,
                                         const s1* url,
                                         u4 requestBodyLength,
                                         ClxHttpHeaderFieldsHandle requestHeaderFieldsHandle);
        

    void (*dataRequired) (struct ClxWebDataBspInterfaceStruct* this_,
                          ClxHttpServerRequestHandle requestHandle,
                          enum ClxHttpRequiredDataType requiredDataType,
                          struct ClxWebDataBspInterfaceBuffer* buffer);

    ClxHttpResponseStatus (*dataReceived) (struct ClxWebDataBspInterfaceStruct* this_,
                                           ClxHttpServerRequestHandle requestHandle,
                                           const u1* data, 
                                           u4 dataLength);

    void (*requestComplete) (struct ClxWebDataBspInterfaceStruct* this_, 
                             ClxHttpServerRequestHandle requestHandle);

} ClxWebDataBspInterface;


extern ClxResult clxMiniHttpServerInit(ClxWebDataBspInterface* webDataInterface, u4 maxNumberOfRequests, u4 connectionIdleTimeout, const struct ClxNetworkServiceAddress* localAddr);
extern void clxMiniHttpServerDestroy();


#define CLX_MINI_HTTP_RESULT_SUCCESS            1
#define CLX_MINI_HTTP_RESULT_NOT_FOUND          -5
#define CLX_MINI_HTTP_RESULT_TOO_BIG            -6

extern ClxResult clxMiniHttpSeachForHeader(ClxHttpHeaderFieldsHandle headerFieldsHandle, const s1* headerName, s1* headerValue, u4* headerValueLength);


#ifdef __cplusplus
}
#endif


#endif // __HttpServer_Bsp__
