#ifndef __NetworkAdapter_Bsp__
#define __NetworkAdapter_Bsp__

/*******************************************************************************
*
* Project             Clarinox Debugger
* File                NetworkAdapter.Bsp.h
* Description         Defines Network Adapter BSP netIF
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/




#define CLX_NETWORK_PACKET_FRAGMENT_TYPE_NULL					0x00
#define CLX_NETWORK_PACKET_FRAGMENT_TYPE_COMPLETE				0x01
#define CLX_NETWORK_PACKET_FRAGMENT_TYPE_START					0x02
#define CLX_NETWORK_PACKET_FRAGMENT_TYPE_CONTINUE				0x03
#define CLX_NETWORK_PACKET_FRAGMENT_TYPE_END					0x04

#define CLX_NETWORK_HEADER_TYPE_ETHERNET						0x00
#define CLX_NETWORK_HEADER_TYPE_IPV4							0x01
#define CLX_NETWORK_HEADER_TYPE_IPV6							0x02

#define CLX_NETWORK_FLAG_PACKET_TYPE_NULL						0x00
#define CLX_NETWORK_FLAG_PACKET_TYPE_INCOMING_LOCAL_HOST		0x01
#define CLX_NETWORK_FLAG_PACKET_TYPE_INCOMING_OTHER_HOST		0x02
#define CLX_NETWORK_FLAG_PACKET_TYPE_OUTGOING					0x03
#define CLX_NETWORK_FLAG_PACKET_TYPE_FORWARDING					0x04



/**
The indication CLX_NETWORK_ACCESS_INTERFACE_INDICATION_STARTED is received by the network interface implementation when the parent virtual interface
has been successfully started. When this indication is received, the implementation of network interface is able to call any API functions defined
in this file. 

NOTE : When this indication is received, the wireless link is still down. The link should be considered up only after reception of the indication
#CLX_NETWORK_ACCESS_INTERFACE_INDICATION_LINK_UP.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxNetworkInterfaceIndication_Started.

Handling this indication is MANDATORY.
*/
#define CLX_NETWORK_ACCESS_INTERFACE_INDICATION_ROLE_STARTED                  0x0001

/**
The indication CLX_NETWORK_ACCESS_INTERFACE_INDICATION_STOPPED is received by the network interface implementation when the parent virtual interface
has been stopped. When this indication is received, the implementation of network interface MUST cancel any ongoing TX/RX activity and return any
resources of the WLAN stack it currently possesses (e.g. TX/RX buffers and descriptors) to the WLAN stack via API functions. When the implementation of 
handleIndication() returns, the implementation of network interface must already have called off any TX/RX activity, and all resources must have been returned
to the WLAN stack. Also, when handleIndication() returns, the implementation of the network stack is NOT allowed to call the API functions defined in this file
any longer.

NOTE : This indication might be received when the link is up. In this case, reception of this indication also implies that the link has gone down.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxNetworkInterfaceIndication.

Handling this indication is MANDATORY.
*/
#define CLX_NETWORK_ACCESS_INTERFACE_INDICATION_ROLE_STOPPED                  0x0002

/**
The indication CLX_NETWORK_ACCESS_INTERFACE_STATUS_LINK_UP is received by the network interface implementation when there is an active wireless link and TX packets
may be sent to the WLAN stack or RX packets may be received by the network interface implementation.

When this indication is received, the link should be considered up until the indication #CLX_NETWORK_ACCESS_INTERFACE_INDICATION_LINK_DOWN or #CLX_NETWORK_ACCESS_INTERFACE_INDICATION_STOPPED
is received.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxNetworkInterfaceIndication.

Handling this indication is optional.
*/
#define CLX_NETWORK_ACCESS_INTERFACE_INDICATION_LINK_UP                       0x0003

/**
The indication CLX_NETWORK_ACCESS_INTERFACE_STATUS_LINK_DOWN is received by the network interface implementation when there is no longer an active wireless link and TX packets
cannot be sent to the WLAN stack any more.

NOTE : When the link is down, any TX packets passed to the WLAN stack will be completed immediately.
NOTE : After receiving this indication, the implementation will not receive any RX frames any more.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxNetworkInterfaceIndication.

Handling this indication is optional.
*/
#define CLX_NETWORK_ACCESS_INTERFACE_INDICATION_LINK_DOWN                     0x0004

/**
The indication CLX_NETWORK_ACCESS_INTERFACE_INDICATION_NEW_CLIENT_ASSOCIATED is received by the network interface implementation when a new station has joined the BSS. This indication is received
ONLY if the local virtual interface has the AccessPoint or P2P-GO role.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxNetworkInterfaceIndication_NewClientAssociatedStruct.

Handling this indication is optional.
*/
#define CLX_NETWORK_ACCESS_INTERFACE_INDICATION_NEW_LINK                    0x0005


/**
The indication CLX_NETWORK_ACCESS_INTERFACE_INDICATION_CLIENT_DEASSOCIATED is received by the network interface implementation when a station has left the BSS. This indication is received
ONLY if the local virtual interface has the AccessPoint or P2P-GO role.

The indication object, as passed to handleIndication() implementation of the network interface, is of type #ClxNetworkInterfaceIndication_ClientDeassociated.

Handling this indication is optional.
*/
#define CLX_NETWORK_ACCESS_INTERFACE_INDICATION_LINK_DISCONNECTED           0x0006


#define CLX_NETWORK_ACCESS_INTERFACE_COMMAND_6LOWPAN_BASE									0x0010

#define CLX_NETWORK_ACCESS_INTERFACE_COMMAND_SET_6LOWPAN_LOCAL_INTERFACE_IDENTIFIER			(CLX_NETWORK_ACCESS_INTERFACE_COMMAND_6LOWPAN_BASE + 0x0001)
#define CLX_NETWORK_ACCESS_INTERFACE_COMMAND_SET_6LOWPAN_NEIGHBOUR_INTERFACE_IDENTIFIER		(CLX_NETWORK_ACCESS_INTERFACE_COMMAND_6LOWPAN_BASE + 0x0002)
#define CLX_NETWORK_ACCESS_INTERFACE_COMMAND_SET_6LOWPAN_ADD_PREFIX_CONTEXT					(CLX_NETWORK_ACCESS_INTERFACE_COMMAND_6LOWPAN_BASE + 0x0003)
#define CLX_NETWORK_ACCESS_INTERFACE_COMMAND_SET_6LOWPAN_REMOVE_PREFIX_CONTEXT				(CLX_NETWORK_ACCESS_INTERFACE_COMMAND_6LOWPAN_BASE + 0x0003)


#define CLX_NETWORK_ADDRESS_MAX_SIZE                                        16    /* Bytes */
#define CLX_NETWORK_IPV6_INTERFACE_IDENTIFIER_SIZE                          8     /* Bytes */
#define CLX_NETWORK_IPV6_NETWORK_PREFIX_SIZE								8     /* Bytes */


#ifdef __cplusplus
extern "C" {
#endif

    typedef enum ClxNetworkAddressTypeEnum
    {
        ClxNetworkAddressType_IPv4 = 0,                     /*!< 32 bit */
        ClxNetworkAddressType_IPv6 = 1,                     /*!< 128 bit */
        ClxNetworkAddressType_EUI64 = 2,                    /*!< 64 bit */
        ClxNetworkAddressType_IEEE80215_SHORT = 3,          /*!< 16 bit short address used in IEEE802.15 networks */
        ClxNetworkAddressType_MAC = 4                       /*!< 48 bit IEEE physical address */
    } ClxNetworkAddressType;


    typedef enum ClxNetworkLinkTypeEnum
    {
        ClxNetworkLinkType_Node2Router = 0,
        ClxNetworkLinkType_Router2Node = 1,
        ClxNetworkLinkType_Node2Node = 2,
        ClxNetworkLinkType_Multicast = 3
    } ClxNetworkLinkType;


    typedef enum ClxNetworkDeviceRoleEnum
    {
        ClxNetworkDeviceRoleEnum_Client = 0,
        ClxNetworkDeviceRoleEnum_Router = 1,
        ClxNetworkDeviceRoleEnum_GroupOwner = 2,
        ClxNetworkDeviceRoleEnum_NonCategorized = 3
    } ClxNetworkDeviceRole;


    typedef struct ClxNetworkAddressStruct
    {
        u4                      value[CLX_NETWORK_ADDRESS_MAX_SIZE/4];
        ClxNetworkAddressType   type;
    } ClxNetworkAddress;


    extern void clxNetworkAddressEncode(const ClxNetworkAddress* address, u1* out);
    extern void clxNetworkAddressDecode(ClxNetworkAddress* address, const u1* in);
    extern void clxNetworkAddressCopy(ClxNetworkAddress* out, const ClxNetworkAddress* in);
    extern bool clxNetworkAddressIsEqual(const ClxNetworkAddress* arg1, const ClxNetworkAddress* arg2);
	extern void clxNetworkAddressSetToBroadcast(ClxNetworkAddress* address);

    extern boolean clxNetworkAddressGetSourceAddress(const u1* data, u4 dataLength, u1 headerType, ClxNetworkAddress* out);
    extern boolean clxNetworkAddressGetDestinationAddress(const u1* data, u4 dataLength, u1 headerType, ClxNetworkAddress* out);



    /**
    Base type for all indications
    */
    typedef struct ClxNetworkInterfaceIndicationStruct
    {
        u2 indicationID;
    } ClxNetworkInterfaceIndication;


	/**
	Base type for all commands
	*/
	typedef struct ClxNetworkInterfaceCommandStruct
	{
		u2 commandID;
	} ClxNetworkInterfaceCommand;


	typedef struct ClxNetworkInterfaceCommand_Set6LowPanLocalInterfaceIdentifierStruct
	{
        ClxNetworkInterfaceCommand	base;         /*!< SHALL be the first member */

		u1							uid[CLX_NETWORK_IPV6_INTERFACE_IDENTIFIER_SIZE];
	} ClxNetworkInterfaceCommand_Set6LowPanLocalInterfaceIdentifier;


	typedef struct ClxNetworkInterfaceCommand_Set6LowPanNeighbourInterfaceIdentifierStruct
	{
        ClxNetworkInterfaceCommand	base;         /*!< SHALL be the first member */

		ClxNetworkAddress			physicalAddress;
		u1							uid[CLX_NETWORK_IPV6_INTERFACE_IDENTIFIER_SIZE];
	} ClxNetworkInterfaceCommand_Set6LowPanNeighbourInterfaceIdentifier;


	typedef struct ClxNetworkInterfaceCommand_Set6LowPanLocalNetworkPrefixStruct
	{
        ClxNetworkInterfaceCommand	base;         /*!< SHALL be the first member */

		u1							prefix[CLX_NETWORK_IPV6_NETWORK_PREFIX_SIZE];
	} ClxNetworkInterfaceCommand_Set6LowPanLocalNetworkPrefix;


    /**
    Object type for the indication CLX_NETWORK_ACCESS_INTERFACE_INDICATION_ROLE_STARTED
    */
    typedef struct ClxNetworkInterfaceIndication_RoleStartedStruct
    {
        ClxNetworkInterfaceIndication base;         /*!< SHALL be the first member */

        const ClxNetworkAddress* localPhysicalAddress;      /*!< The local MAC address which is going to be used by the parent virtual interface */
        ClxNetworkDeviceRole     localDeviceRole;

    } ClxNetworkInterfaceIndication_RoleStarted;


    /**
    Object type for the indication CLX_NETWORK_ACCESS_INTERFACE_INDICATION_NEW_LINK
    */
    typedef struct ClxNetworkInterfaceIndication_NewLinkStruct
    {
        ClxNetworkInterfaceIndication base;         /*!< SHALL be the first member */

        const ClxNetworkAddress*   destPhysicalAddress;           /*!< MAC address of the client associated to the local AccessPoint or P2P GO */
        ClxNetworkLinkType         linkType;
        u2                         linkID;                       /*!< The numeric unique ID the client associated to the local AccessPoint or P2P GO (assigned by ClarinoxWLAN) */
    }  ClxNetworkInterfaceIndication_NewLink;


    /**
    Object type for the indication CLX_NETWORK_ACCESS_INTERFACE_INDICATION_CLIENT_DEASSOCIATED
    */
    typedef struct ClxNetworkInterfaceIndication_LinkDisconnectedStruct
    {
        ClxNetworkInterfaceIndication base;         /*!< SHALL be the first member */

        const ClxNetworkAddress*   destPhysicalAddress;           /*!< MAC address of the client associated to the local AccessPoint or P2P GO */
        ClxNetworkLinkType         linkType;
        u2                         linkID;                       /*!< The numeric unique ID the client associated to the local AccessPoint or P2P GO (assigned by ClarinoxWLAN) */
    }  ClxNetworkInterfaceIndication_LinkDisconnected;


    typedef struct ClxNetworkBufferDescriptorBaseStruct
    {
        void* rtti;                                 /*!< Not to be modified */
    } ClxNetworkBufferDescriptorBase;


    typedef struct ClxNetworkBufferDescriptorStruct
    {
        ClxNetworkBufferDescriptorBase				base;                  /*!< Initialized by the transport layer. SHALL NOT be modified by the network interface implementation */
        void*                                       userData;
        struct ClxNetworkBufferDescriptorStruct*    next;                   /*!< Pointer to the next buffer in the buffer chain */
        
        u1                                          fragmentType:3;
        u2                                          flags:13;

        /** 
        header members shall be set if this is the first fragment of a network packet 
        (e.g. when fragmentType is either CLX_NETWORK_PACKET_FRAGMENT_TYPE_COMPLETE or CLX_NETWORK_PACKET_FRAGMENT_TYPE_START).
        Otherwise, it shall be ignored. 
        */
        struct Header
        {
            u2 totalPacketLength;            /*!< Total length of this packet */
        } header;

        u1*                                         address;
        u2                                          bufferSize;
        u2                                          dataLength;
    } ClxNetworkBufferDescriptor;


    typedef struct ClxNetworkBufferDescriptorChainStruct
    {
        ClxNetworkBufferDescriptor* first;
        ClxNetworkBufferDescriptor* current;
    } ClxNetworkBufferDescriptorChain;


    typedef ClxNetworkBufferDescriptorChain ClxNetworkPacket;


    extern void                         clxNetworkBufferDescriptorChainInit				(ClxNetworkBufferDescriptorChain* list);
    extern void                         clxNetworkBufferDescriptorChainPushAll			(ClxNetworkBufferDescriptorChain* list, ClxNetworkBufferDescriptor* descriptorChain);
    extern void                         clxNetworkBufferDescriptorChainAppend			(ClxNetworkBufferDescriptorChain* list1, const ClxNetworkBufferDescriptorChain* list2);
    extern void                         clxNetworkBufferDescriptorChainAppendAndReset	(ClxNetworkBufferDescriptorChain* list1, ClxNetworkBufferDescriptorChain* list2);
    extern ClxNetworkBufferDescriptor*  clxNetworkBufferDescriptorChainPopFirst			(ClxNetworkBufferDescriptorChain* list);
    extern ClxNetworkBufferDescriptor*  clxNetworkBufferDescriptorChainPopAll			(ClxNetworkBufferDescriptorChain* list);


    typedef struct ClxNetworkTransportHandleStruct
    {} ClxNetworkTransportHandle;


    struct ClxNetworkAccessInterface 
    {
        ClxResult (*init) (_in_ struct ClxNetworkAccessInterface*    thisObj,
                           _in_ ClxNetworkTransportHandle*           transportHandle,
                           _in_ const ClxConfigList*                 configList);


        /**
        This function is called when an indication has been received from the WLAN stack. An indication has the base type #ClxWlanNetworkInterfaceIndication.
        The indicationID member of #ClxWlanNetworkInterfaceIndication determined the actual type of the indication. Based on indicationID, the indication object
        can be cast to the correct type (if there is a special structure defined for that indication) and the other members of the indication can be accessed.

        NOTE : An implementation is able to call network interface API functions within this function, or after this function returns.

        THIS FUNCTION MUST ALWAYS BE DEFINED and it must handle the mandatory indications. However, optional indications might be not handled. The documentation
        of each indication specifies whether or the indication is mandatory to handle.

        \param[ in ] thisObj A pointer to this object.
        \param[ in ] indication A pointer to the indication object. The indicationID member of the object determines the actual type of the object.
        */
        void (*handleIndication) (_in_ struct ClxNetworkAccessInterface*    thisObj, 
                                  _in_ const ClxNetworkInterfaceIndication* indication);


        /**
        Destroys the given interface. An implementation must destroy all of the internal variables, and the implementation object (if dynamically allocated).

        When this function is called, the indication #CLX_NETWORK_ACCESS_INTERFACE_INDICATION_STOPPED has already been received and must have been handled by
        the implementation of #handleIndication function.

        THIS FUNCTION MUST ALWAYS BE DEFINED.

        IMPORTANT : NONE of the network interface API functions may be called within the implementation of this function.

        Note : This function will not be called if #(*init) has returned an error.

        \param[ in ] thisObj A pointer to this object.
        */
        void (*destroy) (_in_ struct ClxNetworkAccessInterface* thisObj);

        /**
        Called when there are one or more packets received from the wireless link. The interface must use 
        #clxWlanGetPendingRxDescriptors() function to retrieve the packets. The interface must be in enabled
        state when this function is called.
        
        THIS FUNCTION MUST ALWAYS BE DEFINED.

        IMPORTANT NOTE: This call is RX protected. This means that #lockRx has been called before this function is called. 
        #unlockRx will be called when this function returns.

        Note: The function #clxWlanGetPendingRxDescriptors() may be called inside the implementation of this function, 
        alternatively it may called later on from the context of another thread. In the latter case, the RX path 
        must be manually locked by the interface before the function #clxWlanGetPendingRxDescriptors() is called. 

        \param[ in ] thisObj A pointer to this object.
        \param[ in ] totalNumOfPendingRxDescriptors Total number of RX packets which have been received but not obtained by the interface yet.

        \return TRUE if the implementation has already obtained all the pending RX descriptors using #clxWlanGetPendingRxDescriptors() 
        function within this call-back function.
        FALSE if the implementation will try to obtain the RX descriptors later on.
        */
        void  (*rxPacketPending) (_in_ struct ClxNetworkAccessInterface* thisObj);

        ClxNetworkBufferDescriptor* (*getFreeRxBuffer) (_in_ struct ClxNetworkAccessInterface* thisObj);

        /**
        Called when there are one or more TX packets have been sent to the wireless link either with success or in error. 
        The interface must call #clxWlanGetCompletedTxDescriptorsID() function to retrieve the TX descriptors IDs.
        The interface must be in enabled state when this function is called.

        THIS FUNCTION SHALL MUST BE DEFINED.

        IMPORTANT NOTE: This call is TX protected. This means that #lockTx has been called before this function is called. 
        #unlockTx will be called when this function returns.

        Note: The function #clxWlanGetCompletedTxDescriptorsID() may be called inside the implementation of this function, 
        alternatively it may called later on from the context of another thread. In the latter case, The TX path 
        must be manually locked by the interface before the function #clxWlanGetCompletedTxDescriptorsID() is called. 

        \param[ in ] thisObj A pointer to this object.
        \param[ in ] totalNumOfCompletedTxDescriptors Total number of TX packets which have been sent.

        \return TRUE if the implementation has already obtained all the completed TX descriptors (by a call to #clxWlanGetCompletedTxDescriptorsID() 
        function within this call-back function.
        FALSE if the implementation will try to obtain the TX descriptors later on.
        */
        void  (*txBuffersComplete) (_in_ struct ClxNetworkAccessInterface* thisObj);
    };



        /**
        Queues a packet, described by a TX descriptor, into the WLAN stack. Note that this function will not send the packet 
        to the wireless link. The function #clxWlanFlushPendingNetworkTxDescriptors() must be called to flush all 
        of the queued TX descriptors to the WLAN driver.
    
        NOTE: TX path protection is not required if the functions #clxWlanQueueNetworkTxDescriptor(), 
        #clxWlanQueueNetworkTxDescriptors(), and #clxWlanFlushPendingNetworkTxDescriptors() are always called in the context of 
        the same thread. Otherwise, TX path protection is mandatory.

        NOTE: When a descriptor is passed to this function to be queued, it is not free any more. 
        The interface must not modify the contents of the descriptor after it has been passed to this function.

        IMPORTANT NOTE: The interface MUST NOT pass an interface-allocated TX descriptor object to this function. 
        Only TX descriptor object obtained by a call to #clxWlanGetFreeNetworkTxDescriptors() function may be used.

        \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.
        \param[ in ] txDescriptor A pointer to a TX descriptor object. This object has been obtained by a call to 
        #clxWlanGetFreeNetworkTxDescriptors() function, and has been filled with the information of the TX packet.
        */
        extern void clxNetworkInterfaceQueueTxPacket (_in_ ClxNetworkTransportHandle*   transport,
                                                      _in_ const ClxNetworkPacket*  packet);


        /**
        Flushes all pending (queued) TX descriptors to the WLAN driver. An implementation must call this function frequently 
        in order to make sure the packets will be sent to the wireless link. When the packets are sent to the wireless link 
        (either with success or in error), the associated descriptor objects will be marked as complete. These descriptors 
        will be free for reuse after #clxWlanGetCompletedTxDescriptorsID() function is called.

        NOTE: TX path protection is not required if the functions #clxWlanQueueNetworkTxDescriptor(), 
        #clxWlanQueueNetworkTxDescriptors(), and #clxWlanFlushPendingNetworkTxDescriptors() are always called in the context of 
        the same thread. Otherwise, TX path protection is mandatory.
    
        \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.

        \return The number of pending TX descriptors which could not be sent to the driver since the wireless link was not operational. 
        These descriptors are marked complete and will be free after the function #clxWlanGetCompletedTxDescriptorsID() is called. 
        Note that the interface call-back function txPdusSendingComplete inside #ClxNetworkAccessInterface WILL NOT be called for these descriptors.
        */
        extern void clxNetworkInterfaceFlushPendingTxPackets (_in_ ClxNetworkTransportHandle* transport);

        /**
        Retrieves one or more pending RX descriptor. Each descriptor points to a packet received from the wireless link.

        IMPORTANT NOTE: RX descriptor objects are allocated by the WLAN stack. They MUST NOT be deleted by the interface.

        IMPORTANT NOTE: RX path MUST always be protected when this function is called.

        \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.
        \param[ in ] rxDescriptorsList An array of type ClxWlanRxPacketDescriptor* and size numberOfDescriptors. 
        On return, it will contain zero or more RX descriptors.
        \param [ out ] numberOfDescriptors Contains the maximum number of RX descriptors which the caller is prepared to obtain. 
        On return, it will contain the actual number of RX descriptors returned in the rxDescriptorsList.
        \return The current number of pending RX descriptors upon this function's return. This number does not include 
        the RX descriptors just returned in rxDescriptorsList.
        */
        extern ClxNetworkBufferDescriptor* clxNetworkInterfaceGetPendingRxPackets (_in_ ClxNetworkTransportHandle* transport);

        extern ClxNetworkBufferDescriptor* clxNetworkInterfaceGetReleasedTxBuffers (_in_ ClxNetworkTransportHandle* transport);

        /**
        Releases the used RX descriptors back to the WLAN stack. All RX descriptors returned by a call to #clxWlanGetPendingRxDescriptors() 
        function shall be released after the interface is no longer using them.

        IMPORTANT NOTE: RX PATH MUST always be protected when this function is called.

        \param[ in ] interfaceHandle The handle of the parent virtual interface, as passed to #(*init) implementation of the interface.
        \param[ in ] rxDescriptorsList A pointer to an array of type ClxWlanRxPacketDescriptor and the size numberOfDescriptors.
        \param[ in ] numberOfDescriptors Number of descriptors inside rxDescriptorsList array.
        */
        extern void clxNetworkInterfaceRxBuffersReleased (_in_ ClxNetworkTransportHandle* transport, _in_ const ClxNetworkBufferDescriptorChain* buffers);

        /**
        Retrieves a list of one or more TX Buffers. These buffers may be used to store the TX packets for
        sending them to the wireless link.

        IMPORTANT NOTE: TX buffer objects are allocated by the WLAN stack. They MUST NOT be deleted by the interface.

        IMPORTANT NOTE: TX path SHALL always be protected when this function is called.

        \param[ in ] interfaceHandle handle for the parent virtual interface as passed to #(*init) implementation of the interface.
        \param[ out ] buffersList A pointer to an array of type ClxWlanBuffer* with the size of numberOfRequiredBuffers. 
        On return, the array will contain the TX buffer objects.
        \param[ in ] numberOfRequiredBuffers maximum number of TX buffers required by the interface. 
        The actual number of returned buffers may be less than this number if there is not enough 
        free buffers in the queue of free TX buffers.

        \return The actual number of TX buffers returned in list. A value of 0 to numberOfRequiredBuffers may be returned. 
        A value of 0 means there are no free buffer available (not an error condition).
        */
        extern ClxNetworkBufferDescriptor* clxNetworkInterfaceAllocateBufferDescriptor (_in_ ClxNetworkTransportHandle* transport);

        extern void clxNetworkInterfaceFreeBufferDescriptor (_in_ ClxNetworkTransportHandle*  transport, 
                                                             _in_ ClxNetworkBufferDescriptor* desc);


		extern ClxResult clxNetworkInterfaceIssueCommand(_in_ ClxNetworkTransportHandle*  transport,
														 _inout_ ClxNetworkInterfaceCommand* command);

#ifdef __cplusplus
}
#endif

#endif  // _ClarinoxBlueNetworkInterface_


