#ifndef __NetworkStackServiceInterface_Bsp__
#define __NetworkStackServiceInterface_Bsp__

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                NetworkStackServiceInterface.Bsp.h
* Description         Network Stack Service BSP Interface
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


#define CLX_NETWORK_ETHERNET_MAC_ADDRESS_SIZE					        6   /* Bytes */


/**
Initializes a variable of type #ClxNetworkServiceIp4Address with the specified IPv4 address
and the TCP/UDP port number. The provided IPv4 address and the port number shall both be 
in the platform natural byte order.
*/
#define CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS(obj, ip_, port_)					    \
    obj.base.type = ClxNetworkServiceAddressType_IP4;								\
    obj.base.length = sizeof(ClxNetworkServiceIp4Address);							\
    obj.addr = ip_;																	\
    obj.port = port_

/**
Initializes a variable of type #ClxNetworkServiceEthernetAddress.
*/
#define CLX_NETWORK_SERVICE_INIT_ETHERNET_ADDRESS(obj, ethAddress)					\
    obj.base.type = ClxNetworkServiceAddressType_Ethernet;							\
    obj.base.length = sizeof(ClxNetworkServiceEthernetAddress);						\
    memcpy(obj.addr, ethAddress, sizeof(obj.add))


/**
Returns the IP4 address 'a.b.c.d' as an integer. 
NOTE : The return value is in the natural byte order of the platform:
*/
#define CLX_IP4_ADDRESS(a, b, c, d)    (((u4)a << 24) | ((u4)b << 16) | ((u4)c << 8) | (u4)d)

/**
Returns the first (MSB) byte of the given IP address. For an IP address in the format a.b.c.d, this macro returns a.
addr is assumed to be in the natural byte order of the platform.
*/
#define CLX_IP4_ADDRESS_A(addr)        (((u4)addr & 0xFF000000) >> 24)

/**
Returns the second (MSB) byte of the given IP address. For an IP address in the format a.b.c.d, this macro returns b.
addr is assumed to be in the natural byte order of the platform.
*/
#define CLX_IP4_ADDRESS_B(addr)        (((u4)addr & 0x00FF0000) >> 16)

/**
Returns the third (MSB) byte of the given IP address. For an IP address in the format a.b.c.d, this macro returns c.
addr is assumed to be in the natural byte order of the platform.
*/
#define CLX_IP4_ADDRESS_C(addr)        (((u4)addr & 0x0000FF00) >> 8)

/**
Returns the forth (MSB) byte of the given IP address. For an IP address in the format a.b.c.d, this macro returns d.
addr is assumed to be in the natural byte order of the platform.
*/
#define CLX_IP4_ADDRESS_D(addr)         ((u4)addr & 0x000000FF)


/**
Socket Error Codes. The values of these error codes are independent from
the underlying platform error codes.
*/
#define CLX_SOCKET_INVALID_SOCKET_TYPE          -699
#define CLX_SOCKET_INVALID_ADDR                 -700
#define CLX_SOCKET_INVALID_ADDR_TYPE            -701
#define CLX_SOCKET_EINTR                        -702
#define CLX_SOCKET_EBADF                        -703
#define CLX_SOCKET_EACCES                       -704
#define CLX_SOCKET_EFAULT                       -705
#define CLX_SOCKET_EINVAL                       -706
#define CLX_SOCKET_EMFILE                       -707
#define CLX_SOCKET_EWOULDBLOCK                  -708
#define CLX_SOCKET_EINPROGRESS                  -709
#define CLX_SOCKET_EALREADY                     -710
#define CLX_SOCKET_ENOTSOCK                     -711
#define CLX_SOCKET_EDESTADDRREQ                 -712
#define CLX_SOCKET_EMSGSIZE                     -713
#define CLX_SOCKET_EPROTOTYPE                   -714
#define CLX_SOCKET_ENOPROTOOPT                  -715
#define CLX_SOCKET_EPROTONOSUPPORT              -716
#define CLX_SOCKET_ESOCKTNOSUPPORT              -717
#define CLX_SOCKET_EOPNOTSUPP                   -718
#define CLX_SOCKET_EPFNOSUPPORT                 -719
#define CLX_SOCKET_EAFNOSUPPORT                 -720
#define CLX_SOCKET_EADDRINUSE                   -721
#define CLX_SOCKET_EADDRNOTAVAIL                -722
#define CLX_SOCKET_ENETDOWN                     -723
#define CLX_SOCKET_ENETUNREACH                  -724
#define CLX_SOCKET_ENETRESET                    -725
#define CLX_SOCKET_ECONNABORTED                 -726
#define CLX_SOCKET_ECONNRESET                   -727
#define CLX_SOCKET_ENOBUFS                      -728
#define CLX_SOCKET_EISCONN                      -729
#define CLX_SOCKET_ENOTCONN                     -730
#define CLX_SOCKET_ESHUTDOWN                    -731
#define CLX_SOCKET_ETIMEDOUT                    -732
#define CLX_SOCKET_ECONNREFUSED                 -733
#define CLX_SOCKET_EHOSTDOWN                    -734
#define CLX_SOCKET_EHOSTUNREACH                 -735
#define CLX_SOCKET_UNKNOWN_ERROR                -736
#define CLX_SOCKET_NOTINITIALISED               -737
#define CLX_SOCKET_ENOMEM                       -738


#define CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP              0x01     /*!< The handle represents a single TCP connection between the local machine and a remote machine */
#define CLX_NETWORK_SOCKET_HANDLE_TYPE_TCP_LISTENER     0x02     /*!< The handle represents a TCP listening endpoint in the local machine which listens for incoming connection requests */
#define CLX_NETWORK_SOCKET_HANDLE_TYPE_UDP              0x03     /*!< The handle represents a UDP endpoint in the local machine */
#define CLX_NETWORK_SOCKET_HANDLE_TYPE_RAW              0x04     /*!< The handle represents a layer-3 raw endpoint in the local machine (the handle data contains IP header and all upper layer headers) */
#define CLX_NETWORK_SOCKET_HANDLE_TYPE_ETHERNET         0x05     /*!< The service uses Ethernet frames (the handle data contains Ethernet v2 header and all upper layer headers) */

#define CLX_NETWORK_SOCKET_BUFFER_FLAG_COPY                     0x0001
#define CLX_NETWORK_SOCKET_BUFFER_FLAG_FLUSH_QUEUED_DATA        0x0002
#define CLX_NETWORK_SOCKET_BUFFER_FLAG_SET_TCP_PUSH_BIT         0x0004

#ifdef __cplusplus
extern "C" {
#endif


extern u4 CLX_HTONL(u4 value);
extern u2 CLX_HTONS(u2 value);

#define CLX_NTOHL(long_value)        CLX_HTONL(long_value)
#define CLX_NTOHS(short_value)       CLX_HTONS(short_value)


enum ClxNetworkServiceAddressType
{
    ClxNetworkServiceAddressType_IP4 = 0,
    ClxNetworkServiceAddressType_IP6,
    ClxNetworkServiceAddressType_Ethernet
};

/**
The base structure for network addresses. Derived structure shall have a member of type
ClxNetworkServiceAddress as their very first member.
*/
struct ClxNetworkServiceAddress
{
    enum ClxNetworkServiceAddressType	type;		/*!< Actual type of this structure */
    u4									length;		/*!< Actual length of this structure */
};

/**
An IPv4 address.
The macro #CLX_NETWORK_SERVICE_INIT_IP4_ADDRESS shall be used to initialize an instance of this structure.
*/
struct ClxNetworkServiceIp4Address
{
    struct ClxNetworkServiceAddress    base;		/*!< Base structure. 
														 base.type shall be set to ClxNetworkServiceAddressType_IP4. 
														 base.length shall be set to sizeof(ClxNetworkServiceIp4Address) */

    u4									addr;		/*!< IPv4 address in the platform natural byte order */
    u2									port;		/*!< TCP or UDP port in the platform natural byte order */
};

/**
An Ethernet MAC address.
The macro #CLX_NETWORK_SERVICE_INIT_ETHERNET_ADDRESS shall be used to initialize an instance of this structure.
*/
struct ClxNetworkServiceEthernetAddress
{
    struct ClxNetworkServiceAddress    base;		/*!< Base structure. 
														 base.type shall be set to ClxNetworkServiceAddressType_Ethernet. 
														 base.length shall be set to sizeof(ClxNetworkServiceEthernetAddress) */

	u1								   addr[CLX_NETWORK_ETHERNET_MAC_ADDRESS_SIZE];		/*!< Ethernet MAC address */
};

enum ClxNetworkServicePriority
{
    ClxNetworkServicePriority_High = 0,
    ClxNetworkServicePriority_Normal,
    ClxNetworkServicePriority_Low,
    ClxNetworkServicePriority_Idle
};


struct ClxNetworkSocketHandle
{
    void* userData;
};


struct ClxNetworkService
{
    void (*connectionComplete) (struct ClxNetworkService* service,
                                struct ClxNetworkSocketHandle* handle,
                                const struct ClxNetworkServiceAddress* remoteAddr);   
    
    void (*terminated) (struct ClxNetworkService* service,
                        void* handleUserData,                     
                        ClxResult reason);

    void (*newConnection) (struct ClxNetworkService* service,
                           struct ClxNetworkSocketHandle* newHandle,
                           const struct ClxNetworkServiceAddress* localAddr, 
                           const struct ClxNetworkServiceAddress* remoteAddr);

    void (*sendComplete) (struct ClxNetworkService* service,
                          struct ClxNetworkSocketHandle* handle, 
                          const u1* sentBuf);
    
    void (*dataReceived) (struct ClxNetworkService* service,
                          struct ClxNetworkSocketHandle* handle,
                          u1* data, 
                          size_t dataLength,
                          u4 flags,
                          struct ClxNetworkServiceAddress* remoteAddr);

    void (*connectionIdle) (struct ClxNetworkService* service,
                            struct ClxNetworkSocketHandle* handle,
                            u4 idleTime);
};


extern ClxResult clxNetworkStackServiceInterface_Open(struct ClxNetworkService* service,
                                                      u1 socketType,
                                                      const struct ClxNetworkServiceAddress* localAddr,
                                                      ClxNetworkSocketHandle** handle);


extern ClxResult clxNetworkStackServiceInterface_Connect(struct ClxNetworkSocketHandle* handle,
                                                         const struct ClxNetworkServiceAddress* remoteAddr);

extern ClxResult clxNetworkStackServiceInterface_Terminate(struct ClxNetworkSocketHandle* handle);

extern void clxNetworkStackServiceInterface_Close(struct ClxNetworkSocketHandle* handle);

extern ClxResult clxNetworkStackServiceInterface_Send(struct ClxNetworkSocketHandle* handle, 
                                                      const u1* buf,
                                                      size_t length,
                                                      u4 flags,
                                                      const struct ClxNetworkServiceAddress* remoteAddr);

extern ClxResult clxNetworkStackServiceInterface_RecvComplete(struct ClxNetworkSocketHandle* handle);


#ifdef __cplusplus
}
#endif


#endif // __NetworkStackServiceInterface_Bsp__
