#ifndef _TI_VendorSpecific_
#define _TI_VendorSpecific_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                TI_VendorSpecific.h
* Description         BSP Support for TI Bluetooth Chip
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

extern boolean clxBluetoothUartSetBaudRate(const s1* portName, u4  baudRate);
extern boolean clxBluetooth_HardResetController();
extern boolean clxEnableBluetoothHardware();
extern boolean clxSerialIoIntegrity_SetBaudRate(const s1* portName, u4  baudRate);
extern boolean clxBluetoothIntegrity_HardResetController();
/* Extracts the OCF part from HCI command op-code */
extern u2 clxGetHciOcf(u2* opcodePTR);

/* Extracts the OGF part from HCI command op-code */
extern u2 clxGetHciOgf(u2* opcodePTR);

u1 clxInitBluetoothController( void )
{
#if defined (CLX_INTEGRITY) 
  printf("+clxInitBluetoothController\n");
  
  clxSerialIoIntegrity_SetBaudRate("SerialDev3", 115200);
#endif         
  /*
  This function is called by ClarinoxBlue right before starting the Bluetooth controller.
  This example indicates how to send vendor-specific commands to the chip.
  The function clxSendVendorSpecificCommand() is blocking and only returns when a
  response (COMMAND COMPLETE event) is received, or the timer expires. if this function
  returns 0, the stack initialization will be immediately complete in failure.
  Set responseTimeout parameter of clxSendVendorSpecificCommand() to 0 in order to indicate
  infinite timeout.

  If a response is received, the address of the buffer storing the response message will be returned 
  by eventPacket parameter of clxSendVendorSpecificCommand(). 
  This buffer is allocated by the stack and MUST NOT be modified or deleted.
  The response buffer will remain valid until clxSendVendorSpecificCommand() is called again, 
  or this function returns.
  */

  ClxError ret = CLX_SUCCESS;
  u1* eventPacket = NULL;
#if defined (CLX_INTEGRITY)  
  /* Before doing anything, we hard reset the Bluetooth controller: */
  if (!clxBluetoothIntegrity_HardResetController())
  {
    return 0;
  }
#elif defined( CLX_UCOS_III ) || defined( CLX_THREADX ) || defined (CLX_FREE_RTOS)
  clxEnableBluetoothHardware();
#endif  
  printf("->RESET\n");
  
  /* We soft reset the controller by sending standard RESET command to the chip
     (This is necessary. Otherwise, the chip may not work properly).
     We try up to 3 times. If not successful, we will give up:*/
  u4 i;
  for (i = 0; i < 3; i++)
  {
    u2 resetCmdOpCode = 0x0C03;
    u4 ogf = clxGetHciOgf( (&resetCmdOpCode) );
    u4 ocf = clxGetHciOcf( (&resetCmdOpCode) );
    ret = clxSendVendorSpecificCommand(ogf, ocf, NULL /*No payload*/, 0, &eventPacket, 3000 /* milliseconds */);

    if (ret == CLX_SUCCESS)
    {
      break; 
    }
  }
  if (i == 3)
  {
    return 0;
  }
  
#if !defined(LPC4330)
  printf("->Set baud to 921600\n");
#else
  printf("->Set baud to 115200\n");
#endif
  
  /* Now we send the following command to the chip to set the chip baud rate to 921600 bps:*/
  const u1 setBaudRateCommandPayload[] = {0x00, 0x10, 0x0E, 0x00}; // 921600
  u2 setBaudRateOpCode = 0xFF36;
  u2 setBaudRateOgf = clxGetHciOgf( (&setBaudRateOpCode) );
  u2 setBaudRateOcf = clxGetHciOcf( (&setBaudRateOpCode) );
  
#if !defined(LPC4330)  
  
  ret = clxSendVendorSpecificCommand(setBaudRateOgf, setBaudRateOcf, setBaudRateCommandPayload, sizeof(setBaudRateCommandPayload), &eventPacket, 3000 /* milliseconds */);

  if (ret != CLX_SUCCESS)
  {
    return 0; 
  }
#if defined( CLX_UCOS_III ) || defined( CLX_THREADX ) || defined (CLX_FREE_RTOS)
  /*Set the baud rate of the USART port to 921600 bps*/
  if(clxBluetoothUartSetBaudRate("BLUETOOTH_PORT", CLX_BLUETOOTH_UART_MAX_BAUDRATE) != TRUE)
  {
    return 0;
  }
#endif  
  
#endif
  
  printf("->FW\n");
  
  /* Downloading the Bluetooth firmware to the chip: */
  u4 arraySize = sizeof(tiwiBluetoothFirmware);
  u4 j =0;
  while(arraySize )
  {
      u1* packetP  = (u1*)&tiwiBluetoothFirmware[j];

      /* Sending the current command to the chip */ 
      u1* opCode = packetP + 1;
      u4 ogf = clxGetHciOgf( ((u2*)opCode) );
      u4 ocf = clxGetHciOcf( ((u2*)opCode) );
      u4 length = (u4)(*(packetP + 3));
      ret = clxSendVendorSpecificCommand(ogf, ocf, packetP + 4, length, &eventPacket, 3000 /* milliseconds */);

      if (ret != CLX_SUCCESS)
      {
        return 0; 
      }
      j = (j + length + 4);
      arraySize = arraySize - length - 4;
  }
  
#define SCO_GOES_TO_HCI
#if defined( SCO_GOES_TO_HCI )  
  // send a TI vendor specific command to divert SCO to HCI
  const u1 setScoConfigurationWiLink8CommandPayload[] = {0x01, 0xb4, 0xd0, 0x02, 0x01};
  // type:1 HCI, 
  // size:b4 180 bytes, 
  // latency:0x02d0, 
  // accept packet with bad crc: 1
  
  setBaudRateOpCode = 0xFE10;
  setBaudRateOgf = clxGetHciOgf( (&setBaudRateOpCode) );
  setBaudRateOcf = clxGetHciOcf( (&setBaudRateOpCode) );
  ret = clxSendVendorSpecificCommand(setBaudRateOgf, setBaudRateOcf, setScoConfigurationWiLink8CommandPayload, sizeof(setScoConfigurationWiLink8CommandPayload), &eventPacket, 3000 /* milliseconds */);

 // send a TI vendor specific command to read SCO configuration. For each enabled SCO channel the following will be read;   
  // type: 1 HCI, 0 Codec
  // size: 1 bytes,
  // latency: 2 bytes, 
  // accept packet with bad crc: 1: enable, 0: disable
  setBaudRateOpCode = 0xFE11;
  setBaudRateOgf = clxGetHciOgf( (&setBaudRateOpCode) );
  setBaudRateOcf = clxGetHciOcf( (&setBaudRateOpCode) );
  ret = clxSendVendorSpecificCommand(setBaudRateOgf, setBaudRateOcf,  NULL /*No payload*/, 0, &eventPacket, 3000 /* milliseconds */);
#endif 
  
  /* We are good to go: */
  return 1;
}

#endif // _TI_VendorSpecific_
