/*******************************************************************************
*
* Project             ClarinoxBlue
* File                FileBsp.cpp
* Description         BSP Support for File Operations
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/
#include <stdlib.h>
#include "ClxBsp.h"
#include "ClxUserFile.h"
#include <string.h>


//#define CLX_PRINTF printf(x, ##__VA_ARGS__)
#define CLX_PRINTF(x, ...)

#ifdef __cplusplus
extern "C" {
#endif

// Must define only one of the below MACROs (but not both). MUST BE COMING FROM the PROJECT
//
// #define RAM_FILE_SUPPORTED
// #define FLASH_FILE_SUPPORTED  


/* IMPORTANT
 *   TODO -
 * FILE RECORDS NEED TO BE WRAPPED IN PROPER HEADER AND CHECKSUM FORMAT
 * TO DETECT AND HANDLE CORRUPTION PROPERLY.  
 */

// User application must provide the file list. An example is provided below
extern const VirtualFile fileList[];

const VirtualFile* clxVirtualFileSystemMap = NULL;

#if 0

here is an example setup of a RAM based file system for Clarinox stack use: 

/*
These below tables are used as the fixed values if RAM_FILE_SUPPORTED, and CANNOT be changed
If FLASH_FILE_SUPPORTED, then these values are used as the default values, 
and they will be changed through out the execution
*/

// The following list specifies the system-defined debugger features:
// Bit 0   : Stack Tracing (function calls)
// Bit 1   : Warnings
// Bit 2   : Logging messages
// Bit 3   : Errors and Fatal Errors
// Bit 4   : Memory Analysis messages (allocations and deallocations)
// Bit 5   : Bluetooth Protocol Monitor messages
// Bit 6   : RFID Protocol Monitor messages
// Bit 7   : GPRS Protocol Monitor messages
// Bit 8   : Inter module Messaging
// Bit 9   : Virtual Console (Both input and output) messages
// Bit 10  : Co-operative Tasking Monitoring
static s1 debugConfig[] =
"DEBUG_MEDIUM = UART\n"
"DEBUG_LOCAL_IP = 0.0.0.0\n"
"DEBUG_LOCAL_PORT = 4002\n"
"DEBUG_REMOTE_IP = 192.168.0.10\n"
"DEBUG_REMOTE_PORT = 14321\n"
"DEBUG_PORT_NAME = DEBUG_PORT\n"
"DEBUG_BAUD_RATE = 921600\n"
"DEBUG_DATA_BITS = 8\n"
"DEBUG_STOP_BITS = 0\n"
"TE_MEDIUM = UART\n"
"TE_PORT_NAME = DEBUG_PORT\n"
"TE_BAUD_RATE = 921600\n"
"TE_DATA_BITS = 8\n"
"TE_STOP_BITS = 0\n"
"DEBUG_TIMEOUT = 2000\n"
"DEBUG_BUFFER_SIZE = 8196\n"
"DEBUG_FLAG = 01000111111\n\n";
//"DEBUG_FLAG = 11111111111\n\n";
 

static s1 bluetoothConfig[] =
"Role = Master\n"
"TraceMode = TraceOn\n"
"TransportType = TT_UART_H4\n"
"ComPortName = BLUETOOTH_PORT\n"
"ComPortSpeed = 115200\n"
"DeviceID = TRUE\n"
"PageScanInterval = 640\n"
"PageScanWindow = 150\n"
"InquiryScanInterval = 2048\n"
"InquiryScanWindow = 400\n"
"MaxInquiryPeriod = 10\n"
"MinInquiryPeriod = 9\n"
"MaxInquiryTimeUnits = 3\n"
"MaxDevicesToReport = 4\n"
"ExtendedInquiryResponse = TRUE\n\n";

// This license file is generated for MAC Address range of xx - yy
static s1 licence[] = "E5AE2F43E4CAE0CF3F5C46DB7F4DC7E5401EF8E7E422D52E938D9E1562681649F04D191C\n";

#if defined( FLASH_FILE_SUPPORTED )

#define MAX_FLASH_BLOCK_SIZE        8192
u1 flashBuffer[MAX_FLASH_BLOCK_SIZE];

static const VirtualFile fileList[] = { { StorageType_Flash, "SoftFrame.cfg",    {sizeof(debugConfig),     debugConfig}     },
                                        { StorageType_Flash, "ClarinoxBlue.cfg", {sizeof(bluetoothConfig), bluetoothConfig} },
                                        { StorageType_Flash, "licence.txt",      {sizeof(licence),         licence}         },
                                        { StorageType_Flash,   NULL,             { 0,                        NULL}          } };

#elif defined( READ_ONLY_FILE_SUPPORTED )


static const VirtualFile fileList[] = { { StorageType_RAM, "SoftFrame.cfg",    {sizeof(debugConfig),     debugConfig}      },
                                        { StorageType_RAM, "ClarinoxBlue.cfg", {sizeof(bluetoothConfig), bluetoothConfig}  },
                                        { StorageType_RAM, "licence.txt",      {sizeof(licence),         licence}          },
                                        { StorageType_RAM,   NULL,             { 0,                        NULL}           } };
#endif
#endif


#if defined( FLASH_FILE_SUPPORTED )

extern ClxResult clxNonVolatileEraseBlock (u4 blockIndex);
extern ClxResult clxNonVolatileReadBlock  (u4 blockIndex, u1* nonVolatileBuffer, u4 size);
extern ClxResult clxNonVolatileWriteBlock (u4 blockIndex, u1* nonVolatileBuffer, u4 size);

#endif

static ClxResult open (const s1* name, u2 flags, void** file, u4* currentSize)
{
    u4 i = 0;
    //u4 numOfFiles = sizeof(fileList)/sizeof(VirtualFile);
    CLX_PRINTF("file open %s\n", name);
    //for (u4 i = 0; i < numOfFiles; i++)
    while(clxVirtualFileSystemMap[i].fileName != NULL)
    {
	    if (strcmp(name, clxVirtualFileSystemMap[i].fileName) == 0)
	    {
            *file = (void*)&clxVirtualFileSystemMap[i];
            switch(clxVirtualFileSystemMap[i].type)
            {
#if defined( READ_ONLY_FILE_SUPPORTED )
            case StorageType_RAM:
                *currentSize = clxVirtualFileSystemMap[i].ram.fileLen;
                CLX_PRINTF("file size %u\n", *currentSize);
                return CLX_SUCCESS;
#endif // defined( READ_ONLY_FILE_SUPPORTED )

#if defined( FLASH_FILE_SUPPORTED )
            case StorageType_Flash:
                {
///////////////// erase the file if NEEDED
#if 0
	                if ( clxNonVolatileEraseBlock(clxVirtualFileSystemMap[i].flash.blockIndex) != CLX_SUCCESS )
                    {
                        CLX_PRINTF("file write error - flash write\n");
                        return CLX_FILE_ERROR_WRITING;
                    }
                    return CLX_SUCCESS;
#endif
//////////////// erase the file
                    // read  size information
	                if ( clxNonVolatileReadBlock(clxVirtualFileSystemMap[i].flash.blockIndex, flashBuffer, sizeof(size_t)) != CLX_SUCCESS )
                    {
                        CLX_PRINTF("file open failed %s\n", name);
                        return CLX_FILE_OPEN_FAILED;
                    }
                    // Check the beginning of the file for size information
                    u4 j;
                    for (j = 0; j < sizeof(size_t); j++)
                    {
                        if (flashBuffer[j] != 0xFF)
                        {
                            break;
                        }
                    }
                    if(j == sizeof(size_t))
                    {
                        /*
                        Flash block is blank, but we cannot leave the flash empty, 
                        we should copy the default contents from the RAM buffer into Flash
                        */
	                    u4 fileLen = clxVirtualFileSystemMap[i].ram.fileLen;
	                    const s1* fileBuffer = clxVirtualFileSystemMap[i].ram.fileBuffer;

                        *currentSize = fileLen;
                        // CLX_PRINTF("file read %u bytes\n", *currentSize);
                        // copy data into the flash buffer
	                    memcpy(flashBuffer + sizeof(size_t), fileBuffer, fileLen);
                        // write the size information into the beginning of the Flash block
                        *(size_t*)flashBuffer = fileLen; 
                        // write buffer into the flash memory
	                    if ( clxNonVolatileWriteBlock(clxVirtualFileSystemMap[i].flash.blockIndex, (UInt8*)flashBuffer, fileLen + sizeof(size_t)) != CLX_SUCCESS )
                        {
                            CLX_PRINTF("file open error - Flash write failed\n");
                            return CLX_FILE_OPEN_FAILED;
                        }
                    }
                    else
                    {
                        // copy the size information into the user supplied pointer
                        *currentSize = *(size_t*)flashBuffer;
                    }
                    CLX_PRINTF("file size %u\n", *currentSize);
                    return CLX_SUCCESS;
                }
#endif // defined( FLASH_FILE_SUPPORTED )

            default:
                BLACKBOX;
            }
        }
        i++;
    }
    CLX_PRINTF("file does not exist %s\n", name);
	return CLX_FILE_NOT_EXIST;
}

static void close (void* file)
{
#if defined( READ_ONLY_FILE_SUPPORTED )
    // nothing required
#elif defined( FLASH_FILE_SUPPORTED )
    // NonVolatile functions for close file
#else
#   error "define only one of the following macros: READ_ONLY_FILE_SUPPORTED or FLASH_FILE_SUPPORTED"
#endif
}

static ClxResult write (void* file, u4 indexMsb, u4 indexLsb, const void* data, size_t len, size_t* written)
{
    VirtualFile* vf = (VirtualFile*)file;
    CLX_PRINTF("file write %s\n", vf->fileName);
    switch(vf->type)
    {
#if defined( READ_ONLY_FILE_SUPPORTED )
    case StorageType_RAM:
        // No write support for volatile file implementation, it is read only
        *written = len;
        // CLX_PRINTF("file write %u bytes\n", *written);
	    return CLX_SUCCESS;
#endif // defined( READ_ONLY_FILE_SUPPORTED )

#if defined( FLASH_FILE_SUPPORTED )
    case StorageType_Flash:
        if ( len > MAX_FLASH_BLOCK_SIZE )
        {
            BLACKBOX;
        }
        // copy data into the flash buffer
	    memcpy(flashBuffer + sizeof(size_t), data, len);
        // write the size information into the beginning of the Flash block
        *(size_t*)flashBuffer = len; 
        // write buffer into the flash memory
	    if ( clxNonVolatileWriteBlock(vf->flash.blockIndex, (UInt8*)data, len + sizeof(size_t)) != CLX_SUCCESS )
        {
            CLX_PRINTF("file write error - flash write\n");
            return CLX_FILE_ERROR_WRITING;
        }
        *written = len;
        // CLX_PRINTF("file write %u bytes\n", *written);
        return CLX_SUCCESS;
#endif // defined( FLASH_FILE_SUPPORTED )

    default:
        BLACKBOX;
    }
    CLX_PRINTF("file write error\n");
    return CLX_FILE_ERROR_WRITING;
}

static ClxResult read (void* file, u4 indexMsb, u4 indexLsb, void* buf, size_t len, size_t* read)
{
    VirtualFile* vf = (VirtualFile*)file;
    // CLX_PRINTF("file read %s\n", vf->fileName);
    switch(vf->type)
    {
#if defined( READ_ONLY_FILE_SUPPORTED )
    case StorageType_RAM:
        {
	    *read = 0;

	    if (indexMsb)
	    {
            CLX_PRINTF("file read error LEN_OVER_4GB_NOT_SUPPORTED\n");
		    return CLX_FILE_LEN_OVER_4GB_NOT_SUPPORTED;
	    }
	    else if (!file)
	    {
            CLX_PRINTF("file read error INVALID_HANDLE\n");
		    return CLX_FILE_INVALID_HANDLE;
	    }

	    u4 fileLen = vf->ram.fileLen;
	    const s1* fileBuffer = vf->ram.fileBuffer;

	    if (fileLen <= indexLsb)
	    {
            // CLX_PRINTF("file read END_OF_FILE\n");
		    return CLX_FILE_END_OF_FILE;
	    }

	    if ((fileLen - indexLsb) < len)
	    {
		    *read = fileLen - indexLsb;
	    }
	    else
	    {
		    *read = len;
	    }
        // CLX_PRINTF("file read %u bytes\n", *read);
	    memcpy(buf, fileBuffer + indexLsb, *read);
	    return CLX_SUCCESS;
        }
#endif // defined( READ_ONLY_FILE_SUPPORTED )

#if defined( FLASH_FILE_SUPPORTED )
    case StorageType_Flash:
        if ( len > MAX_FLASH_BLOCK_SIZE )
        {
            CLX_PRINTF("file read error FILE TOO LARGE FOR FLASH BLOCK\n");
            BLACKBOX;
        }

        // read the size information
	    if ( clxNonVolatileReadBlock(vf->flash.blockIndex, flashBuffer, sizeof(size_t)) != CLX_SUCCESS )
        {
            CLX_PRINTF("file read error FLASH READ ERROR\n");
            return CLX_FILE_ERROR_READING;
        }
        // Check the beginning of the file for size information
        u4 i;
        for (i = 0; i < sizeof(size_t); i++)
        {
            if (flashBuffer[i] != 0xFF)
            {
                break;
            }
        }
        if(i == sizeof(size_t))
        {
            *read = 0;
        }
        else
        {
            if ( clxNonVolatileReadBlock(vf->flash.blockIndex, flashBuffer, MAX_FLASH_BLOCK_SIZE) != CLX_SUCCESS )
            {
                CLX_PRINTF("file read error FLASH READ ERROR\n");
                return CLX_FILE_ERROR_READING;
            }
            /*
            copy the length of characters from the flash buffer to the user buf
            copy the size information into the user supplied pointer
            */
            //*read = *(size_t*)flashBuffer;
            UInt16 fileLen = *(size_t*)flashBuffer;
            if (fileLen <= indexLsb)
            {
                // CLX_PRINTF("file read END_OF_FILE\n");
                return CLX_FILE_END_OF_FILE;
            }
            if ((fileLen - indexLsb) < len)
            {
                *read = fileLen - indexLsb;
            }
            else
            {
                *read = len;
            }
            memcpy(buf, (flashBuffer + sizeof(size_t) + indexLsb), *read);
        }
        // CLX_PRINTF("file read %u bytes\n", *read);
        return CLX_SUCCESS;
#endif // defined( FLASH_FILE_SUPPORTED )

    default:
        BLACKBOX;
    }
    CLX_PRINTF("file read error\n");
    return CLX_FILE_ERROR_READING;
}

static boolean isIndexSupported (void* file, u4 indexMsb, u4 indexLsb)
{
    VirtualFile* vf = (VirtualFile*)file;
    CLX_PRINTF("file isIndexSupported %s\n", vf->fileName);
	if (indexMsb)
	{
        CLX_PRINTF("file index error too large\n");
		return FALSE;
	}
	else if (!file)
	{
        CLX_PRINTF("file index error no file\n");
		return FALSE;
	}

    switch(vf->type)
    {
#if defined( READ_ONLY_FILE_SUPPORTED )
    case StorageType_RAM:
        {
	    u4 fileLen = vf->ram.fileLen;
	    return (indexLsb < fileLen) ? TRUE : FALSE;	
        }
#endif // defined( READ_ONLY_FILE_SUPPORTED )

#if defined( FLASH_FILE_SUPPORTED )
    case StorageType_Flash:
        // Flash file system does not support indexing
        CLX_PRINTF("file index error indexing not supported\n");
        return FALSE;
#endif // defined( FLASH_FILE_SUPPORTED )

    default:
        BLACKBOX;
    }
    CLX_PRINTF("file index error\n");
    return FALSE;
}

ClxFileBspInterface clxMemoryFileInterface = {open,
											  close,
											  write,
											  read,
											  isIndexSupported};
                                              

#ifdef __cplusplus
}
#endif
