/*******************************************************************************************************************************
*
* Project             Bluetooth Application
* File                BtModuleControl.cpp
* Description         This file defines funtions to enable Bluetooth hardware.
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************************************************************/
#include <stdlib.h>
#include "ClxBsp.h"
#include "UartBsp.h"
#include "stm32f4xx_conf.h"

#ifdef __cplusplus
extern "C" {
#endif

boolean clxInitBtEnGpio(uint32_t clockSource, GPIO_TypeDef *port, uint16_t pin)
{
  GPIO_InitTypeDef        GPIO_InitStructure;
  
  RCC_AHB1PeriphClockCmd(clockSource, ENABLE);
  GPIO_InitStructure.GPIO_Pin 	= pin;
  GPIO_InitStructure.GPIO_Mode 	= GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd 	= GPIO_PuPd_NOPULL;
  GPIO_Init(port, &GPIO_InitStructure);
  
  return true;
}

boolean clxBluetoothEnable(GPIO_TypeDef *port, uint16_t pin)
{ 
    /*Set the BT_EN pin high*/
    GPIO_SetBits  (port, pin);   
    return true ;
}

boolean clxBluetoothDisable(GPIO_TypeDef *port, uint16_t pin)
{
    /*Set the BT_EN pin low*/
    GPIO_ResetBits  (port, pin);
    return true ;
}


boolean clxEnableBluetoothHardware()
{
	boolean ret = true;        
    /* Enable Bluetooth hardware by toggling BT_EN pin*/
#if defined (BLUETOOTH_ENABLE)   
	clxInitBtEnGpio(BLUETOOTH_ENABLE_PORT_CLOCK_SETUP_GPIO, BLUETOOTH_ENABLE_PORT, BLUETOOTH_ENABLE_PIN);
    ret = clxBluetoothDisable(BLUETOOTH_ENABLE_PORT, BLUETOOTH_ENABLE_PIN);
    clxSleep(100);
    ret = clxBluetoothEnable(BLUETOOTH_ENABLE_PORT, BLUETOOTH_ENABLE_PIN);
    clxSleep(100);
    ret = clxBluetoothDisable(BLUETOOTH_ENABLE_PORT, BLUETOOTH_ENABLE_PIN);
    clxSleep(100);
    ret = clxBluetoothEnable(BLUETOOTH_ENABLE_PORT, BLUETOOTH_ENABLE_PIN);
    clxSleep(100);
#elif defined (BLUETOOTH_SHUTDOWN)
	clxInitBtEnGpio(BLUETOOTH_SHUTDOWN_PORT_CLOCK_SETUP_GPIO, BLUETOOTH_SHUTDOWN_PORT, BLUETOOTH_SHUTDOWN_PIN);
	ret = clxBluetoothDisable(BLUETOOTH_SHUTDOWN_PORT, BLUETOOTH_SHUTDOWN_PIN);
    clxSleep(100);
    ret = clxBluetoothEnable(BLUETOOTH_SHUTDOWN_PORT, BLUETOOTH_SHUTDOWN_PIN);
    clxSleep(100);
    ret = clxBluetoothDisable(BLUETOOTH_SHUTDOWN_PORT, BLUETOOTH_SHUTDOWN_PIN);
    clxSleep(100);
    ret = clxBluetoothEnable(BLUETOOTH_SHUTDOWN_PORT, BLUETOOTH_SHUTDOWN_PIN);
    clxSleep(100);
#endif
  
    return ret;
}

#ifdef __cplusplus
}
#endif

