/*******************************************************************************
*
* Project             Clarinox SoftFrame
* File                OsBsp.cpp
* Description         Implementation of BSP interfaces and functions for SysBios
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>

#include "ClxBsp.h"
#include "ClxCommon.h"
#include "Thread.Bsp.h"

#include "ClxSemaphore.h"
#include "ClxThread.h"
#include "ClxMutex.h"
#include "ClxTime.h"
#include "ClarinoxErrorCodes.h" 

#   define CLARINOX_SOFTFRAME_TIMER_THREAD_STACK_SIZE                              200u  
#   define CLARINOX_SOFTFRAME_DEBUG_THREAD_STACK_SIZE                              120u
#   define CLARINOX_SOFTFRAME_UI_THREAD_STACK_SIZE                                 220u
#   define CLARINOX_SOFTFRAME_TERMINAL_EMULATOR_THREAD_STACK_SIZE                  120u  
#   define CLARINOX_BLUETOOTH_APPLICATION_THREAD_STACK_SIZE                        400u  
#   define CLARINOX_BLUETOOTH_STACK_THREAD_STACK_SIZE                              900u 
#   define CLARINOX_BLUETOOTH_UARTRX_THREAD_STACK_SIZE                             200u   
#   define CLARINOX_WLAN_STACK_THREAD_STACK_SIZE	                              1300u 
#   define CLARINOX_WLAN_APPLICATION_THREAD_STACK_SIZE          				   250u 

#ifdef __cplusplus
extern "C" {
#endif
#include  <os.h> 	
  

u2 clarinoxuCosSoftFrameTaskStackSizeTable[4]	= { CLARINOX_SOFTFRAME_TIMER_THREAD_STACK_SIZE, 
                                                    CLARINOX_SOFTFRAME_DEBUG_THREAD_STACK_SIZE,
                                                    CLARINOX_SOFTFRAME_UI_THREAD_STACK_SIZE,
                                                    CLARINOX_SOFTFRAME_TERMINAL_EMULATOR_THREAD_STACK_SIZE }; 

u2 clarinoxuCosBluetoothTaskStackSizeTable[3] 	= { CLARINOX_BLUETOOTH_APPLICATION_THREAD_STACK_SIZE,                    
													CLARINOX_BLUETOOTH_STACK_THREAD_STACK_SIZE,   
                                                    CLARINOX_BLUETOOTH_UARTRX_THREAD_STACK_SIZE };
													  
u2 clarinoxuCosWlanTaskStackSizeTable[2]		= { CLARINOX_WLAN_STACK_THREAD_STACK_SIZE,
                                                    CLARINOX_WLAN_APPLICATION_THREAD_STACK_SIZE };													  
/**
 *
 * Thread Interface:
 */
ClxOSThread clxBspThreadInterfaceCreate (void* handle,
                                        const char* name,
                                        u4 stackSize,
                                        u4 priority)
{
    OS_TCB* tcb = (OS_TCB*)clxPoolsetAlloc ( 0, __LINE__, sizeof(OS_TCB) );
    void* stack = clxPoolsetAlloc( 0, __LINE__, stackSize * sizeof(u4) );

    OS_ERR        err;
    // handle =  (ClxOSThread)tcb;
    OSTaskCreate((OS_TCB     *)tcb,                /* Create the start task                                */
                 (CPU_CHAR   *)name,
                 (OS_TASK_PTR )clxThreadEntryDispatcher,
                 (void       *)handle,
                 (OS_PRIO     )priority,
                 (CPU_STK    *)stack,
                 (CPU_STK_SIZE)stackSize / 10,
                 (CPU_STK_SIZE)stackSize,
                 (OS_MSG_QTY  )0,
                 (OS_TICK     )0,
                 (void       *)0,
                 (OS_OPT      )(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                 (OS_ERR     *)&err);
    if (err == OS_ERR_NONE)
    {
        return tcb;
    }
    else
    {
        BLACKBOX;            //(err == OS_ERR_ILLEGAL_CREATE_RUN_TIME || OS_ERR_NAME || OS_ERR_PRIO_INVALID || OS_ERR_STK_INVALID || OS_ERR_STK_SIZE_INVALID || OS_ERR_STK_LIMIT_INVALID 
                            //|| OS_ERR_TASK_CREATE_ISR || OS_ERR_TASK_INVALID || OS_ERR_TCB_INVALID)
		return NULL;
    }  
}

ClxResult clxBspThreadInterfaceSuspend (ClxOSThread thread)
{
    OS_ERR err;
    /* Individual thread suspension not supported */
    OSTaskSuspend((OS_TCB*)thread, &err);
    if (err == OS_ERR_NONE)
    {
        return CLX_SUCCESS;
    }
    else
    {
        BLACKBOX;
        return CLX_ERROR;
    }
}

ClxResult clxBspThreadInterfaceResume (ClxOSThread thread)
{
    OS_ERR err;
    /* Individual thread suspension not supported */    
	OSTaskResume((OS_TCB*)thread, &err);
    if (err == OS_ERR_NONE)
    {
        return CLX_SUCCESS;
    }
    else
    {
        BLACKBOX;
        return CLX_ERROR;
    }   
}

void clxBspThreadInterfaceDestroy (ClxOSThread thread)
{

    clxPoolsetFree(((OS_TCB*)thread)->StkBasePtr);
    clxPoolsetFree(thread);
}

ClxResult clxBspThreadInterfaceEndCurrentThread (void)
{
    OS_TCB* thread = (OS_TCB*)clxBspThreadInterfaceGetCurrentThreadOsID();

   // printf( "thread name        : %s", ((OS_TCB*)thread)->NamePtr );
   // printf( "thread stack size  : %u", ((OS_TCB*)thread)->StkSize);
   // printf( "thread stack usage : %u ( %u )", (((OS_TCB*)thread)->StkUsed), ((((OS_TCB*)thread)->StkUsed)*100/((OS_TCB*)thread)->StkSize) );  
     
    OS_ERR        err;
    OSTaskDel ((OS_TCB*)0,
             (OS_ERR  *)&err);
    if (err == OS_ERR_NONE)
    {
		return CLX_SUCCESS;
    }
    else
    {
        BLACKBOX;
		return NULL;
    }
}

void clxBspThreadInterfaceYieldCurrentThread (void)
{
    clxSleep(10);
}

void* clxBspThreadInterfaceGetCurrentThreadOsID(void)
{
    OS_ERR  Regerr;
    return(ClxThreadID)OSTaskRegGet ((OS_TCB     *)NULL,
                                   (OS_REG_ID   )0,
                                   (OS_ERR     *)&Regerr);
}

ClxResult clxBspThreadInterfaceBindToProcessor (ClxOSThread thread, u4 processorIndex)
{
    while(1);
}

/**
 *
 * Semaphore Interface:
 */
ClxSemaphore clxCreateSemaphore_( s4 initialValue, const char* file, unsigned int line )
{
    ClxSemaphore semaphore;
    OS_ERR        err;
    semaphore = clxPoolsetAlloc(  0, __LINE__, sizeof ( OS_SEM ) );
    OSSemCreate  ((OS_SEM      *)semaphore,
                  (CPU_CHAR    *)"",
                  (OS_SEM_CTR   )initialValue,                                      
                  (OS_ERR  *)&err);
    if (err == OS_ERR_NONE)        
    {
    }
    else
    {
      BLACKBOX; //(err == OS_ERR_CREATE_ISR || OS_ERR_ILLEGAL_CREATE_RUN_TIME || OS_ERR_NAME || OS_ERR_OBJ_CREATED || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE)
    }
    return semaphore;
}

ClxSemaphore clxCreateBinarySemaphore_( s4 initialValue, const char* file, unsigned int line )
{
    ClxSemaphore semaphore;
    OS_ERR        err;
    semaphore = clxPoolsetAlloc(   0, __LINE__, sizeof ( OS_SEM ) );
    OSSemCreate  ((OS_SEM      *)semaphore,
                  (CPU_CHAR    *)"",
                  (OS_SEM_CTR   )initialValue,                                      
                  (OS_ERR  *)&err);
    if (err == OS_ERR_NONE)
    {        
    }
    else
    {
      BLACKBOX; //(err == OS_ERR_CREATE_ISR || OS_ERR_ILLEGAL_CREATE_RUN_TIME || OS_ERR_NAME || OS_ERR_OBJ_CREATED || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE)
    }
    return semaphore;
}

void clxDeleteSemaphore( ClxSemaphore semaphore )
{
    OS_ERR        err;
    OSSemDel ((OS_SEM  *)semaphore,
              (OS_OPT   )OS_OPT_DEL_NO_PEND,
              (OS_ERR  *)&err);
    if (err == OS_ERR_NONE)
    {
    }
    else
    {
      BLACKBOX; //(err == OS_ERR_DEL_ISR || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE || OS_ERR_OPT_INVALID || OS_ERR_TASK_WAITING)
    }
    clxPoolsetFree( semaphore );
}

void clxAcquireSemaphore( ClxSemaphore semaphore )
{
    OS_ERR        err;
    OSSemPend((OS_SEM *)semaphore,
              (OS_TICK )0,
              (OS_OPT  )OS_OPT_PEND_BLOCKING,
              (CPU_TS *)0,
              (OS_ERR *)&err);
    if (err == OS_ERR_NONE)
    {
    }
    else
    {
      BLACKBOX;     //(err == OS_ERR_OBJ_DEL || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE || OS_ERR_OPT_INVALID || OS_ERR_PEND_ABORT || OS_ERR_PEND_ISR || OS_ERR_PEND_WOULD_BLOCK || OS_ERR_SCHED_LOCKED 
                   // || OS_ERR_STATUS_INVALID || OS_ERR_TIMEOUT)
    }
}

void clxReleaseSemaphore( ClxSemaphore semaphore )
{
    OS_ERR        err;
    OSSemPost((OS_SEM *)semaphore,
              (OS_OPT  )OS_OPT_POST_1,
              (OS_ERR *)&err);
    if (err == OS_ERR_NONE)
    {
    }
    else
    {
      BLACKBOX;         //(err == OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE || OS_ERR_SEM_OVF)
    }
}

ClxResult clxTryAcquireSemaphore( ClxSemaphore semaphore )
{
    OS_ERR        err;    
    OSSemPend ((OS_SEM  *)semaphore,
                    (OS_TICK   )0,
                    (OS_OPT  )OS_OPT_PEND_NON_BLOCKING,
                    (CPU_TS  )0,
                    (OS_ERR *)&err);
    if (err == OS_ERR_NONE)
    {
         return CLX_SUCCESS;
    }
    else if (err == OS_ERR_PEND_WOULD_BLOCK)
	{
        BLACKBOX;         
		return CLX_ERROR;
	}
	else
    {
        BLACKBOX;         //(err == OS_ERR_MUTEX_OWNER || OS_ERR_OBJ_DEL || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE || OS_ERR_OPT_INVALID || OS_ERR_PEND_ABORT| || OS_ERR_SCHED_LOCKED
                        //|| OS_ERR_STATE_INVALID || OS_ERR_STATUS_INVALID || OS_ERR_TIMEOUT)
        return CLX_ERROR;
    }
}

ClxResult clxAcquireSemaphoreTimed( ClxSemaphore semaphore, s4 milliseconds )
{
    OS_ERR        err;
    if(OSSemPend ((OS_SEM  *)semaphore,
                  (OS_TICK   )milliseconds,
                  (OS_OPT  )OS_OPT_PEND_BLOCKING,
                  (CPU_TS  )0,
                  (OS_ERR *)&err));
    if( err == OS_ERR_NONE )
    {
        return CLX_SUCCESS;
    } 
    else if( err == OS_ERR_TIMEOUT )
    {
        return CLX_ERROR;
    }
    else
    {
        BLACKBOX;
        return CLX_ERROR;
    }    
}
/**
 *
 * Mutex Interface:
 */
#if defined( CLX_DEBUG )
ClxMutex clxCreateMutex_( const char* file, unsigned int line )
#else
ClxMutex clxCreateMutex_( void )
#endif
{
    ClxMutex mutex;
    OS_ERR        err;
    mutex = ( ClxMutex ) clxPoolsetAlloc(   0, __LINE__, sizeof ( OS_MUTEX ) );
    OSMutexCreate((OS_MUTEX    *)mutex,
                  (CPU_CHAR    *)"App Mutex",
                  (OS_ERR      *)&err);
    if (err == OS_ERR_NONE)
    {
    }
    else
    {
      BLACKBOX;         //(err == OS_ERR_CREATE_ISR || OS_ERR_ILLEGAL_CREATE_RUN_TIME || OS_ERR_NAME || OS_ERR_OBJ_CREATED || OS_ERR_OBJ_PTR_NULL)
    }
    return mutex;
}


void clxDeleteMutex( ClxMutex mutex )
{
    OS_ERR err;
    OSMutexDel ((OS_MUTEX    *)mutex,
                        OS_OPT_DEL_NO_PEND,
                        (OS_ERR      *)&err);
    if (err == OS_ERR_NONE)
    {
    }
    else
    {
      BLACKBOX;         //(err == OS_ERR_DEL_ISR || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE || OS_ERR_OPT_INVALID || OS_ERR_STATE_INVALID || OS_ERR_TASK_WAITING)
    }
    clxPoolsetFree( mutex );
}

void clxAcquireMutex( ClxMutex mutex )
{
    OS_ERR err;
    OSMutexPend ((OS_MUTEX    *)mutex,
                   (OS_TICK   )0,
                   (OS_OPT    )OS_OPT_PEND_BLOCKING,
                   (CPU_TS *)0,
                   (OS_ERR   *)&err);
    if (err == OS_ERR_NONE)
    {
    }
    else 
    {
      BLACKBOX;         //(err == OS_ERR_MUTEX_OWNER || OS_ERR_OBJ_DEL || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE || OS_ERR_OPT_INVALID || OS_ERR_PEND_ABORT|| OS_ERR_OPT_INVALID || OS_ERR_PEND_ABORT
                        //|| OS_ERR_PEND_ISR || OS_ERR_PEND_WOULD_BLOCK || OS_ERR_SCHED_LOCKED || OS_ERR_STATE_INVALID || OS_ERR_STATUS_INVALID || OS_ERR_TIMEOUT)
    }
}

void clxReleaseMutex( ClxMutex mutex )
{
    OS_ERR err;
    OSMutexPost((OS_MUTEX *)mutex,
                (OS_OPT    )OS_OPT_POST_NONE,
                (OS_ERR    *)&err);
    if (err == OS_ERR_NONE)
    {
    }
    else
    {
      BLACKBOX;         //(err == OS_ERR_MUTEX_NESTING || OS_ERR_MUTEX_NOT_OWNER || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE || OS_ERR_POST_ISR)
    }
}

ClxResult clxTryAcquireMutex( ClxMutex mutex )
{
    OS_ERR err;
      OSMutexPend ((OS_MUTEX    *)mutex,
                   (OS_TICK   )0,
                   (OS_OPT    )OS_OPT_PEND_NON_BLOCKING,
                   (CPU_TS *)0,
                   (OS_ERR   *)&err);
    if (err == OS_ERR_NONE)
    {
        return CLX_SUCCESS;
    }
    else if (err == OS_ERR_PEND_WOULD_BLOCK)
	{
        BLACKBOX;         
		return CLX_ERROR;
	}
	else
    {
        BLACKBOX;         //(err == OS_ERR_MUTEX_OWNER || OS_ERR_OBJ_DEL || OS_ERR_OBJ_PTR_NULL || OS_ERR_OBJ_TYPE || OS_ERR_OPT_INVALID || OS_ERR_PEND_ABORT| || OS_ERR_SCHED_LOCKED
                        //|| OS_ERR_STATE_INVALID || OS_ERR_STATUS_INVALID || OS_ERR_TIMEOUT)
		return CLX_ERROR;
    }
}

/**
 *
 * Required Time Functions:
 */
void clxSleep( u4 milliseconds )
{
    static OS_ERR err;
    static u4 seconds = 0;
    if(milliseconds > 1000)
    {
      seconds = milliseconds/1000;
      milliseconds = milliseconds - (seconds*1000);
    }
    OSTimeDlyHMSM(0, 0, seconds, milliseconds,
                  OS_OPT_TIME_HMSM_STRICT,
                  &err);
}

void clxGetCurrentTime( ClxTimeval* t )
{
    /*
     * No of ticks are equal to milliseconds we can use ticks instead of milliseconds
     */
    //clxMemSet( t, 0, 18 );
}

u4 clxTickTime()
{
    OS_ERR        err;
    u4 currTime = OSTimeGet((OS_ERR  *)&err);
    return currTime;
}

u4 clxHighResolutionTime()
{
	/*
	Return value is in microseconds:
	*/
	u4 currentTick = (OS_TS_GET()/CLX_CPU_CLOCK);
	return currentTick;
}

void enableOsScheduler()
{
    //ToDo
}


void disableOsScheduler()
{
    //ToDo
}

u4 clxGetCurrentDate() 
{
	return 0;
}

#ifdef __cplusplus
}
#endif

void vendorSpecificEvent(u1 *buffer, u4 length, u4 eventCode)
{
}

void vendorSpecificReturnCode(u4 ocf, u1* data)
{
}

void clxTimeInit()
{
    //ToDo
}

