/*******************************************************************************
*
* Project             ClarinoxBlue
* File                UartBsp.cpp
* Description         UART interface for Bluetooth Chip
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2014
*
*******************************************************************************/

// _____________________________________________________________________________
//
#undef  CLX_MODULE_ID
#define CLX_MODULE_ID  1123
// _____________________________________________________________________________
//
#include <stdlib.h>
#include <map>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <stdint.h>
#include <assert.h>

#include "ClxBsp.h"
#include "ClarinoxWlanConst.h"
#include "ClxMemoryPoolset.h"
#include "ClarinoxErrorCodes.h"
#include "WlanErrors.h"
#include "ClarinoxWLanConst.h"
#include "WlanBufferManagerInterface.h"
#include "WlanNetworkStackInterface.h"

#include "stm32f4xx_sdio_sd.h"
#include "stm32f4xx_sdio.h"
#include "stm32f4_discovery.h"

#include "SdioBsp.h"
typedef s4 ClxResult;
SDIO_DataInitTypeDef Clx_SDIO_DataInitStructure;
SDIO_InitTypeDef Clx_SDIO_InitStructure;
SDIO_CmdInitTypeDef Clx_SDIO_CmdInitStructure;

extern  void* _clxAlloc_  (u2 moduleID, u4 line, u4 size );
extern  void  clxFree  ( void* ptr );
ClxSdioEventHandler SDIO_eventHandler;

void* SDIO_eventHandlerData;

#if defined(CLX_DEBUG)
#	define CLX_DEBUG_CONSOLE(str) printf (str) 
#else
#	define CLX_DEBUG_CONSOLE(str)
#endif

#define SD_DATATIMEOUT                  ((uint32_t)0xFFFF)

extern const s1* clxSdioIntegrity11ExternalInterruptIoDeviceName;
extern ClxSdioBspInterface clxSdioBspInterface_Integrity11;


#ifdef __cplusplus
extern "C" {
#endif

#include "StmBsp.h"
#include "os.h" 

SD_Error        SDIO_Send_Cmd_53			(ClxSdioRequest* cmd_struct, int write, int incr_addr, unsigned int no_of_blocks);
SD_Error        SDIO_Send_Cmd_52			(ClxSdioRequest* cmd_struct, int write, unsigned char *out);
SD_Error        SDIO_Send_Cmd_52_Check		(int write, unsigned int fn,unsigned int addr, unsigned char in, unsigned char *out);
extern          SD_Error CmdResp1Error		(uint8_t cmd);
void            EXTILine0_Config			(void);
void            CLX_DMA_ISR_Handler_Debug   (void); 
void            Clx_BSP_IntHandlerSDIO      (void);
void            Clx_BSP_IntHandlerDMA2_CH4  (void);
/**************************************************************************************
  * brief  Initializes SDIO port.
  * param  deviceName: 
  * param  eventHandler: 
  * param  eventHandlerData: 
  * param  handle: SDIO port handle
  * retval CLX_SUCCESS/CLX_FAIL.
  ************************************************************************************/
static ClxResult openHandle(const s1* deviceName, ClxSdioEventHandler eventHandler, void* eventHandlerData, ClxSdio* handle)
{
    SD_Error status = SD_OK;    
    SDIO_eventHandler = eventHandler;
    SDIO_eventHandlerData = eventHandlerData;
    /* Configure the NVIC Preemption Priority Bits */
    BSP_IntVectSet(SDIO_IRQn, Clx_BSP_IntHandlerSDIO);
    BSP_IntEn(SDIO_IRQn);
    BSP_IntVectSet(SD_SDIO_DMA_IRQn, Clx_BSP_IntHandlerDMA2_CH4);
    BSP_IntEn(SD_SDIO_DMA_IRQn);
    status = SD_Init();
    if(status != SD_OK)
    {
        STM_EVAL_LEDOn(LED4); 
        return CLX_SDIO_FAILED_TO_OPEN_DRIVER;
    }  
     
    *handle = (ClxSdio)SDIO;

    return CLX_SUCCESS;
}
    
// SDIO init controller function
static ClxResult initControllerHardware(ClxSdio handle, void* platformSpecificConfigParams)
{
    // .... implementation ....
    return CLX_SUCCESS;
}

// SDIO check card insert function
static ClxResult isCardInserted(ClxSdio handle, boolean& inserted)
{
    // .... implementation ....
    return CLX_SUCCESS;
}

// SDIO get list of IO functions
static ClxResult getListOfIOFunctions(ClxSdio handle, ClxSdioFunction* funtionsList)
{
    // .... implementation ....
    return CLX_SUCCESS;
}

/**************************************************************************************
  * brief  SDIO sends a request to the inserted SDIO card 
  * param  handle: 
  * param  request: 
  * retval CLX_SUCCESS/CLX_FAIL.
  ************************************************************************************/
static ClxResult sendRequest(ClxSdio handle, ClxSdioRequest* request)
{
  SD_Error errorstatus = SD_OK;
  enum {Read, Write} transferType;
  enum {Fixed, Inc} addressType;
  uint8_t cmd;
  
    // .... implementation ....
    switch (request -> type)
    {
        case ClxSdio_Init:
             break;
      
        case ClxSdio_Reset:
             break;         
             
        case ClxSdio_GetFunctionBlockSize:
            break;
        
        case ClxSdio_SetFunctionBlockSize:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Write;
            break;
          }
        case ClxSdio_WriteRegister:
          {
            cmd = SD_CMD_SDIO_RW_DIRECT;
            transferType = Write;
            break;
          }        
        case ClxSdio_ReadRegister:
          {
            cmd = SD_CMD_SDIO_RW_DIRECT;
            transferType = Read;
            break;
          }
        case ClxSdio_WriteDataBuffer_FixedAddr:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Write;
            addressType = Fixed;
            break;
          }
        case ClxSdio_WriteDataBuffer_IncAddr:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Write;
            addressType = Inc;
            break;
          }
        case ClxSdio_ReadDataBuffer_FixedAddr:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Read;
            addressType = Fixed;
            break;
          }
        case ClxSdio_ReadDataBuffer_IncAddr:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Read; 
            addressType = Inc;            
            break;
          }
        case ClxSdio_WriteDataBlocks_FixedAddr:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Write;
            addressType = Fixed;
            break;
          }
        case ClxSdio_WriteDataBlocks_IncAddr:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Write;
            addressType = Inc;            
            break;
          }
        case ClxSdio_ReadDataBlocks_FixedAddr:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Read;
            addressType = Fixed;
            break;
          }
        case ClxSdio_ReadDataBlocks_IncAddr:
          {
            cmd = SD_CMD_SDIO_RW_EXTENDED;
            transferType = Read;
            addressType = Inc;
            break;
          }
    }
 
    if(cmd == SD_CMD_SDIO_RW_DIRECT)
    {
        if (transferType == Read)
        {
            errorstatus = SDIO_Send_Cmd_52(request, 0, NULL);   
        }
        else
        {
            errorstatus = SDIO_Send_Cmd_52(request, 1, NULL);        
        } 
    }   
    else if(cmd == SD_CMD_SDIO_RW_EXTENDED)
    {
        if (transferType == Read)
        {
          errorstatus = SDIO_Send_Cmd_53(request, 0, ((addressType == Fixed) ? 0 : 1), 0);
        }
        else
        {
          errorstatus = SDIO_Send_Cmd_53(request, 1, ((addressType == Fixed) ? 0 : 1), 0);
        } 
    }
    if (errorstatus != SD_OK)
    {
        CLX_DEBUG_CONSOLE("failed to write SDIO Command\n");
        return(CLX_FAIL);
    }          

    request -> requestResult = CLX_SUCCESS;
    return CLX_SUCCESS;
}

// SDIO close port function
static ClxResult closeHandle(ClxSdio handle)
{
    // .... implementation ....
    return CLX_SUCCESS;
}

// SDIO returns the last platform-specific error code
static s4 getLastPlatformDependentError(ClxSdio handle)
{
    // .... implementation ....
    return 0;
}

ClxResult      maskCardInterrupt(ClxSdio handle, u4 ioFunctionsMaskFlag)
{
         switch(ioFunctionsMaskFlag)
        {
            case 0:
               
                break;
            case 0xFFFFFFFF:
                
                break;
            default:
                break;
        }
        return CLX_SUCCESS; 
}
/**************************************************************************************
  * brief  Allows to write/read blocks/stream of data starting from a specified 
						address in the module using CMD53.
  *        The Data transfer can be managed by DMA mode only. 
  * note   This operation should be followed by a function to check the 
  *        DMA Controller.
  *         - SD_WriteWaitOperation(): this function insure that the DMA
  *           controller has finished all data transfer.
  * param  write: Specifies whether the operation is read/write.
  * param  incr_addr: specifies whether the operation is to be done with address increment.
  * param  no_of_blocks: specifies the no. of blocks of data (1 block = 512 bytes)
  * retval SD_Error: SD Card Error code.
  ************************************************************************************/
SD_Error SDIO_Send_Cmd_53(ClxSdioRequest* cmd_struct, int write, int incr_addr, unsigned int no_of_blocks)
{
    SD_Error errorstatus = SD_OK;
    SD_Error waitStatus = SD_OK;	
    u4 dataLength  = cmd_struct->length;
    u4 dataAddress = cmd_struct->registerAddress;  
    SDIO->DCTRL = 0x0;	/*Data control register*/
    /***********************Set the data config. to write data in the buffer***************************************/
    Clx_SDIO_DataInitStructure.SDIO_DataTimeOut = SD_DATATIMEOUT;    
    if (dataLength < 512)
    {
        Clx_SDIO_DataInitStructure.SDIO_DataLength = dataLength;
        Clx_SDIO_DataInitStructure.SDIO_DataBlockSize = 0;
        Clx_SDIO_DataInitStructure.SDIO_TransferMode = SDIO_TransferMode_Stream; 
    }
    else
    {
        Clx_SDIO_DataInitStructure.SDIO_DataLength = dataLength;
        Clx_SDIO_DataInitStructure.SDIO_DataBlockSize = SDIO_DataBlockSize_512b; 
        Clx_SDIO_DataInitStructure.SDIO_TransferMode = SDIO_TransferMode_Block; 
    }
    if(write)
    {
        Clx_SDIO_DataInitStructure.SDIO_TransferDir = SDIO_TransferDir_ToCard;
    }
    else
    {
        Clx_SDIO_DataInitStructure.SDIO_TransferDir = SDIO_TransferDir_ToSDIO;
    }    
    Clx_SDIO_DataInitStructure.SDIO_DPSM = SDIO_DPSM_Enable;
    SDIO_DataConfig(&Clx_SDIO_DataInitStructure);

    /*******************************************************SetUp DMA*******************************************/
    SDIO_ITConfig(SDIO_IT_DCRCFAIL | SDIO_IT_DTIMEOUT | SDIO_IT_DATAEND | SDIO_IT_RXOVERR | SDIO_IT_STBITERR, ENABLE);    
    if(write)
    {
        if (dataLength < 512)
        {
            SD_LowLevel_DMA_TxConfig((uint32_t *)cmd_struct->data, dataLength);        
        }
        else
        {
            SD_LowLevel_DMA_TxConfig((uint32_t *)cmd_struct->data, dataLength);
        }
    }
    else
    {
        SDIO_DMACmd(ENABLE);
        if (dataLength < 512)
        {
            SD_LowLevel_DMA_RxConfig((uint32_t *)cmd_struct->data, dataLength);        
        }
        else
        {
            SD_LowLevel_DMA_RxConfig((uint32_t *)cmd_struct->data, dataLength);
        }
    }
    /*******************************************************Send CMD53*******************************************/
    Clx_SDIO_CmdInitStructure.SDIO_Argument = write ? 0x80000000 : 0x00000000;                         /*Write or Read*/
    Clx_SDIO_CmdInitStructure.SDIO_Argument |= cmd_struct->function << 28;                             /*function number*/
    Clx_SDIO_CmdInitStructure.SDIO_Argument |= incr_addr ? 0x04000000 : 0x00000000;                    /*RAW flag*/
    Clx_SDIO_CmdInitStructure.SDIO_Argument |= dataAddress << 9;                                       /*address*/
    if (dataLength < 512)
    {
         Clx_SDIO_CmdInitStructure.SDIO_Argument |= dataLength;                                        /* byte mode */
    }
    else
    {
         Clx_SDIO_CmdInitStructure.SDIO_Argument |= 0x08000000 | (dataLength/512);                    /* block mode */
    }
    Clx_SDIO_CmdInitStructure.SDIO_CmdIndex = SD_CMD_SDIO_RW_EXTENDED;
    Clx_SDIO_CmdInitStructure.SDIO_Response = SDIO_Response_Short;
    Clx_SDIO_CmdInitStructure.SDIO_Wait = SDIO_Wait_No;
    Clx_SDIO_CmdInitStructure.SDIO_CPSM = SDIO_CPSM_Enable;
    
    SDIO_SendCommand(&Clx_SDIO_CmdInitStructure);
    /*********************************************Wait for CMD53 Response*******************************************/
    errorstatus = CmdResp1Error(SD_CMD_SDIO_RW_EXTENDED);
    if (errorstatus != SD_OK)
    {
        return(errorstatus);
    }
    /*************************************Send Data****************************************************************/
    if(write)
    {
        SDIO_DMACmd(ENABLE);
        waitStatus = SD_WaitWriteOperation();
        if (waitStatus != SD_OK)
        {
            while(1);        
        }
    }
    else
    {
      waitStatus = SD_WaitReadOperation();
      if (waitStatus != SD_OK)
      {
          while(1);
      }
    }
    return SD_OK;
}
/**************************************************************************************
  * brief  Allows to write&read a byte of data in the specified address of the module 
  *					using CMD52. There is no data transfer in lines D3-D0
  * param  write: Specifies whether the operation is read/write.
  * param  fn: function number used with the command.
  * param  addr: address location from where the data is to be read/written.
  * param  in: data to be written to the specified addr location.
  * param  out: 
  * retval SD_Error: SD Card Error code.
  ************************************************************************************/
SD_Error SDIO_Send_Cmd_52(ClxSdioRequest* cmd_struct, int write, unsigned char *out)
{
	SD_Error errorstatus = SD_OK;
	SDIO->DCTRL = 0x0;/*Data control register*/


	Clx_SDIO_CmdInitStructure.SDIO_Argument = write ? 0x80000000 : 0x00000000;
	Clx_SDIO_CmdInitStructure.SDIO_Argument |= cmd_struct->function << 28;
	if(write)
	{
		Clx_SDIO_CmdInitStructure.SDIO_Argument |= 0x00000000;//0x08000000;
	}
	else
	{
		Clx_SDIO_CmdInitStructure.SDIO_Argument |= (write && out) ? 0x08000000 : 0x00000000;
	}
	Clx_SDIO_CmdInitStructure.SDIO_Argument |= cmd_struct->registerAddress << 9;
	Clx_SDIO_CmdInitStructure.SDIO_Argument |= (uint32_t)cmd_struct->data;
	Clx_SDIO_CmdInitStructure.SDIO_CmdIndex = SD_CMD_SDIO_RW_DIRECT;
	Clx_SDIO_CmdInitStructure.SDIO_Response = SDIO_Response_Short;
	Clx_SDIO_CmdInitStructure.SDIO_Wait = SDIO_Wait_No;
	Clx_SDIO_CmdInitStructure.SDIO_CPSM = SDIO_CPSM_Enable;
	SDIO_SendCommand(&Clx_SDIO_CmdInitStructure);
	
	errorstatus = CmdResp1Error(SD_CMD_SDIO_RW_DIRECT);
	if (errorstatus != SD_OK)
	{
		/*!< CMD Response TimeOut (wait for CMDSENT flag) */
		return(errorstatus);
	}
	return SD_OK;
}
/*************************************************************************************
  * brief  SDIO interrupt handler 
  ************************************************************************************/
void Clx_BSP_IntHandlerSDIO(void)
{
  /*Allows to process all the interrupts that are high*/
  SD_ProcessIRQSrc();
}
/*************************************************************************************
  * brief  SDIO DMA interrupt handler 
  ************************************************************************************/
void Clx_BSP_IntHandlerDMA2_CH4(void)
{
  /*This function waits until the SDIO DMA data transfer is finished*/
  SD_ProcessDMAIRQ();
}

ClxSdioBspInterface clxSdioSTM32F407BspInterface = {openHandle,
                                                    initControllerHardware,
                                                    isCardInserted,
                                                    getListOfIOFunctions,
                                                    sendRequest,                                                    
                                                    getLastPlatformDependentError,
                                                    maskCardInterrupt,
                                                    closeHandle};


#ifdef __cplusplus
}
#endif
