#ifndef _AVRCP_API_H_
#define _AVRCP_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Avrcp.Api.h
* Description         Declares ClarinoxBlue AVRCP API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#define Status u1
/**
Definition of the status call-back function
*/
typedef Status (*ClxAvrcpStatusCallbackFunc) (u1 &playStatus);

/**
Definition for the browse player ID
*/
typedef u2  ClxAvrcpBrowsePlayerId;

/*
Local service supported categories:
*/
enum ClxAvrcpSupportedFeatures
{
     AVRCP_CATEGORY_PLAYER_RECORDER      = 0x01,
     AVRCP_CATEGORY_MONITOR_AMPLIFIER    = 0x02,
     AVRCP_CATEGORY_TUNER                = 0x04,
     AVRCP_CATEGORY_MENU                 = 0x08,
     AVRCP_PLAYER_APPLICATION_SETTINGS   = 0x10, /*!<*** TARGET ONLY *** Bit 0 should be set for this bit to be set. */
     AVRCP_GROUP_NAVIGATION              = 0x20, /*!<*** TARGET ONLY *** Bit 0 should be set for this bit to be set. */
     AVRCP_SUPPORTS_BROWSING             = 0x40,
     AVRCP_SUPPORTS_MULTIPLE_MEDIA       = 0x80, /*! *** TARGET ONLY ***                                             */
     AVRCP_CATEGORY_ALL                  = 0x0F
};

enum ClxAvrcpNotificationCapabilities
{
  EVENT_PLAYBACK_STATUS_CHANGED_CAPABILITY             = 0x0001, /*!<Change in playback status of the current track.                                                              */
  EVENT_TRACK_CHANGED_CAPABILITY                       = 0x0002, /*!<Change of current track.                                                                                     */
  EVENT_TRACK_REACHED_END_CAPABILITY                   = 0x0004, /*!<Reached end of a track.                                                                                      */
  EVENT_TRACK_REACHED_START_CAPABILITY                 = 0x0008, /*!<Reached start of a track                                                                                     */
  EVENT_PLAYBACK_POS_CHANGED_CAPABILITY                = 0x0010, /*!<Change in playback position. Returned after the specified playback notification change notification interval */
  EVENT_BATT_STATUS_CHANGED_CAPABILITY                 = 0x0020, /*!<Change in battery status                                                                                     */
  EVENT_SYSTEM_STATUS_CHANGED_CAPABILITY               = 0x0080, /*!<Change in system status                                                                                      */
  EVENT_PLAYER_APPLICATION_SETTING_CHANGED_CAPABILITY  = 0x0100, /*!<Change in player application setting                                                                         */
  EVENT_NOW_PLAYING_CONTENT_CHANGED_CAPABILITY         = 0x0200, /*!<The content of the Now Playing list has changed, see 6.9.5.                                                  */
  EVENT_AVAILABLE_PLAYERS_CHANGED_CAPABILITY           = 0x0400, /*!<The available players have changed, see 6.9                                                                  */
  EVENT_ADDRESSED_PLAYER_CHANGED_CAPABILITY            = 0x0800, /*!<The Addressed Player has been changed, see 6.9.2.                                                            */
  EVENT_UIDS_CHANGED_CAPABILITY                        = 0x1000, /*!<The UIDs have changed, see 6.10.3.3.                                                                         */
  EVENT_VOLUME_CHANGED_CAPABILITY                      = 0x2000  /*!<The volume has been changed locally on the TG, see 6.13.3.                                                   */
  /*0x0e-0xFF Reserved for future use*/
};

enum ClxAvrcpRegisterNotification
{
  EVENT_PLAYBACK_STATUS_CHANGED             = 0x01, /*!<Change in playback status of the current track.                                                              */
  EVENT_TRACK_CHANGED                       = 0x02, /*!<Change of current track.                                                                                     */
  EVENT_TRACK_REACHED_END                   = 0x03, /*!<Reached end of a track.                                                                                      */
  EVENT_TRACK_REACHED_START                 = 0x04, /*!<Reached start of a track                                                                                     */
  EVENT_PLAYBACK_POS_CHANGED                = 0x05, /*!<Change in playback position. Returned after the specified playback notification change notification interval */
  EVENT_BATT_STATUS_CHANGED                 = 0x06, /*!<Change in battery status                                                                                     */
  EVENT_SYSTEM_STATUS_CHANGED               = 0x07, /*!<Change in system status                                                                                      */
  EVENT_PLAYER_APPLICATION_SETTING_CHANGED  = 0x08, /*!<Change in player application setting                                                                         */
  EVENT_NOW_PLAYING_CONTENT_CHANGED         = 0x09, /*!<The content of the Now Playing list has changed, see 6.9.5.                                                  */
  EVENT_AVAILABLE_PLAYERS_CHANGED           = 0x0a, /*!<The available players have changed, see 6.9                                                                  */
  EVENT_ADDRESSED_PLAYER_CHANGED            = 0x0b, /*!<The Addressed Player has been changed, see 6.9.2.                                                            */
  EVENT_UIDS_CHANGED                        = 0x0c, /*!<The UIDs have changed, see 6.10.3.3.                                                                         */
  EVENT_VOLUME_CHANGED                      = 0x0d  /*!<The volume has been changed locally on the TG, see 6.13.3.                                                   */
  /*0x0e-0xFF Reserved for future use                                                                                                                                */
};

/**
Valid values for AVRCP commands, used as an argument for #clxAvrcpSendButtonEvent function
*/
enum ClxAvrcpCommand
{
    ClxAvrcpVolUp           = 0x41,
    ClxAvrcpVolDown         = 0x42,
    ClxAvrcpMute            = 0x43,
    ClxAvrcpPlay            = 0x44,
    ClxAvrcpStop            = 0x45,
    ClxAvrcpPause           = 0x46,
    ClxAvrcpRecord          = 0x47,
    ClxAvrcpRewind          = 0x48,
    ClxAvrcpFastfw          = 0x49,
    ClxAvrcpEject           = 0x4a,
    ClxAvrcpForward         = 0x4b,
    ClxAvrcpBackward        = 0x4c
};

/**
Button press state, used as parameters in #clxAvrcpSendButtonEvent function.
*/
typedef enum AvrcpButtonEventType
{
    PRESSED,
    RELEASED,
    CLICKED
}AvrcpButtonEventType;

/**
Play status events of the current media track on the target, used as parameters to 
CLX_AVRCP_PLAY_STATUS_INDICATION indication.
*/
enum ClxAvrcpPlayStatusEvents
{
    ClxAvrcpStopped         = 0x00,
    ClxAvrcpPlaying         = 0x01,
    ClxAvrcpPaused          = 0x02,
    ClxAvrcpSeekForward     = 0x03,
    ClxAvrcpSeekRewind      = 0x04,
    ClxAvrcpPlayStatusError = 0xFF
};

/**
Track change events on the target, used as parameters to CLX_AVRCP_TRACK_CHANGE_INDICATION indication.
*/
enum ClxAvrcpTrackChangeEvents
{
    ClxAvrcpChangeOfTrack,
    ClxAvrcpStartOfTrack,
    ClxAvrcpEndOfTrack
};

/**
Volume change events on the target, used as parameters to CLX_AVRCP_VOLUME_CHANGE_INDICATION indication.
*/
enum ClxAvrcpVolumeChangeEvents
{
    ClxAvrcpVolumeUpEvent,
    ClxAvrcpVolumeDownEvent,
    ClxAvrcpMuteEvent
};

/**
Possible roles for AVRCP, used as an argument for #clxAvrcpCreate function
*/
enum ClxAvrcpRole
{
    ClxAvrcpTarget,
    ClxAvrcpController,
    ClxAvrcpTargetController
};


/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpBrowseCommand -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpBrowseCommand defines the possible commands and their corresponding values
** The user can identify the response for which command the response is sent, using these values 
** These are the available and supported commands used to browse through
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpBrowseCommandEnum
{
    ClxAvrcpBrowseCommandEnum_CmdNone                  = 0x00,  /*!<Indicates no commands                                       */
    ClxAvrcpBrowseCommandEnum_SetBrowsePlayer          = 0x70,  /*!<Command to set the target player which will be browsed      */
    ClxAvrcpBrowseCommandEnum_GetFolderItems           = 0x71,  /*!<Command to get particular folder contents details           */
    ClxAvrcpBrowseCommandEnum_GetTotalNumberOfItems    = 0x75,  /*!<Command to get the total number of items count              */
    ClxAvrcpBrowseCommandEnum_ChangeBrowsePath         = 0x72,  /*!<Command to change the path of folder to browse              */
    ClxAvrcpBrowseCommandEnum_PlayItem                 = 0x74,  /*!<Command to play a particular item                           */
    ClxAvrcpBrowseCommandEnum_RegisterNotification     = 0x31,  /*!<Command to register to receive any notification from target */
    ClxAvrcpBrowseCommandEnum_AddToNowPlayingList      = 0x90,  /*!<Command to Add item to Now Playing List                     */
    ClxAvrcpBrowseCommandEnum_SearchMediaItems         = 0x80   /*!<Command to search the media items                           */
}ClxAvrcpBrowseCommand;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpBrowseDirection -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpBrowseDirection defines the possible directions to navigate across the folders
** The user can pass these values to the browse command to browse through the folders in 
** the remote machine file system
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpBrowseDirectionEnum
{
    ClxAvrcpBrowseDirection_FolderUp        = 0x00,  /*!<To move one folder up/parent folder  */
    ClxAvrcpBrowseDirection_FolderDown      = 0x01,  /*!<To move one folder down/child folder */
    ClxAvrcpBrowseDirection_Reserved        = 0x02   /*!<Reserved for future use              */
} ClxAvrcpBrowseDirection;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpBrowseScope -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpBrowseScope defines the possible scopes in which the content navigation can take place
** The user can pass these values to the browse command to choose the scope in which the browsing is done in 
** the remote machine file system
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpBrowseScopeEnum
{
    ClxAvrcpBrowseScope_MediaPlayerList                 = 0x00,  /*!<Browse scope in media player list   */
    ClxAvrcpBrowseScope_MediaPlayerVirtualFileSystem    = 0x01,  /*!<Browse scope in virtual file system */
    ClxAvrcpBrowseScope_Search                          = 0x02,  /*!<Browse scope in search list         */
    ClxAvrcpBrowseScope_NowPlaying                      = 0x03   /*!<Browse scope in now playing list    */
} ClxAvrcpBrowseScope;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpMediaAttributeList -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpMediaAttributeList defines the list of IDs for attributes. These IDs are used to 
** uniqely identify media information. The user can use these values to identify the attribute 
** from the command response from the remote machine file system
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpMediaAttributeListEnum
{
    ClxAvrcpMediaAttributeList_Title        = 0x01,  /*!<Requests title attribute value of element to be returned from   target */
    ClxAvrcpMediaAttributeList_ArtistName   = 0x02,  /*!<Requests artists attribute value of element to be returned from target */
    ClxAvrcpMediaAttributeList_AlbumName    = 0x03,  /*!<Requests album attribute value of element to be returned from   target */
    ClxAvrcpMediaAttributeList_TrackNumber  = 0x04,  /*!<Requests track number attribute field to be returned from       target */
    ClxAvrcpMediaAttributeList_TotalTracks  = 0x05,  /*!<Requests total tracks attribute of element to be returned from  target */
    ClxAvrcpMediaAttributeList_Genre        = 0x06,  /*!<Requests genre attribute value of element to be returned from   target */
    ClxAvrcpMediaAttributeList_PlayingTime  = 0x07   /*!<Requests playing time attribute value of element to be returned        */
} ClxAvrcpMediaAttributeList;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpMediaType -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpMediaType defines the list of media types available for the media element item. 
** The user can use these values to identify the media type and take respective action
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpMediaTypeEnum
{
    ClxAvrcpMediaType_Audio     = 0x00,  /*!<Indicates the audio type of media available in target device */
    ClxAvrcpMediaType_Video     = 0x01   /*!<Indicates the video type of media available in target device */
} ClxAvrcpMediaType;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpFolderType -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpFolderType defines the list of folder type available for the folder element item. 
** This indicates the content type of the folder which is browsed through.
** The user can use these values to identify the folder type and take respective action.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpFolderTypeEnum
{
    ClxAvrcpFolderType_Mixed        = 0x00,  /*!<Indicates that the folder consists of mixed type of elements    */
    ClxAvrcpFolderType_Titles       = 0x01,  /*!<Indicates that the folder consists of titles type of elements   */
    ClxAvrcpFolderType_Albums       = 0x02,  /*!<Indicates that the folder consists of album type of elements    */
    ClxAvrcpFolderType_Artists      = 0x03,  /*!<Indicates that the folder consists of artist type of elements   */
    ClxAvrcpFolderType_Genres       = 0x04,  /*!<Indicates that the folder consists of genre type of elements    */
    ClxAvrcpFolderType_Playlists    = 0x05,  /*!<Indicates that the folder consists of playlist type of elements */
    ClxAvrcpFolderType_Years        = 0x06   /*!<Indicates that the folder consists of year type of elements     */
} ClxAvrcpFolderType;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpBrowseItemsType -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpBrowseItemsType defines the list of browse able items in the remote device. 
** The remote device contains any of the listed browse able items in any database.
** The user can use these values to identify the browse able element type and take respective action.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpBrowseItemsTypeEnum
{
    ClxAvrcpBrowseItemsType_MediaPlayer     = 0x01,  /*!<Indicates the items type as media player related  */
    ClxAvrcpBrowseItemsType_Folder          = 0x02,  /*!<Indicates the items type as folder related        */
    ClxAvrcpBrowseItemsType_MediaElement    = 0x03   /*!<Indicates the items type as media element related */
} ClxAvrcpBrowseItemsType;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpBrowsePlayerType -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpBrowsePlayerType defines the list of media players supported in the remote device. 
** The remote device contains any of the listed type of media player which plays the supported media element.
** The user can use these values to identify the supported players in the remote device.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpBrowsePlayerTypeEnum
{
    ClxAvrcpBrowsePlayerType_Audio              = 0x01,  /*!<Indicates the player as audio, which can play audio file elements */
    ClxAvrcpBrowsePlayerType_Video              = 0x02,  /*!<Indicates the player as video, which can play video file elements */
    ClxAvrcpBrowsePlayerType_BroadcastingAudio  = 0x03,  /*!<Indicates the player which broadcasts audio                       */
    ClxAvrcpBrowsePlayerType_BroadcastingVideo  = 0x04   /*!<Indicates the player which broadcasts video                       */
} ClxAvrcpBrowsePlayerType;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpBrowsePlayerSubType -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpBrowsePlayerSubType defines the list of media player sub types supported in the remote device. 
** The remote device contains any of the listed type of media player which plays the supported media element.
** The user can use these values to identify the supported players in the remote device.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpBrowsePlayerSubTypeEnum
{
    ClxAvrcpBrowsePlayerSubType_AudioBook   = 0x00000001,  /*!<Indicates the player sub type as audio book */
    ClxAvrcpBrowsePlayerSubType_Podcast     = 0x00000002   /*!<Indicates the player sub type as podcast    */  
} ClxAvrcpBrowsePlayerSubType;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpBrowsePlayStatus -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpBrowsePlayStatus defines the possible status of a playing item in the remote device player. 
** The user gets these values as a response to the browse command which requests the status of the playing item in the 
** remote device player.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpBrowsePlayStatusEnum
{
    ClxAvrcpBrowsePlayStatus_Stopped        = 0x00,  /*!<Indicates the track has stopped playing             */
    ClxAvrcpBrowsePlayStatus_Playing        = 0x01,  /*!<Indicates the track is playing                      */
    ClxAvrcpBrowsePlayStatus_Paused         = 0x02,  /*!<Indicates the track is paused                       */
    ClxAvrcpBrowsePlayStatus_ForwardSeek    = 0x03,  /*!<Indicates the track is forwarded                    */
    ClxAvrcpBrowsePlayStatus_RewindSeek     = 0x04,  /*!<Indicates the track is rewound                      */
    ClxAvrcpBrowsePlayStatus_Error          = 0xFF   /*!<Indicates an error in getting the play status value */
} ClxAvrcpBrowsePlayStatus;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxAvrcpBrowseResultStatus -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpBrowseResultStatus defines the possible result of the browse command issued by the user. 
** The user gets these values in the response to the browse command. This result is related to the command issued by the user. 
** It also indicates the result of the operation requested in the command. When the result is not success, then the user shall
** not consider the rest of the result output buffer as it may contain junk. The rest of the result buffer contains valid
** content only if the status is ClxAvrcpBrowseResultStatus
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxAvrcpBrowseResultStatusEnum
{
    ClxAvrcpBrowseResultStatus_CmdInvalid           = 0x00,  /*!<Target doesn't understand the given browse command           */
    ClxAvrcpBrowseResultStatus_InvalidParam         = 0x01,  /*!<Target doesn't understand the given parameter                */
    ClxAvrcpBrowseResultStatus_ParamError           = 0x02,  /*!<Parameter contents are wrong                                 */
    ClxAvrcpBrowseResultStatus_InternalError        = 0x03,  /*!<Some other error occurred                                    */
    ClxAvrcpBrowseResultStatus_CmdSuccess           = 0x04,  /*!<Command OK. Now the other output values returned are valid   */
    ClxAvrcpBrowseResultStatus_UidChanged           = 0x05,  /*!<The UIDs on the device has changed                           */
    ClxAvrcpBrowseResultStatus_Reserved             = 0x06,  /*!<Reserved for future use                                      */
    ClxAvrcpBrowseResultStatus_InvalidDirection     = 0x07,  /*!<The direction parameter is invalid                           */
    ClxAvrcpBrowseResultStatus_NotADirectory        = 0x08,  /*!<The UID provided doesn't refer to a folder item              */
    ClxAvrcpBrowseResultStatus_InvalidUid           = 0x09,  /*!<The UID provided does not refer to any currently valid item  */
    ClxAvrcpBrowseResultStatus_InvalidScope         = 0x0A,  /*!<The scope parameter is invalid                               */
    ClxAvrcpBrowseResultStatus_ErrorRange           = 0x0B,  /*!<The start of range provided is not valid                     */
    ClxAvrcpBrowseResultStatus_ErrorNotPlayable     = 0x0C,  /*!<Media player cannot handle the directory referred by the UID */
    ClxAvrcpBrowseResultStatus_MediaInUse           = 0x0D,  /*!<Media player cannot use this item at this time               */
    ClxAvrcpBrowseResultStatus_ListFull             = 0x0E,  /*!<No more items can be added to the Now Playing List           */
    ClxAvrcpBrowseResultStatus_NotSupportedSearch   = 0x0F,  /*!<The Browsed Media Player does not support search             */
    ClxAvrcpBrowseResultStatus_InProgressSearch     = 0x10,  /*!<A search operation is already in progress                    */
    ClxAvrcpBrowseResultStatus_InvalidPlayerId      = 0x11,  /*!<The specified Player Id does not refer to a valid player     */
    ClxAvrcpBrowseResultStatus_PlayerNotBrowseable  = 0x12,  /*!<The referred player does not support browsing                */
    ClxAvrcpBrowseResultStatus_PlayerNotAddressed   = 0x13,  /*!<The player is not addressed, for this command to be executed */
    ClxAvrcpBrowseResultStatus_NoValidSearchResults = 0x14,  /*!<The Search result list does not contain valid entries        */
    ClxAvrcpBrowseResultStatus_NotAvailablePlayers  = 0x15,  /*!<No available players                                         */
    ClxAvrcpBrowseResultStatus_PlayerChanged        = 0x16   /*!<Addressed Player Changed                                     */
} ClxAvrcpBrowseResultStatus;

/**
Refer to #clxAvrcpConnect function.
*/
#define CLX_AVRCP_CONNECT_COMPLETE                              0x5400

/**
Refer to #clxAvrcpSendButtonEvent function.
*/
#define CLX_AVRCP_SEND_BUTTON_COMPLETE                          0x5401

/**
Refer to #clxAvrcpSendButtonEvent function.
*/
#define CLX_AVRCP_REGISTER_NOTIFICATION_COMPLETE                0x5402

/**
Refer to #clxAvrcpUnitInfo function.
*/
#define CLX_AVRCP_UNITINFO_COMPLETE                             0x5403

/**
Refer to #clxAvrcpDisconnect function.
*/
#define CLX_AVRCP_DISCONNECT_COMPLETE                           0x5404

/**
FolderUID & Media Element UID length
*/
#define CLX_AVRCP_FOLDER_MEDIA_ELEMENT_UID_SIZE                 8

/**
Total number of features for the player
*/
#define CLX_AVRCP_TOTAL_FEATURE_BIT_MASK                        16

/**
Minimum length of the out buffer to hold browsing content
*/
#define CLX_AVRCP_MIN_BROWSE_OUT_BUFFER_LEN                     335

/**
Refer to #clxAvrcpSelectBrowsePlayer function.
*/
#define CLX_AVRCP_SELECT_BROWSE_PLAYER_COMPLETE                 0x6400

/**
Refer to #clxAvrcpGetFolderItems function.
*/
#define CLX_AVRCP_GET_FOLDER_ITEMS_COMPLETE                     0x6401

/**
Refer to #clxAvrcpGetTotalItemsCount function.
*/
#define CLX_AVRCP_GET_TOTAL_ITEMS_COUNT_COMPLETE                0x6402

/**
Refer to #clxAvrcpPlayItem function.
*/
#define CLX_AVRCP_PLAY_ITEM_COMPLETE                            0x6403

/**
Refer to #clxAvrcpChangeBrowsePath function.
*/
#define CLX_AVRCP_CHANGE_BROWSE_PATH_COMPLETE                   0x6404

/**
Refer to #clxAvrcpConnectBrowse function.
*/
#define CLX_AVRCP_CONNECT_BROWSE_COMPLETE                       0x6405

/**
Refer to #clxAvrcpDisconnectBrowse function.
*/
#define CLX_AVRCP_DISCONNECT_BROWSE_COMPLETE                    0x6406

/**
Refer to #clxAvrcpUIDCounterChangeRegisterNotification function.
*/
#define CLX_AVRCP_U_I_D_CHANGE_REGISTER_NOTIFICATION_COMPLETE   0x6407

/**
Refer to #clxAvrcpAddToNowPlayingList function.
*/
#define CLX_AVRCP_ADD_TO_NOW_PLAYING_LIST_COMPLETE              0x6408

/**
Refer to #clxAvrcpSearchMediaItems function.
*/
#define CLX_AVRCP_SEARCH_MEDIA_ITEMS_COMPLETE                   0x6409

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpResultConnect -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpResultConnect defines the structure of the response result buffer returned for the clxAvrcpConnect and  
** clxAvrcpConnectBrowse API issued by the user. On getting the response result back from the remote device, user can access the 
** result buffer in this structure format. 
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpResultConnectStruct
{
    ClxDeviceId deviceId;                   /*!<Device ID of the remote device which is connected                         */
    ClxError result;                        /*!<Result of the operation. Used only in case of command complete indication */
}ClxAvrcpResultConnect;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpResultDisconnect -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpResultDisconnect defines the structure of the response result buffer returned for the clxAvrcpDisconnectBrowse and  
** clxAvrcpDisconnectBrowse API issued by the user. On getting the response result back from the remote device, user can access 
** the result buffer in this structure format. 
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef ClxAvrcpResultConnect ClxAvrcpResultDisconnect;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpResultPlayerItem -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpResultPlayerItem defines the structure of the response result buffer returned for the clxAvrcpSelectBrowsePlayer 
** API issued by the user. On getting the response result back from the remote device, user can access the result buffer 
** in this structure format. 
** - The character set format ID is of the list mentioned in the website link 
**   (as per http://www.iana.org/assignments/character-sets/character-sets.xhtml)
** - itemsReturned value less than folderDepth indicates that the user provided buffer is not enough to hold all folder names 
**   returned from the target device
** - folderNameLenPairs format shall be [nameLen][name string].. ..[N](folderDepth times)
**   Maximum size is decided by the user application. But contents are limited to L2cap Max MTU size
**   Minimum size is CLX_AVRCP_MIN_BROWSE_OUT_BUFFER_LEN (335 bytes)
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpResultPlayerItemStruct
{
    ClxAvrcpBrowseResultStatus     status;              /*!<Result of the command issued by the user                           */
    u2                             uidCounter;          /*!<UID version of the target device database.                         */
    u4                             itemsCount;          /*!<Total number of items in the current folder                        */
    u2                             characterSet;        /*!<Character set format in which the folder names shall be displayed  */
    u1                             folderDepth;         /*!<Depth level of current folder                                      */
    u1                             itemsReturned;       /*!<Number of folder names returned                                    */
    u1                             folderNameLenPairs;  /*!<User allocated buffer which is filled by the stack during response */
} ClxAvrcpResultPlayerItem;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpResultFolderItems -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpResultFolderItems defines the structure of the response result buffer returned for the clxAvrcpGetFolderItems 
** API issued by the user. On getting the response result back from the remote device, user can access the result buffer 
** in this structure format. 
** - itemsReturned value less than itemsCount indicates that the user provided buffer is not enough to hold all item details
**   returned from the target device
** - listOfItems can be of any of the following structures ClxAvrcpMediaPlayerItem, ClxAvrcpFolderItem, ClxAvrcpMediaElementItem
**   Maximum size is decided by the user application. But contents are limited to L2cap Max MTU size
**   Minimum size is CLX_AVRCP_MIN_BROWSE_OUT_BUFFER_LEN (335 bytes)
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpResultFolderItemsStruct
{
    ClxAvrcpBrowseResultStatus     status;          /*!<Result of the command issued by the user                           */
    u2                             uidCounter;      /*!<UID version of the target device database.                         */
    u4                             itemsCount;      /*!<Total number of items in the remote device folder which is browsed */
    u1                             itemsReturned;   /*!<Total number of items returned                                     */
    u1                             listOfItems;     /*!<Contents of the item name list returned by the stack               */
} ClxAvrcpResultFolderItems;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpMediaPlayerItem -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpMediaPlayerItem defines the structure of the response result buffer returned for the clxAvrcpGetFolderItems 
** API issued by the user. If the listOfItems field of the structure ClxAvrcpResultFolderItems is of type 
** ClxAvrcpBrowseItemsType_MediaPlayer. Then the list buffer values can be accessed in this structure format. 
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpMediaPlayerItemStruct
{
    ClxAvrcpBrowseItemsType         type;           /*!<Type of the browse able item (ClxAvrcpBrowseItemsType_MediaPlayer) */
    u2                              itemLength;     /*!<Total length of this item                                          */
    u2                              playerId;       /*!<ID of this player which this structure represents                  */
    ClxAvrcpBrowsePlayerType        playerType;     /*!<Type of this player                                                */
    ClxAvrcpBrowsePlayerSubType     playerSubType;  /*!<Sub Type of this player                                            */
    ClxAvrcpBrowsePlayStatus        playStatus;     /*!<Current Play status of the player                                  */
    u1                              featureBits[CLX_AVRCP_TOTAL_FEATURE_BIT_MASK];  /*!<Supported features, by this player */
    u2                              characterSet;   /*!<Character set format in which the player names shall be displayed  */
    u2                              nameLength;     /*!<Length of the player name                                          */
    u1                              nameString;     /*!<Player name string of length nameLength                            */
} ClxAvrcpMediaPlayerItem;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpFolderItem -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpFolderItem defines the structure of the response result buffer returned for the clxAvrcpGetFolderItems 
** API issued by the user. If the listOfItems field of the structure ClxAvrcpResultFolderItems is of type 
** ClxAvrcpBrowseItemsType_Folder. Then the list buffer values can be accessed in this structure format. 
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpFolderItemStruct
{
    ClxAvrcpBrowseItemsType     type;               /*!<Type of the browse able item (ClxAvrcpBrowseItemsType_Folder)     */
    u2                          itemLength;         /*!<Total length of this item                                         */
    u1                          folderUid[CLX_AVRCP_FOLDER_MEDIA_ELEMENT_UID_SIZE]; /*!<UID of this element               */
    ClxAvrcpFolderType          folderType;         /*!<Type of this folder element of enum ClxAvrcpFolderType            */
    u1                          isPlayable;         /*!<Indicates whether this folder is playable by a player or not      */
    u2                          characterSet;       /*!<Character set format in which the folder names shall be displayed */
    u2                          nameLength;         /*!<Length of the folder name                                         */
    u1                          nameString;         /*!<Folder name string of length nameLength                           */
} ClxAvrcpFolderItem;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpMediaElementItem -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpMediaElementItem defines the structure of the response result buffer returned for the clxAvrcpGetFolderItems 
** API issued by the user. If the listOfItems field of the structure ClxAvrcpResultFolderItems is of type 
** ClxAvrcpBrowseItemsType_MediaElement. Then the list buffer values can be accessed in this structure format. 
** The nameString variable is followed by the values of structure ClxAvrcpMediaElementItemAttrib. The number of such structures
** followed by each other is the number of times of attributeCount
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpMediaElementItemStruct
{
    ClxAvrcpBrowseItemsType     type;           /*!<Type of the browse able item (ClxAvrcpBrowseItemsType_MediaElement)      */
    u2                          itemLength;     /*!<Total length of this item                                                */
    u1                          mediaElementUid[CLX_AVRCP_FOLDER_MEDIA_ELEMENT_UID_SIZE];   /*!<UID of this element          */
    ClxAvrcpMediaType           mediaType;      /*!<Type of this media element of enum ClxAvrcpMediaType                     */
    u2                          characterSet;   /*!<Character set format in which the media element names shall be displayed */
    u1                          attributeCount; /*!<Number of attribute items associated with this element                   */
    u2                          nameLength;     /*!<Length of the media element name                                         */
    u1                          nameString;     /*!<Element name string of length nameLength                                 */
} ClxAvrcpMediaElementItem;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpMediaElementItemAttrib -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpMediaElementItemAttrib defines the structure of the response result buffer returned for the clxAvrcpGetFolderItems 
** API issued by the user. Each attribute information is of this structure type 
** The nameString variable is followed by the values of this structure ClxAvrcpMediaElementItemAttrib. The number of such 
** structures followed by each other is the number of times of attributeCount of structure ClxAvrcpMediaElementItem
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpMediaElementItemAttribStruct
{
    u4              attribID;           /*!<Unique ID of the attribute in this scope                                     */
    u2              characterSet;       /*!<Character set format in which the attribute element names shall be displayed */
    u2              valueLength;        /*!<Length of the attribute element name                                         */
    u1              valueString;        /*!<Attribute element name string of length valueLength                          */
} ClxAvrcpMediaElementItemAttrib;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpResultUIDCounterNotify -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpResultUIDCounterNotify defines the structure of the response result buffer returned for the 
** notification registered with the stack for notifying if there is any change in the UID Counter value of the remote device 
** database system. 
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpResultUIDCounterNotifyStruct
{
    u2              uidCounter;         /*!< UID Counter value of the remote device database */
} ClxAvrcpResultUIDCounterNotify;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: clxAvrcpGetTotalItemsCount -->
**
** <!-- DESCRIPTION:-->
** \brief
** clxAvrcpGetTotalItemsCount defines the structure of the response result buffer returned for the clxAvrcpGetTotalItemsCount
** API for getting the total number of items count in the particular path/folder in the remote device database. 
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpResultTotalItemsCountStruct
{
    ClxAvrcpBrowseResultStatus     status;      /*!<Result of the command issued by the user                           */
    u2                             uidCounter;  /*!<UID Counter value of the remote device database                    */
    u4                             itemsCount;  /*!<Total number of items in the remote device folder which is browsed */
} ClxAvrcpResultTotalItemsCount;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpResultChangeBrowsePath -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpResultChangeBrowsePath defines the structure of the response result buffer returned for the clxAvrcpChangeBrowsePath 
** API.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpResultChangeBrowsePathStruct
{
    ClxAvrcpBrowseResultStatus     status;      /*!<Result of the command issued by the user                           */
    u4                             itemsCount;  /*!<Total number of items in the remote device folder which is browsed */
} ClxAvrcpResultChangeBrowsePath;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxAvrcpResultSearchMediaItems -->
**
** <!-- DESCRIPTION:-->
** \brief
** ClxAvrcpResultSearchMediaItems defines the structure of the response result buffer returned for the clxAvrcpSearchMediaItems
** API for getting the total number of items count in the search operation performed in the remote device media database. 
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxAvrcpResultSearchMediaItemsStruct
{
    ClxAvrcpBrowseResultStatus     status;      /*!<Result of the command issued by the user                           */
    u2                             uidCounter;  /*!<UID Counter value of the remote device database                    */
    u4                             itemsCount;  /*!<Total number of items in the remote device folder which is browsed */
} ClxAvrcpResultSearchMediaItems;

/**
Data Structure for the indication CLX_AVRCP_SELECT_BROWSE_PLAYER_COMPLETE 
*/
typedef struct ClxAvrcpSelectBrowsePlayerCompleteStruct 
{
    _inout_ u4*                                 resultSize;
    _user_out_ ClxAvrcpResultPlayerItem*        result;
} ClxAvrcpSelectBrowsePlayerComplete;

/** 
Data Structure for the indication CLX_AVRCP_GET_FOLDER_ITEMS_COMPLETE 
*/
typedef struct ClxAvrcpGetFolderItemsCompleteStruct 
{
    u4*                                          resultSize;
    _user_out_ ClxAvrcpResultFolderItems*        result;
} ClxAvrcpGetFolderItemsComplete;

/** 
Data Structure for the indication CLX_AVRCP_GET_TOTAL_ITEMS_COUNT_COMPLETE 
*/
typedef struct ClxAvrcpGetTotalItemsCountCompleteStruct 
{
    _user_out_ ClxAvrcpResultTotalItemsCount*     result;
} ClxAvrcpGetTotalItemsCountComplete;

/** 
Data Structure for the indication CLX_AVRCP_CHANGE_BROWSE_PATH_COMPLETE 
*/
typedef struct ClxAvrcpChangeBrowsePathCompleteStruct 
{
    _user_out_ ClxAvrcpResultChangeBrowsePath*    result;
} ClxAvrcpChangeBrowsePathComplete;

/** 
Data Structure for the indication CLX_AVRCP_PLAY_ITEM_COMPLETE 
*/
typedef struct ClxAvrcpPlayItemCompleteStruct 
{
    _user_out_ ClxAvrcpBrowseResultStatus*     status;
} ClxAvrcpPlayItemComplete;

/** 
Data Structure for the indication CLX_AVRCP_U_I_D_CHANGE_REGISTER_NOTIFICATION_COMPLETE 
*/
typedef struct ClxAvrcpUIDCounterChangeRegisterNotificationCompleteStruct 
{
    _user_out_ ClxAvrcpResultUIDCounterNotify*  result;
} ClxAvrcpUIDCounterChangeRegisterNotificationComplete;

/** 
Data Structure for the indication CLX_AVRCP_ADD_TO_NOW_PLAYING_LIST_COMPLETE 
*/
typedef struct ClxAvrcpAddToNowPlayingListCompleteStruct
{
    ClxAvrcpBrowseResultStatus     status;
} ClxAvrcpAddToNowPlayingListComplete;

/** 
Data Structure for the indication CLX_AVRCP_SEARCH_MEDIA_ITEMS_COMPLETE 
*/
typedef struct ClxAvrcpSearchMediaItemsCompleteStruct
{
    _inout_ u4*                                    resultSize;
    _user_out_ ClxAvrcpResultSearchMediaItems*     result;
} ClxAvrcpSearchMediaItemsComplete;

/** INDICATIONS*/
/**
This indication is sent to the application when a connection is made to AVRCP service.
This indication is only received when the connection request has been initiated by the
remote device. The parameter of this indication (params argument of the call-back function) will be NULL.
*/
#define CLX_AVRCP_CONNECT_INDICATION                       0x9400

/**
This indication is sent to the application when play status of the current media track changes in target device.
This indication will be received if the device role is Controller.
The parameter of this indication is an enum of type #ClxAvrcpPlayStatusEvents.
*/
#define CLX_AVRCP_PLAY_STATUS_INDICATION                   0x9401

/**
This indication is sent to the application when a change in track detected.
This indication will be received if the device role is Target.
The parameter of this indication is an enum of type #ClxAvrcpTrackChangeEvents.
*/
#define CLX_AVRCP_TRACK_CHANGE_INDICATION                  0x9402

/**
This indication is sent to the application when a change in volume detected.
This indication will be received if the device role is Target.
The parameter of this indication is an enum of type #ClxAvrcpVolumeChangeEvents.
*/
#define CLX_AVRCP_VOLUME_CHANGE_INDICATION                 0x9403

/**
This indication is sent to the application when a pass-through command received form controller.
This indication will be received if the device role is Target.
The parameter of this indication is an enum of type #ClxAvrcpCommand.
*/
#define CLX_AVRCP_BUTTON_PRESSED_INDICATION                0x9404

/**
This indication is sent to the application when an AVRCP signalling disconnect happens.
The parameter of this indication (params argument of the call-back function) will be NULL.
*/
#define CLX_AVRCP_SIGNALING_DISCONNECT_INDICATION          0x9405

/**
This indication is sent to the application when an AVRCP browse notification happens.
The parameter of this indication (params argument of the call-back function) will be of structure 
ClxAvrcpUIDCounterChangeIndication.
*/
#define CLX_AVRCP_U_I_D_CHANGE_INDICATION                  0x9406

/**
This indication is sent to the application when a browse connection is made to AVRCP service.
This indication is only received when the connection request has been initiated by the
remote device. The parameter of this indication (params argument of the call-back function) will be NULL.
*/
#define CLX_AVRCP_BROWSE_CONNECT_INDICATION                0x9407

/**
This indication is sent to the application when an AVRCP browse signalling disconnect happens.
The parameter of this indication (params argument of the call-back function) will be NULL.
*/
#define CLX_AVRCP_BROWSE_DISCONNECT_INDICATION             0x9408 


/**
Creates an  AVRCP service on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] role defines the role of the AVRCP which is of type ClxAvrcpRole.
\param[ in ] localDeviceCategory 
\param[ in ] companyID shall contain the 24-bit unique ID obtained from the IEEE Registration Authority
Committee (RAC), If the vendor of a TG device does not have the unique ID above, the value 0xFFFFFF may be used.
\param[ in ] eventIDs supported event IDs.
\param[ in ] avrcpStatusFunc Status of AVRCP such as play, pause, stop etc....
\param[ in ] indicationCallbackFunc A pointer to the call-back function which will be called upon 
                 reception of indication events related to this handle.
\return ClxHandle AVRCP handle. This handle must be used for any further operations on this instance of AVRCP profile.
*/
ClxHandle clxAvrcpCreate(_in_ ClxStack                        stack,
                         _in_ ClxAvrcpRole                    role,
                         _in_ enum ClxAvrcpSupportedFeatures  localDeviceCategory,
                         _in_ u4                              companyID,
                         _in_ u2                              eventIDs,
                         _in_ ClxAvrcpStatusCallbackFunc      avrcpStatusFunc,
                         _in_ ClxApplicationCallbackFunc      indicationCallbackFunc);

/**
Request to create an AVRCP connection.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type 
CLX_AVRCP_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  avrcp handle.
\param[ in ]  remoteDeviceId ID of the remote Bluetooth device, this id must be obtained via GAP APIs.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxAvrcpConnect(_in_ ClxHandle     avrcp,
                         _in_ ClxDeviceId   remoteDeviceId,
                         _in_ boolean       block);
                                
/**
Request a specific command when a button has been pressed by user, This function shall be called after #clxAvrcpConnect.
This command is only valid if the AVRCP role is Controller.
Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type 
CLX_AVRCP_SEND_BUTTON_COMPLETE.
The indication does not have any parameters.

\param[ in ]  avrcp handle.
\param[ in ]  buttenPressed type of button pressed.
\param[ in ]  buttonEventType indicates whether the button is pressed or released or clicked
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxAvrcpSendButtonEvent(_in_ ClxHandle                 avrcp,
                                 _in_ ClxAvrcpCommand           buttenPressed,
                                 _in_ AvrcpButtonEventType      buttonEventType,
                                 _in_ boolean                   block);

/********************************************************************************************
* \brief        Request to connect an AVRCP browsing channel. Pre-requisite is that the control connection should have been 
*                already established.
*
*                 Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
*                 type CLX_AVRCP_CONNECT_BROWSE_COMPLETE.
*                 The indication does not have any parameters.
*
* \param[ in ]  avrcp handle.
* \param[ in ]  remoteDeviceId ID of the remote Bluetooth device, this id must be obtained via GAP APIs.
* \param[ in ]  block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
                    will be raised upon command completion.
* \return ClxError error code for the operation.
*
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                   In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*****************************************************************************************/
ClxResult clxAvrcpConnectBrowse(_in_ ClxHandle       avrcp,
                                _in_ ClxDeviceId     remoteDeviceId,
                                _in_ boolean         block);

/********************************************************************************************
* \brief         Request to disconnect an existing AVRCP browsing connection.
*
*                 Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
*                 type CLX_AVRCP_DISCONNECT_BROWSE_COMPLETE.
*                 The indication does not have any parameters.
*
* \param[ in ]  avrcp handle.
* \param[ in ]  block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
                    will be raised upon command completion.
* \return ClxError error code for the operation.
*
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                   In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*****************************************************************************************/
ClxResult clxAvrcpDisconnectBrowse(_in_ ClxHandle     avrcp,
                                   _in_ boolean       block);

/******************************************************************************
* \brief Request a specific command when a specific player is selected by user, 
*            This function shall be called after #clxAvrcpConnect.
*            This command is only valid if the AVRCP role is Controller.
*            Non-blocking mode : When the command is complete, the call-back 
*            function will be called with an event of type 
*            CLX_AVRCP_SELECT_BROWSE_PLAYER_COMPLETE.
*            The indication does not have any parameters.
*            result the result buffer of structure ClxAvrcpResultPlayerItem to be filled by the stack. 
*                Maximum size is decided by the user application. But contents are limited to L2cap Max MTU size
*                Minimum size is CLX_AVRCP_MIN_BROWSE_OUT_BUFFER_LEN (335 bytes)
*
* \param[ in   ]  avrcp handle.
* \param[ in   ]  browsePlayerId ID of the player.
* \param[ inout]  resultSize size of the result buffer. In return the stack will update with buffer actually filled.
* \param[ out  ]  result is the out result buffer allocated and given to the stack. This will be filled by stack during callback
*                 the result buffer will be of structure ClxAvrcpResultPlayerItem. 
*                 Maximum size is decided by the user application. But contents are limited to L2cap Max MTU size
*                 Minimum size is CLX_AVRCP_MIN_BROWSE_OUT_BUFFER_LEN (335 bytes)
*
* \param[ in   ]  block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
*                  will be raised upon command completion.
* \return ClxError error code for the operation.
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
******************************************************************************/
ClxResult clxAvrcpSelectBrowsePlayer(_in_ ClxHandle                         avrcp,
                                     _in_ ClxAvrcpBrowsePlayerId            browsePlayerId,
                                     _inout_ u4*                            resultSize,
                                     _user_out_ ClxAvrcpResultPlayerItem*   result,
                                     _in_ boolean                           block);



/******************************************************************************
* \brief Request a specific command when user needs the folder item information, 
*            This function shall be called after #clxAvrcpSelectBrowsePlayer.
*            This command is only valid if the AVRCP role is Controller.
*            Non-blocking mode : When the command is complete, the call-back 
*            function will be called with an event of type 
*            CLX_AVRCP_FOLDER_ITEMS_INFO_RECEIVED_INDICATION.
*            The indication have structure ClxAvrcpResultFolderItems parameters.
*
*            If this function to be called after there was a change in the UID Counter of remote device (on receiving 
*            CLX_AVRCP_U_I_D_CHANGE_INDICATION), then a UID Counter change notification has to be registered using the function
*            clxAvrcpUIDCounterChangeRegisterNotification(), before this call.
*
* \param[ in    ]  avrcp handle.
* \param[ in    ]  scope scope of the browse of type ClxAvrcpBrowseScope
* \param[ in    ]  from start element number to return
* \param[ in    ]  to end element number to return
* \param[ in    ]  attributeCount number of attributes to return
* \param[ in    ]  attributeList types of attributes of type ClxAvrcpMediaAttributeList
*                The buffer can be a list of any of these structures ClxAvrcpMediaPlayerItem, 
*                ClxAvrcpResultFolderItems and ClxAvrcpMediaElementItem.
*                Maximum size is infinite- up to user application. But contents limited to L2cap Max MTU size
*                Minimum size is CLX_AVRCP_MIN_BROWSE_OUT_BUFFER_LEN (335 bytes)
* \param[ inout ]  resultSize size of the result buffer. In return the stack will update with buffer actually filled.
* \param[ out   ]  result the result buffer of structure ClxAvrcpResultFolderItems to be filled by the stack. 
* \param[ in    ]  block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
                    will be raised upon command completion.
* \return ClxError error code for the operation.
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
******************************************************************************/
ClxResult clxAvrcpGetFolderItems(_in_ ClxHandle                                  avrcp,
                                 _in_ ClxAvrcpBrowseScope                        scope,
                                 _in_ u4                                         from,
                                 _in_ u4                                         to,
                                 _in_ u1                                         attributeCount,
                                 _user_in_ const ClxAvrcpMediaAttributeList*     attributeList,
                                 _inout_ u4*                                     resultSize,
                                 _user_out_ ClxAvrcpResultFolderItems*           result,
                                 _in_ boolean                                    block);

/******************************************************************************
* \brief Request a specific command when user wants to know the total number of items in a folder, 
*            This function shall be called after #clxAvrcpSelectBrowsePlayer.
*            This command is only valid if the AVRCP role is Controller.
*            Non-blocking mode : When the command is complete, the call-back 
*            function will be called with an event of type 
*            CLX_AVRCP_TOTAL_ITEMS_COUNT_RECEIVED_INDICATION.
*            The indication have ClxAvrcpResTotalItemsCount structure as parameters.
*
* \param[ in    ]  avrcp handle.
* \param[ in    ]  scope scope of the command of type ClxAvrcpBrowseScope.
* \param[ out   ]  result the result buffer of structure ClxAvrcpResultTotalItemsCount to be filled by the stack. 
* \param[ in    ]  block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
                    will be raised upon command completion.
* \return ClxError error code for the operation.
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
******************************************************************************/
ClxResult clxAvrcpGetTotalItemsCount(_in_ ClxHandle                               avrcp,
                                     _in_ ClxAvrcpBrowseScope                     scope,
                                     _user_out_ ClxAvrcpResultTotalItemsCount*    result,
                                     _in_ boolean                                 block);

/******************************************************************************
* \brief Request a specific command when user plays a media element, 
*            This function shall be called after #clxAvrcpSelectBrowsePlayer.
*            This command is only valid if the AVRCP role is Controller.
*            Non-blocking mode : When the command is complete, the call-back 
*            function will be called with an event of type CLX_AVRCP_PLAY_ITEM_COMPLETE.
*            The indication does not have any parameters.
*
* \param[ in  ]  avrcp handle.
* \param[ in  ]  scope scope of the play command of type ClxAvrcpBrowseScope.
* \param[ in  ]  uidOfElement UID of the media element to be played
* \param[ in  ]  uidCounter indicates the uid change version of the target device
* \param[ out ]  result the result buffer of variable ClxAvrcpBrowseResultStatus to be filled by the stack. 
* \param[ in  ]  block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
                    will be raised upon command completion.
* \return ClxError error code for the operation.
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
******************************************************************************/
ClxResult clxAvrcpPlayItem(_in_ ClxHandle                           avrcp,
                           _in_ ClxAvrcpBrowseScope                 scope,
                           _user_in_ u1*                            uidOfElement,
                           _in_ u2                                  uidCounter,
                           _user_out_ ClxAvrcpBrowseResultStatus*   result,
                           _in_ boolean                             block);

/******************************************************************************
* \brief Request a specific command when the browse path is changed by user, 
*            This function shall be called after #clxAvrcpSelectBrowsePlayer.
*            This command is only valid if the AVRCP role is Controller.
*            Non-blocking mode : When the command is complete, the call-back 
*            function will be called with an event of type 
*            CLX_AVRCP_CHANGE_PATH_COMPLETE_INDICATION.
*            The indication have ClxAvrcpResChangeBrowsePath structure as parameters.
*
* \param[ in    ]  avrcp handle.
* \param[ in    ]  uidCounter indicates the uid change version of the target device
* \param[ in    ]  direction of the navigation
* \param[ in    ]  folderUid UID of the folder to be navigated in. If the navigation command is Folder Up this field is reserved.
* \param[ out   ]  result the result buffer of structure ClxAvrcpResultChangeBrowsePath to be filled by the stack. 
* \param[ in    ]  block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
                    will be raised upon command completion.
* \return ClxError error code for the operation.
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
******************************************************************************/
ClxResult   clxAvrcpChangeBrowsePath(_in_ ClxHandle                              avrcp,
                                     _in_ u2                                     uidCounter,
                                     _in_ ClxAvrcpBrowseDirection                direction,
                                     _user_in_ u1*                               folderUid,
                                     _user_out_ ClxAvrcpResultChangeBrowsePath*  result,
                                     _in_ boolean                                block);

/********************************************************************************************
* \brief          Request to register the change in the UID counter of the remote device database.
*
*                 Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
*                 type CLX_AVRCP_U_I_D_CHANGE_REGISTER_NOTIFICATION_COMPLETE.
*                 The indication has the parameter of structure ClxAvrcpUIDCounterChangeRegisterNotificationComplete.
*                 There when there is a change in the UID Counter of the remote device, an indication 
*                 CLX_AVRCP_U_I_D_CHANGE_INDICATION will be received.
*
* \param[ in ]     avrcp handle.
* \param[ out   ]  result the result buffer of structure ClxAvrcpResultUIDCounterNotify to be filled by the stack. 
* \param[ in ]     block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
*                  will be raised upon command completion.
* \return ClxError error code for the operation.
*
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                   In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*****************************************************************************************/
ClxResult clxAvrcpUIDCounterChangeRegisterNotification(_in_ ClxHandle                              avrcp,
                                                       _user_out_ ClxAvrcpResultUIDCounterNotify*  result,
                                                       _in_ boolean                                block);

/********************************************************************************************
* \brief          Request to add the items in the Now Playing List of the media player
*
*                 Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
*                 type CLX_AVRCP_ADD_TO_NOW_PLAYING_LIST_COMPLETE.
*                 The indication has the parameter of structure ClxAvrcpAddToNowPlayingListComplete.
*
* \param[ in    ]  avrcp handle.
* \param[ in    ]  scope scope of the command of type ClxAvrcpBrowseScope.
* \param[ in    ]  uid of the folder item to be added in Now Playing List
* \param[ in    ]  uidCounter indicates the uid change version of the target device
* \param[ in    ]  block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
*                  will be raised upon command completion.
* \return ClxError error code for the operation.
*
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                   In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*****************************************************************************************/
ClxResult clxAvrcpAddToNowPlayingList(_in_ ClxHandle                              avrcp,
                                      _in_ ClxAvrcpBrowseScope                    scope,
                                      _user_in_ u1*                               uid,
                                      _in_ u2                                     uidCounter,
                                      _in_ boolean                                block);

/********************************************************************************************
* \brief          Request to search the remote device media database for the presence of media elements based on the seach 
*                 string provided. This API will only perform the search and returns only the number of elements found in the 
*                 search operation and not the list. To get the list details, use API clxAvrcpGetFolderItems with scope as search
*
*                 Some players may not support search or unless it is addressed (can be found using feature bits returned
*                 by the clxAvrcpGetFolderItems API
*                 Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
*                 type CLX_AVRCP_SEARCH_MEDIA_ITEMS_COMPLETE.
*                 The indication has the parameter of structure ClxAvrcpSearchMediaItemsComplete.
*
* \param[ in ]     avrcp handle.
* \param[ in ]     charSet character set of the search string provided as the input.
* \param[ in ]     length length of the search string.
* \param[ in ]     searchString the string name of the media files to be searched.
* \param[ inout ]  resultSize size of the result buffer. In return the stack will update with buffer actually filled.
* \param[ out   ]  result the result buffer of structure ClxAvrcpResultSearchMediaItems to be filled by the stack. 
* \param[ in ]     block type of the operation.
*                - TRUE:  API will be blocked until this command is completed (successfully or failed).
*                - FALSE: API will return immediately and then a corresponding API completion event 
*                  will be raised upon command completion.
* \return ClxError error code for the operation.
*
*                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
*                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
*                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
*                - CLX_ERROR_INVALID_HANDLE: invalid handle.
*                   In case of blocking mode the following error codes may also be returned.
*                - CLX_SUCCESS: operation is successful
*                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*****************************************************************************************/
ClxResult clxAvrcpSearchMediaItems(_in_ ClxHandle                                 avrcp,
                                   _in_ u2                                        characterSet,
                                   _in_ u2                                        length,
                                   _user_in_ const u1*                            searchString,
                                   _inout_ u4*                                    resultSize,
                                   _user_out_ ClxAvrcpResultSearchMediaItems*     result,
                                   _in_ boolean                                   block);

/**
This command registers with the TG to receive notifications asynchronously based on specific events occurring, 
This function shall be called after #clxAvrcpConnect.
This command is only valid if the AVRCP role is Controller.
Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type 
CLX_AVRCP_SEND_BUTTON_COMPLETE.
The indication does not have any parameters.

\param[ in ]  avrcp handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxAvrcpRegisterNotification(_in_  ClxHandle              avrcp,
                                      _in_  boolean                block);

/**
The UNIT INFO status command is used to obtain information that pertains to the unit as a whole, 
This function shall be called after #clxAvrcpConnect.
Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type 
CLX_AVRCP_SEND_BUTTON_COMPLETE.
The indication does not have any parameters.

\param[ in ]  avrcp handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxAvrcpUnitInfo(_in_  ClxHandle                    avrcp,
                          _in_  boolean                      block);


/**
Request to close an AVRCP connection.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type 
CLX_AVRCP_DISCONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  avrcp handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
*/
ClxError clxAvrcpDisconnect(_in_ ClxHandle   avrcp,
                            _in_ boolean     block);

#ifdef __cplusplus
}
#endif


#endif // _AVRCP_API_H_
