#ifndef __Bpps_Api_h__
#define __Bpps_Api_h__

/*******************************************************************************
*
* Project             Bpp
* File                Bpps.Api.h
* Description         Declares API Functions and Definitions For Bpp
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Represents the handle to an ongoing OBEX transaction (BPP request).
The transaction may have been initiated by either the printer or the client
*/
typedef void* ClxBppTransactionHandle;

/**
Refer to #clxBppAssignUserDataToTransactionHandle function.
*/
#define CLX_BPP_ASSIGN_USER_DATA_TO_TRANSACTION_HANDLE_COMPLETE                             0x6000

/**
Refer to #clxBppSendRequest function.
*/
#define CLX_BPP_SEND_REQUEST_COMPLETE                                                       0x6001

/**
Refer to #clxBppAttachResponseAttributes function.
*/
#define CLX_BPP_ATTACH_RESPONSE_ATTRIBUTES_COMPLETE                                         0x6002

/**
Refer to #clxBppSendResponseCode function.
*/
#define CLX_BPP_SEND_RESPONSE_CODE_COMPLETE                                                 0x6003

/**
Refer to #clxBppReceiveAttributes function.
*/
#define CLX_BPP_RECEIVE_ATTRIBUTES_COMPLETE                                                 0x6004

/**
Refer to #clxBppPrinterRoleInitiateGetReferenceObjectRequest function.
*/
#define CLX_BPP_PRINTER_ROLE_INITIATE_GET_REFERENCE_OBJECT_REQUEST_COMPLETE                 0x6005

/**
Refer to #clxBppAbortRequest function.
*/
#define CLX_BPP_ABORT_REQUEST_COMPLETE                                                      0x6006

/**
This indication is received when an OBEX channel is established by the remote BPP client.
The parameter of this indication is of type #ClxBppObexChannelEstablishedIndication.
*/
#define CLX_BPP_OBEX_CHANNEL_ESTABLISHED_INDICATION                                         0xa000

/**
This indication is received when an OBEX channel is disconnected either by the remote BPP client,
or as a result of lower layer (e.g. RFCOMM, L2CAP or ACL) channel disconnection.
The parameter of this indication is of type #ClxBppObexChannelDisconnectedIndication.
*/
#define CLX_BPP_OBEX_CHANNEL_DISCONNECTED_INDICATION                                        0xa001

/**
This indication is received when a new request is received from the connected remote BPP client.
The parameter of this indication is of type #ClxBppNewRequestReceivedIndication.
*/
#define CLX_BPP_NEW_REQUEST_RECEIVED_INDICATION                                             0xa002

/**
This indication is received when one or more attributes attached to a transaction is received.
The parameter of this indication is of type #ClxBppAttributesReceivedIndication.
*/
#define CLX_BPP_ATTRIBUTES_RECEIVED_INDICATION                                              0xa003

/**
This indication is received when an ongoing request is aborted by the remote BPP agent.

NOTE : This indication is received only if the transaction has been initiated by the remote device.
The parameter of this indication is of type #ClxBppRequestAbortedIndication.
*/
#define CLX_BPP_REQUEST_ABORTED_INDICATION                                                  0xa004

/**
This indication is received when a transaction request is completely received, and now the local device needs to send a response code, and zero or more response attributes,
to the remote machine.
A transaction cannot be completed if a response is not sent back to the sender of the request.

NOTE : This indication is received only if the transaction has been initiated by the remote device.
The parameter of this indication is of type #ClxBppResponseRequiredIndication.
*/
#define CLX_BPP_RESPONSE_REQUIRED_INDICATION                                                0xa005

/**
This indication is received when an OBEX transaction is complete either with success or in error.
The indication may have been initiated by the local or remote device.
The parameter of this indication is of type #ClxBppTransactionCompleteIndication.
*/
#define CLX_BPP_TRANSACTION_COMPLETE_INDICATION                                             0xa006

typedef enum ClxBppRoleEnum
{
    ClxBppRole_Printer  = 0x00,                 /*!< BPP Host Role. */
    ClxBppRole_Sender   = 0x01                  /*!< BPP Client Role. */
}ClxBppRole;

/**
Enumerates different types of the BPP OBEX channel
*/
typedef enum ClxBppChannelTypeEnum
{
    ClxBppChannelType_Job = 0,                 /*!< Job (main) channel (printer is the server) */
    ClxBppChannelType_Status = 1,              /*!< Status channel (printer is the server) */
    ClxBppChannelType_Object = 2,              /*!< Object channel (printer is the client) */
    ClxBppChannelType_RuiReferencedJob = 3     /*!< RUI referenced job channel (not supported) */
} ClxBppChannelType;

/**
Enumerates the different phases of a BPP transaction
*/
typedef enum ClxBppTransactionPhaseEnum
{
    ClxBppTransactionPhase_Request = 0,     /*!< Request phase */
    ClxBppTransactionPhase_Response = 1     /*!< Response phase */
} ClxBppTransactionPhase;

/**
Enumerates different types of the BPP request to be sent, or which has been received
*/
typedef enum ClxBppRequestTypeEnum
{
    ClxBppRequestType_CreateJob = 0,               /*!< This operation is used to request creation of a new printing job.
                                                        This is issued to the printer by the client. */
    ClxBppRequestType_SendDocument = 1,            /*!< The SendDocument action is used to send the print data.
                                                        For each CreateJob, one and only one SendDocument action shall be issued.
                                                        This is issued to the printer by the client. */
    ClxBppRequestType_GetJobAttributes = 2,        /*!< This operation is used to request details about the status and attributes of an ongoing printing job.
                                                        If the RequestedJobAttributes attribute is included in the request, the Printer
                                                        shall respond only with those attributes specifically requested as long as all of those
                                                        attributes are valid, else the Printer shall respond with all attributes that it supports.
                                                        This is issued to the printer by the client. */
    ClxBppRequestType_GetPrinterAttributes = 3,    /*!< This operation is used to request details about the printers capabilities and status.
                                                        This is issued to the printer by the client. */
    ClxBppRequestType_CancelJob = 4,               /*!< This operation is used to cancel a particular print job.
                                                        This is issued to the printer by the client. */
    ClxBppRequestType_GetReferenceObjects = 5,     /*!< Some print data formats (e.g., XHTML-Print) can require the Printer be able to retrieve
                                                        objects or parts of objects from the Sender. Typical examples of objects that the Printer
                                                        retrieves from the Sender are image(s) that need to be interleaved with text, or
                                                        externally referenced style sheets.
                                                        This is issued to the client by the printer. */
    ClxBppRequestType_GetEvent = 6,                /*!< This operation is used by the client to allow the Printer to provide notification of
                                                        changes in its status and changes in the status of an ongoing printing job
                                                        This is issued to the printer by the client. */
    ClxBppRequestType_CreatePreciseJob = 7,        /*!< This operation is not supported in this version */
    ClxBppRequestType_GetMargins = 8,              /*!< This operation is not supported in this version */
    ClxBppRequestType_SendReference = 9,           /*!< This operation is not supported in this version */
    ClxBppRequestType_GetRUI = 10                  /*!< This operation is not supported in this version */
} ClxBppRequestType;

/**
Identifies the current state of the printer
*/
typedef enum ClxBppPrinterStateEnum
{
    ClxBppPrinterState_Idle = 0,          /*!< new jobs can start processing without waiting */
    ClxBppPrinterState_Processing = 1,    /*!< jobs are processing; new jobs may wait before */
    ClxBppPrinterState_Stopped = 2,       /*!< no jobs can be processed and intervention is required */
    ClxBppPrinterState_Unknown = 3        /*!< Unknown (not to be used when sending the printer state to a client) */
} ClxBppPrinterState;

/**
Indicates the reason why the Printer is in its current state
*/
typedef enum ClxBppPrinterReasonEnum
{
    ClxBppPrinterReason_None = 0,                    /*!< Indicates that there are no current state reasons */
    ClxBppPrinterReason_AttentionRequired = 1,       /*!< Indicates reason other than those listed */
    ClxBppPrinterReason_MediaJam = 2,                /*!< The device has a media jam. */
    ClxBppPrinterReason_Paused = 3,                  /*!< Someone has paused the printer and the PrinterState is stopped. In this state, a Printer may not produce printed output */
    ClxBppPrinterReason_DoorOpen = 4,                /*!< One or more covers on the device are open */
    ClxBppPrinterReason_MediaLow = 5,                /*!< At least one input tray is low on media */
    ClxBppPrinterReason_MediaEmpty = 6,              /*!< At least one input tray is empty */
    ClxBppPrinterReason_OutputAreaAlmostFull = 7,    /*!< One or more output area is almost full: e.g., tray, stacker, collator */
    ClxBppPrinterReason_OutputAreaFull = 8,          /*!< One or more output area is full: e.g., tray, stacker, collator */
    ClxBppPrinterReason_MarkerSupplyLow = 9,         /*!< The device is low on at least one marker supply: e.g., toner, ink, ribbon */
    ClxBppPrinterReason_MarkerSupplyEmpty = 10,      /*!< The device is out of at least one marker supply: e.g., toner, ink, ribbon */
    ClxBppPrinterReason_MarkerFailure = 11           /*!< The device has at least one marking device which has failed and requires service or replacement: e.g., pen */
} ClxBppPrinterReason;

/**
Data Structure for the indication #CLX_BPP_OBEX_CHANNEL_ESTABLISHED_INDICATION
*/
typedef struct ClxBppObexChannelEstablishedIndicationStruct
{
    ClxDeviceId          remoteDeviceId;        /*!< The ID of the remote device to which a BPP OBEX channel has been established */
    ClxBppChannelType    channelType;           /*!< The type of the BPP OBEX channel which has been established */
} ClxBppObexChannelEstablishedIndication;

/**
Data Structure for the indication #CLX_BPP_OBEX_CHANNEL_DISCONNECTED_INDICATION
*/
typedef struct ClxBppObexChannelDisconnectedIndicationStruct
{
    ClxDeviceId          remoteDeviceId;        /*!< The ID of the remote device from which the BPP OBEX channel has been disconnected */
    ClxBppChannelType    channelType;           /*!< The type of the BPP OBEX channel which has been disconnected */
} ClxBppObexChannelDisconnectedIndication;

/**
Data Structure for the indication #CLX_BPP_NEW_REQUEST_RECEIVED_INDICATION
*/
typedef struct ClxBppNewRequestReceivedIndicationStruct
{
    ClxBppTransactionHandle    transactionHandle;        /*!< The handle of the new request */
    ClxBppRequestType          requestType;              /*!< The type of the request received */
    ClxBppChannelType          channelType;              /*!< The type of the BPP channel over which the request has been received */
} ClxBppNewRequestReceivedIndication;

/**
Data Structure for the indication #CLX_BPP_ATTRIBUTES_RECEIVED_INDICATION
*/
typedef struct ClxBppAttributesReceivedIndicationStruct
{
    ClxBppTransactionHandle    transactionHandle;        /*!< The handle of the new request */
    void*                      userData;                 /*!< The user data associated to this request */
    ClxBppTransactionPhase     transactionPhase;         /*!< The transaction phase in which the attributes have been received */
    ClxConfigList*             attributeList;            /*!< A stack-allocated list of attributes attached to the transaction. The list and its elements are valid until the indication call-back function
                                                              returns. The list and elements SHALL not be modified or deleted by the application. Also, they SHALL NOT be accessed any more when the call-back function returns. */
    boolean                    moreAttributes;           /*!< Indicates if there are more attributes in the request or the response of this transaction, which need to be received.
                                                              When TRUE, the application SHALL call #clxBppReceiveAttributes in order to receive the rest of the attributes.
                                                              If FALSE, all the attributes for the current phase of the transaction has been received. Depending on the current phase,
                                                              an indication of type #CLX_BPP_RESPONSE_REQUIRED or #CLX_BPP_TRANSACTION_COMPLETE will be subsequently received. */
} ClxBppAttributesReceivedIndication;

/**
Data Structure for the indication #CLX_BPP_REQUEST_ABORTED_INDICATION
*/
typedef struct ClxBppRequestAbortedIndicationStruct
{
    ClxBppTransactionHandle    transactionHandle;        /*!< The handle of the new request */
    void*                      userData;                 /*!< The user data associated to this request */
} ClxBppRequestAbortedIndication;

/**
Data Structure for the indication #CLX_BPP_RESPONSE_REQUIRED_INDICATION
*/
typedef struct ClxBppResponseRequiredIndicationStruct
{
    ClxBppTransactionHandle    transactionHandle;        /*!< The handle of the new request */
    void*                      userData;                 /*!< The user data associated to this request */
} ClxBppResponseRequiredIndication;

/**
Data Structure for the indication #CLX_BPP_TRANSACTION_COMPLETE_INDICATION
*/
typedef struct ClxBppTransactionCompleteIndicationStruct
{
    ClxBppTransactionHandle    transactionHandle;        /*!< The handle of the new request */
    void*                      userData;                 /*!< The user data associated to this request */
    ClxError                   result;                   /*!< The final result of the transaction */
} ClxBppTransactionCompleteIndication;


/**
Creates a Basic Printing service (Printer role) on the local machine.

\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\param[ in ] securityRequirement level of security needed for authentication.

\return Handle to the BPP service. NULL if the initialization of the service has failed.
*/
ClxHandle clxBppPrinterRoleCreate(_in_ ClxStack                       stack,
                                  _in_ ClxApplicationCallbackFunc     callbackFunc,
                                  _in_ ClxSecurityRequirement         securityRequirement);

/**
                Assigns user data to a transaction handle. The details of the user data is application specific. The pointer to the user data will be
                returned back to the application as the argument of the indications, along with the transaction handle. The application may use this data
                to store any details related to a transaction.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_BPP_ASSIGN_USER_DATA_TO_TRANSACTION_HANDLE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] bpp                Service handle
\param[  in   ] transactionHandle  Handle of a valid (ongoing) transaction
\param[  in   ] userData           A pointer (of type void*) to the user data
\param[  in   ] block              Indicates mode of operation:
                                   TRUE : Blocking mode
                                   FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxBppAssignUserDataToTransactionHandle(_in_ ClxHandle                  bpp,
                                                  _in_ ClxBppTransactionHandle    transactionHandle,
                                                  _in_ void*                      userData,
                                                  _in_ boolean                    block);

/**
                Attaches one or more attributes to a request (as indicated by its transaction handle). The transaction shall be in the request phase. Otherwise,
                an error will be returned.
                
                This API function may be called several times for the same request. The application SHALL keep calling this API function until the final bit in
                the request (as indicated by the third argument of this function) is set, or a previous call to this function (for the same transaction) has returned
                an error.
                
                NOTE : When the last call to this function (which is indicated by the final bit being set) has been successful, the application SHALL call #clxBppReceiveAttributes
                in order to receive the attributes in the response.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_BPP_SEND_REQUEST_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] bpp                Service handle
\param[  in   ] transactionHandle  Handle of a valid (ongoing) transaction
\param[  in   ] attributeList      List of attributes to attach to the request
\param[  in   ] setFinalBit        Specifies whether or not the final bit must be set in the request:
                                   TRUE : There is no more attributes to attach to the request. The final bit must be set
                                   FALSE : There are more attributes to attach to the request. This function will be called again to feed more attributes
\param[  in   ] block              Indicates mode of operation:
                                   TRUE : Blocking mode
                                   FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_CONNECTION_NOT_EXIST: There is no BPP connection to the remote device
                - CLX_ERROR_INVALID_REQUEST: Invalid request
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxBppSendRequest(_in_ ClxHandle                    bpp,
                            _in_ ClxBppTransactionHandle      transactionHandle,
                            _user_in_ const ClxConfigList*    attributeList,
                            _in_ boolean                      setFinalBit,
                            _in_ boolean                      block);

/**
                Attaches one or more attributes to the response of an ongoing transaction (as indicated by its transaction handle). The transaction shall be in the response phase. Otherwise,
                an error will be returned.
                
                This API function may be called several times for the same transaction.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_BPP_ATTACH_RESPONSE_ATTRIBUTES_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] bpp                Service handle
\param[  in   ] transactionHandle  Handle of a valid (ongoing) transaction
\param[  in   ] attributeList      A list of one or more caller-allocated attributes to attach to the response. The list and all its members SHALL remain valid (not be deleted or modified)
                                   until this command is complete. The attributes which may be attached to the response depends on the request type. Please refer to the documentation of each request for more information.
\param[  in   ] block              Indicates mode of operation:
                                   TRUE : Blocking mode
                                   FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_CONNECTION_NOT_EXIST: There is no BPP connection to the remote device
                - CLX_ERROR_INVALID_REQUEST: There is no ongoing transaction, the transaction has not been initiated by the remote device,
                                             or the transaction is not in the response phase
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxBppAttachResponseAttributes(_in_ ClxHandle                    bpp,
                                         _in_ ClxBppTransactionHandle      transactionHandle,
                                         _user_in_ const ClxConfigList*    attributeList,
                                         _in_ boolean                      block);

/**
                Signals the completion of the response to a request by attaching a response code to the last response frame.
                If the response is to have any attributes, it must already have been attached to it by one or more calls to #clxBppAttachResponseAttributes.
                After this function is called, no more attributes may be attached to the response.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_BPP_SEND_RESPONSE_CODE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] bpp                Service handle
\param[  in   ] transactionHandle  Handle of a valid (ongoing) transaction
\param[  in   ] response           The response code. May be any value other than CLX_OBEX_RSP_CONTINUE
\param[  in   ] block              Indicates mode of operation:
                                   TRUE : Blocking mode
                                   FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_INVALID_REQUEST: There is no ongoing transaction, or the transaction is not in response phase, or it has not been initiated by the remote device
                - CLX_ERROR_CONNECTION_NOT_EXIST: There is no BPP connection to the remote device
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxBppSendResponseCode(_in_ ClxHandle                  bpp,
                                 _in_ ClxBppTransactionHandle    transactionHandle,
                                 _in_ ClxObexResponse            response,
                                 _in_ boolean                    block);

/**
                Signals the stack to receive some or entire attributes attached either the request or response of a transaction.
                
                This command gets complete immediately, without waiting for attributes to be received. When the attributes are received, an indication of type #CLX_BPP_ATTRIBUTES_RECEIVED_INDICATION will be sent to the application.
                
                NOTE : When this command is issued successfully, it shall not be issued again until an indication of type #CLX_BPP_ATTRIBUTES_RECEIVED_INDICATION is received.
                
                NOTE : This function SHALL be called again if the #CLX_BPP_ATTRIBUTES_RECEIVED_INDICATION indication specifies that there are more attributes to be received.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_BPP_RECEIVE_ATTRIBUTES_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] bpp                Service handle
\param[  in   ] transactionHandle  Handle of a valid (ongoing) transaction
\param[  in   ] block              Indicates mode of operation:
                                   TRUE : Blocking mode
                                   FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_CONNECTION_NOT_EXIST: There is no BPP connection to the remote device
                - CLX_ERROR_INVALID_REQUEST: There is no ongoing transaction
*/
ClxResult clxBppReceiveAttributes(_in_ ClxHandle                  bpp,
                                  _in_ ClxBppTransactionHandle    transactionHandle,
                                  _in_ boolean                    block);

/**
Data Structure for the indication CLX_BPP_PRINTER_ROLE_INITIATE_GET_REFERENCE_OBJECT_REQUEST_COMPLETE
*/
typedef struct ClxBppPrinterRoleInitiateGetReferenceObjectRequestCompleteStruct 
{
    _user_out_ ClxBppTransactionHandle*    transactionHandle;        /*!< Pointer to a variable which upon a successful completion of this command will contain the handle to this request */
} ClxBppPrinterRoleInitiateGetReferenceObjectRequestComplete;

/**
                Initiates a Get Reference Object request. This function may only be called if the local device has the printer roll. If the command is successful, a transaction handle
                will be returned.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_BPP_PRINTER_ROLE_INITIATE_GET_REFERENCE_OBJECT_REQUEST_COMPLETE.
                The parameter of this indication is of type #ClxBppPrinterRoleInitiateGetReferenceObjectRequestComplete.

\param[  in   ] bpp                Service handle
\param[  out  ] transactionHandle  Pointer to a variable which upon a successful completion of this command will contain the handle to this request
\param[  in   ] block              Indicates mode of operation:
                                   TRUE : Blocking mode
                                   FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxBppPrinterRoleInitiateGetReferenceObjectRequest(_in_ ClxHandle                         bpp,
                                                             _user_out_ ClxBppTransactionHandle*    transactionHandle,
                                                             _in_ boolean                           block);

/**
                Aborts a valid (ongoing) request. A request may be aborted only and only if it has been initiated by the local machine.

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_BPP_ABORT_REQUEST_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] bpp                Service handle
\param[  in   ] transactionHandle  Handle of a valid (ongoing) transaction
\param[  in   ] block              Indicates mode of operation:
                                   TRUE : Blocking mode
                                   FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

                Furthermore, the result of the command may be one of the following codes.
                In blocking mode, the result will be returned by this function. In non-blocking mode, the result will be passed to the call-back function:
                - CLX_SUCCESS: The operation has been successful
                - CLX_ERROR_CONNECTION_NOT_EXIST: There is no BPP connection to the remote device
                - CLX_ERROR_INVALID_REQUEST: There is no ongoing transaction, or the transaction has not been initiated by the local device
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxBppAbortRequest(_in_ ClxHandle                  bpp,
                             _in_ ClxBppTransactionHandle    transactionHandle,
                             _in_ boolean                    block);

#ifdef __cplusplus
}
#endif

#endif  // __Bpps_Api_h__

