#ifndef _ClarinoxBlueConfigParms_h_
#define _ClarinoxBlueConfigParms_h_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                ClarinoxBlueConfigParms.h
* Description         Declares ClarinoxBlue configuration parameters
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/



struct ClarinoxBlueConfigParameters
{
    struct Stack
    {

        boolean                         initSoftFrameWithStack;
        u4                              stackServicesCleanupTimeout;
        u2                              maxNoOfPairedDevices;
        enum ClxGapIoCapability         ioCapabilities;                 
        s1                              localDeviceName[249];
        enum ClxMajorDeviceClass        majorClassOfDevice; 
        u1                              minorClassOfDevice;             
        u4                              serviceClasses; 
        u4                              linkRequestTimeout;         
        boolean                         supportSecureSimplePairing;     
        boolean                         enableLinkLevelAuthentication; 
        s1                              usbPortName[16];
        s1                              transportType[16];
        s1                              extendedInquiryResponse[6];
        s1                              deviceID[6];
        s1                              comPortName[32];
        u4                              comPortSpeed;
        u2                              pageScanInterval;
        u2                              pageScanWindow;
        u2                              inquiryScanInterval;
        u2                              inquiryScanWindow;
        u2                              maxInquiryPeriod;
        u2                              minInquiryPeriod;
        u1                              maxInquiryTimeUnits;
        u1                              maxDevicesToReport;
        u1                              inquiryTxPower;
        u4                              maxNoOfRemoteDevices;

    } stack;
    struct L2cap
    {
        u4                              l2capOutgoingBufferSize;
        u4                              l2capIncomingBufferSize;
        u4                              l2capSignalChannelInputContainerSize;
        u4                              l2capSignalChannelOutputContainerSize;
        u4                              l2capMaxNumberOfStackWideL2capConnections;
        u2                              l2capSignalChannelMtu;
    } l2cap;

    struct Network
    {
       u4                               numberOfNetworkBufferDescriptors;
    } network;

    struct Ble
    {
        u4                              l2capSignalChannelInputContainerSize;
        u4                              l2capSignalChannelOutputContainerSize;

        u4                              attChannelInputContainerSize;
        u4                              attChannelOutputContainerSize;
        u2                              attChannelMaxMtu;
        u2                              attChannelQueuedWritesContainerSize;
    } ble;
    struct Hci
    {
        u4                              hciCommandBufferSize;
        u4                              hciLinkSupervisionTimeout;

        u4                              hciNumberOfRxAclBuffers;
        u4                              hciRxAclSingleBufferSize;
        u4                              hciEventBufferSize;
    } hci;
    struct Rfcomm
    {
        u4                              rfcommIncomingQueueSize;
        u4                              rfcommOutgoingQueueSize;
        u4                              rfcommMaxNumberOfL2capConnections;
		u2								rfcommMaxRemoteCredit;
        u2                              rfcommIncomingMtu;
        u2                              rfcommMaxOutgoingDataSize;
        ClxSecurityRequirement          rfcommSecurityRequirement;
    } rfcomm;
    struct Sdp
    {
        u4                              sdpMaxNumberOfL2capConnections;
        u2                              sdpIncomingMtu;
        ClxSecurityRequirement          sdpSecurityRequirement;
    } sdp;
    struct Avrcp
    {
        u4                              avrcpIncomingBufferSize;
        u4                              avrcpOutgoingBufferSize;
        u4                              avrcpMaxNumberOfL2capConnections;
        u2                              avrcpIncomingMtu;
        u2                              avrcpMaxOutgoingDataSize;
        ClxSecurityRequirement          avrcpSecurityRequirement;
    } avrcp;

#if defined(CLARINOX_BLUE_HDP_PROFILE_ENABLED)
    struct Hdp
    {
        u1                              mcapRole;
        u4                              hdpIncomingBufferSize;
        u4                              hdpOutgoingBufferSize;
        u2                              hdpIncomingMtu;
        u2                              hdpMaxOutgoingDataSize;
        ClxSecurityRequirement          hdpSecurityRequirement;
    } hdp;
#endif

#if defined(CLARINOX_BLUE_HID_CLIENT_PROFILE_ENABLED)
    struct Hid
    {
        u4                              IncomingBufferSize;
        u4                              OutgoingBufferSize;
        u4                              MaxNumberOfL2capConnections;
        u2                              IncomingMtu;
        u2                              MaxOutgoingDataSize;
        ClxSecurityRequirement          SecurityRequirement;
    } hid;
#endif /* CLARINOX_BLUE_HID_CLIENT_PROFILE_ENABLED */

#if defined(CLARINOX_BLUE_BNEP_PROTOCOL_ENABLED)
    struct Bnep
    {
        u4                              bnepIncomingBufferSize;
        u4                              bnepOutgoingBufferSize;
        u2                              bnepIncomingMtu;
        u2                              bnepMaxOutgoingDataSize;
        u2                              bnepMaxNumberOfL2capConnections;
        u4                              bnepMaxPacketRetransmitTimeout;
        u4                              bnepMaxRetransmitAttempt;
        ClxSecurityRequirement          bnepSecurityRequirement;
    } bnep;
#endif

};

extern ClarinoxBlueConfigParameters clarinoxBlueConfigParameters;



#endif // _ClarinoxBlueConfigParms_h_
