#ifndef _ClarinoxBlueScoInterface_h_
#define _ClarinoxBlueScoInterface_h_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                ClarinoxBlueScoInterface.h
* Description         ClarinoxBlue Bluetooth Protocol SCO Interface
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/



#ifdef __cplusplus
extern "C" {
#endif

/** 
SCO TX Buffer is filled by the application and used by stack
*/
struct ClxTxScoBuffer
{
    struct ClxCQueueable    queueable;      /*!< Internal use */

    u1*                     buffer;         /*!< buffer indicating data to be send to a SCO channel */
    u2                      bufferSize;     /*!< buffer size */

    u2                      scoHandle;      /*!< SCO channel connection handle */
    u2                      dataLength;     /*!< send data size */
};


/** 
SCO RX Buffer
*/
struct ClxRxScoBuffer
{
    struct ClxCQueueable    queueable;      /*!< Queueable object */

    u1*                     buffer;         /*!< buffer indicating data received from a SCO channel */

    u2                      scoHandle;      /*!< SCO channel connection handle */
    u2                      dataLength;     /*!< received data size */
};


/** 
SCO Interface structure declares bindings via the stack SCO function pointers
*/
struct ClarinoxBlueScoInterface
{
    ClxResult (*init) (_in_ struct ClarinoxBlueScoInterface* thisObj, u4* numberOfTxBuffers, u4* numberOfRxBuffers);    /*!< initialize SCO interface */
    void (*destroy) (_in_ struct ClarinoxBlueScoInterface* thisObj);                                                    /*!< destroy SCO interface */
    void (*rxReceived) (_in_ struct ClarinoxBlueScoInterface* thisObj, _in_ struct ClxRxScoBuffer* packet);             /*!< SCO data received callback */
};

/**
Returns a TX SCO buffer to the application for filling with SCO data.
\return ClxTxScoBuffer contains the details of the TX buffer.
*/
extern struct ClxTxScoBuffer* clxBlueGetTxScoBuffer();

/**
Releases unused TX SCO buffer
\param[ in ] buffer ClxTxScoBuffer, that would be released.
*/
extern void clxBlueReleaseTxScoBuffer(struct ClxTxScoBuffer* buffer);

/**
Releases RX SCO buffer after it is being consumed by the application
\param[ in ] buffer ClxRxScoBufferccontained the RX data.
*/
extern void clxBlueReleaseRxScoBuffer(struct ClxRxScoBuffer* buffer);

/**
Queue a TX SCO buffer for sending to the SCO channel
\param[ in ] packet ClxTxScoBuffer packet containing the TX data.
*/
extern void clxBlueQueueTxScoPacketForSending(_in_ struct ClxTxScoBuffer* packet);

/**
Flushes queued TX SCO buffer for sending
*/
extern void clxBlueFlushQueuedTxScoBuffers();

/** 
SCO Interface structure instance declaration
*/
extern struct ClarinoxBlueScoInterface* clarinoxBlueScoInterface;



#ifdef __cplusplus
}
#endif


#endif // _ClarinoxBlueScoInterface_h_

