#ifndef _GAP_API_H_
#define _GAP_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Gap.Api.h
* Description         Declares ClarinoxBlue Generic Access Profile API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Refer to #clxGapStartInquiry function.
*/
#define CLX_GAP_START_INQUIRY_COMPLETE                      0x4500

/**
Refer to #clxGapStartLimitedInquiry function.
*/
#define CLX_GAP_START_LIMITED_INQUIRY_COMPLETE              CLX_GAP_START_INQUIRY_COMPLETE

/**
Refer to #clxGapStopInquiry function.
*/
#define CLX_GAP_STOP_INQUIRY_COMPLETE                       0x4501

/**
Refer to #clxGapGetListOfDiscoveredDevices function.
*/
#define CLX_GAP_GET_LIST_OF_DISCOVERED_DEVICES_COMPLETE     0x4502

/**
Refer to #clxGapInitiateBonding function.
*/
#define CLX_GAP_INITIATE_BONDING_COMPLETE                   0x4503

/**
Refer to #clxGapPinCodeRequestReply function.
*/
#define CLX_GAP_PIN_CODE_REQUEST_REPLY_COMPLETE             0x4504

/**
Refer to #clxGapAcceptConnectionRequest function.
*/
#define CLX_GAP_ACCEPT_CONNECTION_REQUEST_COMPLETE          0x4505

/**
Refer to #clxGapRejectConnectionRequest function.
*/
#define CLX_GAP_REJECT_CONNECTION_REQUEST_COMPLETE          0x4506

/**
Refer to #clxGapSetDiscoverability function.
*/
#define CLX_GAP_SET_DISCOVERABILITY_COMPLETE                0x4507

/**
Refer to #clxGapSetConnectability function.
*/
#define CLX_GAP_SET_CONNECTABILITY_COMPLETE                 0x4508

/**
Refer to #clxGapGetListOfPairedDevices function.
*/
#define CLX_GAP_GET_LIST_OF_PAIRED_DEVICES_COMPLETE         0x4509

/**
Refer to #clxGapUserPasskeyRequestReply function.
*/    
#define CLX_GAP_USER_PASSKEY_REQUEST_REPLY_COMPLETE         0x450A

/**
Refer to #clxGapUserConfirmationRequestReply function.
*/
#define CLX_GAP_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE    0x450B

/**
Refer to #clxGapDeletePairedDeviceInfo function.
*/
#define CLX_GAP_DELETE_PAIRED_DEVICE_INFO_COMPLETE          0x450C

/**
Refer to #clxGapDeleteAllPairedDevicesInfo function.
*/

#define CLX_GAP_DELETE_ALL_PAIRED_DEVICES_INFO_COMPLETE     0x450D

/**
Refer to #clxGapSetBondability function.
*/
#define CLX_GAP_SET_BONDABILITY_COMPLETE                    0x450E

/**
Refer to #clxRequestRemoteDeviceName function.
*/
#define CLX_GAP_REQUEST_REMOTE_DEVICE_NAME_COMPLETE         0x450F

/**
Refer to #clxGapGetLocalBtAddress function.
*/
#define CLX_GAP_GET_LOCAL_BT_ADDRESS_COMPLETE               0x4510

/**
Refer to #clxGapDisconnectPhysicalLink function.
*/
#define CLX_GAP_DISCONNECT_PHYSICAL_LINK_COMPLETE           0x4511


/**
Refer to #clxGapWriteConfigurationParameters function.
*/
#define CLX_GAP_WRITE_CONFIGURATION_PARAMETERS_COMPLETE    0x4512


/**
This indication is received when the initialization of the stack is
complete either with success or in error.

The argument errorCode of the call-back function will be one of the following:
 
 - CLX_SUCCESS : the stack is successfully initialized and ready to use.
 - CLX_ERROR_CLARINOX_BLUE_FAILED : the stack could not be initialized. Although the
   stack is not initialized, but still the stack object has been created (and received
   as the argument stack of the call-back function). Therefore, the object still needs
   to be destroyed by calling #clxDestroyClarinoxBlue (or the equivalent in the target platform). 
   Note that, in this case, the function #clxTerminateClarinoxBlue (or the equivalent in the target platform)
   MUST NOT be called since the stack has never been initialized, hence no need for termination.

This indication does not have any parameter (params argument of the call-back function
will be NULL).
*/
#define CLX_GAP_INITIALIZATION_COMPLETE_INDICATION          0x850F

/**
This indication is received during inquiry procedure, which indicates
a device has been discovered in the vicinity. A ClxDeviceDetail object will be passed
to the user with this message. This object will be valid until the next clxGapStartInquiry
or clxGapStartLimitedInquiry API function call.

The parameter for this indication is of type #clxGapDeviceDiscoveredIndication.
*/
#define CLX_GAP_DEVICE_DISCOVERED_INDICATION                0x8510

/**
This indication is received during connection/authentication procedure,
which indicates that the user of the local device must enter a pin code or reject
the authentication request. Upon reception of this indication, the function
#clxGapPinCodeRequestReply MUST be called.

This indication does not have any parameter (params argument of the call-back function
will be NULL).
*/
#define CLX_GAP_PIN_CODE_REQUEST_INDICATION                 0x8511

/**
This indication is received when there is a physical-layer (ACL) connection request
from a remote device. Upon reception of this indication, either the function
#clxGapAcceptConnectionRequest or #clxGapRejectConnectionRequest MUST be called.

This indication does not have any parameter (params argument of the call-back function
will be NULL).
*/
#define CLX_GAP_INCOMING_CONNECTION_REQUEST_INDICATION      0x8512

/**
This indication is received when the physical (ACL) link to a remote device has failed. When the physical link
fails, all service-level connections to the remote device will also fail automatically. Therefore, there is no need
to disconnect the service-level connections manually.

The parameter for this indication is of type #clxGapLinkDisconnectionIndication.
*/
#define CLX_GAP_LINK_DISCONNECTION_INDICATION               0x8513

/**
This indication is received during authentication procedure when the Bluetooth controller asks the local device
to provide a passkey or reject the authentication request. Upon reception of this indication, the function
#clxGapUserPasskeyRequestReply MUST be called.

The parameter for this indication is of type #clxGapUserPasskeyRequestIndication.
*/
#define CLX_GAP_USER_PASSKEY_REQUEST_INDICATION             0x8514

/**
This indication is received during authentication procedure when the Bluetooth controller asks the local device
to display a passkey on its display/monitor/LCD. This indication is only received when the local device has display
capabilities. Upon reception of this indication, the local application MUST display the passkey to the user, and must
keep displaying the passkey until the indication #CLX_GAP_PAIRING_COMPLETE_INDICATION is received.

The parameter for this indication is of type #clxGapUserPasskeyNotificationIndication.
*/
#define CLX_GAP_USER_PASSKEY_NOTIFICATION_INDICATION        0x8515

/**
This indication is received during authentication procedure when the Bluetooth controller asks the local device
to confirm or reject the connection to a remote device. The indication contains a passkey as well. If the local device
has display capabilities, it must display the passkey to the user and asks for confirmation or rejection. Otherwise, it may
directly confirm or reject the request by calling the function #clxGapUserConfirmationRequestReply.

The parameter for this indication is of type #clxGapUserConfirmationRequestIndication.
*/
#define CLX_GAP_USER_CONFIRMATION_REQUEST_INDICATION        0x8516

/**
This indication is received during authentication procedure when the pairing procedure is complete. In case of legacy pairing, failure will
not cause this indication to occur. Whereas when simple secure pairing is used, this indication will occur either with success
or in error. This indication is received by both the initiator of the authentication procedure, and the responder to the authentication procedure.
If the local device is currently displaying a passkey to the user, or asking the user to provide a passkey, or confirming or rejecting an authentication
request, it may cancel the operation upon reception of this indication.

The parameter for this indication is of type #clxGapPairingCompleteIndication.
*/
#define CLX_GAP_PAIRING_COMPLETE_INDICATION                 0x8517

/**
This indication is received when the local application has requested for the stack to terminate, and the first phase of termination is complete.
Upon reception of this indication, the stack MUST be destroyed by calling the function provided to destroy the stack (this function may be different
for different platforms). Note that, the stack CANNOT be destroyed within the context of the call-back function thread.

This indication does not have any parameter (params argument of the call-back function
will be NULL). Also, all other arguments of the call-back function (except for indicationID) will have INVALID values.
*/
#define CLX_GAP_TERMINATION_COMPLETE_INDICATION             0x85FF

/**
Parameter type of following indications:

- #CLX_GAP_DEVICE_DISCOVERED_INDICATION
- #CLX_GAP_PIN_CODE_REQUEST_INDICATION
- #CLX_GAP_INCOMING_CONNECTION_REQUEST_INDICATION
- #CLX_GAP_LINK_DISCONNECTION_INDICATION
- #CLX_GAP_USER_PASSKEY_REQUEST_INDICATION
*/
struct ClxDeviceDetail
{
    s1                          deviceName[257];        /*!< Name of the device as a null-terminated UTF-8 encoded string */  
    u1                          bdAddr[6];              /*!< The universally unique Bluetooth address of the device */
    ClxDeviceId                 deviceId;               /*!< The ClarinoxBlue-assigned device ID which is fixed throughout a single session of ClarinoxBlue stack */
    u4                          serviceClasses;         /*!< The combination (bit-wide or-ed) value of generic services which are supported by the device. Refer to #ClxDeviceServiceClass enum for more information */
    enum ClxMajorDeviceClass    majorClassOfDevice;     /*!< The major class of device. Refer to ClarinoxBlueConst.h for more information */
    u1                          minorClassOfDevice;     /*!< The minor class of device which is interpreted in the context of the major class. Refer to ClarinoxBlue.h for more information */
    s1                          rssi;                   /*!< The RSSI value obtained by the remote device during device discovery process */
};

/**
Parameter type of following indications:
- #CLX_GAP_USER_PASSKEY_NOTIFICATION_INDICATION
- #CLX_GAP_USER_CONFIRMATION_REQUEST_INDICATION
*/
struct ClxDeviceDetailAndNumericValue
{
    struct ClxDeviceDetail    deviceDetail;             /*!< Details of the remote device for which this indication has been generated*/
    u4                        value;                    /*!< The value of the passkey which is to be displayed on the screen and/or confirmed by the local user. It will be 0 through 999999 decimal */
};

/**
Parameter type of following command complete indications:
- #CLX_GAP_GET_LIST_OF_DISCOVERED_DEVICES_COMPLETE
- #CLX_GAP_GET_LIST_OF_PAIRED_DEVICES_COMPLETE
*/
typedef struct ClxDevicesListStruct
{
    _user_out_ ClxDeviceDetail*   list;                     /*!< The list of discovered/paired devices. The list is allocated and owned by the stack */
    _user_out_ u4*                numberOfDevices;          /*!< The number of discovered/paired devices*/
}ClxDevicesList;

/**
Parameter type of the indication #CLX_GAP_PAIRING_COMPLETE_INDICATION
*/
struct clxGapPairingCompleteIndication
{
    struct ClxDeviceDetail    deviceDetail;             /*!< The details of the remote device for which simple pairing procedure has just completed */
    u1                        status;                   /*!< The result of the simple pairing procedure, a value of 0 indicates success */
};

/**
Parameter type of the indication #CLX_GAP_DEVICE_DISCOVERED_INDICATION.
The same as #ClxDeviceDetail. 
*/
#define clxGapDeviceDiscoveredIndication                ClxDeviceDetail

/**
Parameter type of the indication #CLX_GAP_PIN_CODE_REQUEST_INDICATION.
The same as #ClxDeviceDetail. 
*/
#define clxGapPinCodeRequestIndication                  ClxDeviceDetail

/**
Parameter type of the indication #CLX_GAP_INCOMING_CONNECTION_REQUEST_INDICATION.
The same as #ClxDeviceDetail. 
*/
#define clxGapIncomingConnectionRequestIndication       ClxDeviceDetail

/**
Parameter type of the indication #CLX_GAP_LINK_DISCONNECTION_INDICATION.
The same as #ClxDeviceDetail. 
*/
typedef struct ClxGapLinkDisconnectionIndicationStruct
{
    ClxDeviceDetail device;                  /*!< The details of the remote device from which the link has been disconnected */
    u1              disconnectionReason;     /*!< The reason of disconnection as defined in Bluetooth Core Specification */
} ClxGapLinkDisconnectionIndication;

#define clxGapLinkDisconnectionIndication ClxGapLinkDisconnectionIndication

/**
Parameter type of the indication #CLX_GAP_USER_PASSKEY_REQUEST_INDICATION.
The same as #ClxDeviceDetail.
*/
#define clxGapUserPasskeyRequestIndication              ClxDeviceDetail

/**
Parameter type of the indication #CLX_GAP_USER_PASSKEY_NOTIFICATION_INDICATION.
The same as #ClxDeviceDetailAndNumericValue.
*/
#define clxGapUserPasskeyNotificationIndication         ClxDeviceDetailAndNumericValue

/**
Parameter type of the indication #CLX_GAP_USER_CONFIRMATION_REQUEST_INDICATION.
The same as #ClxDeviceDetailAndNumericValue.
*/
#define clxGapUserConfirmationRequestIndication         ClxDeviceDetailAndNumericValue

/**
Parameter type of the command complete indication #CLX_GAP_GET_LIST_OF_DISCOVERED_DEVICES_COMPLETE.
The same as #ClxDevicesList. This list will be valid until the next clxGapStartInquiry
or clxGapStartLimitedInquiry API function call.
*/
#define clxGapGetListOfDiscoveredDevicesComplete        ClxDevicesList



/** Data Structure for the indication #CLX_GAP_GET_LIST_OF_PAIRED_DEVICES_COMPLETE. */
typedef struct ClxGapGetListOfPairedDevicesCompleteStruct 
{
    _user_out_ struct ClxDeviceDetail* list;
    _user_out_ u4* numberOfPairedDevices;
} ClxGapGetListOfPairedDevicesComplete;

/**
Determines the reason why an incoming connection request is being rejected by the local device.
*/
enum ClxConnectionRequestRejectReason 
{
    ClxLimitedResources                 = 0x0D, 
    ClxSecurity                         = 0x0E, 
    ClxUnacceptableBluetoothAddress     = 0x0F, 
    ClxUnspecified                      = 0x1F
};


/**
Determines which of parameters of the structure #ClxGapInquiryFilter have valid values.
*/
enum ClxGapInquiryFilterMask
{
    ClxNoInquiryFilter                  = 0x00,         /*!< All parameters have invalid values (no filtering is to be performed) */
    ClxMajorClass                       = 0x01,         /*!< Only #ClxGapInquiryFilter.majorClass has a valid value */
    ClxMajorAndMinorClass               = 0x03,         /*!< Only #ClxGapInquiryFilter.majorClass and #ClxGapInquiryFilter.minorClass have valid values */
    ClxServiceClass                     = 0x04,         /*!< Only #ClxGapInquiryFilter.serviceClass has a valid value */
    ClxMajorAndServiceClass             = 0x05,         /*!< Only #ClxGapInquiryFilter.majorClass and #ClxGapInquiryFilter.serviceClass have valid values */
    ClxMajorMinorServiceClass           = 0x07          /*!< All three parameters have valid values */
};

/**
Determines the type of filtering which is to be used when #clxGapStartInquiry function is called.
*/
struct ClxGapInquiryFilter
{
    enum ClxGapInquiryFilterMask    filterMask;         /*!< Determines which of the following three parameters have valid values. 
                                                             If set to ClxNoInquiryFilter, then no filtering is applied */
    enum ClxMajorDeviceClass        majorClass;         /*!< The major device class according to which discovered devices must be filtered */
    u1                              minorClass;         /*!< The minor device class according to which discovered devices must be filtered */ 
    u4                              serviceClass;       /*!< The service class(es) according to which discovered devices must be filtered */
};

/**
It gets the local device Bluetooth address.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_GET_LOCAL_BT_ADDRESS_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ out ] btAddress a pointer to the local Bluetooth address.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_INQUIRY_IN_PROGRESS: inquiry process is currently in progress
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: issuing the start inquiry command was not successful
*/
ClxError clxGapGetLocalBtAddress( _in_       ClxStack   stack, 
                                  _user_out_ u1*        btAddress,
                                  _in_       boolean    block );
/**
Starts inquiry from the local device. If any Bluetooth devices in the vicinity are found, then 
#CLX_GAP_DEVICE_DISCOVERED_INDICATION message will be received via the callback function. A name 
request will be performed for each new discovered device which is not already in the pairing list.

Optionally, a filter may be used to limit the discovered devices to a specific device class (or a combination of device classes).

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_START_INQUIRY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  inquiryFilter a pointer to a data structure determining the type of filtering (if any) which is to be used. This parameter CANNOT be NULL.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_INQUIRY_IN_PROGRESS: inquiry process is currently in progress
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: issuing the start inquiry command was not successful
*/
ClxError clxGapStartInquiry( _in_ ClxStack stack, 
                             _in_ const struct ClxGapInquiryFilter* inquiryFilter,
                             _in_ boolean block );


/**
Starts a limited inquiry from the local device. If any Bluetooth devices in the vicinity are found, then 
#CLX_GAP_DEVICE_DISCOVERED_INDICATION message will be received via the callback function. A name request 
will be performed for each new discovered device which is not already in the pairing list.

The limited inquiry is similar to standard inquiry (as performed by #clxGapStartInquiry), but it is only able to discover
devices which are in limited discovery mode. All parameters, and indications of this functions is the same as #clxGapStartInquiry.

Optionally, a filter may be used to limit the discovered devices to a specific device class (or a combination of device classes).

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_START_INQUIRY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  inquirytFilter a pointer to a data structure determining the type of filtering (if any) which is to be used. This parameter CANNOT be NULL.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_INQUIRY_IN_PROGRESS: inquiry process is currently in progress
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: issuing the start inquiry command was not successful
*/
ClxError clxGapStartLimitedInquiry( _in_ ClxStack stack, 
                                    _in_ const struct ClxGapInquiryFilter* inquirytFilter,
                                    _in_ boolean block );


/**
Stop inquiry from the local device. This operation will stop any on-going inquiry operation, as has been initiated by #clxGapStartInquiry or
#clxGapStartLimitedInquiry command.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_STOP_INQUIRY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_INQUIRY_NOT_IN_PROGRESS: there is no inquiry process currently in progress
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: issuing the stop inquiry command was not successful
*/
ClxError clxGapStopInquiry( _in_ ClxStack stack, 
                            _in_ boolean block );

/**
Get list of Discovered Devices function will return a list of devices found during the inquiry operation.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_GET_LIST_OF_DISCOVERED_DEVICES_COMPLETE.
The indication parameters will be of type #clxGapGetListOfDiscoveredDevicesComplete. Note that in the non-blocking mode, 
the list of discovered devices will be stored in a stack-provided buffer. The discovered devices list will be valid until the next clxGapStartInquiry
or clxGapStartLimitedInquiry API function call.

The following code example shows how to use inquiry related functions.
First, the code segment shows an example start inquiry call, this call will be blocked until inquiry operation is started.
Then this same thread will be waiting for an user input to stop the inquiry. Meanwhile, for each device found during inquiry,
the callback handler will be invoked, within the #CLX_GAP_DEVICE_DISCOVERED_INDICATION message handler, the name of the discovered
device will be printed. Finally once the user presses 's' to stop the inquiry then the list of discovered devices will be returned to the user.
\code
    ClxError err = clxGapStartInquiry(stack, 0x200400, 0x200400, TRUE);
    printf ("\nAt any time, press s to stop searching\n");
    while ((clxGetCharEcho() != 's') && (ch != 'S'));
    err = clxGapStopInquiry(stack, TRUE);
    ClxDeviceDetail list[20];
    u4 numberOfDiscoveredDevices;
    err = clxGapGetListOfDiscoveredDevices(stack, list, 20, &numberOfDiscoveredDevices, TRUE);
    if (numberOfDiscoveredDevices)
    {
        for (u4 i = 0; i < numberOfDiscoveredDevices; i++)
        {
            printf ("    %u. %s\n", i, list[i].deviceName);
        }
    }

    // callback handler function
    bool stackMessageHandler(ClxStack stack, ClxHandle serviceHandle, u4 messageID, const void* params, ClxError errorCode)
    {
        if (messageID == CLX_GAP_DEVICE_DISCOVERED_INDICATION)
        {
            clxGapDeviceDiscoveredIndication* arg = (clxGapDeviceDiscoveredIndication*)params;
            printf ("device discovered: %s\n", arg->deviceName);
        }    
        return true;
    }
\endcode

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param [out]  list A caller-allocated array to store the list of discovered devices (only in blocking mode).
\param [in]   listSize Size of the above-mentioned list. If more devices are found during inquiry, returned list will be limited.
\param [out]  numberOfDiscoveredDevices A caller-allocated buffer to store the actual number of discovered devices. If this value 
              is greater than /p listSize, only the first "listSize" number of discovered devices have been returned in the 
              caller-provided list. If this value is smaller than /p listSize, only the first "listSize" elements of the list 
              have valid values (only in blocking mode).

\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful  
*/
ClxError clxGapGetListOfDiscoveredDevices( _in_         ClxStack            stack, 
										   _user_out_   ClxDeviceDetail*    list,
										   _in_         u4                  listSize,
										   _user_out_   u4*                 numberOfDiscoveredDevices, 
										   _in_         boolean             block );


/**
clxGapInitiateBonding() function is used to initiate bonding procedure to a remote device. The remote device id is retrieved from an inquiry operation.
For faster operation, inquiry operation must be stopped before this function is called. In order to bonding operation
to be successful, both local device and remote device must be in bondable mode.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_INITIATE_BONDING_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Remote device id.
\param[ in ]  securityRequirement The security level required for the bonding process. It is highly recommended that the value of #ClxAutomaticSecurity be set for 
              this parameter. Refer to the documentation of #ClxSecurityRequirement for more information.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_GAP_LOCAL_DEVICE_NOT_BONDABLE: bonding is not possible since the local device is in non-bondable mode
                - CLX_ERROR_GAP_REMOTE_DEVICE_NOT_BONDABLE: bonding is not possible since the remote device is in non-bondable mode
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful (connection to the remote device or authentication operation failed)

\remarks
Bonding is the procedure of pairing to a remote device permanently, by storing pairing information in a local storage area. When bonding is successful,
the devices can communicate without authentication and security measures as long as bonding information is not deleted by either the local device or the
remote device.
clxGapInitiateBonding() is used to pair to a device permanently without an intention to connect to any specific service on the remote device. Any attempt
to connect to a remote device to use a service (except for SDAP) will automatically result in bonding as well (as long as both the local device and
the remote device are in bondable mode). Therefore, calling this function to pair to a remote device in order to use a service on that device is not
necessary.

The following code example shows how to initiate bonding to the first remote device 
of the discovered devices list.
\code
    ClxDeviceDetail list[20];
    u4 listSize = 20;

    ClxError err = clxGapGetListOfDiscoveredDevices(gap, list, &listSize, TRUE);

    if (listSize)
    {
        err = clxGapInitiateBonding(gap, list[0].deviceId, TRUE);
    }

\endcode
                
*/
ClxError clxGapInitiateBonding( _in_ ClxStack stack, 
                                _in_ ClxDeviceId remoteDeviceId,
                                _in_ enum ClxSecurityRequirement securityRequirement,
                                _in_ boolean block );


/**
Pin code response during a pairing operation. This function is only used if either the local or the remote device does not support
Simple Secure Pairing.

NOTE: this function must only be called upon the reception of a #CLX_GAP_PIN_CODE_REQUEST_INDICATION indication.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_PIN_CODE_REQUEST_REPLY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Id of the remote device.
\param[ in ]  pinCode Pin code (up to 16 bytes long). If NULL, the pin code request will be rejected.
\param[ in ]  pinCodeLength Length of the provided pin code. Maximum 16 bytes.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful.
*/
ClxError clxGapPinCodeRequestReply(_in_ ClxStack stack,
                                   _in_ ClxDeviceId remoteDeviceId,
                                   _in_ const u1* pinCode,
                                   _in_ u4 pinCodeLength,
                                   _in_ boolean block);

/**
Accept connection request function is used, if a remote device is trying to connect to the local device and user wishes to accept the connection request.
This command will be complete when the connection to the remote device is complete (either with success or in error). 

NOTE: this function must only be called upon the reception of a #CLX_GAP_INCOMING_CONNECTION_REQUEST_INDICATION indication.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_ACCEPT_CONNECTION_REQUEST_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Id of the remote device.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful   
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful (connection to the remote device or authentication operation failed)                
*/
ClxError clxGapAcceptConnectionRequest(_in_ ClxStack stack,
                                       _in_ ClxDeviceId remoteDeviceId,
                                       _in_ boolean block);

/**
Reject connection request function is used, if a remote device is trying to connect to the local device and user does not accept the connection request.

NOTE: this function must only be called upon the reception of a #CLX_GAP_INCOMING_CONNECTION_REQUEST_INDICATION indication.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_REJECT_CONNECTION_REQUEST_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Id of the remote device.
\param[ in ]  reason Reason for the rejection, selections are given in #ClxConnectionRequestRejectReason
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful 
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful (e.g. there has been no incoming connection request from the remote device).
*/
ClxError clxGapRejectConnectionRequest(_in_ ClxStack stack,
                                       _in_ ClxDeviceId remoteDeviceId,
                                       _in_ enum ClxConnectionRequestRejectReason reason,
                                       _in_ boolean block);


/**
Disconnects the physical link to a remote device. This is the link over which all upper layer data is sent to and received from this remote device. When the physical link
is terminated, no data can be sent and received between the local device and the remote device any more. 

If there is a physical link to the specified remote device, this command will not be complete until the connection is successfully terminated, or a timeout occurs.

IMPORTANT : If this command is successful, all service-level connections to the remote device will also be automatically terminated. As a result,
an appropriate indication is sent to the affected services, indicating the loss of the service-level connection.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_GAP_DISCONNECT_PHYSICAL_LINK_COMPLETE.
The response does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Id of the remote device.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_INVALID_DEVICE_ID : The provided device ID is not associated to any remote device.
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS : the physical link is currently disconnecting from the remote device. 
                - CLX_ERROR_CONNECTION_NOT_EXIST : there is no physical link to the remote device.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL : The command was not successful 
*/
ClxResult clxGapDisconnectPhysicalLink(_in_ ClxStack stack,
                                       _in_ ClxDeviceId remoteDeviceId,
                                       _in_ boolean block);


/**
Set discoverability is used to make the local device discoverable or non-discoverable.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_SET_DISCOVERABILITY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  discoverable Set to TRUE to make the local device discoverable.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxGapSetDiscoverability(_in_ ClxStack stack,
                                  _in_ boolean discoverable,
                                  _in_ boolean block);

/**
Set Connectability is used to make the local device connectable or non-connectable.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_SET_CONNECTABILITY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  connectable Set to TRUE to make the local device connectable.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful   
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxGapSetConnectability(_in_  ClxStack stack,
                                  _in_ boolean connectable,
                                  _in_ boolean block);

/**
Set Bondability is used to make the local device bondable or non-bondable. The local device is in bondable mode by default.
When the local device is in non-bondable mode, only already-paired devices are able to make a connection request to the local device. Also, when in
non-bondable mode, a connection attempt by the local device to a not-paired remote device will not result in bonding, and may fail based on the configuration
of the remote device.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_SET_BONDABILITY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  bondable Set to TRUE to make the local device bondable. set to FALSE to make the local device non-bondable.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful   
*/
ClxError clxGapSetBondability(_in_ ClxStack stack,
                              _in_ boolean bondable,
                              _in_ boolean block);



/**
Get list of Paired Devices function will return a list of paired devices. 

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_GET_LIST_OF_PAIRED_DEVICES_COMPLETE.
The indication parameters will be of type #ClxGapGetListOfPairedDevicesComplete. Note that in the non-blocking mode, the list of paired devices will be stored in
a stack-provided buffer.

The following code example shows how to use this function.
The API is first used with a NULL list value to obtain the size of the paired devices list. Then a list with that 
size is created and populated with the paired devices.
\code
    u4 numberOfPairedDevices = 0;
    ClxError ret = clxGapGetListOfPairedDevices(stack, NULL, 0, &numberOfPairedDevices, TRUE);
    
    ClxDeviceDetail* list = NEW ClxDeviceDetail[numberOfPairedDevices];
    clxGapGetListOfPairedDevices(stack, list, numberOfPairedDevices, &numberOfPairedDevices, TRUE);
\endcode

\param[ in ] stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ out ] list A caller-allocated array to store the list of paired devices. If a NULL value is passed, then the size of the paired list will be returned.
\param[ in ] listSize Size of the above-mentioned list, if more devices are paired to the local device, returned list will be limited.
                       This parameter MUST be zero when /p list is NULL.
\param [out]  numberOfPairedDevices A caller-allocated to store he actual number of paired devices. If this value is greater than /p listSize, only
              the first "listSize" number of paired devices have been returned in the caller-provided list. If this value is
              smaller than listSize, only the first "listSize" elements of the list have valid values.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful   
*/
ClxError clxGapGetListOfPairedDevices(_in_          ClxStack            stack,
                                      _user_out_    ClxDeviceDetail*    list,
									  _in_          u4                  listSize,
									  _user_out_    u4*                 numberOfPairedDevices,
									  _in_          boolean             block);


/**
Replies to the Bluetooth controller's request to enter a passkey, by either providing a passkey, or rejecting the request. This function is only used if both the local
device and the remote device support Simple Secure Pairing.

NOTE: this function must only be called upon the reception of a #CLX_GAP_USER_PASSKEY_REQUEST_INDICATION indication.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_USER_PASSKEY_REQUEST_REPLY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Id of the remote device.
\param[ in ]  userEnteredPasskey if TRUE, the user has entered a passkey. If FALSE, the user has rejected the passkey request.
\param[ in ]  userPasskey the passkey entered by the user, as an integer. The value must be 0 through 999999 decimal. This parameter
              will be ignored if userEnteredPasskey is FALSE.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful (e.g. there has been no passkey request from the Bluetooth controller).
*/
ClxError clxGapUserPasskeyRequestReply(_in_ ClxStack stack,
                                       _in_ ClxDeviceId remoteDeviceId,
                                       _in_ boolean userEnteredPasskey,
                                       _in_ u4 userPasskey,
                                       _in_ boolean block);


/**
Replies to the Bluetooth controller's request to confirm an authentication request from a remote device, by either confirming or rejecting the request. 
This function is only used if both the local device and the remote device support Simple Secure Pairing.

NOTE: this function must only be called upon the reception of a #CLX_GAP_USER_CONFIRMATION_REQUEST_INDICATION indication.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Id of the remote device.
\param[ in ]  userSelectedYes if TRUE, the user has confirmed the request. If FALSE, the user has rejected the request.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful (e.g. there has been no confirmation request from the Bluetooth controller).
*/
ClxError clxGapUserConfirmationRequestReply(_in_ ClxStack stack,
                                            _in_ ClxDeviceId remoteDeviceId,
                                            _in_ boolean userSelectedYes,
                                            _in_ boolean block);


/**
Deletes pairing information of an already-paired remote device. If the remote device is also bonded, the pairing information will also be deleted from
the persistent storage.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_DELETE_PAIRED_DEVICE_INFO_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Id of the remote device of which the pairing information is to be deleted, if passed ZERO, the oldest pairing information will be deleted.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_INVALID_DEVICE_ID: the device ID provided is invalid. 
*/
ClxError clxGapDeletePairedDeviceInfo(_in_ ClxStack stack,
                                      _in_ ClxDeviceId remoteDeviceId,
                                      _in_ boolean block);


/**
Deletes pairing information of all already-paired remote devices. If a remote device is also bonded, the pairing information will also be deleted from
the persistent storage.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_DELETE_ALL_PAIRED_DEVICES_INFO_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
*/
ClxError clxGapDeleteAllPairedDevicesInfo(_in_ ClxStack stack,
                                          _in_ boolean block);

/**
Request of the remote name function will return the remote device within the ClxDeviceDetail structure. This function is generally needed
when the name request times out during an inquiry operation. Most Bluetooth chips have trouble responding to a name request if the inquiry
operation is going on. Any connection requests to the same remote device during this request will fail. The reason is there would be a LMP connection
established with the remote device during the remote name request. If this call executes successfully, then the stored device name will be replaced with
the obtained name. This call will always send a name request to the remote device.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_REQUEST_REMOTE_DEVICE_NAME_COMPLETE.
The indication parameters will be of type #ClxDeviceDetail. Note that in the non-blocking mode, the list of discovered devices will be stored in
a stack-provided buffer.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  remoteDeviceId Remote device id.
\param[ in ]  requestTimeout Request timeout in milliseconds.
\param [out]  deviceDetail A caller-allocated structure to store the details of the device (only in blocking mode).
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful  
                - CLX_ERROR_INVALID_DEVICE_ID: the device ID provided is invalid. 
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful.
*/
ClxError clxRequestRemoteDeviceName(_in_        ClxStack            stack,
                                    _in_        ClxDeviceId         remoteDeviceId,
									_in_        u4                  requestTimeout,
									_user_out_  ClxDeviceDetail*    deviceDetail,
                                    _in_        boolean             block);

/**
This function can be used to dynamically configure some global parameters of the stack. The parameters that can be configured with this function are

1. Major Class of Device
2. Minor Class of Device
3. Service Class
4. Local Device Name

NOTE : While this function immediately applies the new values of the parameters, the new values might not be immediately reflected in the remote devices since
many Bluetooth stacks are by default configured to discover these parameters only during pairing. Therefore, the pairing information stored in the remote device may need to be deleted before
the new values of these parameters appear.

Therefore, these parameters should be dynamically modified ONLY if it is deemed necessary.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_WRITE_CONFIGURATION_PARAMETERS_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  parameterConfigList The list of configuration parameters to modify in the stack. If a configuration parameter does not exist in the list,
                                  it's current value will not be modified.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful  
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful.
*/

ClxError clxGapWriteConfigurationParameters(_in_         ClxStack            stack, 
                                            _user_in_    ClxConfigList*      parameterConfigList,
										    _in_         boolean             block);

#ifdef __cplusplus
}
#endif


#endif // _GAP_API_H_

