#ifndef __Gap_Ble_Api_h__
#define __Gap_Ble_Api_h__

/*******************************************************************************
*
* Project             GapBle
* File                Gap.Ble.Api.h
* Description         Declares API Functions and Definitions For GapBle
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#define CLX_BLE_ADVERTISING_DATA_MAX_SIZE                                                   32

/**
LE Security Mode 1:
 - Level 1: No security (no authentication, no encryption)
 - Level 2: Unauthenticated pairing with encryption (no MITM protection)
 - Level 3: Authenticated pairing with encryption (MITM protection)

LE Security Mode 2:
 - Level 1: Unauthenticated pairing with with data signing (MITM protection)
 - Level 2: Authenticated pairing with data signing (MITM protection)

 Mixed LE security modes requirements:
 If security mode 1 and mode 2 level 2 are required, security mode 1 level 3 is used
 If security mode 1 level 3 and mode 2 are required, security mode 1 level 3 is used
 If security mode 1 level 2 and mode 2 level 1 is required, mode 1 level 2 is used


 The security mode is defined by CLX_BLE_GAP_SECURITY_MODE 
 The security level is defined by CLX_BLE_GAP_SECURITY_LEVEL 
*/

#define CLX_BLE_GAP_SECURITY_MODE                                                       1

#define CLX_BLE_GAP_SECURITY_LEVEL                                                      1

/**
LE Privacy:
 Peripheral:
  Privacy features are set by CLX_BLE_GAP_PRIVACY_FLAG and CLX_BLE_GAP_RECCONECTION_ADDRESS

    CLX_BLE_GAP_PERIPHERAL_PRIVACY_FLAG :           1 - Privacy enabled
                                                    0 - Privacy Disabled
    CLX_BLE_GAP_PERPHERAL_RECCONECTION_ADDRESS:     1 - Use reconnection address
                                                    0 - Use resolvable address

    CLX_BLE_GAP_PERIPHERAL_PRIVACY_FLAG            CLX_BLE_GAP_PERPHERAL_RECCONECTION_ADDRESS         Privacy Support
    0                                               x                                                   None
    1                                               0                                                   Privacy only in undirected connectable mode
    1                                               1                                                   Privacy in directed connectable mode and undirected connectable mode                                                    



*/

#define CLX_BLE_GAP_PERIPHERAL_PRIVACY_FLAG                                                 1

#define CLX_BLE_GAP_PERPHERAL_RECCONECTION_ADDRESS                                          1

#define CLX_BLE_GAP_CENTRAL_PRIVACY_ENABLE                                                  1


/**
Refer to #clxGapBleManageWhiteList function.
*/
#define CLX_GAP_BLE_MANAGE_WHITE_LIST_COMPLETE                                              0x5500

/**
Refer to #clxGapBleStartScan function.
*/
#define CLX_GAP_BLE_START_SCAN_COMPLETE                                                     0x5501

/**
Refer to #clxGapBleStopScan function.
*/
#define CLX_GAP_BLE_STOP_SCAN_COMPLETE                                                      0x5502

/**
Refer to #clxGapBleInitiateBonding function.
*/
#define CLX_GAP_BLE_INITIATE_BONDING_COMPLETE                                               0x5503

/**
Refer to #clxGapConnectToPeripheral function.
*/
#define CLX_GAP_CONNECT_TO_PERIPHERAL_COMPLETE                                              0x5504

/**
Refer to #clxGapDisconnectFromPeripheral function.
*/
#define CLX_GAP_DISCONNECT_COMPLETE                                                         0x5505

/**
Refer to #clxGapBleStartAdvertising function.
*/
#define CLX_GAP_BLE_START_ADVERTISING_COMPLETE                                              0x5506

/**
Refer to #clxGapBleStopAdvertising function.
*/
#define CLX_GAP_BLE_STOP_ADVERTISING_COMPLETE                                               0x5507

/**
Refer to #clxGapBleSetScanResponseData function.
*/
#define CLX_GAP_BLE_SET_SCAN_RESPONSE_DATA_COMPLETE                                         0x5508

/**
Refer to #clxGapBleSetBondable function.
*/
#define CLX_GAP_BLE_SET_BONDABLE_COMPLETE                                                   0x5509

/**
Refer to #ClxGapBleGetListOfPairedDevicesCompleteStruct function.
*/
#define CLX_GAP_BLE_GET_LIST_OF_PAIRED_DEVICES_COMPLETE                                     0x550a

/**
Refer to #clxGapBleUserPasskeyRequestReply function.
*/    
#define CLX_GAP_BLE_USER_PASSKEY_REQUEST_REPLY_COMPLETE                                     0x550b

/**
Refer to #clxGapBleUserConfirmationRequestReply function.
*/
#define CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE                                0x550c

/**
Refer to #clxGapBleDeletePairedDeviceInfo function.
*/
#define CLX_GAP_BLE_DELETE_PAIRED_DEVICE_INFO_COMPLETE                                      0x550d

/**
Refer to #clxGapBleDeleteAllPairedDevicesInfo function.
*/
#define CLX_GAP_BLE_DELETE_ALL_PAIRED_DEVICES_INFO_COMPLETE                                 0x550e

/**
Refer to #clxGapBleEncrypt function.
*/
#define CLX_GAP_BLE_ENCRYPT_COMPLETE                                                        0x550f



/**
This indication is received when there is a physical-layer (ACL) connection established
from a remote device (Central Role).

The parameter for this indication is of type #ClxGapBleConnectionEstablishedByRemoteCentralIndicationStruct.
*/
#define CLX_GAP_BLE_CONNECTION_ESTABLISHED_BY_REMOTE_CENTRAL_INDICATION                     0x9500

/**
This indication is received when the physical (ACL) link to a remote device has failed. When the physical link
fails, all service-level connections to the remote device will also fail automatically. Therefore, there is no need
to disconnect the service-level connections manually.

The parameter for this indication is of type #ClxGapBleLinkDisconnectionIndicationStruct.
*/
#define CLX_GAP_BLE_LINK_DISCONNECTION_INDICATION                                           0x9501

/**
This indication is received during scanning procedure, which indicates
a device has been discovered in the vicinity. A ClxBleDeviceDetail object will be passed
to the user with this message. This object will be valid until the next #clxGapBleStartScan
API function call.

The parameter for this indication is of type #ClxGapBleDeviceAdvertisingIndicationStruct.
*/
#define CLX_GAP_BLE_DEVICE_ADVERTISING_INDICATION                                           0x9502

/**
This indication is received during authentication procedure when the Bluetooth controller asks the local device
to provide a passkey or reject the authentication request. Upon reception of this indication, the function
#clxGapBleUserPasskeyRequestReply MUST be called.

The parameter for this indication is of type #ClxGapBleUserPasskeyRequestIndicationStruct.
*/
#define CLX_GAP_BLE_USER_PASSKEY_REQUEST_INDICATION                                         0x9503

/**
This indication is received during authentication procedure when the Bluetooth controller asks the local device
to display a passkey on its display/monitor/LCD. This indication is only received when the local device has display
capabilities. Upon reception of this indication, the local application MUST display the passkey to the user, and must
keep displaying the passkey until the indication # TODO is received.

The parameter for this indication is of type #ClxGapBleUserPasskeyNotificationIndicationStruct.
*/
#define CLX_GAP_BLE_USER_PASSKEY_NOTIFICATION_INDICATION                                    0x9504

/**
This indication is received during authentication procedure when the Bluetooth controller asks the local device
to confirm or reject the connection to a remote device. The indication contains a passkey as well. If the local device
has display capabilities, it must display the passkey to the user and asks for confirmation or rejection. Otherwise, it may
directly confirm or reject the request by calling the function #clxGapBleUserConfirmationRequestReply.

The parameter for this indication is of type #ClxGapBleUserConfirmationRequestIndicationStruct.
*/
#define CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_INDICATION                                    0x9505

/**
This indication is received when a peripheral device has been scanned, with privacy enabled and reconnection address enabled.
The central device has to option of setting the reconnection address of the peripheral device. This can be done with clxGapBleSelectPeripheralAddress.
*/
#define CLX_GAP_SET_BLE_RECONNECTION_ADDRESS_INDICATION                                      0x9506


/**
Determines the addressing mode of a local BLE device.
*/
typedef enum ClxBleAddressTypeEnum
{
    ClxBleAddressType_Public                                        = 0x00,     /*!< The universally unique Bluetooth address of the device is used */
    ClxBleAddressType_Random                                        = 0x01      /*!< A random Bluetooth address is used for each session */
} ClxBleAddressType;


/**
Determines the addressing mode of a remote BLE device.
*/
typedef enum ClxBleAddressModeEnum
{
    ClxBleAddressMode_PublicAddress                                 = 0x00,     /*!< The universally unique Bluetooth address of the device is used */
    ClxBleAddressMode_Resolvable                                    = 0x01,     /*!< Resolvable private address is used, requires IRK to resolve the address */
    ClxBleAddressMode_Unresolvable                                  = 0x02      /*!< Unresolvable private address is used, random address is for a session only */
} ClxBleAddressMode;

/**
Determines the operations on a white list
*/
typedef enum ClxBleWhiteListOperationEnum
{
    ClxBleWhiteListOperation_Add                                    = 0x00,     /*!< Add a device into the white list*/
    ClxBleWhiteListOperation_Remove                                 = 0x01,     /*!< Remove a device from the white list*/
    ClxBleWhiteListOperation_Clear                                  = 0x02      /*!< Clear the current entries from the white list*/
} ClxBleWhiteListOperation;

/**
Determines the operations of advertising filter (applicable to discovery only)
*/
typedef enum ClxBleAdvertisingFilterPolicyEnum
{
    ClxBleAdvertisingFilterPolicy_ScanConnectionAnyone              = 0x00,     /*!< Scan and advertise connectable by anyone */
    ClxBleAdvertisingFilterPolicy_ScanWhitelistConnectionAnyone     = 0x01,     /*!< Scan by whitelist devices, advertise connectable by anyone */
    ClxBleAdvertisingFilterPolicy_ScanAnyoneConnectionWhitelist     = 0x02,     /*!< Scan by anyone, connectable to only whitelist devices */
    ClxBleAdvertisingFilterPolicy_ScanWhitelistConnectionWhitelist  = 0x03      /*!< Scan by whitelist devices and connectable by whitelist devices only */
} ClxBleAdvertisingFilterPolicy;

/**
Determines the advertising type (applicable to connection only)
*/
typedef enum ClxBleAdvertisingTypeEnum
{
    ClxBleAdvertisingType_ConnectableUndirected                     = 0x00,     /*!< connectable  by anyone */
    ClxBleAdvertisingType_ConnectableDirected                       = 0x01,     /*!< connectable  by directed address */
    ClxBleAdvertisingType_ScannableUndirected                       = 0x02,     /*!< scannable  by anyone */
    ClxBleAdvertisingType_NonConnectableUndirected                  = 0x03      /*!< non-connectable */
} ClxBleAdvertisingType;

typedef enum ClxBleSmpRoleEnum
{
    ClxBleSmpRole_Slave     = 0x00,                         /*!< Security Manager Slave Role                                   */
    ClxBleSmpRole_Master    = 0x01                          /*!< Security Manager Master Role                                  */
} ClxBleSmpRole;

typedef enum ClxBleSmpBondingTypeEnum
{
    ClxBleSmpBondingType_NoBonding  = 0x00,                 /*!< Device in non bondable mode                                   */
    ClxBleSmpBondingType_Bonding    = 0x01                  /*!< Device in bondable mode                                       */
} ClxBleSmpBondingType;

typedef enum ClxBleSmpMITMProtectionEnum
{
    ClxBleSmpMITMProtection_Off = 0x00,                     /*!< Device doesn't need MITM protection for bonding               */
    ClxBleSmpMITMProtection_On  = 0x01                      /*!< Device needs MITM protection for bonding                      */
} ClxBleSmpMITMProtection;

typedef enum ClxBleSmpKeyDistributionEnum
{
    ClxBleSmpKeyDistribution_EncryptionKey  = 0x01,         /*!< Distributing Long Term Key                                    */
    ClxBleSmpKeyDistribution_IdentityKey    = 0x02,         /*!< Distributing Identity Resolving Key                           */
    ClxBleSmpKeyDistribution_SignatureKey   = 0x04          /*!< Distributing Connection Signature Resolving Key               */
} ClxBleSmpKeyDistribution;

/**
Determines the scanning type 
*/
typedef enum BleScanTypeEnum
{
    BleScanType_Passive = 0x00,
    BleScanType_Active = 0x01
} BleScanType;

/**
For defining BLE address
*/
typedef struct ClxBleBdAddressStruct
{
    u1                    value[6];                     /*!< The universally unique Bluetooth address of the device */
    ClxBleAddressType     addressType;
} ClxBleBdAddress;


typedef struct ClxBleDeviceDetailAndNumericValueStruct
{
    ClxBleBdAddress     deviceAddr;
    u4                  value;
} ClxBleDeviceDetailAndNumericValue;

/* Data Structure for the indication CLX_GAP_BLE_CONNECTION_ESTABLISHED_BY_REMOTE_CENTRAL_INDICATION */
typedef struct ClxGapBleConnectionEstablishedByRemoteCentralIndicationStruct
{
    ClxBleBdAddress     deviceAddr;
} ClxGapBleConnectionEstablishedByRemoteCentralIndication;

/* Data Structure for the indication CLX_GAP_BLE_LINK_DISCONNECTION_INDICATION */
typedef struct ClxGapBleLinkDisconnectionIndicationStruct
{
    ClxBleBdAddress     deviceAddr;
} ClxGapBleLinkDisconnectionIndication;

/* Data Structure for the indication CLX_GAP_BLE_DEVICE_ADVERTISING_INDICATION */
typedef struct ClxGapBleDeviceAdvertisingIndicationStruct
{
    ClxBleBdAddress     deviceAddr;
    u1                  data[CLX_BLE_ADVERTISING_DATA_MAX_SIZE];
    ClxSize             dataLength;
} ClxGapBleDeviceAdvertisingIndication;

/* Data Structure for the indication CLX_GAP_BLE_USER_PASSKEY_REQUEST_INDICATION */
typedef struct ClxGapBleUserPasskeyRequestIndicationStruct
{
    ClxBleBdAddress     deviceAddr;
} ClxGapBleUserPasskeyRequestIndication;

/* Data Structure for the indication CLX_GAP_BLE_USER_PASSKEY_NOTIFICATION_INDICATION */
typedef struct ClxGapBleUserPasskeyNotificationIndicationStruct
{
    ClxBleBdAddress     deviceAddr;
    u4                  value;
} ClxGapBleUserPasskeyNotificationIndication;

/* Data Structure for the indication CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_INDICATION */
typedef struct ClxGapBleUserConfirmationRequestIndicationStruct
{
    ClxBleBdAddress     deviceAddr;
    u4                  value;
} ClxGapBleUserConfirmationRequestIndication;

/**
Manage white list is used to add and delete white listed devices for the API functions that require white list.  

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_MANAGE_WHITE_LIST_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  operation Type of the white list operation.
\param[ in ]  deviceAddr Device that needs to be added or deleted to the white list. If a white list clear operation is used, this input will be don't care.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful   
*/
ClxResult clxGapBleManageWhiteList(_in_ ClxStack                     stack,
                                   _in_ ClxBleWhiteListOperation     operation,
                                   _in_ const ClxBleBdAddress*       deviceAddr,
                                   _in_ boolean                      block);

/**
Starts scanning from the local device. If any Bluetooth devices in the vicinity are found, then 
#CLX_GAP_BLE_DEVICE_ADVERTISING_INDICATION message will be received via the callback function. A name 
request will be performed for each new discovered device which is not already in the pairing list.

Optionally, a filter may be used to limit the discovered devices to a specific device class (or a combination of device classes).

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_START_SCAN_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  leScanType Passive/Active
\param[ in ]  leScanInterval Time interval between scans
\param[ in ]  leScanWindow Duration of scan, less than or equal to scan interval 
\param[ in ]  useWhitelist whether or not to use whitelist (FALSE = don't use, TRUE = use)
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_SCAN_IN_PROGRESS: scan process is currently in progress
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: issuing the start scan command was not successful
*/
ClxResult clxGapBleStartScan(_in_ ClxStack        stack,
                             _in_ BleScanType     leScanType,
                             _in_ u4              leScanInterval,
                             _in_ u4              leScanWindow,
                             _in_ boolean         useWhitelist,
                             _in_ boolean         block);

/**
Stop scanning from the local device. This operation will stop any on-going scanning operation, as has been initiated by #clxGapBleStartScan

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_STOP_SCAN_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_SCAN_NOT_IN_PROGRESS: there is no scan process currently in progress
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: issuing the stop scan command was not successful
*/
ClxResult clxGapBleStopScan(_in_ ClxStack     stack,
                            _in_ boolean      block);

/**
clxGapBleInitiateBonding() function is used to initiate bonding procedure to a remote device. The remote device id is retrieved from a scan operation.
In order for bonding operation to be successful, both local device and remote device must be in bondable mode. The client requests to store the long-term-key (LTK).

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_INITIATE_BONDING_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  deviceAddr Remote device Bluetooth Address
\param[ in ]  mitmProtectionRequired The security level required for the bonding process. If man-in-the-middle protection is required, this must be set to TRUE.
\param[ in ]  permanentBonding When this variable is set to TRUE, the bonding process will store the LTK (after bonding) if successful. If set to FALSE, pairing will be short term.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_GAP_LOCAL_DEVICE_NOT_BONDABLE: bonding is not possible since the local device is in non-bondable mode
                - CLX_ERROR_GAP_REMOTE_DEVICE_NOT_BONDABLE: bonding is not possible since the remote device is in non-bondable mode
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful (connection to the remote device or authentication operation failed)

\remarks
Bonding is the procedure of pairing to a remote device permanently, by storing pairing information in a local storage area. When bonding is successful,
the devices can communicate without authentication and security measures as long as bonding information is not deleted by either the local device or the
remote device.
clxGapBleInitiateBonding() is used to pair to a device permanently without an intention to connect to any specific service on the remote device. Any attempt
to connect to a remote device to use a service (except for SDAP) will automatically result in bonding as well (as long as both the local device and
the remote device are in bondable mode). Therefore, calling this function to pair to a remote device in order to use a service on that device is not
necessary.

The following code example shows how to initiate bonding to the first remote device 
of the discovered devices list.
\code
    ClxBleDeviceDetail list[20];
    u4 listSize = 20;

    ClxError err = clxGapBleGetListOfDiscoveredDevices(gap, list, &listSize, TRUE);

    if (listSize)
    {
        err = clxGapBleInitiateBonding(gap, list[0].deviceId, TRUE);
    }

\endcode
                
*/
ClxResult clxGapBleInitiateBonding(_in_ ClxStack                   stack,
                                   _in_ ClxBleSmpRole              role,
                                   _in_ ClxBleSmpBondingType       bondingType,
                                   _in_ ClxBleSmpMITMProtection    mitmProtection,
                                   _in_ u1                         maxEncryptionKeySize,
                                   _in_ u1                         localDistributeKeys,
                                   _in_ u1                         remoteDistributeKeys,
                                   _in_ boolean                    block);

/**
Establish an instance of a connection to remote device.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_GAP_CONNECT_TO_PERIPHERAL_COMPLETE.
The response does not have any parameters.

\param[ in ] stack BLE stack handle.
\param[ in ] deviceAddr remote Bluetooth device id. This id must be obtained via other GAP APIs.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxResult clxGapConnectToPeripheral(_in_ ClxStack                   stack,
                                    _in_ const ClxBleBdAddress*     deviceAddr,
                                    _in_ boolean                    block);

/**
Disconnects from the remote connection.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_GAP_DISCONNECT_COMPLETE.
The response does not have any parameters.

\param[ in ]  stack BLE stack handle.
\param[ in ]  deviceAddr Device address to disconnect from
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote device.
*/
ClxResult clxGapDisconnectFromPeripheral(_in_ ClxStack     stack,
                           _in_ const ClxBleBdAddress*     deviceAddr,
                           _in_ boolean                    block);
/**
Start advertising is used to make local device discoverable/connectable. Advertising settings are applied during enabling advertising

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_START_ADVERTISING_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  advertisingIntervalMin 0x0020 to 0x4000, interval = advertising_interval_min * 0.625msec. For non-directed advertising.
\param[ in ]  advertisingIntervalMax 0x0020 to 0x4000, interval = advertising_interval_min * 0.625msec. For non-directed advertising.
\param[ in ]  advertisingType 0 - connectable undirected advertising, 0x01- connectable directed, 0x02 - scannable undirected advertising, 0x03 - non-connectable undirected advertising
\param[ in ]  addressType Own address type: local device's address type: 0x00 for public, 0x01 for random
\param[ in ]  directAddress Address to directly advertise to (set to 0 if not using directed advertising)
\param[ in ]  advertisingFilterPolicy filter policy applied to remote devices; 
    0x00: Allow scan and connection request from anyone, 
    0x01: allow scan from whitelist only, allow connection request from any
    0x02: Allow scan request from any device, allow connection request from white list devices only
    0x03: Allow scan request and connection request from white list only
\param[ in ] advertisingDataLength Set to number of bytes in advertising data: range is 0 to 31
\param[ in ] advertisingData Byte array of data to be sent in advertising packets
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxGapBleStartAdvertising(_in_ ClxStack                          stack,
                                    _in_ u4                                advertisingIntervalMin,
                                    _in_ u4                                advertisingIntervalMax,
                                    _in_ ClxBleAdvertisingType             advertisingType,
                                    _in_ ClxBleAddressType                 addressType,
                                    _in_ const ClxBleBdAddress*            directAddress,
                                    _in_ ClxBleAdvertisingFilterPolicy     advertisingFilterPolicy,
                                    _user_in_ const u1*                    advertisingData,
                                    _in_ u4                                advertisingDataLength,
                                    _in_ boolean                           block);

/**
Stop advertising is used to make device non-discoverable and non-connectable

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_STOP_ADVERTISING_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxGapBleStopAdvertising(_in_ ClxStack     stack,
                                   _in_ boolean      block);

/**
Set scan response data. This is the data that is sent by the device if probed by an active scan

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_SET_SCAN_RESPONSE_DATA_COMPLETE
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  scanResponseData Byte array of data to be sent in scan response packets
\param[ in ]  scanResponseDataLength Set to number of bytes in scan response data: range is 0 to 31
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxResult clxGapBleSetScanResponseData(_in_ ClxStack      stack,
                                       _in_ const u1*     scanResponseData,
                                       _in_ u4            scanResponseDataLength,
                                       _in_ boolean       block);

/**
Set Bondable is used to make the local device bondable or non-bondable. The local device is in bondable mode by default.
When the local device is in non-bondable mode, only already-paired devices are able to make a connection request to the local device. Also, when in
non-bondable mode, a connection attempt by the local device to a not-paired remote device will not result in bonding, and may fail based on the configuration
of the remote device. 

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_SET_BONDABLE_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  bondable Set to TRUE to make the local device bondable. set to FALSE to make the local device non-bondable.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful   
*/
ClxResult clxGapBleSetBondable(_in_ ClxStack     stack,
                               _in_ boolean      bondable,
                               _in_ boolean      block);

/* Data Structure for the indication CLX_GAP_BLE_GET_LIST_OF_PAIRED_DEVICES_COMPLETE */
typedef struct ClxGapBleGetListOfPairedDevicesCompleteStruct 
{
    _user_out_ ClxBleBdAddress*     deviceAddrList;
    _user_out_ u4*                  numberOfPairedDevices;
} ClxGapBleGetListOfPairedDevicesComplete;

/**
Get list of Paired Devices function will return a list of paired devices. 

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_GET_LIST_OF_PAIRED_DEVICES_COMPLETE. 
Note that in the non-blocking mode, the list of paired devices will be stored in
a stack-provided buffer.

The following code example shows how to use this function.
The API is first used with a NULL list value to obtain the size of the paired devices list. Then a list with that 
size is created and populated with the paired devices.
\code
    u4 numberOfPairedDevices = 0;
    ClxError ret = clxGapBleGetListOfPairedDevices(stack, NULL, 0, &numberOfPairedDevices, TRUE);
    
    ClxBleBdAddress* list = NEW ClxBleBdAddress[numberOfPairedDevices];
    clxGapBleGetListOfPairedDevices(stack, list, numberOfPairedDevices, &numberOfPairedDevices, TRUE);
\endcode

\param[ in ] stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ out ] deviceAddrList A caller-allocated array to store the list of paired devices. If a NULL value is passed, then the size of the paired list will be returned.
\param[ in ] listSize Maximum size of the above-mentioned list, if more devices are paired to the local device, returned list will be limited.
                       This parameter MUST be zero when /p list is NULL.
\param [out]  numberOfPairedDevices The actual number of paired devices. If this value is greater than /p listSize, only
              the first "listSize" number of paired devices have been returned in the caller-provided list. If this value is
              smaller than listSize, only the first "listSize" elements of the list have valid values.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful   
*/
ClxResult clxGapBleGetListOfPairedDevices(_in_ ClxStack                   stack,
                                          _user_out_ ClxBleBdAddress*     deviceAddrList,
                                          _in_ u4                         listSize,
                                          _user_out_ u4*                  numberOfPairedDevices,
                                          _in_ boolean                    block);

/**
Replies to the Bluetooth controller's request to enter a passkey, by either providing a passkey, or rejecting the request. This function is only used if both the local
device and the remote device support Simple Secure Pairing.

NOTE: this function must only be called upon the reception of a #CLX_GAP_BLE_USER_PASSKEY_REQUEST_INDICATION indication.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_USER_PASSKEY_REQUEST_REPLY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  deviceAddr Bluetooth Address of the remote device.
\param[ in ]  userEnteredPasskey if TRUE, the user has entered a passkey. If FALSE, the user has rejected the passkey request.
\param[ in ]  userPasskey the passkey entered by the user, as an integer. The value must be 0 through 999999 decimal. This parameter
              will be ignored if userEnteredPasskey is FALSE.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful (e.g. there has been no passkey request from the Bluetooth controller).
*/
ClxResult clxGapBleUserPasskeyRequestReply(_in_ ClxStack                   stack,
                                           _in_ const ClxBleBdAddress*     deviceAddr,
                                           _in_ boolean                    userEnteredPasskey,
                                           _in_ u4                         userPasskey,
                                           _in_ boolean                    block);

/**
Replies to the Bluetooth controller's request to confirm an authentication request from a remote device, by either confirming or rejecting the request. 
This function is only used if both the local device and the remote device support Simple Secure Pairing.

NOTE: this function must only be called upon the reception of a #CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_INDICATION indication.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_USER_CONFIRMATION_REQUEST_REPLY_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  deviceAddr Bluetooth Address of the remote device.
\param[ in ]  userSelectedYes if TRUE, the user has confirmed the request. If FALSE, the user has rejected the request.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful (e.g. there has been no confirmation request from the Bluetooth controller).
*/
ClxResult clxGapBleUserConfirmationRequestReply(_in_ ClxStack                   stack,
                                                _in_ const ClxBleBdAddress*     deviceAddr,
                                                _in_ boolean                    userSelectedYes,
                                                _in_ boolean                    block);

/**
Deletes pairing information of an already-paired remote device. If the remote device is also bonded, the pairing information will also be deleted from
the persistent storage.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_DELETE_PAIRED_DEVICE_INFO_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  deviceAddr Bluetooth Address of the remote device of which the pairing information is to be deleted, if passed ZERO, the oldest pairing information will be deleted.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_INVALID_DEVICE_ID: the device ID provided is invalid. 
*/
ClxResult clxGapBleDeletePairedDeviceInfo(_in_ ClxStack                   stack,
                                          _in_ const ClxBleBdAddress*     deviceAddr,
                                          _in_ boolean                    block);

/**
Deletes pairing information of all already-paired remote devices. If a remote device is also bonded, the pairing information will also be deleted from
the persistent storage.

Non-blocking mode: When the command is complete, the GAP call-back function will be called with an indication of type #CLX_GAP_BLE_DELETE_ALL_PAIRED_DEVICES_INFO_COMPLETE.
The indication does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle to GAP was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
*/
ClxResult clxGapBleDeleteAllPairedDevicesInfo(_in_ ClxStack     stack,
                                              _in_ boolean      block);

/**
If a peripheral device was found during a scan with privacy enabled and reconnection address enabled with indication CLX_GAP_SET_BLE_RECONNECTION_ADDRESS_INDICATION, 
then use this function to set the reconnection address of the peripheral device.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_GAP_SET_BLE_RECONNECTION_ADDRESS_COMPLETE.
The response does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  deviceAddr the address to be set.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
*/
ClxResult clxGapBleSelectPeripheralAddress(_in_ ClxStack                     stack,                                   
                                           _in_ const ClxBleBdAddress*       deviceAddr,
                                           _in_ boolean                      block);

/**
If previously bonded, start encryption. Long term key is handled by the stack. This function will not work the same way for server; 
the client will store the LTK and associated data for server device to generate the LTK. LTK is not stored on server device.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_GAP_BLE_ENCRYPT_COMPLETE.
The response does not have any parameters.

\param[ in ]  stack Local device stack handle. A stack object must be created before a GAP API used.
\param[ in ]  set encryption state change
                - TRUE:  If previously bonded, start encryption. Otherwise return an error to indicate bonding with the remote device required CLX_BLE_REMOTE_DEVIVE_NOT_BONDED.
                - FALSE: Turn encryption off
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
*/
ClxResult clxGapBleEncrypt(_in_ ClxStack     stack,
                           _in_ boolean      set,
                           _in_ boolean      block);

#ifdef __cplusplus
}
#endif

#endif  // __Gap_Ble_Api_h__
