#ifndef __Gatt_Ble_Client_Api_h__
#define __Gatt_Ble_Client_Api_h__

/*******************************************************************************
*
* Project             GattBleClient
* File                Gatt.Ble.Client.Api.h
* Description         Declares API Functions and Definitions For GattBleClient
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

typedef u4 ClxBleCharacteristicHandle;

#define CLX_GATT_CHARACTERISTICS_MAX_BUFFER_SIZE                                            512

/**
Refer to #clxGattClientConnect function.
*/
#define CLX_GATT_CLIENT_CONNECT_COMPLETE                                                    0x5f00

/**
Refer to #clxGattClientDisconnect function.
*/
#define CLX_GATT_CLIENT_DISCONNECT_COMPLETE                                                 0x5f01

/**
Refer to #clxGattClientGetListOfServices function.
*/
#define CLX_GATT_CLIENT_GET_LIST_OF_SERVICES_COMPLETE                                       0x5f02

/**
Refer to #clxGattClientDiscoverPrimaryServices function.
*/
#define CLX_GATT_CLIENT_DISCOVER_PRIMARY_SERVICES_COMPLETE                                  0x5f03

/**
Refer to #clxGattClientDiscoverCharacteristics function.
*/
#define CLX_GATT_CLIENT_DISCOVER_CHARACTERISTICS_COMPLETE                                   0x5f04

/**
Refer to #clxGattClientReadCharacteristicValue function.
*/
#define CLX_GATT_CLIENT_READ_CHARACTERISTIC_VALUE_COMPLETE                                  0x5f05

/**
Refer to #clxGattClientWriteCharacteristicValue function.
*/
#define CLX_GATT_CLIENT_WRITE_CHARACTERISTIC_VALUE_COMPLETE                                 0x5f06

/**
Refer to #clxGattClientReadCharacteristicDescriptor function.
*/
#define CLX_GATT_CLIENT_READ_CHARACTERISTIC_DESCRIPTOR_COMPLETE                             0x5f07

/**
Refer to #clxGattClientUpdateCharacteristicDescriptor function.
*/
#define CLX_GATT_CLIENT_UPDATE_CHARACTERISTIC_DESCRIPTOR_COMPLETE                           0x5f08

/**
The parameter of this indication is of type #ClxGattNotifyIndication.
*/
#define CLX_GATT_NOTIFY_INDICATION                                                          0x9f00

/**
The parameter of this indication is of type #ClxGattClientNotificationReceivedIndication.
*/
#define CLX_GATT_CLIENT_NOTIFICATION_RECEIVED_INDICATION                                    0x9f01

/**
This indication has no arguments.
*/
#define CLX_GATT_CLIENT_INDICATION_RECEIVED_INDICATION                                      0x9f02

typedef struct ClxGattDescriptorDeclarationStruct
{
    u2                                   attributeHandle;
    ClxGattUuid                          uuid;
    ClxGattCharacteristicsDescriptors    type;
    u1                                   descriptorBuffer[CLX_GATT_CHARACTERISTICS_MAX_BUFFER_SIZE];
    u2                                   descriptorSize;
} ClxGattDescriptorDeclaration;

typedef struct ClxGattCharacteristicDeclarationStruct
{
    u2             attributeHandle;
    ClxGattUuid    uuid;
    u1             CharacteristicProperties;
    u2             CharacteristicValueAttributeHandle;
} ClxGattCharacteristicDeclaration;

typedef struct ClxServiceDetailStruct
{
    u2                    firstHandle;
    u2                    lastHandle;
    ClxGattUuid           uuid;
    ClxGattServiceType    type;
} ClxServiceDetail;

typedef struct ClxCharacteristicDescriptorDetailStruct
{
    u2             handle;
    ClxGattUuid    uuid;
} ClxCharacteristicDescriptorDetail;

typedef struct ClxCharacteristicDescriptorListStruct
{
    ClxCharacteristicDescriptorDetail    DescriptorList[4];
    u2                                   numberOfCharacteristicDescriptors;
} ClxCharacteristicDescriptorList;

typedef struct ClxCharacteristicDetailStruct
{
    ClxCharacteristicDescriptorList    descriptorList;
    u2                                 valueHandle;
    u2                                 declarationHandle;
    u1                                 properties;
    u2                                 endHandle;
    ClxGattUuid                        uuid;
} ClxCharacteristicDetail;

typedef struct ClxServiceListStruct
{
    ClxServiceDetail*    list;
    u2                   numberOfServices;
} ClxServiceList;

typedef struct ClxCharacteristicListStruct
{
    ClxCharacteristicDetail*    list;
    u2                          numberOfCharacteristics;
} ClxCharacteristicList;

/**
Data Structure for the indication #CLX_GATT_NOTIFY_INDICATION
*/
typedef struct ClxGattNotifyIndicationStruct
{
    ClxDeviceId    deviceId;
    u2             attributeHandle;
} ClxGattNotifyIndication;

/**
Data Structure for the indication #CLX_GATT_CLIENT_NOTIFICATION_RECEIVED_INDICATION
*/
typedef struct ClxGattClientNotificationReceivedIndicationStruct
{
    _in_ ClxDeviceId                   deviceId;
    _in_ ClxBleCharacteristicHandle    characteristicHandle;
} ClxGattClientNotificationReceivedIndication;

/**
Data Structure for the indication CLX_GATT_CLIENT_CONNECT_COMPLETE
*/
typedef struct ClxGattClientConnectCompleteStruct 
{
    _user_out_ u4*    negotiatedMtu;
} ClxGattClientConnectComplete;

ClxHandle clxGattCreateClient(_in_ ClxStack                     stack, 
                              _in_ ClxApplicationCallbackFunc   callbackFunc);

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_CONNECT_COMPLETE.
                The parameter of this indication is of type #ClxGattClientConnectComplete.

\param[  in   ] gatt            
\param[  in   ] remoteDeviceId  
\param[  in   ] requestedMtu    
\param[  out  ] negotiatedMtu   
\param[  in   ] block           Indicates mode of operation:
                                TRUE : Blocking mode
                                FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientConnect(_in_ ClxHandle      gatt,
                               _in_ ClxDeviceId    remoteDeviceId,
                               _in_ u4             requestedMtu,
                               _user_out_ u4*      negotiatedMtu,
                               _in_ boolean        block);

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_DISCONNECT_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] gatt   
\param[  in   ] block  Indicates mode of operation:
                       TRUE : Blocking mode
                       FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientDisconnect(_in_ ClxHandle    gatt,
                                  _in_ boolean      block);

/**
Data Structure for the indication CLX_GATT_CLIENT_GET_LIST_OF_SERVICES_COMPLETE
*/
typedef struct ClxGattClientGetListOfServicesCompleteStruct 
{
    _user_out_ ClxServiceList*    result;
} ClxGattClientGetListOfServicesComplete;

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_GET_LIST_OF_SERVICES_COMPLETE.
                The parameter of this indication is of type #ClxGattClientGetListOfServicesComplete.

\param[  in   ] gatt    
\param[  in   ] uuid    
\param[  out  ] result  
\param[  in   ] block   Indicates mode of operation:
                        TRUE : Blocking mode
                        FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientGetListOfServices(_in_ ClxHandle                gatt,
                                         _in_ ClxGattUuid              uuid,
                                         _user_out_ ClxServiceList*    result,
                                         _in_ boolean                  block);

/**
Data Structure for the indication CLX_GATT_CLIENT_DISCOVER_PRIMARY_SERVICES_COMPLETE
*/
typedef struct ClxGattClientDiscoverPrimaryServicesCompleteStruct 
{
    _user_out_ ClxServiceList*    services;
} ClxGattClientDiscoverPrimaryServicesComplete;

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_DISCOVER_PRIMARY_SERVICES_COMPLETE.
                The parameter of this indication is of type #ClxGattClientDiscoverPrimaryServicesComplete.

\param[  in   ] gatt      
\param[  in   ] uuid      
\param[  out  ] services  
\param[  in   ] block     Indicates mode of operation:
                          TRUE : Blocking mode
                          FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientDiscoverPrimaryServices(_in_ ClxHandle                gatt,
                                               _in_ ClxGattUuid              uuid,
                                               _user_out_ ClxServiceList*    services,
                                               _in_ boolean                  block);

/**
Data Structure for the indication CLX_GATT_CLIENT_DISCOVER_CHARACTERISTICS_COMPLETE
*/
typedef struct ClxGattClientDiscoverCharacteristicsCompleteStruct 
{
    _user_out_ ClxCharacteristicList*    characteristics;
} ClxGattClientDiscoverCharacteristicsComplete;

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_DISCOVER_CHARACTERISTICS_COMPLETE.
                The parameter of this indication is of type #ClxGattClientDiscoverCharacteristicsComplete.

\param[  in   ] gatt             
\param[  in   ] uuid             
\param[  in   ] service          
\param[  out  ] characteristics  
\param[  in   ] block            Indicates mode of operation:
                                 TRUE : Blocking mode
                                 FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientDiscoverCharacteristics(_in_ ClxHandle                       gatt,
                                               _in_ ClxGattUuid                     uuid,
                                               _user_in_ const ClxServiceDetail*    service,
                                               _user_out_ ClxCharacteristicList*    characteristics,
                                               _in_ boolean                         block);

/**
Data Structure for the indication CLX_GATT_CLIENT_READ_CHARACTERISTIC_VALUE_COMPLETE
*/
typedef struct ClxGattClientReadCharacteristicValueCompleteStruct 
{
    _user_out_ u1*    buffer;
    _user_out_ u4*    readLength;
} ClxGattClientReadCharacteristicValueComplete;

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_READ_CHARACTERISTIC_VALUE_COMPLETE.
                The parameter of this indication is of type #ClxGattClientReadCharacteristicValueComplete.

\param[  in   ] gatt             
\param[  in   ] uuid             
\param[  in   ] characteristics  
\param[  out  ] buffer           
\param[  in   ] bufferSize       
\param[  out  ] readLength       
\param[  in   ] block            Indicates mode of operation:
                                 TRUE : Blocking mode
                                 FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientReadCharacteristicValue(_in_ ClxHandle                            gatt,
                                               _in_ ClxGattUuid                          uuid,
                                               _user_in_ const ClxCharacteristicList*    characteristics,
                                               _user_out_ u1*                            buffer,
                                               _in_ u4                                   bufferSize,
                                               _user_out_ u4*                            readLength,
                                               _in_ boolean                              block);

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_WRITE_CHARACTERISTIC_VALUE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] gatt            
\param[  in   ] characteristic  
\param[  in   ] optionsFlag     
\param[  in   ] data            
\param[  in   ] dataLength      
\param[  in   ] block           Indicates mode of operation:
                                TRUE : Blocking mode
                                FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientWriteCharacteristicValue(_in_ ClxHandle                              gatt,
                                                _user_in_ const ClxCharacteristicDetail*    characteristic,
                                                _in_ u1                                     optionsFlag,
                                                _user_in_ const u1*                         data,
                                                _in_ u4                                     dataLength,
                                                _in_ boolean                                block);

/**
Data Structure for the indication CLX_GATT_CLIENT_READ_CHARACTERISTIC_DESCRIPTOR_COMPLETE
*/
typedef struct ClxGattClientReadCharacteristicDescriptorCompleteStruct 
{
    _user_out_ ClxCharacteristicDescriptorList*    characteristics;
} ClxGattClientReadCharacteristicDescriptorComplete;

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_READ_CHARACTERISTIC_DESCRIPTOR_COMPLETE.
                The parameter of this indication is of type #ClxGattClientReadCharacteristicDescriptorComplete.

\param[  in   ] gatt             
\param[  in   ] characteristic   
\param[  out  ] characteristics  
\param[  in   ] expectedSize     
\param[  in   ] block            Indicates mode of operation:
                                 TRUE : Blocking mode
                                 FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientReadCharacteristicDescriptor(_in_ ClxHandle                                 gatt,
                                                    _user_in_ const ClxCharacteristicDetail*       characteristic,
                                                    _user_out_ ClxCharacteristicDescriptorList*    characteristics,
                                                    _in_ u4                                        expectedSize,
                                                    _in_ boolean                                   block);

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_GATT_CLIENT_UPDATE_CHARACTERISTIC_DESCRIPTOR_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] gatt        
\param[  in   ] handle      
\param[  in   ] data        
\param[  in   ] dataLength  
\param[  in   ] block       Indicates mode of operation:
                            TRUE : Blocking mode
                            FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxGattClientUpdateCharacteristicDescriptor(_in_ ClxHandle         gatt,
                                                      _in_ u2                handle,
                                                      _user_in_ const u1*    data,
                                                      _in_ u4                dataLength,
                                                      _in_ boolean           block);

#ifdef __cplusplus
}
#endif

#endif  // __Gatt_Ble_Client_Api_h__

