#ifndef __Hdp_Api_h__
#define __Hdp_Api_h__

/*******************************************************************************
*
* Project             Hdp
* File                Hdp.Api.h
* Description         Declares API Functions and Definitions For Hdp
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif


/**
Defines the Data end point ID for the data end point which uses HDP. Unique to this HDP-MCAP instance.
*/
typedef u1 ClxHdpDataEndPointID;

/**
Defines the ID for the data link used in HDP identifies a set of two data end points of the connected devices. Unique for the 
pair of devices
*/
typedef u4 ClxHdpChannelID;

/**
Maximum size of the buffer to hold the HDP data end point description buffer string
*/
#define CLX_HDP_ENDPOINT_DESCRIPTION_MAX_SIZE                                               20

/**
Maximum number of data end points allowed
*/
#define CLX_HDP_ENDPOINT_MAX_LIMIT                                                          10


/**
Refer to #clxHdpCreate function.
*/
#define CLX_HDP_CREATE_COMPLETE                                                             0x6400

/**
Refer to #clxHdpGetDataEndPoints function.
*/
#define CLX_HDP_GET_DATA_END_POINTS_COMPLETE                                                0x6401

/**
Refer to #clxHdpCreateDataChannel function.
*/
#define CLX_HDP_CREATE_DATA_CHANNEL_COMPLETE                                                0x6402

/**
Refer to #clxHdpReconnectDataChannel function.
*/
#define CLX_HDP_RECONNECT_DATA_CHANNEL_COMPLETE                                             0x6403

/**
Refer to #clxHdpDeleteDataChannel function.
*/
#define CLX_HDP_DELETE_DATA_CHANNEL_COMPLETE                                                0x6404

/**
Refer to #clxHdpSendData function.
*/
#define CLX_HDP_SEND_DATA_COMPLETE                                                          0x6405

/**
Refer to #clxHdpReceiveData function.
*/
#define CLX_HDP_RECEIVE_DATA_COMPLETE                                                       0x6406


/**
Indication received when the data link which is under create/re-create/establishment/re-establishment state is connected
*/
#define CLX_DATA_LINK_CONNECTED_INDICATION                                                  0xa402

/**
Indication received when the data link which is under delete/disconnect state is complete
*/
#define CLX_DATA_LINK_DISCONNECTED_INDICATION                                               0xa403


/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxHdpDataType -->
**
** <!-- DESCRIPTION:-->
**
** ClxHdpDataType defines the possible data types which uses the HDP defined by the ISO/IEEE 11073 - 10101 Nomenclature Data Type
** Code standards
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxHdpDataTypeEnum
{
    ClxHdpDataType_PulseOximeter                        = 0x1004,   /*!< Pulse oximeter */
    ClxHdpDataType_BasicECG                             = 0x1006,   /*!< Basic ECG */
    ClxHdpDataType_BloodPressureMonitor                 = 0x1007,   /*!< Blood Pressure Monitor */
    ClxHdpDataType_BodyThermometer                      = 0x1008,   /*!< Body Thermometer */
    ClxHdpDataType_BodyWeightScale                      = 0x100F,   /*!< Body Weight Scale */
    ClxHdpDataType_GlucoseMeter                         = 0x1011,   /*!< Glucose Meter */
    ClxHdpDataType_InternationalNormalizedRatioMonitor  = 0x1012,   /*!< INR Monitor */
    ClxHdpDataType_BodyCompositionAnalyzer              = 0x1014,   /*!< Body Composition Analyzer */
    ClxHdpDataType_PeakFlowMonitor                      = 0x1015,   /*!< Peak Flow Monitor */
    ClxHdpDataType_CardiovascularFitnessActivityMonitor = 0x1029,   /*!< CardioVascular Fitness Activity Monitor */
    ClxHdpDataType_StepCounter10441                     = 0x1068,   /*!< Step Counter based on 10441 */
    ClxHdpDataType_StrengthFitnessEquipment             = 0x102A,   /*!< Strength Fitness Equipment */
    ClxHdpDataType_IndependentLivingActivityHub         = 0x1047,   /*!< Independent Living Activity Hub */
    ClxHdpDataType_FallSensor                           = 0x1075,   /*!< Fail Sensor */
    ClxHdpDataType_PersonalEmergencyResponseSensor      = 0x1076,   /*!< Personal Emergency Response Sensor */
    ClxHdpDataType_SmokeSensor                          = 0x1077,   /*!< Smoke Sensor */
    ClxHdpDataType_CarbonMonoxideSensor                 = 0x1078,   /*!< Carbon Monoxide Sensor */
    ClxHdpDataType_WaterSensor                          = 0x1079,   /*!< Water Sensor */
    ClxHdpDataType_GasSensor                            = 0x107A,   /*!< Gas Sensor */
    ClxHdpDataType_MotionSensor                         = 0x107B,   /*!< Motion Sensor */
    ClxHdpDataType_PropertyExitSensor                   = 0x107C,   /*!< Property Exit Sensor */
    ClxHdpDataType_EnuresisSensor                       = 0x107D,   /*!< Enuresis Sensor */
    ClxHdpDataType_ContactClosureSensor                 = 0x107E,   /*!< Contact Closure Sensor */
    ClxHdpDataType_UsageSensor                          = 0x107F,   /*!< Usage Sensor */
    ClxHdpDataType_SwitchSensor                         = 0x1080,   /*!< Switch Sensor */
    ClxHdpDataType_MedicationDosingSensor               = 0x1081,   /*!< Medication Dosing Sensor */
    ClxHdpDataType_TemperatureSensor                    = 0x1082,   /*!< Temperature Sensor */
    ClxHdpDataType_Medicationmonitor                    = 0x1048    /*!< Medication Monitor */
} ClxHdpDataType;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxHdpDataLinkConfig -->
**
** <!-- DESCRIPTION:-->
**
** ClxHdpDataLinkConfig defines the possible configuration options for the data links created in HDP.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxHdpDataLinkConfigEnum
{
    ClxHdpDataLinkConfig_NoPreference   = 0x00,     /*!< Shall by used only by Sink roles to indicate any configuration is OK */
    ClxHdpDataLinkConfig_Reliable       = 0x01,     /*!< Reliable   - configured for first data link of the data end point */
    ClxHdpDataLinkConfig_Streaming      = 0x02      /*!< Streaming  - configured for streaming data */
} ClxHdpDataLinkConfig;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxHdpRole -->
**
** <!-- DESCRIPTION:-->
**
** ClxHdpRole defines the possible HDP roles.
**
**<!-- ----------------------------------------------------------------- -->
*/enum ClxHdpRole
{
    ClxHdpRole_Source       = 0x01,
    ClxHdpRole_Sink         = 0x02
};

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxHdpDataEndPointsDetails -->
**
** <!-- DESCRIPTION:-->
**
** ClxHdpDataEndPointsDetails defines the necessary details of the data end point. This structure explains about the type of 
** data end point supported. Data end point role shall be the same as the HDP application role (Source/Sink).
** The structure to be filled and sent to the command. The remoteDeviceId to be the remote to which the data link to be created.
** NOTE:- description buffer shall be null terminated.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxHdpDataEndPointsDetailsStruct
{
    ClxDeviceId             remoteDeviceId;     /*!< ID of the remote device to which the data link to be created */
    ClxHdpDataEndPointID    dataEndPointId;     /*!< ID of the data end point allocated by the stack */
    ClxHdpDataType          dataType;           /*!< Data type of the data end point. Defined in IEEE 11073 standards */
    ClxHdpRole              role;               /*!< Role of the data end point */
    u1                      description[CLX_HDP_ENDPOINT_DESCRIPTION_MAX_SIZE];     /*!< Description about the data end point */
} ClxHdpDataEndPointsDetails;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxHdpDataChannelConfigParams -->
**
** <!-- DESCRIPTION:-->
**
** ClxHdpDataChannelConfigParams defines the configuration parameters for the configuration of the data links which is retained
** and used during the command clxHdpReconnectDataChannel. 
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxHdpDataChannelConfigParamsStruct
{
    ClxHdpDataLinkConfig       mode;        /*!< Mode of the data link (reliable/streaming for source.no preference for sink */
    u2                         incomingMtu; /*!< Size of the incoming Maximum Transmission Unit for data link. 
                                                 If given 0, then default value configured in the stack will be taken */
} ClxHdpDataChannelConfigParams;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxRemoteConnectedIndication -->
**
** <!-- DESCRIPTION:-->
**
** ClxRemoteConnectedIndication defines the parameters received when a remote device (HDP instance) connects.
** Data structure for the indication CLX_REMOTE_CONNECTED_INDICATION
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxRemoteConnectedIndicationStruct
{
    ClxResult       result;             /*!< Result of the connection operation */
    ClxDeviceId     remoteDeviceId;     /*!< ID of the remote Bluetooth device (HDP) which has connected to the local machine. */
} ClxRemoteConnectedIndication;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxRemoteDisconnectedIndication -->
**
** <!-- DESCRIPTION:-->
**
** ClxRemoteDisconnectedIndication defines the parameters received when a remote device (HDP instance) disconnects.
** Data structure for the indication CLX_REMOTE_DISCONNECTED_INDICATION
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxRemoteDisconnectedIndicationStruct
{
    ClxResult     result;                       /*!< Result of the connection operation */
} ClxRemoteDisconnectedIndication;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxDataLinkConnectedIndication -->
**
** <!-- DESCRIPTION:-->
**
** ClxDataLinkConnectedIndication defines the parameters received when a remote device (HDP instance) MDEP connects.
** Data structure for the indication CLX_DATA_LINK_CONNECTED_INDICATION
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxDataLinkConnectedIndicationStruct
{
    ClxResult               result;            /*!< Result of the connection operation */
    ClxHdpChannelID         channelID;         /*!< Data link which is connected */
} ClxDataLinkConnectedIndication;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxDataLinkDisconnectedIndication -->
**
** <!-- DESCRIPTION:-->
**
** ClxDataLinkDisconnectedIndication defines the parameters received when a remote device (HDP instance) MDEP disconnects.
** Data structure for the indication CLX_DATA_LINK_DISCONNECTED_INDICATION
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxDataLinkDisconnectedIndicationStruct
{
    ClxResult               result;            /*!< Result of the disconnection operation */
    ClxHdpChannelID         channelID;         /*!< Data link which is disconnected */
} ClxDataLinkDisconnectedIndication;

/**
               Creates a HDP Data End point on the local machine. This handle to be used as a reference for any further 
               operations/commands on this end point.

 \param[ in ]  stack stack handle
 \param[ in ]  dataEndPoint local End point details.
 \param[ in ]  indicationCallbackFunc A pointer to the call-back function which will be called upon 
               reception of indication events related to this handle.
 \return       ClxHandle HDP handle. This handle must be used for any further operations on this instance of HDP.
*/
ClxHandle clxHdpCreate(_in_ ClxStack                                    stack,
                       _user_in_ const ClxHdpDataEndPointsDetails*      dataEndPoint,
                       _in_ ClxApplicationCallbackFunc                  indicationCallbackFunc);

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxHdpGetDataEndPointsComplete -->
**
** <!-- DESCRIPTION:-->
**
** ClxHdpGetDataEndPointsComplete defines the structure of the response result buffer returned for the command
** clxHdpGetDataEndPoints (CLX_HDP_GET_DATA_END_POINTS_COMPLETE) to get the number of 
** data end points filled in the buffer and the end points information.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxHdpGetDataEndPointsCompleteStruct 
{
    _user_out_ ClxHdpDataEndPointsDetails*     remoteDataEndPointDetails; /*!< Data end points details filled in */
    _user_out_ u4*                             numberOfEndPoints;         /*!< Number of end points returned in out buffer */
} ClxHdpGetDataEndPointsComplete;


/**
               Request to get the information of all the data end points supported by the remote device which enables the 
               establishment of the data links to any one of these data end points supported by the remote device.

               Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
               type CLX_HDP_GET_DATA_END_POINTS_COMPLETE.
               The indication does not have any parameters.

 \param[ in    ]  hdp handle.
 \param[ in    ]  remoteDeviceId remote device ID for which the end point details are requested.
 \param[ in    ]  count number of end points details the buffer can hold/allocated for.
 \param[ out   ]  remoteDataEndPointDetails details of structure ClxHdpDataEndPointsDetails to be filled in by the stack. 
                  User shall allocated the required memory. Contains the end point details which remote device publish.
 \param[ out   ]  numberOfEndPoints number of data end points information filled in the out buffer by the stack
 \param[ in    ]  block type of the operation.
                  - TRUE:  API will be blocked until this command is completed (successfully or failed).
                  - FALSE: API will return immediately and then a corresponding API completion event 
                    will be raised upon command completion.

 \return       ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                   In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHdpGetDataEndPoints(_in_ ClxHandle                            hdp,
                                _in_ ClxDeviceId                          remoteDeviceId,
                                _in_ u4                                   count,
                                _user_out_ ClxHdpDataEndPointsDetails*    remoteDataEndPointDetails,
                                _user_out_ u4*                            numberOfEndPoints,
                                _in_ boolean                              block);


/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxHdpCreateDataChannelComplete -->
**
** <!-- DESCRIPTION:-->
**
** ClxHdpCreateDataChannelComplete defines the structure of the response result buffer returned for the command
** clxHdpCreateDataChannel (CLX_HDP_CREATE_DATA_CHANNEL_COMPLETE) to get the newly allotted channel ID from stack.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxHdpCreateDataChannelCompleteStruct 
{
    _user_out_ ClxHdpChannelID*     channelID;
} ClxHdpCreateDataChannelComplete;


/**
               Request to create a new data channel to the given end point (includes end point details and remote device ID).

               Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
               type CLX_HDP_CREATE_DATA_CHANNEL_COMPLETE.
               The indication returns the data channel ID (in structure ClxHdpCreateDataChannelComplete) which shall be used 
               as a reference for further operations on the data channel.
               When the indication returns with error, there is a possibility that the remote device has tried to create the data
               channel at the same time. In such case, back off a random amount of time before re-initiating the connection.

 \param[ in ]  hdp handle.
 \param[ in ]  remoteDataEndPoint details of remote end point.
 \param[ in ]  configuration - incoming mtu and mode. Structure ClxHdpDataChannelConfigParams explains more details.
 \param[ out]  channelID - channel ID of the created data link.
 \param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event 
                  will be raised upon command completion.

 \return       ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                   In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHdpCreateDataChannel(_in_ ClxHandle                                     hdp,
                                 _user_in_ const ClxHdpDataEndPointsDetails*        remoteDataEndPoint,
                                 _user_in_ const ClxHdpDataChannelConfigParams*     configuration,
                                 _user_out_ ClxHdpChannelID*                        channelID,
                                 _in_ boolean                                       block);


/**
               Request to re-connect the temporarily suspended data link. The data link might have been intentionally or
               un-intentionally suspended/disconnected due to environmental reasons or by the remote device. 
               In such case, the same data link can be established again using this command. Also re-establishment allows
               devices to retain/memorize and re-use the same data link configuration as it was during the establishment.
               This command can be executed only after a successful call to the command 
               clxHdpCreateDataChannel.

               Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
               type CLX_HDP_RECONNECT_DATA_CHANNEL_COMPLETE.
               The indication does not have any parameters.

 \param[ in ]  hdp handle.
 \param[ in ]  channelID Hdp Data Link (Channel) ID. (Obtained during the establishment of data link connection)
 \param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event 
                  will be raised upon command completion.

 \return       ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                   In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHdpReconnectDataChannel(_in_ ClxHandle              hdp,
                                    _in_ ClxHdpChannelID        channelID,
                                    _in_ boolean                block);


/**
               Request to delete the data channel which was established user command clxHdpCreateDataChannel.
               
               Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
               type CLX_HDP_DELETE_DATA_CHANNEL_COMPLETE.
               The indication does not have any parameters.

 \param[ in ]  hdp handle.
 \param[ in ]  channelID Hdp Data Link (Channel) ID. (Obtained during the establishment of data link connection)
 \param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event 
                  will be raised upon command completion.

 \return       ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                   In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHdpDeleteDataChannel(_in_ ClxHandle         hdp,
                                 _in_ ClxHdpChannelID   channelID,
                                 _in_ boolean           block);


/**
               Request to send the device data on the specific data channel.
               This command can be executed only after a successful call to the command 
               clxHdpCreateDataChannel or clxHdpReconnectDataChannel

               Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
               type CLX_HDP_SEND_DATA_COMPLETE.
               The indication does not have any parameters.

 \param[ in ]  hdp handle.
 \param[ in ]  channelID Hdp Data Link (Channel) ID. (Obtained during the establishment of data link connection)
 \param[ in ]  size of the data to be sent to the remote device.
 \param[ in ]  data to be sent to the remote device.
 \param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event 
                  will be raised upon command completion.

 \return       ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                   In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHdpSendData(_in_ ClxHandle             hdp,
                        _in_ ClxHdpChannelID       channelID,
                        _in_ ClxSize               size,
                        _user_in_ const u1*        data,
                        _in_ boolean               block);


/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxHdpReceiveDataComplete -->
**
** <!-- DESCRIPTION:-->
**
** ClxHdpReceiveDataComplete defines the structure of the response result buffer returned for the command
** clxHdpReceiveData (CLX_HDP_RECEIVE_DATA_COMPLETE) to get the data received on one of the data channel from remote end point.
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxHdpReceiveDataCompleteStruct 
{
    _user_out_ u1*                  buffer;
    _user_out_ ClxSize*             receivedDataLength;
    _user_out_ ClxHdpChannelID*     channelID;
} ClxHdpReceiveDataComplete;

/**
               Request to receive the incoming data on any of the data links associated with this end point (ClxHandle).
               The incoming data can be received only upon request using this command.
               The memory should be allocated by the user for size and data parameters and sent to the stack.

               Non-blocking mode : When the command is complete, the call-back function will be called with an indication of 
               type CLX_HDP_RECEIVE_DATA_COMPLETE.
               The parameter of the indication is of the type structure ClxHdpReceiveDataComplete.

 \param[ in  ]  hdp handle.
 \param[ in  ]  bufferLength Size of the data that the buffer can hold
 \param[ out ]  buffer to be received from remote
 \param[ out ]  receivedDataLength Size of the data that is received on the data link
 \param[ out ]  channelID indicates the data link on which the data is received
 \param[ in  ]  block type of the operation.
                   - TRUE:  API will be blocked until this command is completed (successfully or failed).
                   - FALSE: API will return immediately and then a corresponding API completion event 
                     will be raised upon command completion.

 \return       ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                   In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHdpReceiveData(_in_ ClxHandle                    hdp,
                           _in_ ClxSize                      bufferLength,
                           _user_out_ u1*                    buffer,
                           _user_out_ ClxSize*               receivedDataLength,
                           _user_out_ ClxHdpChannelID*       channelID,
                           _in_ boolean                      block);

#ifdef __cplusplus
}
#endif

#endif  // __Hdp_Api_h__

