#ifndef _HFP_COMMON_API_H_
#define _HFP_COMMON_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Hfp.Common.Api.h
* Description         Defines ClarinoxBlue Hands-free Profile common variables
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#define CLX_HFP_NUMBER_MAX_SIZE                        48    // bytes

enum ClxHfpAgErrors
{
    ClxHfpAgError_0     = 0U,        /*!< AG failure */ 
    ClxHfpAgError_1     = 1U,        /*!< no connection to phone */  
    ClxHfpAgError_3     = 3U,        /*!< operation not allowed */  
    ClxHfpAgError_4     = 4U,        /*!< operation not supported */  
    ClxHfpAgError_5     = 5U,        /*!< PH-SIM PIN required */  
    ClxHfpAgError_10    = 10U,       /*!< SIM not inserted*/ 
    ClxHfpAgError_11    = 11U,       /*!< SIM PIN required */  
    ClxHfpAgError_12    = 12U,       /*!< SIM PUK required */  
    ClxHfpAgError_13    = 13U,       /*!< SIM failure */  
    ClxHfpAgError_14    = 14U,       /*!< SIM busy */  
    ClxHfpAgError_16    = 16U,       /*!< incorrect password */  
    ClxHfpAgError_17    = 17U,       /*!< SIM PIN2 required */  
    ClxHfpAgError_18    = 18U,       /*!< SIM PUK2 required */  
    ClxHfpAgError_20    = 20U,       /*!< memory full */  
    ClxHfpAgError_21    = 21U,       /*!< invalid index  */ 
    ClxHfpAgError_23    = 23U,       /*!< memory failure */ 
    ClxHfpAgError_24    = 24U,       /*!< text string too long */  
    ClxHfpAgError_25    = 25U,       /*!< invalid characters in text string */  
    ClxHfpAgError_26    = 26U,       /*!< dial string too long */   
    ClxHfpAgError_27    = 27U,       /*!< invalid characters in dial string */  
    ClxHfpAgError_30    = 30U,       /*!< no network service */  
    ClxHfpAgError_32    = 32U,       /*!< Network not allowed - Emergency calls only  */
    ClxHfpAgError_None  = 34U
};

typedef struct ClxHfpConnectionIndication
{
    ClxDeviceId deviceId;
} ClxHfpConnectionIndication;

/**
Return parameters of the indication #CLX_HFP_HF_SPEAKER_VOLUME_ADJUSTMENT_REQUEST_INDICATION.
*/
typedef struct ClxHfpSpeakerVolume
{
    u1 speakerVolume;
}ClxHfpSpeakerVolume;

/**
Return parameters of the indication #CLX_HFP_HF_MICROPHONE_GAIN_ADJUSTMENT_REQUEST_INDICATION.
*/
typedef struct ClxHfpMicrophoneGain
{
    u1 microphoneGain;
}ClxHfpMicrophoneGain;


typedef struct ClxHfpNumberInfo
{
    s1 number[CLX_HFP_NUMBER_MAX_SIZE];
    u1 numberType;
} ClxHfpNumberInfo;


typedef enum ClxHfpCallDirection
{
    ClxHfp_OutgoingCall = 0,
    ClxHfp_IncomingCall = 1
} ClxHfpCallDirection;


typedef enum ClxHfpCallStatus
{
    ClxHfp_ActiveCall   = 0,
    ClxHfp_HeldCall     = 1,
    ClxHfp_Dialing      = 2,
    ClxHfp_Alerting     = 3,
    ClxHfp_Incoming     = 4,
    ClxHfp_Waiting      = 5
} ClxHfpCallStatus;


typedef enum ClxHfpCallMode
{
    ClxHfp_VoiceCall    = 0,
    ClxHfp_DataCall     = 1,
    ClxHfp_FaxCall      = 2
} ClxHfpCallMode;


typedef enum ClxHfpCallMultipartyType
{
    ClxHfp_NotMultipartyCall = 0,
    ClxHfp_MultipartyCall    = 1
} ClxHfpCallMultipartyType;


typedef struct ClxHfpCallInformation
{
    u4                          callIndex;
    ClxHfpCallStatus            callStatus;
    ClxHfpCallMode              callMode;
    ClxHfpCallMultipartyType    multipartyType;
    ClxHfpNumberInfo            numberInfo;
} ClxHfpCallInformation;


typedef struct ClxHfpCallIndex
{
    u4 callIndex;
} ClxHfpCallIndex;


#ifdef __cplusplus
}
#endif

#endif // _HFP_COMMON_API_H_
