#ifndef _HFP_AG_API_H_
#define _HFP_AG_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                HfpAg.Api.h
* Description         Declares ClarinoxBlue Hands-free Profile AG role API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "Hfp.Common.Api.h"

#ifdef __cplusplus
extern "C" {
#endif

/** COMMAND COMPLETE MESSAGES */

/**
Refer to #clxHfpAgConnect function.
*/
#define CLX_HFP_AG_CONNECT_COMPLETE                                     0x4D00

/**
Refer to #clxHfpAgBeginIncomingCall function.
*/
#define CLX_HFP_AG_BEGIN_INCOMING_CALL_COMPLETE                         0x4D01

/**
Refer to #clxHfpAgReleaseCall function.
*/
#define CLX_HFP_AG_RELEASE_CALL_COMPLETE                                0x4D02

/**
Refer to #clxHfpAgBeginOutgoingCall function.
*/
#define CLX_HFP_AG_BEGIN_OUTGOING_CALL_COMPLETE                         0x4D03

/**
Refer to #clxHfpAgRemotePartyReached function.
*/
#define CLX_HFP_AG_REMOTE_PARTY_REACHED_COMPLETE                        0x4D04

/**
Refer to #clxHfpAgRemotePartyAnswered function.
*/
#define CLX_HFP_AG_REMOTE_PARTY_ANSWERED_COMPLETE                       0x4D05

/**
Refer to #clxHfpAgConnectAudio function.
*/
#define CLX_HFP_AG_CONNECT_AUDIO_COMPLETE                               0x4D06

/**
Refer to #clxHfpAgDisconnectAudio function.
*/
#define CLX_HFP_AG_DISCONNECT_AUDIO_COMPLETE                            0x4D07

/**
Refer to #clxHfpAgUpdateIndicatorValue function.
*/
#define CLX_HFP_AG_UPDATE_INDICATOR_VALUE_COMPLETE                      0x4D08

/**
Refer to #clxHfpAgDisconnect function.
*/
#define CLX_HFP_AG_DISCONNECT_COMPLETE                                  0x4D09

/**
Refer to #clxHfpAgRejectOugoingCallRequest function.
*/
#define CLX_HFP_AG_REJECT_OUTGOING_CALL_REQUEST_COMPLETE                0x4D0A

/**
Refer to #clxHfpAgActivateCall function.
*/
#define CLX_HFP_AG_ACTIVE_CALL_COMPLETE                                 0x4D0B

/**
Refer to #clxHfpAgSetNetworkOperatorName function.
*/
#define CLX_HFP_AG_SET_NETWORK_OPERATOR_NAME_COMPLETE                   0x4D0C

/**
Refer to #clxHfpAgEnableInbandRing function.
*/
#define CLX_HFP_AG_ENABLE_INBAND_RING_COMPLETE                          0x4D0D

/**
Refer to #clxHfpAgDisableInbandRing function.
*/
#define CLX_HFP_AG_DISABLE_INBAND_RING_COMPLETE                         0x4D0E

/**
Refer to clxHfpAgPlaceCallOnHold function.
*/
#define CLX_HFP_AG_PLACE_CALL_ON_HOLD_COMPLETE                          0x4D0F


/**
Refer to clxHfpAgGetAudioConnectionComplete function.
*/
#define CLX_HFP_AG_GET_AUDIO_CONNECTION_HANDLE_COMPLETE                0x4D10


/** INDICATIONS*/

/**
This indication is received when a RFCOMM connection is made to a remote HFP service.
This indication is only received when the connection has been initiated by the
remote device. The parameter of this indication (params argument of the call-back
function) will of type #ClxHfpConnectionIndication.
*/
#define CLX_HFP_AG_CONNECTION_INDICATION                                0x8D00

/**
This indication is received when a Service Level Connection is made to a remote HFP(HF) service.
This indication is only received when the connection has been initiated by the
remote device (HF). The parameter of this indication (params argument of the call-back
function) will be NULL.
*/
#define CLX_HFP_AG_SLC_CONNECTION_COMPLETION_INDICATION                 0x8D01

/**
This indication is received when a disconnection is made to a remote HFP(HF) service.
This indication is only received when the disconnection has been initiated by the
remote device (HF). The parameter of this indication (params argument of the call-back
function) will of type #ClxHfpConnectionIndication.
*/
#define CLX_HFP_AG_DISCONNECTION_INDICATION                             0x8D02

/**
This indication is received when an outgoing call is  placed from a remote HFP(HF) service.
This indication is only received when the outgoing voice call has been initiated by the
remote device (HF). The parameter of this indication (params argument of the call-back
function) will of type #ClxHfpAgOutgoingDialRequestIndication.
*/
#define CLX_HFP_AG_OUTGOING_DIAL_REQUEST_INDICATION                     0x8D03


/**
This indication is received when an outgoing voice call (using memory dialling of AG)
is placed from a remote HFP(HF) service.

This indication is only received when the outgoing call has been initiated by the
remote device (HF). The parameter of this indication (params argument of the call-back
function) will of type #ClxHfpAgOutgoingMemoryDialRequestIndication.
*/
#define CLX_HFP_AG_OUTGOING_MEMORY_DIAL_REQUEST_INDICATION              0x8D04

/**
This indication is received when a outgoing last number redial request is made by a remote HFP(HF) service.
This indication is only received when the redial request has been initiated by the
remote device (HF). The parameter of this indication (params argument of the call-back
function) will be NULL.
*/
#define CLX_HFP_AG_OUTGOING_LAST_NUMBER_DIAL_REQUEST_INDICATION         0x8D05

/**
This indication is received when a HF aborts/terminates ongoing call.
This indication is only received when the ongoing call terminate/abort request has been initiated by the
remote device (HF). The parameter of this indication (params argument of the call-back
function) will be NULL.
*/
#define CLX_HFP_AG_ONGOING_CALL_TERMINATION_REQUEST_INDICATION          0x8D06

/**
This indication is received when a incoming call request is rejected by a remote HFP(HF) service.
This indication is only received when the incoming call reject has been initiated by the
remote device (HF). The parameter of this indication (params argument of the call-back
function) will be NULL.
*/
#define CLX_HFP_AG_CALL_REJECTED_BY_HF_INDICATION                       0x8D07

/**
This indication is received when a incoming call is accepted/answered by a remote HFP(HF) service.
This indication is only received when the incoming call answer has been sent by the
remote device (HF). The parameter of this indication (params argument of the call-back
function) will be NULL.
*/
#define CLX_HFP_AG_CALL_ANSWERED_BY_HF_INDICATION                       0x8D08

#define CLX_HFP_AG_AUDIO_CONNECTION_INDICATION                          0x8D09

#define CLX_HFP_AG_AUDIO_DISCONNECTION_INDICATION                       0x8D0A

#define CLX_HFP_AG_AUDIO_CONNECTION_FAILED_INDICATION                   0x8D0B



typedef struct ClxHfpAgIndicatorValues
{
    u1 service;
    u1 signal;
    u1 roam;
    u1 batteryCharge;
} ClxHfpAgIndicatorValues;


typedef enum ClxHfAgIndicator
{
    ClxHfAgIndicator_Service       = 0,
    ClxHfAgIndicator_Signal        = 1,
    ClxHfAgIndicator_Roam          = 2,
    ClxHfAgIndicator_BatteryCharge = 3
} ClxHfAgIndicator;


/**
The parameter type of the indication #CLX_HFP_AG_OUTGOING_DIAL_REQUEST_INDICATION.
*/
typedef struct ClxHfpAgOutgoingDialRequestIndication
{
    s1 numberToDial[CLX_HFP_NUMBER_MAX_SIZE];
} ClxHfpAgOutgoingDialRequestIndication;

/**
The parameter type of the indication #CLX_HFP_AG_OUTGOING_MEMORY_DIAL_REQUEST_INDICATION.
*/
typedef struct ClxHfpAgOutgoingMemoryDialRequestIndication
{
    s1 numberMemIndex[CLX_HFP_NUMBER_MAX_SIZE];
} ClxHfpAgOutgoingMemoryDialRequestIndication;


/** Data Structure for the indication #CLX_HFP_AG_GET_AUDIO_CONNECTION_HANDLE_COMPLETE */
typedef struct ClxHfpAgGetAudioConnectionHandleCompleteStruct 
{
    _user_out_ u2*     audioConnectionHandle;
} ClxHfpAgGetAudioConnectionHandleComplete;


/** Data Structure for the indication #CLX_HFP_AG_AUDIO_CONNECTION_INDICATION */
typedef struct ClxHfpAgAudioConnectionindicationStruct 
{
    _in_ u2     audioConnectionHandle;
} ClxHfpAgAudioConnectionindication;



/**
Creates a Hands Free service (Audio Gateway role) on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] initialIndicatorValues pointer to the default values of service, signal, roaming and battery charge
\param[ in ] audioChannelParams pointer to the list of audio channel parameters.
\param[ in ] inbandRingSupported set to true if in band ring is supported else false.
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\return ClxHandle hands free audio gateway profile handle. This handle must be used for any further operations on this instance of hands free profile.
*/
ClxHandle clxHfpAgCreate(_in_ ClxStack stack,
                         _in_ const ClxHfpAgIndicatorValues* initialIndicatorValues,
                         _in_ const ClxBluetoothAudioChannelParams* audioChannelParams,
                         _in_ boolean inbandRingSupported,
                         _in_ ClxApplicationCallbackFunc callbackFunc);

/**
Connect an instance of Hands free audio gateway profile to a remote hands free service. 
Pairing with the remote device may be completed before connecting to the device. 
If there are no pairing at the time of this API, the authentication process will be executed during this API. 
See GAP API for the details of pairing.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HFP_AG_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpAgConnect(_in_ ClxHandle hfpAg, 
                         _in_ ClxDeviceId remoteDeviceId, 
                         _in_ boolean block);

/**
Requests to begin/answer incoming call  to the remote Hands free service (HF Role).
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  callInformation pointer to the call information such as callIndex, mode, status and number.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
*/
ClxError clxHfpAgBeginIncomingCall(_in_ ClxHandle hfpAg,
                                   _user_in_ const ClxHfpCallInformation* callInformation,
                                   _in_ boolean block);

/**
This function is called to activate an incoming/outgoing call based on call Index and inform to the remote Hands free service (HF Role).
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  callIndex it is an unique number allocated to a particular call.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHfpAgActivateCall(_in_ ClxHandle hfpAg,
                            _in_ u4 callIndex,
                            _in_ boolean block);

/**
This function is called when there is a call in progress and another incoming call is activated thus informing the remote 
Hands free device about an incoming call on hold based on call Index.
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  callIndex it is an unique number allocated to a particular call.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHfpAgPlaceCallOnHold(_in_ ClxHandle hfpAg,
                               _in_ u4 callIndex,
                               _in_ boolean block);

/**
This function is called when an outgoing call is initiated by AG.
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  callInformation pointer to the call information such as mode, status and number.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful.
*/
ClxError clxHfpAgBeginOutgoingCall(_in_ ClxHandle hfpAg,
                                   _user_in_ const ClxHfpCallInformation* callInformation,
                                   _in_ boolean block);

/**
This function is called when AG rejects outgoing call request initiated by the HF device.
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  reason List of reasons for HFP AG error.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHfpAgRejectOugoingCallRequest(_in_ ClxHandle hfpAg,
                                       _in_ ClxHfpAgErrors reason,
                                       _in_ boolean block);

/**
This function is called to inform that remote party is ringing.
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHfpAgRemotePartyReached(_in_ ClxHandle hfpAg,
                                    _in_ boolean block);

/**
This function is called when remote party answers outgoing call initiated by local Device.
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHfpAgRemotePartyAnswered(_in_ ClxHandle hfpAg,
                                     _in_ boolean block);

/**
This function is called when either outgoing/incoming call is released/rejected by AG. There can be two reasons:
                  - Incoming call rejected by user(AG).
                  - Outgoing/incoming call interrupted due to internal event.

NOTE: This function is used both for call reject and call terminate actions.

\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  callIndex it is an unique number allocated to a particular call.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
*/
ClxError clxHfpAgReleaseCall(_in_ ClxHandle hfpAg,
                             _in_ u4 callIndex,
                             _in_ boolean block);

/**
This function is called to inform remote HF device that a particular indicator value is changed. Indicator value can change
due to some internal event.
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  indicator type of indicator service, signal, roam or battery charge.
\param[ in ]  indicatorCurrentValue value of the indicator
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful            
*/
ClxError clxHfpAgUpdateIndicatorValue(_in_ ClxHandle hfpAg,
                                      _in_ ClxHfAgIndicator indicator,
                                      _in_ u1 indicatorCurrentValue,
                                      _in_ boolean block);

/**
This function is called when HF wishes to set a particular network operator name or AG updates current network operator name
\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  operatorName type of network operator.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful            
*/
ClxError clxHfpAgSetNetworkOperatorName(_in_ ClxHandle hfpAg,
                                        _user_in_ const s1* operatorName,
                                        _in_ boolean block);

/**
This function is called upon a user action or an internal event and when AG initiates the establishment of an 
Audio Connection whenever necessary. Further internal actions may be needed by AG to internally route the audio paths.

In principle, setting up an Audio Connection is not necessarily related to any call process.
The AG shall keep the audio paths, call related or not, routed towards HF for all the operations 
(e.g. voice, alert, key press tones) involving presence of audio.

As pre-condition for this procedure, an ongoing Service Level Connection between the AG and the HF shall exist.

\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  audioParams  Bluetooth Audio channel parameters such as transmtBandwidth, receiveBandwidth etc. For more info
                           click link #ClxBluetoothAudioChannelParams.
\param[ in ]  block        type of the operation.
                          - TRUE:  API will be blocked until this command is completed (successfully or failed).
                          - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful            
*/
ClxError clxHfpAgConnectAudio(_in_ ClxHandle hfpAg,
                              _user_in_ const ClxBluetoothAudioChannelParams* audioParams,
                              _in_ boolean block);

/**
This function is called upon a user action or an internal event and when AG initiates the release of an 
existing Audio Connection whenever necessary. 

In principle, release of an Audio Connection is not necessarily related to any call process. Usually called upon the termination of a call.
The AG shall keep the audio paths, call related or not, routed towards HF for all the operations 
An Audio Connection removal always means the disconnection of its corresponding Synchronous Connection.
When the audio connection is released, the audio path shall be routed to the AG.

As pre-condition for this procedure, an ongoing Service Level Connection between the AG and the HF shall exist.

\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  block        type of the operation.
                          - TRUE:  API will be blocked until this command is completed (successfully or failed).
                          - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful            
*/
ClxError clxHfpAgDisconnectAudio(_in_ ClxHandle hfpAg,
                                 _in_ boolean block);

/**
Disconnect an instance of Hands free audio gateway profile to a remote Hands free service. 
Disconnect means service level disconnection.
Note: The disconnection of a Service Level Connection shall result in the immediate removal of the corresponding 
RFCOMM data link channel between the HF and the AG. 
Also, an existing audio connection shall be removed as consequence of the removal of the Service Level Connection. 

Disconnection may be called due to following reasons:
 - Explicit user requests AG to release SLC.
 - When Bluetooth functionality is disabled on AG.
 - The Service Level Connection release procedure may be initiated if an Audio Connection release by AG,
   is performed during an ongoing call in the AG.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HFP_AG_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHfpAgDisconnect(_in_ ClxHandle hfpAg,
                            _in_ boolean block);

/**
This function is called when In-band ringing option is enabled in AG. 
If an in-band ring tone is used, besides sending RING(ALERT), the AG shall also send the In-band ring tone
to the HF via the established Audio Connection.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HFP_AG_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHfpAgEnableInbandRing(_in_ ClxHandle hfpAg,
                                  _in_ boolean block);

/**
This function is called to disable In-band ringing option in AG. 
Under these circumstances, the AG shall send RING(ALERT) to the HF via the established Audio Connection.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_HFP_AG_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ]  hfpAg HFP/AG profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHfpAgDisableInbandRing(_in_ ClxHandle hfpAg,
                                   _in_ boolean block);


/**
Returns the numeric handle to the current Audio Connection. The handle can be passed to the implementation of ClarinoxBlue SCO interface to receive
and audio packets over this audio connection.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_AG_GET_AUDIO_CONNECTION_HANDLE_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpAg HFP/AG profile handle.
\param[ out ] audioConnectionHandle A caller-allocated variable which on a successful completion of the command will hold the handle to the audio connection.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_AUDIO_CONNECTION_NOT_EXIST: there is no audio connection at this moment
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpAgGetAudioConnectionHandle(_in_ ClxHandle hfpAg,
                                          _user_out_ u2* audioConnectionHandle,
                                          _in_ boolean block);


#ifdef __cplusplus
}
#endif


#endif // _HFP_AG_API_H_
