#ifndef _HFP_HF_API_H_
#define _HFP_HF_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                HfpHf.Api.h
* Description         Declares ClarinoxBlue Hands-free Profile HF role API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
HFP/HF SupportedFeatures, used as an input parameters for #clxHfpHfCreate API function.
*/
enum  HfpHfSupportedFeatures
{
  HF_FEATURE_EC_AND_OR_NR                = 0x0001,   /*!<Setting this flag indicates that this device implements noise reduction / echo cancellation. */
  HF_FEATURE_CALL_WAITING_THREE_WAY_CALL = 0x0002,   /*!<Setting this flag indicates that this device supports call waiting and  three way calling*/
  HF_FEATURE_CLI_CAPABILITY              = 0x0004,   /*!<Setting this flag indicates that this device can display Caller Line Identification information */
  HF_FEATURE_VOICE_RECOGNITION           = 0x0008,   /*!<Setting this flag indicates that this device can enable voice recognition on the AG */
  HF_FEATURE_REMOTE_VOLUME_CONTROL       = 0x0010,   /*!<Setting this flag indicates that the AG can control this device's local volume. */
  HF_FEATURE_ENHANCED_CALL_STATUS        = 0x0020,   /*!<Setting this flag indicates can request a list of current calls from the AG and also receive call status indicators from the AG. */
  HF_FEATURE_ENHANCED_CALL_CONTROL       = 0x0040    /*!<Setting this flag indicates that this device can use the extended three-way calling features of the AG. */
};

/**
Three way call handling options, used as an input parameter for #clxHfpHfHandleThreeWayCall API function.
*/
enum CallHoldOptions
{
    ClxHfpHfReleaseHeldCalls             = 0x0001,    /*!<Releases all held calls or sets User Determined User Busy (UDUB) for a waiting call. */
    ClxHfpHfReleaseActiveCall            = 0x0002,    /*!<Releases all active calls (if any exist) and accepts the other (held or waiting) call. 1[idx] = Releases specified active call only ([idx]). */
    ClxHfpHfPutOnHoldActiveCall          = 0x0004,    /*!<Places all active calls (if any exist) on hold and accepts the other (held or waiting) call. 2[idx] = Request private consultation mode with specified call ([idx]). (Place all calls on hold EXCEPT the call indicated by [idx].)*/
    ClxHfpHfAddHeldToConversation        = 0x0008,    /*!<Adds a held call to the conversation.*/
    ClxHfpHfConnectTwoCallsAndHangup     = 0x0010,    /*!<Connects the two calls and disconnects the subscriber from both calls (Explicit Call Transfer). Support for this value and its associated functionality is optional for the HF.*/
    ClxHfpHfReleaseActiveCallSpecified   = 0x0020,    /*!<Releases the call specified by index.*/
    ClxHfpHfPutOnHoldActiveCallSpecified = 0x0040     /*!<Puts on hold the active call specified by index. */
};

/**
RoamingStatus options, used in #ClxHfpHfIndicators struct.
*/
enum RoamingStatus
{
    ClxHfpHfRoamingActive,                  /*!<Indicates that Roaming is active*/
    ClxHfpHfRoamingInactive                 /*!<Indicates that Roaming is not active*/
};

/**
Type of indicators, used in #ClxHfpHfIndicators struct.
*/
enum IndicatorType
{
    ClxHfpHfNoIndicator,                    /*!<This can be used as an initial value*/
    ClxHfpHfBatteryCharge,                  /*!<Battery Charge indicator of AG*/
    ClxHfpHfSignalStrength,                 /*!<Signal Strength indicator of AG*/
    ClxHfpHfRoamingStatus                   /*!<Roaming status indicator of AG*/
};

/**
Return parameters of the indication #CLX_HFP_HF_INDICATOR_RECEIVED_INDICATION.
*/
typedef struct ClxHfpHfIndicators
{
    u1   battLevel;                                  /*!<Battery level value can be (0-5)*/
    u1   signalStr;                                  /*!<Signal strength value can be (0-5)*/
    enum RoamingStatus roamingStat;                  /*!<Roaming status can be (0 or 1)*/    
    enum IndicatorType indicatorReceivedType;        /*!<Type of the indicators*/   
}ClxHfpHfIndicators;

/**
ResponseHoldStatus options, used in #ClxHfpHfHeldCallResponse struct.
*/
enum ResponseHoldStatus
{
    ClxHfpHfOnHold                       = 0,        /*!<Incoming call is put on hold in the AG */
    ClxHfpHfAccepted                     = 1,        /*!<Held incoming call is accepted in the AG */
    ClxHfpHfRejected                     = 2,        /*!<Held incoming call is rejected in the AG */
    ClxHfpHfNotDefinedResponseHoldStatus = 3         /*!<Defined to be set as an initial value */
};

/**
Return parameters of the indication #CLX_HFP_HF_HELD_CALL_INDICATION.
*/
typedef struct ClxHfpHfHeldCallResponse
{
    enum ResponseHoldStatus responseHoldStatus;
}HeldCallResponse;

/**
Return parameters of the indication #CLX_HFP_HF_ERROR_RECV_INDICATION.
*/
typedef struct ClxHfpAgErrorCodes
{
    u1 errorCodes;
}ClxHfpAgErrorCodes;

/**
Return parameters of the indication #CLX_HFP_HF_HOLD_OPTIONS_RECV_INDICATION.
*/
typedef struct ClxHfpHfHoldCallOptions
{
    u1 holdCallOptions;
}ClxHfpHfHoldCallOptions;

/**
Refer to #clxHfpHfSLCConnect function.
*/
#define CLX_HFP_HF_SLC_CONNECT_COMPLETE                                     0x4C00

/**
Refer to #clxHfpHfAudioConnect function.
*/
#define CLX_HFP_HF_AUDIO_CONNECT_COMPLETE                                   0x4C01


/**
Refer to #ClxHfpHfGetAudioConnectionHandleComplete function.
*/
#define CLX_HFP_HF_GET_AUDIO_CONNECTION_HANDLE_COMPLETE                     0x4C16


/**
Refer to #clxHfpHfAnswerCallRequest function.
*/
#define CLX_HFP_HF_ANSWER_COMPLETE                                          0x4C02

/**
Refer to #clxHfpHfRejectCallRequest function.
*/
#define CLX_HFP_HF_REJECT_COMPLETE                                          0x4C03

/**
Refer to #clxHfpHfHangupCallRequest function.
*/
#define CLX_HFP_HF_HANGUP_COMPLETE                                          0x4C04

/**
Refer to #clxHfpHfQueryResponseHoldStatus function.
*/
#define CLX_HFP_HF_QUERY_RESPONSE_HOLD_STATUS_COMPLETE                      0x4C05

/**
Refer to #clxHfpHfAcceptHeldIncoming function.
*/
#define CLX_HFP_HF_ACCEPT_HELD_INCOMING_COMPLETE                            0x4C06

/**
Refer to #clxHfpHfRejectHeldIncoming function.
*/
#define CLX_HFP_HF_REJECT_HELD_INCOMING_COMPLETE                            0x4C07

/**
Refer to #clxHfpHfInitiateOutgoingCall function.
*/
#define CLX_HFP_HF_INITIATE_OUTGOING_CALL_COMPLETE                          0x4C08

/**
Refer to #clxHfpHfDialTheLastNumber function.
*/
#define CLX_HFP_HF_DIAL_LAST_NUMBER_COMPLETE                                0x4C09

/**
Refer to #clxHfpHfDialFromMemory function.
*/
#define CLX_HFP_HF_DIAL_FROM_MEMORY_COMPLETE                                0x4C0A

/**
Refer to #clxHfpHfVoiceRecognitionSetup function.
*/
#define CLX_HFP_HF_VOICE_RECOGNITION_SETUP_COMPLETE                         0x4C0B

/**
Refer to #clxHfpHfCallWaitingNotificationSetup function.
*/
#define CLX_HFP_HF_CALL_WAITING_SETUP_COMPLETE                              0x4C0C

/**
Refer to #clxHfpHfCallingLineIdNotificationSetup function.
*/
#define CLX_HFP_HF_CALLING_LINE_ID_SETUP_COMPLETE                           0x4C0D

/**
Refer to #clxHfpHfHandleThreeWayCall function.
*/
#define CLX_HFP_HF_HANDLE_THREE_WAY_CALL_COMPLETE                           0x4C0E

/**
Refer to #clxHfpHfTransmitDtmfCode function.
*/
#define CLX_HFP_HF_TRANSMIT_DTMF_CODE_COMPLETE                              0x4C0F

/**
Refer to #clxHfpHfReportSpeakerVolume function.
*/
#define CLX_HFP_HF_REPORT_SPEAKER_VOLUME_COMPLETE                           0x4C10

/**
Refer to #clxHfpHfReportMicrophoneGain function.
*/
#define CLX_HFP_HF_REPORT_MICROPHONE_GAIN_COMPLETE                          0x4C11

/**
Refer to #clxHfpHfQuerySubscriberNumber function.
*/
#define CLX_HFP_HF_QUERY_SUBSCRIBER_NUMBER_COMPLETE                         0x4C12

/**
Refer to #clxHfpHfQuerySubscriberNumber function.
*/
#define CLX_HFP_HF_QUERY_LIST_OF_CURRENT_CALLS_COMPLETE                     0x4C13

/**
Refer to #clxHfpHfDisconnectAudio function.
*/
#define CLX_HFP_HF_DISCONNECT_AUDIO_COMPLETE                                0x4C14

/**
Refer to #clxHfpHfDisconnectSLC function.
*/
#define CLX_HFP_HF_DISCONNECT_SLC_COMPLETE                                  0x4C15

/**
This indication is sent to the application when a Service Level Connection is made to a remote HFP/AG service.
This indication is only received when the connection has been initiated by the remote device (HFP/AG). 
The parameter of this indication (params argument of the call-back function) will be of type #ClxHfpConnectionIndication.
*/
#define CLX_HFP_HF_SLC_CONNECTION_INDICATION                                0x8C00

/**
This indication is received when an audio connection is made to a remote HFP/AG device.
This indication is only received when the connection has been initiated by the remote device (HFP/AG). 
The parameter of this indication (there are no parameter arguments of the call-back function) will be NULL.
*/
#define CLX_HFP_HF_AUDIO_CONNECTION_INDICATION                              0x8C01

/**
This indication is sent to the application when a Service Level Connection has been disconnected.
This indication is only received when the disconnection has been initiated by the remote device(HFP/AG). 
The parameter of this indication (params argument of the call-back function) will be of type #ClxHfpConnectionIndication.
*/
#define CLX_HFP_HF_SLC_DISCONNECTION_INDICATION                             0x8C02

/**
This indication is sent to the application when the audio connection (SCO/eSCO) has been disconnected .
This indication is only received when the disconnection has been initiated by the remote device(HFP/AG). 
The parameter of this indication (there are no parameter arguments of the call-back function) will be NULL.
*/
#define CLX_HFP_HF_AUDIO_DISCONNECTION_INDICATION                           0x8C03

/**
This indication is sent to the application when a Service Level Connection has been lost. 
and stack is automatically trying to reconnect, this can be configured in hfp_init_params. 
If all reconnection attempts fail then CLX_HFP_HF_SLC_DISCONNECTION_INDICATION will be sent.

The parameter of this indication (there are no parameter arguments of the call-back function) will be NULL.
*/
#define CLX_HFP_HF_SLC_LINK_LOSS_INDICATION                                 0x8C04

/**
This indication is sent to the application when a Ring signal is sent to a remote HFP/AG device.
This indication is only received when the Ring signal Request has been initiated by the remote device(HFP/AG).
The parameter of this indication (there are no parameter arguments of the call-back function) will be NULL.
*/
#define CLX_HFP_HF_RING_SIGNAL_INDICATION                                   0x8C05

/**
This indication is sent to the application when a call is established with a remote HFP/AG device.
This indication is only received when the call establish request has been initiated by the remote device(HFP/AG).
The parameter of this indication (there are no parameter arguments of the call-back function) will be NULL.
*/
#define CLX_HFP_HF_CALL_ESTABLISHMENT_INDICATION                            0x8C06

/**
This indication is sent to the application when a call is terminated with a remote HFP/AG device.
This indication is only received when the terminate call request has been initiated by the remote device(HFP/AG).
The parameter of this indication (there are no parameter arguments of the call-back function) will be NULL.
*/
#define CLX_HFP_HF_CALL_TERMINATION_INDICATION                              0x8C07

/**
This indication is sent to the application whenever an indicator is received from the AG.
The parameter for this indication is of type #ClxHfpHfIndicators.
*/
#define CLX_HFP_HF_INDICATOR_RECEIVED_INDICATION                            0x8C08

/**
This indication is sent to the application whenever a call waiting notification is received from the AG .
The parameter for this indication is of type #ClxHfpNumberInfo.
*/
#define CLX_HFP_HF_CALL_WAITING_NOTIFICATION_INDICATION                     0x8C09

/**
This indication is sent to the application when a set microphone gain request received from AG.
The parameter of this indication is of type #ClxHfpMicrophoneGain.
*/
#define CLX_HFP_HF_MICROPHONE_GAIN_ADJUSTMENT_REQUEST_INDICATION            0x8C0A

/**
This indication is sent to the application when a set Speaker Volume request received from AG.
The parameter for this indication is of type #ClxHfpSpeakerVolume.
*/
#define CLX_HFP_HF_SPEAKER_VOLUME_ADJUSTMENT_REQUEST_INDICATION             0x8C0B

/**
This indication is sent to the application when a response to put on hold received from AG.
The parameter for this indication is of type #ClxHfpHfHeldCallResponse.
*/
#define CLX_HFP_HF_HELD_CALL_INDICATION                                     0x8C0C

/**
This indication is sent to the application when a CLI(Calling Line Identification) notification received from AG.
The parameter for this indication is of type #ClxHfpNumberInfo.
*/
#define CLX_HFP_HF_CLI_NOTIFICATION_INDICATION                              0x8C0D

/**
This indication is sent to the application whenever a current calls information received from AG.
The parameter for this indication is of type #ClxHfpCallInformation.
*/
#define CLX_HFP_HF_CURRENT_CALLS_INFO_RECV_INDICATION                       0X8C0E

/**
This indication is sent to the application whenever an error received from AG.
The parameter for this indication is of type #ClxHfpAgErrorCodes.
*/
#define CLX_HFP_HF_ERROR_RECV_INDICATION                                    0X8C0F

/**
This indication is sent to the application whenever information about Hold and Call options, supported in AG received from AG.
The parameter for this indication is of type #ClxHfpHfHoldCallOptions.
*/
#define CLX_HFP_HF_HOLD_OPTIONS_RECV_INDICATION                         0X8C10

#define CLX_HFP_HF_AUDIO_CONNECTION_FAILED_INDICATION                   0x8C11


typedef struct HfpHfConfig
{
  u2            hfpHfSupportedFeatures;
  boolean       linkLossRecover;
} HfpHfConfig;


/** Data Structure for the indication #CLX_HFP_HF_GET_AUDIO_CONNECTION_HANDLE_COMPLETE */
typedef struct ClxHfpHfGetAudioConnectionHandleCompleteStruct 
{
    _user_out_ u2*     audioConnectionHandle;
} ClxHfpHfGetAudioConnectionHandleComplete;


/** Data Structure for the indication #CLX_HFP_HF_AUDIO_CONNECTION_INDICATION */
typedef struct ClxHfpHfAudioConnectionindicationStruct 
{
    _in_ u2     audioConnectionHandle;
} ClxHfpHfAudioConnectionindication;



/**
Creates a Hands Free service (Hands Free role) on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] hfpHfConfig is a bitwise combination of values defined in #HfpHfConfig.
\param[ in ] audioChannelParams pointer to the list of audio channel parameters.
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\return ClxHandle HFP/HF handle. This handle must be used for any further operations on this instance of HFPHF profile.
*/
ClxHandle clxHfpHfCreate(_in_ ClxStack                               stack,
                         _in_ HfpHfConfig                            hfpHfConfig,
                         _in_ const ClxBluetoothAudioChannelParams*  audioChannelParams,
                         _in_ ClxApplicationCallbackFunc             callbackFunc);

/**
Establishes a RFCOMM channel and then a Service Level Connection, if there is no RFCOMM signalling connection,
it establishes the RFCOMM signalling connection before establishing the RFCOMM channel.

Pairing with the remote server may be completed before connecting to the remote device. 
If there are no pairing at the time of this API, the authentication process may or may not be executed during this API function execution. 
See GAP API for the details of pairing.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_SLC_CONNECT_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfSLCConnect(_in_ ClxHandle   hfpHf,
                            _in_ ClxDeviceId remoteDeviceId,
                            _in_ boolean     block);

/**
Initiates the establishment of an Audio Connection to the remote HFP/AG device. 

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_AUDIO_CONNECT_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfAudioConnect(_in_            ClxHandle                       hfpHf,
                              _in_            boolean                         block);


/**
Returns the numeric handle to the current Audio Connection. The handle can be passed to the implementation of ClarinoxBlue SCO interface to receive
and audio packets over this audio connection.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_GET_AUDIO_CONNECTION_HANDLE_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ out ] audioConnectionHandle A caller-allocated variable which on a successful completion of the command will hold the handle to the audio connection.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_AUDIO_CONNECTION_NOT_EXIST: there is no audio connection at this moment
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfGetAudioConnectionHandle(_in_ ClxHandle hfpHf,
                                          _user_out_ u2* audioConnectionHandle,
                                          _in_ boolean block);

/**
Sends a request to AG to answer the incoming voice call.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_ANSWER_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] putOnHold determines the type of answering the incoming call.
                - TRUE: put the incoming call on hold.  
                - FALSE: answer the incoming call. 
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfAnswerCallRequest(_in_ ClxHandle hfpHf,
                                   _in_ boolean   putOnHold,
                                   _in_ boolean   block);

/**
Sends a request to AG to reject the incoming voice call. 

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_REJECT_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle. 
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfRejectCallRequest(_in_ ClxHandle hfpHf,
                                   _in_ boolean   block);

/**
Sends a request to AG to terminate the ongoing call process. 

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_HANGUP_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfHangupCallRequest(_in_ ClxHandle hfpHf,
                                   _in_ boolean   block);
                         
/**
This function allows the user to query the status of the Response and Hold state of the AG.

If AG replies to the query, the application will receive #CLX_HFP_HF_HELD_CALL_INDICATION indication,
but if the AG is not in the Response and Hold states, then no response shall be sent.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_QUERY_RESPONSE_HOLD_STATUS_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] agInResHoldStates declares if AG is currently in any of the Response and Hold states or not.
                - TRUE: AG is in one of the Response and Hold states.
                - FALSE: AG is not in any of the Response and Hold states.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfQueryResponseHoldStatus(_in_      ClxHandle hfpHf,
                                         _user_out_ boolean* agInResHoldStates,
                                         _in_      boolean block);

/**
This function allows the user to accept a previously held incoming call.

The application will receive #CLX_HFP_HF_HELD_CALL_INDICATION indicating the success or failure of the request.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_ACCEPT_HELD_INCOMING_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfAcceptHeldIncoming(_in_  ClxHandle hfpHf,
                                    _in_  boolean block);

/**
This function allows the user to reject a previously held incoming call

The application will receive #CLX_HFP_HF_HELD_CALL_INDICATION indicating the success or failure of the request.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_REJECT_HELD_INCOMING_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfRejectHeldIncoming(_in_  ClxHandle hfpHf,
                                    _in_  boolean block);

/**
Initiates outgoing voice calls by providing the destination phone number to the AG

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_INITIATE_OUTGOING_CALL_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] callInformation pointer to the call information such as callIndex, mode, status and number.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/

ClxError clxHfpHfInitiateOutgoingCall(_in_            ClxHandle              hfpHf,
                                      _user_in_ const ClxHfpCallInformation* callInformation,
                                      _in_            boolean                block);

/**
Initiates outgoing voice call to the last number dialled in the AG

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_DIAL_LAST_NUMBER_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfDialTheLastNumber(_in_ ClxHandle  hfpHf,
                                   _in_ boolean    block);

/**
Initiates outgoing voice call to the last number dialled in the AG

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_DIAL_FROM_MEMORY_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] memoryLocation a decimal number representing a memory location of the saved number in AG memory. 
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfDialFromMemory (_in_ ClxHandle hfpHf,
                                 _in_ u1        memoryLocation,
                                 _in_ boolean   block);

/**
Enables/Disables the voice recognition function resident in the AG.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_VOICE_RECOGNITION_SETUP_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] enable determines the action of disabling or enabling the voice recognition.
                - TRUE: enable the voice recognition.  
                - FALSE: disable the voice recognition.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfVoiceRecognitionSetup(_in_ ClxHandle hfpHf,
                                       _in_ boolean   enable,
                                       _in_ boolean   block);

/**
Enables/Disables the call waiting notifications in the AG.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_CALL_WAITING_SETUP_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] enable determines the action of disabling or enabling the call waiting notification.
                - TRUE: enable the call waiting notification.  
                - FALSE: disable the call waiting notification .
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfCallWaitingNotificationSetup(_in_ ClxHandle hfpHf,
                                              _in_ boolean   enable,
                                              _in_ boolean   block);

/**
Enables/Disables the Calling Line Identification notifications in the AG.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_CALLING_LINE_ID_SETUP_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] enable determines the action of disabling or enabling the Calling Line Identification notification.
                - TRUE: enable the call waiting notification.  
                - FALSE: disable the call waiting notification .
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfCallingLineIdNotificationSetup(_in_ ClxHandle hfpHf,
                                                _in_ boolean   enable,
                                                _in_ boolean   block);

/**
It handles the three way calling, which is the standard call hold and multiparty handling.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_HANDLE_THREE_WAY_CALL_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] callHoldOption is an enum of type CallHoldOptions which decides the type of activating and holding the calls.
\param[ in ] callIndex is an unique number allocated to a particular call. Enhanced call control must be enabled on both the HF and AG to use this feature. 
             Shall set to zero, to use normal command sets.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfHandleThreeWayCall(_in_  ClxHandle               hfpHf,
                                    _in_  enum CallHoldOptions    callHoldOption,
                                    _in_  u4                      callIndex,
                                    _in_  boolean                 block);
                         
/**
Transmits the key pressed to the AG, so AG can transmit a specific DTMF code to its network connection.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_TRANSMIT_DTMF_CODE_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] keyPressed ASCII code of the key pressed.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfTransmitDtmfCode(_in_ ClxHandle hfpHf,
                                  _in_ s1*       keyPressed,
                                  _in_ boolean   block);

/**
Reports its current speaker volume level setting to the AG.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_REPORT_SPEAKER_VOLUME_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] speakerVolume is the HF speaker volume, has to be 0-15, where 0 = Minimum gain and 15 = Maximum gain.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfReportSpeakerVolume(_in_ ClxHandle hfpHf,
                                     _in_ u1        speakerVolume,
                                     _in_ boolean   block);

/**
Reports its current Microphone Gain level setting to the AG.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_REPORT_SPEAKER_VOLUME_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] microphoneGain is the HF current Microphone Gain, has to be 0-15, where 0 = Minimum gain and 15 = Maximum gain.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfReportMicrophoneGain(_in_ ClxHandle hfpHf,
                                      _in_ u1        microphoneGain,
                                      _in_ boolean   block);

/**
Queries for the AG Subscriber Number Information.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_QUERY_SUBSCRIBER_NUMBER_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ out ] numberInfo a caller-provided structure of type ClxHfpNumberInfo which on return, will contain AG Subscriber number information.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfQuerySubscriberNumber(_in_       ClxHandle           hfpHf,
                                       _user_out_ ClxHfpNumberInfo*   numberInfo,
                                       _in_       boolean             block);

/**
Queries for list of current calls in AG.

If AG replies to the query, the application will receive #CLX_HFP_HF_CURRENT_CALLS_INFO_RECV_INDICATION indication,
but if the AG is not in the Response and Hold states, then no response shall be sent.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_QUERY_LIST_OF_CURRENT_CALLS_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfQueryListOfCurrentCalls(_in_       ClxHandle              hfpHf,
                                         _in_       boolean                block);

/**
Disconnects the established Audio Connection to the remote HFP/AG device. 

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_DISCONNECT_AUDIO_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfDisconnectAudio(_in_ ClxHandle hfpHf,
                                 _in_ boolean   block);

/**
Disconnects the RFCOMM channel.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HFP_HF_DISCONNECT_SLC_COMPLETE.
The response does not have any parameters.

\param[ in ] hfpHf HFP/HF profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.

\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHfpHfDisconnectSLC(_in_ ClxHandle hfpHf,
                               _in_ boolean   block);
 

#ifdef __cplusplus
}
#endif

#endif // _HFP_HF_API_H_
