#ifndef __Hidc_Api_h__
#define __Hidc_Api_h__

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Hidc.Api.h
* Description         Declares API Functions and Definitions For Hid
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Refer to #clxHidConnect function.
*/
#define CLX_HID_CONNECT_COMPLETE                                                            0x6401

/**
Refer to #clxHidConnectInterrupt function.
*/
#define CLX_HID_CONNECT_INTERRUPT_COMPLETE                                                  0x6402

/**
Refer to #clxHidSendReport function.
*/
#define CLX_HID_SEND_REPORT_COMPLETE                                                        0x6403

/**
Refer to #clxHidSendData function.
*/
#define CLX_HID_SEND_DATA_COMPLETE                                                          0x6404

/**
Refer to #clxHidSendProtocol function.
*/
#define CLX_HID_SEND_PROTOCOL_COMPLETE                                                      0x6405

/**
Refer to #clxHidSendControlOperation function.
*/
#define CLX_HID_SEND_CONTROL_OPERATION_COMPLETE                                             0x6406

/**
Refer to #clxHidGetHostReport function.
*/
#define CLX_HID_GET_HOST_REPORT_COMPLETE                                                    0x6407

/**
Refer to #clxHidGetHostData function.
*/
#define CLX_HID_GET_HOST_DATA_COMPLETE                                                      0x6408

/**
Refer to #clxHidErrorSendInvalidReportId function.
*/
#define CLX_HID_ERROR_SEND_INVALID_REPORT_ID_COMPLETE                                       0x6409

/**
Refer to #clxHidDisconnect function.
*/
#define CLX_HID_DISCONNECT_COMPLETE                                                         0x640a

/**
Refer to #clxHidDisconnectInterrupt function.
*/
#define CLX_HID_DISCONNECT_INTERRUPT_COMPLETE                                               0x640b

/**
This indication is received when the host establishes the control channel connection.
The parameter of this indication (params argument of the call back function)
will be of type #ClxHidConnectionRequestIndication
*/
#define CLX_HID_CONNECTION_REQUEST_INDICATION                                               0xa400

/**
This indication is received when the host disconnects the established control channel 
connection. The parameter of this indication (params argument of the call back function)
will be of type #ClxHidDisconnectionRequestIndication
*/
#define CLX_HID_DISCONNECTION_REQUEST_INDICATION                                            0xa401

/**
This indication is received when the host sends control operation command to device.
The parameter of this indication (params argument of the call back function)
will be of type #ClxHidControlOperationIndication
*/
#define CLX_HID_CONTROL_OPERATION_INDICATION                                                0xa402

/**
This indication is received when the host requests the report from HID device.
The parameter of this indication (params argument of the call back function)
will be of type #ClxHidGetReportIndication

The parameter reportSize in #ClxHidGetReportIndication is used to inform that
whether the application has to send the entire report or partial report.
If the value of reportSize is 0 then the application has to send the entire
report otherwise limits the report based on the value of reportSize

Once receiving this indication, the application has required to call 
the api #clxHidSendReport to send the report.
*/
#define CLX_HID_GET_REPORT_INDICATION                                                       0xa403

/**
This indication is received when the host sends a report to HID device.
The parameter of this indication (params argument of the call back function)
will be of type #ClxHidSetReportIndication

Once receiving this indication, the application has required to call 
the api #clxHidGetHostReport to get the host report.
*/
#define CLX_HID_SET_REPORT_INDICATION                                                       0xa404

/**
This indication is received when the host requests a device to send the current protocol
mode of the device.
The parameter of this indication (params argument of the call back function) will be of 
type #ClxHidGetProtocolIndication

Once receiving this indication, the application has required to call 
the api #clxHidSendProtocol to send the protocol mode.
*/
#define CLX_HID_GET_PROTOCOL_INDICATION                                                     0xa405

/**
This indication is received when the host sends the protocol mode to configure by device.
The parameter of this indication (params argument of the call back function)
will be of type #ClxHidSetProtocolIndication
*/
#define CLX_HID_SET_PROTOCOL_INDICATION                                                     0xa406

/**
This indication is received when the host sends a data to device.
The parameter of this indication (params argument of the call back function)
will be of type #ClxHidDataIndication

Once receiving this indication, the application has required to call 
the api #clxHidGetHostData to get the host data.
*/
#define CLX_HID_DATA_INDICATION                                                             0xa407

/**
This indication is received when the host establishes the interrupt channel connection.
The parameter of this indication (params argument of the call back function)
will be of type #ClxHidInterruptConnectionRequestIndication
*/
#define CLX_HID_INTERRUPT_CONNECTION_REQUEST_INDICATION                                     0xa408

/**
This indication is received when the host disconnects the established
interrupt channel connection.
The parameter of this indication (params argument of the call back function) will be of
type #ClxHidInterruptDisconnectionRequestIndication
*/
#define CLX_HID_INTERRUPT_DISCONNECTION_REQUEST_INDICATION                                  0xa409

/**
* <!-- -----------------------------------------------------------------
* <!-- ENUM: ClxHidRole 
*
* <!-- DESCRIPTION: 
*       
* ClxHidRole defines the HID role.
*
*<!-- ------------------------------------------------------------------
*/
typedef enum ClxHidRoleEnum
{
    ClxHidRole_Host     = 0x00,                                 /*!<HID Host Role.                                             */
    ClxHidRole_Device   = 0x01                                /*!<HID Device Role.                                             */
} ClxHidRole;

/**
* <!-- -----------------------------------------------------------------
* <!-- ENUM: ClxHidDeviceType 
*
* <!-- DESCRIPTION: 
*       
* ClxHidDeviceType defines the list of supported HID devices.
*
*<!-- ------------------------------------------------------------------
*/
typedef enum ClxHidDeviceTypeEnum
{
    ClxHidDeviceType_UnCategorizedDevice    = 0x00,         /*!<UnCategorized Device.                                          */
    ClxHidDeviceType_JoyStick               = 0x04,         /*!<JoyStick.                                                      */
    ClxHidDeviceType_GamePad                = 0x08,         /*!<GamePad.                                                       */
    ClxHidDeviceType_RemoteControl          = 0x0C,         /*!<Remote Control.                                                */
    ClxHidDeviceType_SensingDevice          = 0x10,         /*!<Sensing Device.                                                */
    ClxHidDeviceType_DigitizerTablet        = 0x14,         /*!<Digitizer Tablet.                                              */
    ClxHidDeviceType_CardReader             = 0x18,         /*!<Card Reader.                                                   */
    ClxHidDeviceType_KeyBoard               = 0x40,         /*!<Key Board.                                                     */
    ClxHidDeviceType_Mice                   = 0x80,         /*!<Pointing Device or Mice.                                       */
    ClxHidDeviceType_ComboKeyBoardMice      = 0xC0          /*!<Combo device for Keyboard and Mice.                            */
} ClxHidDeviceType;

/**
* <!-- ----------------------------------------------------------------- 
* <!-- ENUM: ClxHidDeviceProtocolMode 
*
* <!-- DESCRIPTION: 
*       
* ClxHidDeviceProtocolMode defines the list of supported device protocols.
*
*<!-- ------------------------------------------------------------------
*/
typedef enum ClxHidDeviceProtocolModeEnum
{
    ClxHidDeviceProtocolMode_Boot   = 0x00,                 /*!<Boot protocol mode.                                            */
    ClxHidDeviceProtocolMode_Report = 0x01                  /*!<Report protocol mode.                                          */
} ClxHidDeviceProtocolMode;

typedef enum ClxHidDeviceCountryCodeEnum
{
    ClxHidDeviceCountryCode_NotSupported        = 0x00,
    ClxHidDeviceCountryCode_Arabic              = 0x01,
    ClxHidDeviceCountryCode_Belgian             = 0x02,
    ClxHidDeviceCountryCode_CanadianBilingual   = 0x03,
    ClxHidDeviceCountryCode_CandianFrench       = 0x04,
    ClxHidDeviceCountryCode_CzechRepublic       = 0x05,
    ClxHidDeviceCountryCode_Danish              = 0x06,
    ClxHidDeviceCountryCode_Finnish             = 0x07,
    ClxHidDeviceCountryCode_French              = 0x08,
    ClxHidDeviceCountryCode_German              = 0x09,
    ClxHidDeviceCountryCode_Greek               = 0x0a,
    ClxHidDeviceCountryCode_Hebrew              = 0x0b,
    ClxHidDeviceCountryCode_Hungary             = 0x0c,
    ClxHidDeviceCountryCode_InternationalISO    = 0x0d,
    ClxHidDeviceCountryCode_Italian             = 0x0e,
    ClxHidDeviceCountryCode_JapanKatakana       = 0x0f,
    ClxHidDeviceCountryCode_Korean              = 0x10,
    ClxHidDeviceCountryCode_LatinAmerican       = 0x11,
    ClxHidDeviceCountryCode_NetherlandsDutch    = 0x12,
    ClxHidDeviceCountryCode_Norwegian           = 0x13,
    ClxHidDeviceCountryCode_PersianFarsi        = 0x14,
    ClxHidDeviceCountryCode_Poland              = 0x15,
    ClxHidDeviceCountryCode_Portuguese          = 0x16,
    ClxHidDeviceCountryCode_Russia              = 0x17,
    ClxHidDeviceCountryCode_Slovakia            = 0x18,
    ClxHidDeviceCountryCode_Spanish             = 0x19,
    ClxHidDeviceCountryCode_Swedish             = 0x1a,
    ClxHidDeviceCountryCode_SwissFrench         = 0x1b,
    ClxHidDeviceCountryCode_SwissGerman         = 0x1c,
    ClxHidDeviceCountryCode_Switzerland         = 0x1d,
    ClxHidDeviceCountryCode_Taiwan              = 0x1e,
    ClxHidDeviceCountryCode_TurkishQ            = 0x1f,
    ClxHidDeviceCountryCode_UK                  = 0x20,
    ClxHidDeviceCountryCode_US                  = 0x21,
    ClxHidDeviceCountryCode_Yugoslavia          = 0x22,
    ClxHidDeviceCountryCode_TurkishF            = 0x23,
    ClxHidDeviceCountryCode_Reserved            = 0xff      /*!<36-255  Reserved                                               */
} ClxHidDeviceCountryCode;

/**
* <!-- ----------------------------------------------------------------- 
* <!-- ENUM: ClxHidProtocolMessageType 
*
* <!-- DESCRIPTION: 
*       
* ClxHidProtocolMessageType defines the list of supported message types.
*
*<!-- ------------------------------------------------------------------
*/
typedef enum ClxHidProtocolMessageTypeEnum
{
    ClxHidProtocolMessageType_HandShake     = 0x00,         /* <!-- Message used by device to acknowledge                      */
    ClxHidProtocolMessageType_HidControl    = 0x01,         /* <!-- Message used to request a major state change in the device */
    /* 0x02 - 0x03  Reserved */
    ClxHidProtocolMessageType_GetReport     = 0x04,         /* <!-- Message used by host to request the transfer of HID report */
    ClxHidProtocolMessageType_SetReport     = 0x05,         /* <!-- Message used by host to initiate the transfer of report    */
    ClxHidProtocolMessageType_GetProtocol   = 0x06,         /* <!-- Message used by host to retrieve a protocol mode           */
    ClxHidProtocolMessageType_SetProtocol   = 0x07,         /* <!-- Message used by host to set a protocol mode on device      */
    /*0x08 - 0x09 Deprecated */
    ClxHidProtocolMessageType_Data          = 0x0a,         /* <!-- Message used by both host and device to transfer data      */
    /* 0x0b Deprecated */
    ClxHidProtocolMessageType_Reserved      = 0x0c          /* <!-- 0x0c - 0xff  Reserved                                      */
} ClxHidProtocolMessageType;

/**
* <!-- ----------------------------------------------------------------- 
* <!-- ENUM: ClxHidHandshakeResultCode 
*
* <!-- DESCRIPTION: 
*       
* ClxHidHandshakeResultCode defines the list of supported error codes
* in #ClxHidProtocolMessageType_HandShake
*
*<!-- ------------------------------------------------------------------
*/
typedef enum ClxHidHandshakeResultCodeEnum
{
    ClxHidHandshakeResultCode_Successful            = 0x00, /**
                                                            * <!-- Device has properly received GetReport/SetReport and 
                                                            * GetProtocol/SetProtocol messages
                                                            */
    ClxHidHandshakeResultCode_Not_Ready             = 0x01, /* <!-- Device is too busy to accept the data                      */
    ClxHidHandshakeResultCode_ErrInvalidReportId    = 0x02, /* <!-- Device receives invalid report id                          */
    ClxHidHandshakeResultCode_ErrUnsupportedRequest = 0x03, /* <!-- Device receives unsupported or reserved message types      */
    ClxHidHandshakeResultCode_ErrInvalidParameter   = 0x04, /* <!-- Device receives a parameter value is out of range          */
    /* 0x05 - 0x0d  Reserved */
    ClxHidHandshakeResultCode_ErrUnknown            = 0x0e, /* <!-- Device count not identify the error condition             */
    ClxHidHandshakeResultCode_ErrFatal              = 0x0f  /* <!-- Restart is essential to resume the functionality           */
} ClxHidHandshakeResultCode;

/**
* <!-- ----------------------------------------------------------------- 
* <!-- ENUM: ClxHidControlOperation 
*
* <!-- DESCRIPTION: 
*       
* ClxHidControlOperation defines the list of supported control commands.
*
*<!-- ------------------------------------------------------------------
*/
typedef enum ClxHidControlOperationEnum
{
    ClxHidControlOperation_Suspend              = 0x03,     /* <!-- Go to reduced power mode                                   */
    ClxHidControlOperation_ExitSuspend          = 0x04,     /* <!-- Exit reduced power mode                                    */
    ClxHidControlOperation_VirtualCableUnplug   = 0x05      /* <!-- Unplugging the virtual cable                               */
} ClxHidControlOperation;

/**
* <!-- ----------------------------------------------------------------- 
* <!-- ENUM: ClxHidReportType 
*
* <!-- DESCRIPTION: 
*       
* ClxHidReportType defines the list of available report types.
*
*<!-- ------------------------------------------------------------------
*/
typedef enum ClxHidReportTypeEnum
{
    ClxHidReportType_Other      = 0x00,                     /* <!-- Other                                                      */
    ClxHidReportType_Input      = 0x01,                     /* <!-- Input Report                                               */
    ClxHidReportType_Output     = 0x02,                     /* <!-- Output Report                                              */
    ClxHidReportType_Feature    = 0x03                      /* <!-- Feature Report                                             */
} ClxHidReportType;

typedef struct ClxHidDescriptorStruct
{
    u1  type;
    u1  length;
    u1* descriptorBuffer;
}ClxHidDescriptor;

/**
* <!-- ----------------------------------------------------------------- 
* <!-- STRUCT: ClxHidDeviceConfiguration 
*
* <!-- DESCRIPTION: 
*       
* ClxHidDeviceConfiguration defines the device configuration parameters.
* These details are required for HID SDP records. All these parameters are
* filled by the Application and pass these details via #clxHidCreate
*
*<!-- ------------------------------------------------------------------
*/
typedef struct ClxHidDeviceConfigurationStruct
{
    u2                          parserVersion;              /* <!-- HID parser version in device                               */
    ClxHidDeviceType            deviceType;                 /* <!-- device type                                                */
    ClxHidDeviceCountryCode     countryCode;                /* <!-- country code                                               */
    boolean                     isVirtualCableConnect;      /* <!-- Indicates whether device supports virtual cable            */
    boolean                     isReConnectInitiate;        /* <!-- Indicates whether device initiates reconnection process    */
    boolean                     isBootDevice;               /* <!-- Indicates whether a device supports boot protocol mode     */
    u2                          descriptorCount;            /* <!-- Count of DescriptorList                                    */
    ClxHidDescriptor*           descriptorList;             /* <!-- DescriptorList                                             */
} ClxHidDeviceConfiguration;

/* Data Structure for the indication CLX_HID_CONNECTION_REQUEST_INDICATION */
typedef struct ClxHidConnectionRequestIndicationStruct
{
    ClxDeviceId     deviceId;
} ClxHidConnectionRequestIndication;

/* Data Structure for the indication CLX_HID_DISCONNECTION_REQUEST_INDICATION */
typedef struct ClxHidDisconnectionRequestIndicationStruct
{
    ClxDeviceId     deviceId;
} ClxHidDisconnectionRequestIndication;

/* Data Structure for the indication CLX_HID_CONTROL_OPERATION_INDICATION */
typedef struct ClxHidControlOperationIndicationStruct
{
    ClxHidControlOperation     controlOperation;
} ClxHidControlOperationIndication;

/* Data Structure for the indication CLX_HID_GET_REPORT_INDICATION */
typedef struct ClxHidGetReportIndicationStruct
{
    ClxHidReportType     reportType;
    u1                   reportId;
    u2                   reportSize;
} ClxHidGetReportIndication;

/* Data Structure for the indication CLX_HID_SET_REPORT_INDICATION */
typedef struct ClxHidSetReportIndicationStruct
{
    ClxHidReportType     reportType;
    u4                   reportSize;
} ClxHidSetReportIndication;

/* Data Structure for the indication CLX_HID_GET_PROTOCOL_INDICATION */
typedef struct ClxHidGetProtocolIndicationStruct
{
    ClxResult     result;
} ClxHidGetProtocolIndication;

/* Data Structure for the indication CLX_HID_SET_PROTOCOL_INDICATION */
typedef struct ClxHidSetProtocolIndicationStruct
{
    ClxHidDeviceProtocolMode     protocolMode;
} ClxHidSetProtocolIndication;

/* Data Structure for the indication CLX_HID_DATA_INDICATION */
typedef struct ClxHidDataIndicationStruct
{
    ClxHidReportType     reportType;
    u4                   dataLength;
} ClxHidDataIndication;

/* Data Structure for the indication CLX_HID_INTERRUPT_CONNECTION_REQUEST_INDICATION */
typedef struct ClxHidInterruptConnectionRequestIndicationStruct
{
    ClxDeviceId     deviceId;
} ClxHidInterruptConnectionRequestIndication;

/* Data Structure for the indication CLX_HID_INTERRUPT_DISCONNECTION_REQUEST_INDICATION */
typedef struct ClxHidInterruptDisconnectionRequestIndicationStruct
{
    ClxDeviceId     deviceId;
} ClxHidInterruptDisconnectionRequestIndication;

/* Data Structure for the indication CLX_HID_GET_HOST_DATA_COMPLETE */
typedef struct ClxHidResultHostDataStruct
{
    u2                  dataLength;
    _user_out_  u1*     dataBuffer;
} ClxHidResultHostData;

/* Data Structure for the indication CLX_HID_GET_HOST_REPORT_COMPLETE */
typedef struct ClxHidResultHostReportStruct
{
    u2                  reportLength;
    _user_out_  u1*     reportBuffer;
} ClxHidResultHostReport;

/**
        Request to create the services of HID device role on local machine.

 \param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
 \param[ in ] role the role of HID.
 \param[ in ] deviceConfiguration the configuration list for HID.
 \param[ in ] indicationCallbackFunc A callback function which will be invoked based on the indication events 
              which is related to this handle.
 \return ClxHandle HID handle. This handle is used for any future operations on this instance of HID profile.
*/
ClxHandle clxHidCreate(_in_ ClxStack                        stack,
                       _in_ ClxHidRole                      role,
                       _user_in_ ClxHidDeviceConfiguration* deviceConfiguration,
                       _in_ ClxApplicationCallbackFunc      indicationCallbackFunc);

/**
        Request to establish the control channel connection, which used to exchange the reports #ClxHidReportType between host and device.

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_CONNECT_COMPLETE.
        The indication does not have any parameters.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in  ]  deviceId the remote device Id
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidConnect(_in_ ClxHandle       hidHandle,
                       _in_ ClxDeviceId     deviceId,
                       _in_ boolean         block);

/**
        Request to establish the interrupt channel connection, which used to exchange the data between host and device.

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_CONNECT_INTERRUPT_COMPLETE.
        The indication does not have any parameters.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in  ]  deviceId the remote device Id
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidConnectInterrupt(_in_ ClxHandle       hidHandle,
                                _in_ ClxDeviceId     deviceId,
                                _in_ boolean         block);

/**
        Request to send the input report from device to host.
        This function shall be called once receiving the notification CLX_HID_GET_REPORT_INDICATION from host on the
        call-back function. This indication has the parameter of structure #ClxHidGetReportIndication.
        Based on the indication parameter values, the corresponding report has to be sent.

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_SEND_REPORT_COMPLETE.
        The indication does not have any parameters.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in  ]  reportSize the length of data
 \param[ in  ]  inputReport the buffer which contains the user report.
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidSendReport(_in_ ClxHandle           hidHandle,
                          _in_ u2                  reportSize,
                          _user_in_ u1*            inputReport,
                          _in_ boolean             block);

/**
        Request to send the instantaneous data from device to host.
        This API shall be used to send the data from device to host. For example: mouse movement, keyboard text and etc.,

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_SEND_DATA_COMPLETE.
        The indication does not have any parameters.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in  ]  dataLength the length of data
 \param[ in  ]  inputData the buffer which contains the user data.
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidSendData(_in_ ClxHandle         hidHandle,
                        _in_ u2                dataLength,
                        _user_in_ u1*          inputData,
                        _in_ boolean           block);

/**
        Request to send the current protocol mode from device to host.
        This function shall be called once receiving the notification CLX_HID_GET_PROTOCOL_INDICATION from host on the
        call-back function.

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_SEND_PROTOCOL_COMPLETE.
        The indication does not have any parameters.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in  ]  protocol the buffer which contains the protocol mode used by device.
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidSendProtocol(_in_ ClxHandle                              hidHandle,
                            _in_ ClxHidDeviceProtocolMode               protocol,
                            _in_ boolean                                block);

/**
        Request to send the control operational command #ClxHidControlOperation from device to host.

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_SEND_PROTOCOL_COMPLETE.
        The indication does not have any parameters.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in  ]  controlOperation the control operation #ClxHidControlOperation.
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidSendControlOperation(_in_ ClxHandle                  hidHandle,
                                    _in_ ClxHidControlOperation     controlOperation,
                                    _in_ boolean                    block);

/**
        Request to retrieve the host report.        
        This function shall be called once receiving the notification CLX_HID_SET_REPORT_INDICATION from host on the call-back function.
        This indication has the parameter of structure #ClxHidSetReportIndication.

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_GET_HOST_REPORT_COMPLETE.
        This indication has the parameter of structure #ClxHidResultHostReport.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ out ]  result the user allocated buffer and given to stack. The stack will fill the host report.
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidGetHostReport(_in_ ClxHandle                        hidHandle,
                             _user_out_ ClxHidResultHostReport*    result,
                             _in_ boolean                          block);

/**
        Request to retrieve the host data.
        This function shall be called once receiving the notification CLX_HID_DATA_INDICATION from host on the call-back function.
        The above indications has the parameter of structure #ClxHidDataIndication.

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_GET_HOST_DATA_COMPLETE.
        This indication has the parameter of structure #ClxHidResultHostData.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ out ]  result the user allocated buffer and given to stack. The stack will fill the host data.
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidGetHostData(_in_ ClxHandle                       hidHandle,
                           _user_out_ ClxHidResultHostData*     result,
                           _in_ boolean                         block);

/**
        Request to send the error message from device to host.
        This function shall be called when the device found that the report Id is invalid from the host output report which is
        retrieved from the API #clxHidGetHostReport.

        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_ERROR_SEND_INVALID_REPORT_ID_COMPLETE.
        The indication does not have any parameters.

 \param[ in  ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in  ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                   will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_ERROR_INVALID_REQUEST: command is not supported by HID device

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidErrorSendInvalidReportId(_in_ ClxHandle     hidHandle,
                                        _in_ boolean       block);

/**
        Request to close the HID control channel connection. This API shall be called after #clxHidDisconnectInterrupt.
        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_DISCONNECT_COMPLETE.
        The indication does not have any parameters.

 \param[ in ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                  will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidDisconnect(_in_ ClxHandle     hidHandle,
                          _in_ boolean       block);

/**
        Request to close the HID interrupt channel connection.
        Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type 
        CLX_HID_DISCONNECT_INTERRUPT_COMPLETE.
        The indication does not have any parameters.

 \param[ in ]  hidHandle handle to the HID which is retrieved from #clxHidCreate.
 \param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (either success or fail).
                - FALSE: API will return immediately and then a corresponding API completion event 
                  will be raised upon command completion.
 \return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
*/
ClxError clxHidDisconnectInterrupt(_in_ ClxHandle     hidHandle,
                                   _in_ boolean       block);

#ifdef __cplusplus
}
#endif

#endif  // __Hidc_Api_h__

