#ifndef _HSP_HS_API_H_
#define _HSP_HS_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                HspHs.Api.h
* Description         Declares ClarinoxBlue Headset Profile HS role API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

/**
Opcodes for Command Complete confirmations:
*/

/**
Refer to #clxHspHsConnect function.*/
#define CLX_HSP_HS_CONNECT_COMPLETE                                      0x4800

/**
Refer to #clxHspHsReportSpeakerGain function.*/
#define CLX_HSP_HS_REPORT_SPEAKER_GAIN_COMPLETE                          0x4801

/**
Refer to #clxHspHsReportMicrophoneGain function.*/
#define CLX_HSP_HS_REPORT_MICROPHONE_GAIN_COMPLETE                       0x4802

/**
Refer to #clxHspHsReportButtonPress function.*/
#define CLX_HSP_HS_REPORT_BUTTON_PRESS_COMPLETE                          0x4803

/**
Refer to #clxHspHsDisconnect function.*/
#define CLX_HSP_HS_DISCONNECT_COMPLETE                                   0x4804

/**
Opcodes for indications:
*/

/**
This indication is received when a connection is made to a remote HSP device.
This indication is only received when the connection has been initiated by the
remote device. The parameter of this indication (params argument of the call-back
function) will be of type #CLX_HSP_HS_CONNECTION_INDICATION.
*/
#define CLX_HSP_HS_CONNECTION_INDICATION                                 0x8800

/**
This indication is received when a Ring signal is sent to a remote HSP device.
This indication is only received when the Ring signal Request has been initiated by the
remote device. The parameter of this indication (there are no params arguments for the call-back
function).
*/
#define CLX_HSP_HS_RING_SIGNAL_INDICATION                                0x8801

/**
This indication is received when a Disconnect is made to a remote HSP device.
This indication is only received when the disconnection has been initiated by the
remote device. The parameter of this indication (params argument of the call-back
function) will be of type #CLX_HSP_HS_CONNECTION_INDICATION.
*/
#define CLX_HSP_HS_DISCONNECTION_INDICATION                              0x8802

/**
This indication is received when a Speaker Gain Adjustment signal is sent to a remote HSP device.
This indication is only received when the connection has been initiated by the
remote device. The parameter of this indication (params argument of the call-back
function) will be of type #ClxHspGainReport.
*/
#define CLX_HSP_HS_SPEAKER_GAIN_ADJUSTMENT_REQUEST_INDICATION            0x8803

/**
This indication is received when a Microphone Gain Adjustment signal is sent to a remote HSP device.
This indication is only received when the request has been initiated by the
remote device. The parameter of this indication (params argument of the call-back
function) will be of type #ClxHspGainReport.
*/
#define CLX_HSP_HS_MICROPHONE_GAIN_ADJUSTMENT_REQUEST_INDICATION         0x8804

/**
Creates a Headset service on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\return ClxHandle HSPHS handle. This handle must be used for any further operations on this instance of HSPHS profile.
*/
ClxHandle clxHspHsCreate(_in_ ClxStack stack,
                         _in_ ClxApplicationCallbackFunc callbackFunc);

/**
Establish an instance of headset profile connection to remote HSP device. Pairing with the remote server may be completed before connecting to the remote device. 
If there are no pairing at the time of this API, the authentication process may or may not be executed during this API function execution. 
See GAP API for the details of pairing.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HSP_HS_CONNECT_COMPLETE.
The response does not have any parameters.

\param[ in ] hspHs HspHs profile handle.
\param[ in ] remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHspHsConnect(_in_ ClxHandle hspHs,
                         _in_ ClxDeviceId remoteDeviceId, 
                         _in_ boolean block);

/**
Report speaker volume settings to remote HSP device. 

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HSP_HS_REPORT_SPEAKER_GAIN_COMPLETE.
The response does not have any parameters.

\param[ in ] hspHs HspHs profile handle.
\param[ in ] speakerLevel volume level of the speaker, has to be 0-15, where 0 = Minimum gain and 15 = Maximum gain.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHspHsReportSpeakerGain(_in_ ClxHandle hspHs,
                                   _in_ u1 speakerLevel, 
                                   _in_ boolean block);

/**
Report microphone gain settings to remote HSP device. 

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HSP_HS_REPORT_MICROPHONE_GAIN_COMPLETE.
The response does not have any parameters.

\param[ in ] hspHs HspHs profile handle.
\param[ in ] micLevel gain level of the microphone, has to be 0-15, where 0 = Minimum gain and 15 = Maximum gain.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHspHsReportMicrophoneGain(_in_ ClxHandle hspHs,
                                      _in_ u1 micLevel, 
                                      _in_ boolean block);

/**
Report button press information to remote HSP device. 

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HSP_HS_REPORT_BUTTON_PRESS_COMPLETE.
The response does not have any parameters.

\param[ in ] hspHs HspHs profile handle.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxHspHsReportButtonPress(_in_ ClxHandle hspHs,
                                   _in_ boolean block);

/**
Disconnects from the remote HSPHS connection.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_HSP_HS_DISCONNECT_COMPLETE.
The response does not have any parameters.

\param[ in ]  hspHs HSPHS profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote HSP-AG service.
*/
ClxError clxHspHsDisconnect(_in_ ClxHandle hspHs,
                            _in_ boolean block);


#endif // _HSP_HS_API_H_
