#ifndef _MCE_API_H_
#define _MCE_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Mce.Api.h
* Description         Declares ClarinoxBlue Message Access Profile (MAP) client role API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Refer to #clxMceBind function.
*/
#define CLX_MCE_BIND_COMPLETE                                   0x4B00

/**
Refer to #clxMceConnect function.
*/
#define CLX_MCE_CONNECT_COMPLETE                                0x4B01

/**
Refer to #clxMceSetPath function.
*/
#define CLX_MCE_SET_PATH_COMPLETE                               0x4B02

/**
Refer to #clxMceGetFolderListing function.
*/
#define CLX_MCE_GET_FOLDER_LISTING_COMPLETE                     0x4B03

/**
Refer to #clxMceGetMessageListing function.
*/
#define CLX_MCE_GET_MESSAGE_LISTING_COMPLETE                    0x4B04

/**
Refer to #clxMceGetMessage function.
*/
#define CLX_MCE_GET_MESSAGE_COMPLETE                            0x4B05

/**
Refer to #clxMceSetNotification function.
*/
#define CLX_MCE_SETNOTIFICATION_COMPLETE                        0x4B06

/**
Refer to #clxMceDisconnect function.
*/
#define CLX_MCE_DISCONNECT_COMPLETE                             0x4B07

/**
Received when there is any data received from the remote MAP server.
The parameter of this indication is of the type #ClxMseObjectDataReceptionIndication.

IMPORTANT : The parameter of this indication contains a data buffer which has been allocated by the stack.
When the application is done with this buffer, it shall call #clxMceFreeDataBuffer in order to free this buffer.
Otherwise, a memory leak will occur.
*/
#define CLX_MCE_OBJECT_DATA_RECEPTION_INDICATION                0x8B00

/**
Received when a connection to a remote MAP server fails. This indication is NOT received
when the local client has initiated the disconnection procedure.
The indication does not have any parameters.
*/
#define CLX_MCE_DISCONNECTED_INDICATION                         0x8B01

/**
According to MAP specification, up to 12 MAP server may co-exist on the same device.
*/
#define MAX_NUM_OF_MESSAGE_SERVERS              12

#define CLX_MAP_EMAIL_SUPPORTED                 0x01    /*! The specified remote MAP server supports Email repositories */
#define CLX_MAP_GSM_SMS_SUPPORTED               0x02    /*! The specified remote MAP server supports GSM SMS repositories */
#define CLX_MAP_CDMA_SMS_SUPPORTED              0x04    /*! The specified remote MAP server supports CDMA SMS repositories */
#define CLX_MAP_MMS_SUPPORTED                   0x08    /*! The specified remote MAP server supports MMS repositories */

#define CLX_MAP_MAX_LIST_COUNT                  0x00000001
#define CLX_MAP_START_OFFSET                    0x00000002
#define CLX_MAP_FILTER_MESSAGE_TYPE             0x00000004
#define CLX_MAP_FILTER_PERIOD_BEGIN             0x00000008
#define CLX_MAP_FILTER_PERIOD_END               0x00000010
#define CLX_MAP_FILTER_READ_STATUS              0x00000020
#define CLX_MAP_FILTER_RECIPIENT                0x00000040
#define CLX_MAP_FILTER_ORIGINATOR               0x00000080
#define CLX_MAP_FILTER_PRIORITY                 0x00000100
#define CLX_MAP_ATTACHEMENT                     0x00000200
#define CLX_MAP_TRANSPARENT                     0x00000400
#define CLX_MAP_RETRY                           0x00000800
#define CLX_MAP_NEW_MESSAGE                     0x00001000
#define CLX_MAP_NOTIFICATION_STATUS             0x00002000
#define CLX_MAP_MAS_INSTANCE_ID                 0x00004000
#define CLX_MAP_PARAMETER_MASK                  0x00008000
#define CLX_MAP_FOLDER_LISTING_SIZE             0X00010000
#define CLX_MAP_MESSAGE_LISTING_SIZE            0x00020000
#define CLX_MAP_SUBJECT_LENGTH                  0x00040000
#define CLX_MAP_CHARSET                         0x00080000
#define CLX_MAP_FRACTION_REQUEST                0x00100000
#define CLX_MAP_FRACTION_DELIVER                0X00200000
#define CLX_MAP_STATUS_INDICATOR                0x00400000
#define CLX_MAP_STATUS_VALUE                    0x00800000
#define CLX_MAP_MSE_TIME                        0x01000000

/**
Specifies the possible values for #ClxMessageListingParams.paramFieldsMask parameter.
The values may be combined in order to achieve the desired filter.
*/
enum ClxMceMessageListingMask
{
    ClxMceMessageListingMask_Subject                        = 0x0001,
    ClxMceMessageListingMask_Datetime                       = 0x0002,
    ClxMceMessageListingMask_Sender_name                    = 0x0004,
    ClxMceMessageListingMask_Sender_addressing              = 0x0008,
    ClxMceMessageListingMask_Recipient_name                 = 0x0010,
    ClxMceMessageListingMask_Recipient_addressing           = 0x0020,
    ClxMceMessageListingMask_Type                           = 0x0040,
    ClxMceMessageListingMask_Size                           = 0x0080,
    ClxMceMessageListingMask_Reception_status               = 0x0100,
    ClxMceMessageListingMask_Text                           = 0x0200,
    ClxMceMessageListingMask_Attachment_size                = 0x0400,
    ClxMceMessageListingMask_Priority                       = 0x0800,
    ClxMceMessageListingMask_Read                           = 0x1000,
    ClxMceMessageListingMask_Sent                           = 0x2000,
    ClxMceMessageListingMask_Replyto_addressing             = 0x8000
};

/**
Specifies the possible values for #ClxMessageListingParams.filterMessageType parameter.
The values may be combined in order to achieve the desired filter.
*/
enum ClxMceMessageListingMessageTypes
{
    ClxMceMessageListingMessageTypes_SMS_GSM                = 0x0001,
    ClxMceMessageListingMessageTypes_SMS_CDMA               = 0x0002,
    ClxMceMessageListingMessageTypes_EMAIL                  = 0x0004,
    ClxMceMessageListingMessageTypes_MMS                    = 0x0008
};

/**
An object of type ClxMse represents a single MAP server on a remote machine.
*/
typedef struct ClxMseStruct
{
    u4 serverId;                          /*! The unique ID of the MAP server (only unique for the owning remote device) */
    s1 serverName[33];                    /*! The name of the MAP server as a null-terminated UTF-8 string of the maximum length of 32 bytes */
    u1 supportedMessageTypes;             /*! Supported message types stored in the server. Can be a combination of the following values:
                                            #CLX_MAP_EMAIL_SUPPORTED
                                            #CLX_MAP_GSM_SMS_SUPPORTED
                                            #CLX_MAP_CDMA_SMS_SUPPORTED
                                            #CLX_MAP_MMS_SUPPORTED */
}ClxMse;

/**
An object which holds the list of all MAP servers on a single remote device.
*/
typedef struct ClxMseListStruct
{
    ClxMse     server[MAX_NUM_OF_MESSAGE_SERVERS];     /*! An array of type ClxMse and size #numberOfServers which holds the details of all MAP servers on a remote device */
    ClxSize    numberOfServers;                        /*! Number of the elements valid inside #server array */
}ClxMseList;

/**
The parameter type of #CLX_MCE_BIND_COMPLETE indication.
*/
typedef struct clxMceBindCommandInputStruct
{
    ClxDeviceId deviceId;               /*! The ID of the remote device to which the local MCE service is bound */    
}clxMceBindCommandInput;

/**
The parameter type of #CLX_MCE_BIND_COMPLETE indication.
*/
typedef struct clxMceBindCommandCompleteStruct
{
    _user_out_  ClxMseList* mseList;    /*! A user allocated structure of type #ClxMseList, containing the list of all MAP servers on the remote device */
}clxMceBindCommandComplete;

/**
The parameter type of #CLX_MCE_GET_FOLDER_LISTING_COMPLETE indication.
*/
typedef struct clxMceGetFolderListingCommandCompleteStruct
{
    _user_out_ u4*  folderListingSize;              /*! The user allocated buffer contains the number of folders available in the current path of the remote server. The returned value of this parameter is only valid 
                                                    if the parameter #MaxListCount of the function #clxMceGetFolderListing has been set to 0 */
}clxMceGetFolderListingCommandComplete;

/**
A structure which holds all the parameters and filters which must be applied to the message listing object, as retrieved by #clxMceGetMessageListing.
*/
struct ClxMessageListingParams
{
    u4            paramFieldsMask;        /*! Indicates which parameters of this structure have valid values. Must be a combination of the following values:
                                            #CLX_MAP_MAX_LIST_COUNT
                                            #CLX_MAP_START_OFFSET
                                            #CLX_MAP_FILTER_MESSAGE_TYPE
                                            #CLX_MAP_FILTER_PERIOD_BEGIN
                                            #CLX_MAP_FILTER_PERIOD_END
                                            #CLX_MAP_FILTER_READ_STATUS
                                            #CLX_MAP_FILTER_RECIPIENT
                                            #CLX_MAP_FILTER_ORIGINATOR
                                            #CLX_MAP_FILTER_PRIORITY 
                                            
                                            Set to 0 if none of the parameters have valid values */

    u2            maxListCount;            /*! Indicates the maximum number of messages listed in the Messages-Listing object */
    u2            startOffset;             /*! Indicates the offset of the first entry of the returned MessagesListing object */
    u1            subjectLength;           /*! Indicates the maximum string-length of the "subject" parameter in the entries of the Messages-Listing object. 
                                            The value range shall be 1 to 255 */
    u4            parameterMask;           /*! Indicates the parameters to be contained in the requested Messages-Listing object. May be a combination of the values defined in #ClxMceMessageListingMask.
                                            Set to 0 in order for the message listing object to contain all the defined parameters */
    u1            filterMessageType;       /*! Specifies the type of the messages that are to be returned in the Messages-Listing object. 
                                            May be a combination of the values defined in #ClxMceMessageListingMessageTypes */
    s1*           filterPeriodBegin;       /*! Defined as YYYYMMDDTHHMMSS to specify the date and time of earliest message to be retrieved */ 
    s1*           filterPeriodEnd;         /*! Defined as YYYYMMDDTHHMMSS to specify the date and time of latest message to be retrieved */ 
    u1            filterReadStatus;        /*! Specifies the read status of the messages that are to be returned in the Messages-Listing object.
                                            Must be 1 (unread), 2 (read), or 0 (no filtering is required) */
    s1*           filterRecipient;         /*! used to filter by message-recipient the messages
                                            that are returned in the Messages-Listing object. A related bMessage shall be included
                                            in the listing if the delivered FilterRecipient string matches a sub-string of one of the
                                            vCard attributes "N", "TEL" and "EMAIL" of at least one <bmessage-recipient>
                                            contained by this message. This parameter shall not be used if the MCE intends to
                                            select messages with any recipient. The text string in this application parameter shall be
                                            coded in UTF-8 */
    s1*           filterOriginator;        /*! Used to filter by message-originator the messages
                                            that are returned in the Messages-Listing object. A related bMessage shall be included
                                            in the listing if the delivered FilterOriginator string matches a sub-string of one of the
                                            vCard attributes "N", "TEL" and "EMAIL" of the <bmessage-originator> contained by
                                            this message. This parameter shall not be used if the MCE intends to select messages
                                            with any originator. The text string in this application parameter shall be coded in UTF-8 */
    u1            filterPriority;          /*! Specifies the priority status of the messages that are to be returned in the Messages-Listing object.
                                            Must be 1 (high), 2 (non-high), or 0 (no filtering is required) */
};

/**
Specifies the encoding system required for the body the messages downloaded from the remote MAP server.
*/
enum clxMapCharset
{
    clxMapNative = 0,
    clxMapUTF8   = 1
};

/**
The parameter type of #CLX_MCE_GET_MESSAGE_LISTING_COMPLETE indication.
*/
typedef struct clxMseGetMessageListingCompleteStruct
{
    _user_out_ s1* MSETime;                         /*! Refer to the function #clxMceGetMessageListing for full details */
    _user_out_ u4* MessagesListingSize;             /*! Refer to the function #clxMceGetMessageListing for full details */
}clxMseGetMessageListingComplete;

/**
The parameter type of #CLX_MCE_OBJECT_DATA_RECEPTION_INDICATION indication.
*/
struct ClxMseObjectDataReceptionIndication
{
    u1  serverId;                    /*! The ID of the MAP server from which the data has been received */
    u1* data;                        /*!< A stack-allocated buffer which contains the received data. 
                                          Shall be passed to #clxMceFreeDataBuffer() to be freed. */
    u4  dataLength;                  /*! The length of the data stored in #data, in bytes */
};

/**
The parameter type of #CLX_MCE_DISCONNECTED_INDICATION indication.
*/
struct clxMseDisconnectedIndication
{
    u1 serverId;                    /*! The ID of the MAP server to which the connection has failed */
};

/**
This is a callback function used while receiving data.
*/
typedef void (*ClxMnsServerDataReceivedCallbackFunc) ( const u1*                  dataReceived,
                                                       u4*                        len);
/**
Create MAP Client Equipment (MCE) role service to be able to use the MAP profile as a client.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] dataTransferCallbackFunc call back function of type ClxMnsServerDataReceivedCallbackFunc used to receive data
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\return ClxHandle MAP MCE profile handle. This handle must be used for any further operations on this instance of the MCE service.
*/
ClxHandle clxMceCreate(_in_ ClxStack                             stack, 
                       _in_ ClxMnsServerDataReceivedCallbackFunc dataTransferCallbackFunc,
                       _in_ ClxApplicationCallbackFunc           callbackFunc);

/**
Binds a MCE handle to a remote device. When the local MCE handle is bound to a remote machine, it will be able to connect to one or more
MSE (MAP Server Equipment) service on the remote machine.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_MCE_BIND_COMPLETE.
The indication's parameter will be of type #clxMceBindCommandInput.

\param[ in ] mce MCE profile handle.
\param[ in ] remoteDeviceId the device ID of the remote machine to which this handle is to be bound. This ID is to obtained from GAP API functions.
\param[ out ] remoteServersList A caller-allocated structure of type ClxMseList which, upon completion of this command, will contain the list of all MSE
services (MAP servers) available on the remote machine. The provided structure must not be deleted or accessed until this command is complete (in either
blocking or non-blocking mode).

\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful (At least one MAP server exists on the remote device).
                - CLX_ERROR_INVALID_DEVICE_ID : The provided device ID is invalid.
                - CLX_ERROR_MCE_NO_SERVERS_AVAILABLE : No MAP server is available on the remote device.
                - CLX_ERROR_INVALID_COMMAND_ARGUMENT : The argument remoteServersList is NULL.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS : Another command has been issued to this handle, which has not been completed yet.
                - CLX_ERROR_MCE_BINDING_IN_PROGRESS : Binding procedure to the same remote device, or a different device, is currently in progress.
                - CLX_ERROR_MCE_ALREADY_BOUND : The handle is already bound to the same device, or a different device.
                - CLX_ERROR_MCE_UNBINDING_IN_PROGRESS : Unbinding of this handle from a remote device is currently in progress.

\remark According to Bluetooth MAP specification, a device may have more than one MSE service (MAP server) running at the same time.
A local MCE service is able to connect to more than one MSE service at the same time, as long as all MSE services belong to the same
remote device. Therefore, the local MCE service must be bound to a single remote device before any connection to remote MSE services
can be established.

When successfully bound to a remote machine, the local handle CANNOT be unbound and rebound to the same or a different remote device.
*/
ClxError clxMceBind(_in_        ClxHandle    mce,
                    _in_        ClxDeviceId  remoteDeviceId,
                    _user_out_  ClxMseList*  remoteServersList,
                    _in_        boolean      block);

/**
Connects the local MCE handle to a MSE service (MAP server) on the bound remote machine. The local MCE handle must have already been bound
to a remote device (by calling #clxMceBind function) before this command can be used.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_MCE_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ] mce MCE profile handle.
\param[ in ] serverId The ID of the MAP server as found on the remote machine. This ID must be obtained from the list of MAP servers obtained by #clxMceBind
function.
\param[ in ] securityRequirement The security level required for the MAP connection to the remote device. Refer to GAP documentation for more
information. The recommended value is ClxAutomaticSecurity.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: Operation is successful.
                - CLX_ERROR_CONNECTION_FAILED : An attempt to connect to the remote device failed.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_MCE_INVALID_SERVER_ID : The provided server ID is invalid.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS : Another command has been issued to this handle, which has not been completed yet.
                - CLX_ERROR_MCE_NO_BOUND_DEVICE : The handle is not currently bound to any remote device.
                - CLX_ERROR_MCE_BINDING_IN_PROGRESS : Binding procedure to the same remote device, or a different device, is currently in progress.
                - CLX_ERROR_MCE_UNBINDING_IN_PROGRESS : Unbinding of this handle from a remote device is currently in progress.
                - CLX_ERROR_CONNECTION_EXISTS : A MAP connection already exists to the remote MAP server.
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS : A MAP connection establishment to the remote MAP server is currently in progress.
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS : A disconnection procedure from the remote MAP server is currently in progress.
*/
ClxError clxMceConnect(_in_ ClxHandle              mce,
                       _in_ u1                     serverId,
                       _in_ ClxSecurityRequirement securityRequirement,
                       _in_ boolean                block);

/**
Sets the notification in MSE.
The default mode for the Notification is "off", i.e. the notification is inactive at the MSE if the MAP session has been started.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_MCE_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ] mce MCE profile handle.
\param[ in ] serverId The ID of the MAP server as found on the remote machine. This ID must be obtained from the list of MAP servers obtained by #clxMceBind
function.
\param[ in ] status determines the status of the notification.
                  TRUE: Enables the notification
                  FALSE: Disables the notification.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: Operation is successful.
                - CLX_ERROR_CONNECTION_FAILED : An attempt to connect to the remote device failed.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_MCE_INVALID_SERVER_ID : The provided server ID is invalid.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS : Another command has been issued to this handle, which has not been completed yet.
                - CLX_ERROR_MCE_NO_BOUND_DEVICE : The handle is not currently bound to any remote device.
                - CLX_ERROR_MCE_BINDING_IN_PROGRESS : Binding procedure to the same remote device, or a different device, is currently in progress.
                - CLX_ERROR_MCE_UNBINDING_IN_PROGRESS : Unbinding of this handle from a remote device is currently in progress.
                - CLX_ERROR_CONNECTION_EXISTS : A MAP connection already exists to the remote MAP server.
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS : A MAP connection establishment to the remote MAP server is currently in progress.
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS : A disconnection procedure from the remote MAP server is currently in progress.
*/
ClxError clxMceSetNotification(_in_ ClxHandle mce,
                               _in_ u1        serverId,
                               _in_ boolean   status,
                               _in_ boolean   block);

/**
Retrieves the list of folders which exist in the current path on the specified remote MAP server. The list of folders is a XML document, details of which have been
defined in the following specification:

IrDA Object Exchange Protocol OBEX with Published Errata, Version 1.2, April
1999, Counterpoint Systems Foundry Inc. and Microsoft Corporation

If the request is successful, the XML document data is received as a sequence of #CLX_MCE_OBJECT_DATA_RECEPTION_INDICATION indications, before this command
is complete.

A connection to the remote MAP server must already exist (by calling #clxMceConnect function).

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_MCE_GET_FOLDER_LISTING_COMPLETE.
The indication's parameter will be of type #clxMceGetFolderListingCommandComplete.

\param[ in ] mce MCE profile handle.
\param[ in ] serverId The ID of the MAP server as found on the remote machine. This ID must be obtained from the list of MAP servers obtained by #clxMceBind
function.
\param[ in ] MaxListCount The maximum number of folders to be listed in the resulting XML document. This parameter must be set to 0, if the number of 
folders in the current path of the remote server is to be obtained (and not the actual list of them as a XML document).
\param[ in ] StartOffset The offset (zero-based index) of the first folder to be listed in the resulting XML document.
\param[ out ] folderListingSize (Only in blocking mode) A pointer to a variable of type u4 which, upon completion of the command, will contain the number of
folders available in the current path of the remote server. The returned value of this parameter is only valid if MaxListCount has been set to 0. This parameter is ignored in non-blocking mode and
should be set to NULL.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: Operation is successful.
                - CLX_ERROR_CONNECTION_FAILED : Connection to the remote MAP server failed during the operation.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_MCE_INVALID_SERVER_ID : The provided server ID is invalid.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : An internal error occurred on the server side.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS : Another command has been issued to this handle, which has not been completed yet.
                - CLX_ERROR_MCE_NO_BOUND_DEVICE : The handle is not currently bound to any remote device.
                - CLX_ERROR_MCE_BINDING_IN_PROGRESS : Binding procedure to the same remote device, or a different device, is currently in progress.
                - CLX_ERROR_MCE_UNBINDING_IN_PROGRESS : Unbinding of this handle from a remote device is currently in progress.
                - CLX_ERROR_CONNECTION_NOT_EXIST : No MAP connection exists to the remote MAP server.
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS : A MAP connection establishment to the remote MAP server is currently in progress.
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS : A disconnection procedure from the remote MAP server is currently in progress.
*/
ClxError clxMceGetFolderListing(_in_        ClxHandle mce,
                                _in_        u1        serverId,
                                _in_        u2        MaxListCount,
                                _in_        u2        StartOffset,
                                _user_out_  u4*       folderListingSize,
                                _in_        boolean   block);

/**
Changes the current path on the specified remote MAP server. This function is only able to change the current path one level (up or down) at a time. 
The name of the required folder may be acquired from #clxMceGetFolderListing.

A connection to the remote MAP server must already exist (by calling #clxMceConnect function).

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_MCE_SET_PATH_COMPLETE.
The indication does not have any parameters.

\param[ in ] mce MCE profile handle.
\param[ in ] serverId The ID of the MAP server as found on the remote machine. This ID must be obtained from the list of MAP servers obtained by #clxMceBind
function.
\param[ in ] pathName The name of folder down to which the current path is to be changed. This parameter may only specify one single folder name (e.g. telecom),
and NOT a chain of folder names hierarchy (e.g. telecom/msg). This parameter can be NULL if not required.
\param[ in ] backUpALevel If TRUE, the remote MAP server will change the current folder to one level up, before applying the parameter pathName.
set pathName to NULL if it is only required that the current path on the server go one level back up. 
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: Operation is successful.
                - CLX_ERROR_CONNECTION_FAILED : Connection to the remote MAP server failed during the operation.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_MCE_INVALID_SERVER_ID : The provided server ID is invalid.
                - CLX_ERROR_OBEX_NOT_FOUND : The required path is not found on the remote server.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : An internal error occurred on the server side.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS : Another command has been issued to this handle, which has not been completed yet.
                - CLX_ERROR_MCE_NO_BOUND_DEVICE : The handle is not currently bound to any remote device.
                - CLX_ERROR_MCE_BINDING_IN_PROGRESS : Binding procedure to the same remote device, or a different device, is currently in progress.
                - CLX_ERROR_MCE_UNBINDING_IN_PROGRESS : Unbinding of this handle from a remote device is currently in progress.
                - CLX_ERROR_CONNECTION_NOT_EXIST : No MAP connection exists to the remote MAP server.
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS : A MAP connection establishment to the remote MAP server is currently in progress.
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS : A disconnection procedure from the remote MAP server is currently in progress.

\remark Upon connection to a MAP server, the current path on the server will be the root. If more than one level of path change is required, #clxMceSetPath
must be called more than once.
*/
ClxError clxMceSetPath(_in_ ClxHandle  mce,
                       _in_ u1         serverId,
                       _in_ const s1*  pathName,
                       _in_ boolean    backUpALevel,
                       _in_ boolean    block);

/**
Retrieves the list of messages which exist in the current path on the specified remote MAP server. The list of messages is a XML document, details of which have been
defined in the following specification:

Message Access Profile Specification Version 1.0, June 2009
Copyright (c) Bluetooth Special Interest Group (SIG)

If the request is successful, the XML document data is received as a sequence of #CLX_MCE_OBJECT_DATA_RECEPTION_INDICATION indications, before this command
is complete.

A connection to the remote MAP server must already exist (by calling #clxMceConnect function).

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_MCE_GET_MESSAGE_LISTING_COMPLETE.
The indication's parameter will be of type #clxMseGetMessageListingComplete.

\param[ in ] mce MCE profile handle.
\param[ in ] serverId The ID of the MAP server as found on the remote machine. This ID must be obtained from the list of MAP servers obtained by #clxMceBind
function.
\param[ in ] folderName The name of the folder from which the messages is to be listed. This must be the name of a folder which is a direct child of the current
path on the remote MAP server. Therefore, the name must NOT include any path or hierarchy information.
Set this parameter to an empty string ("") if the list of the messages in the current path is to be retrieved. This parameter CANNOT be NULL.
\param[ in ] parameters A pointer to a structure (of type ClxMessageListingParams) containing the desired parameters to be applied to the resulting message list.
\param[ out ] mseTime (Only in blocking mode) A caller-allocated buffer of the size 21 bytes (or more) which, upon completion of the command, will contain current time
and date of the remote MAP server, as a null-terminated string in the following format:
    
    YYYYMMDDTHHMMSS+hhmm

where "hhmm" refers to the UTC offset. For instance, for Central European Time (CET) the offset is UTC+1h so the delivered value is "YYYYMMDDTHHMMSS+0100". If the UTC offset
is not known by the server, the returned value will be in the following format:

    YYYYMMDDTHHMMSS

This parameter is ignored in non-blocking mode, and should be set to NULL.

\param[ out ] MessagesListingSize (Only in blocking mode) A pointer to a variable of type u4 which, upon completion of the command, will contain the number of
messages fulfilling the required filter (if any) in the specified path of the remote server. This parameter is ignored in non-blocking mode and
should be set to NULL.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: Operation is successful.
                - CLX_ERROR_CONNECTION_FAILED : Connection to the remote MAP server failed during the operation.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_MCE_INVALID_SERVER_ID : The provided server ID is invalid.
                - CLX_ERROR_OBEX_NOT_FOUND : The required path is not found on the remote server.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : An internal error occurred on the server side.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS : Another command has been issued to this handle, which has not been completed yet.
                - CLX_ERROR_MCE_NO_BOUND_DEVICE : The handle is not currently bound to any remote device.
                - CLX_ERROR_MCE_BINDING_IN_PROGRESS : Binding procedure to the same remote device, or a different device, is currently in progress.
                - CLX_ERROR_MCE_UNBINDING_IN_PROGRESS : Unbinding of this handle from a remote device is currently in progress.
                - CLX_ERROR_CONNECTION_NOT_EXIST : No MAP connection exists to the remote MAP server.
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS : A MAP connection establishment to the remote MAP server is currently in progress.
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS : A disconnection procedure from the remote MAP server is currently in progress.
*/
ClxError clxMceGetMessageListing(_in_       ClxHandle                      mce,
                                 _in_       u1                             serverId,
                                 _in_       const s1*                      folderName,
                                 _in_       const ClxMessageListingParams* parameters,
                                 _user_out_ s1*                            mseTime,
                                 _user_out_ u4*                            MessagesListingSize,
                                 _in_       boolean                        block);

/**
Retrieves the message, specified by its message handle, from the current path of the remote MAP server. The message body is in bMessage format, details of which have been provided
in the following specification:

Message Access Profile Specification Version 1.0, June 2009
Copyright (c) Bluetooth Special Interest Group (SIG)


If the request is successful, the message body data is received as a sequence of #CLX_MCE_OBJECT_DATA_RECEPTION_INDICATION indications, before this command
is complete.

A connection to the remote MAP server must already exist (by calling #clxMceConnect function).

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_MCE_GET_MESSAGE_COMPLETE.
The indication does not have any parameters.

\param[ in ] mce MCE profile handle.
\param[ in ] serverId The ID of the MAP server as found on the remote machine. This ID must be obtained from the list of MAP servers obtained by #clxMceBind
function.
\param[ in ] messageHandle The handle of the desired message as a null-terminated string which has been obtained from #clxMceGetMessageListing. 
This parameter CANNOT be NULL.
\param[ in ] receiveAttachements If TRUE, the attachments of the message will also be downloaded along with the message body, before this command is complete.
\param[ in ] messageCharset The character set to which the message body must be encoded before being sent to the local device. It must be a value from #clxMapCharset
enumerator.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: Operation is successful.
                - CLX_ERROR_CONNECTION_FAILED : Connection to the remote MAP server failed during the operation.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_MCE_INVALID_SERVER_ID : The provided server ID is invalid.
                - CLX_ERROR_OBEX_NOT_FOUND : The required path is not found on the remote server.
                - CLX_ERROR_OBEX_INTERNAL_SERVER_ERROR : An internal error occurred on the server side.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS : Another command has been issued to this handle, which has not been completed yet.
                - CLX_ERROR_MCE_NO_BOUND_DEVICE : The handle is not currently bound to any remote device.
                - CLX_ERROR_MCE_BINDING_IN_PROGRESS : Binding procedure to the same remote device, or a different device, is currently in progress.
                - CLX_ERROR_MCE_UNBINDING_IN_PROGRESS : Unbinding of this handle from a remote device is currently in progress.
                - CLX_ERROR_CONNECTION_NOT_EXIST : No MAP connection exists to the remote MAP server.
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS : A MAP connection establishment to the remote MAP server is currently in progress.
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS : A disconnection procedure from the remote MAP server is currently in progress.
*/
ClxError clxMceGetMessage(_in_ ClxHandle     mce,
                          _in_ u1            serverId,
                          _in_ const s1*     messageHandle,
                          _in_ boolean       receiveAttachements,
                          _in_ clxMapCharset messageCharset,
                          _in_ boolean       block);

/**
Disconnects the local MCE handle from a MSE service (MAP server) on the bound remote machine. A connection to the MSE server must already have been
established.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_MCE_DISCONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ] mce MCE profile handle.
\param[ in ] serverId The ID of the MAP server to be disconnected from.
function.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: Operation is successful.
                - CLX_ERROR_INTERNAL_ERROR : An internal error occurred.
                - CLX_ERROR_MCE_INVALID_SERVER_ID : The provided server ID is invalid.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS : Another command has been issued to this handle, which has not been completed yet.
                - CLX_ERROR_MCE_NO_BOUND_DEVICE : The handle is not currently bound to any remote device.
                - CLX_ERROR_MCE_BINDING_IN_PROGRESS : Binding procedure to the same remote device, or a different device, is currently in progress.
                - CLX_ERROR_MCE_UNBINDING_IN_PROGRESS : Unbinding of this handle from a remote device is currently in progress.
                - CLX_ERROR_CONNECTION_NOT_EXIST : No MAP connection exists to the remote MAP server.
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS : A MAP connection establishment to the remote MAP server is currently in progress.
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS : A disconnection procedure from the remote MAP server is currently in progress.

\remark In order to carry out a clean shut-down, all connections to the remote MAP servers should be closed before
the local handle is closed.
*/
ClxError clxMceDisconnect(_in_ ClxHandle mce,
                          _in_ u1        serverId,
                          _in_ boolean   block);


/**
Frees the buffer containing the received data, which has been delivered to the application with an indication of type CLX_MCE_OBJECT_DATA_RECEPTION_INDICATION.

This function MUST be called for EACH indication of type CLX_MCE_OBJECT_DATA_RECEPTION_INDICATION, after the application is done with the buffer. Otherwise, memory leak will occur.

This function may be called inside the indication call-back function, or in another context.

\param[ in ] mce MCE profile handle.
\param[ in ]  arg The data buffer which has been delivered to the application via an indication of CLX_MCE_OBJECT_DATA_RECEPTION_INDICATION.
                  this shall be set to ClxMseObjectDataReceptionIndication.data member of the CLX_PCE_OBJECT_DATA_RECEPTION_INDICATION indication argument (which is of type ClxMseObjectDataReceptionIndication).
*/
void clxMceFreeDataBuffer(_in_ ClxHandle mce,
                          _in_ u1* buffer);



#ifdef __cplusplus
}
#endif

#endif // _MCE_API_H_

