#ifndef __Pan_Api_h__
#define __Pan_Api_h__

/*******************************************************************************
*
* Project             Pan
* File                Pan.Api.h
* Description         Declares API Functions and Definitions For Pan
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Represents Group Ad-Hoc Network user/service
*/
#define CLX_PAN_GROUP_NETWORK_SERVICE                                                       1

/**
Represents Network Access Point service
*/
#define CLX_PAN_NETWORK_ACCESS_POINT_SERVICE                                                2

/**
Represents PAN Client user
*/
#define CLX_PAN_USER_SERVICE                                                                3

/**
Refer to #clxPanCreate function.
*/
#define CLX_PAN_CREATE_COMPLETE                                                             0x6400

/**
Refer to #clxPanConnect function.
*/
#define CLX_PAN_CONNECT_COMPLETE                                                            0x6401

/**
Refer to #clxPanDisconnect function.
*/
#define CLX_PAN_DISCONNECT_COMPLETE                                                         0x6402

/**
The parameter of this indication is of type #clxPanConnectedIndication.
*/
#define CLX_PAN_CONNECTED_INDICATION                                                        0xa400

/**
The parameter of this indication is of type #clxPanDisconnectedIndication.
*/
#define CLX_PAN_DISCONNECTED_INDICATION                                                     0xa401

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- ENUM: ClxPanRole -->
**
** <!-- DESCRIPTION:-->
**
** ClxPanRole defines the possible roles of the PAN user
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef enum ClxPanRoleEnum
{
    ClxPanRole_Pan                   = 0x00,        /*!< PAN User */
    ClxPanRole_GroupAdHocNetwork     = 0x01,        /*!< Group Ad-Hoc Network User, connecting other devices in PAN network */
    ClxPanRole_NetworkAccessPoint    = 0x02         /*!< Network Access Point, has access to external network */
} ClxPanRole;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxPanConnectedIndication -->
**
** <!-- DESCRIPTION:-->
**
** ClxPanConnectedIndication defines the parameters received when a remote device (PAN User) connects.
** Data structure for the indication CLX_PAN_CONNECTED_INDICATION
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxPanConnectedIndicationStruct
{
    ClxResult      result;                      /*!< Result of the connection operation */
    ClxDeviceId    remoteDeviceId;              /*!< Remote Bluetooth device (PAN) ID, which connected to the local device. */
} ClxPanConnectedIndication;

/**
** <!-- ----------------------------------------------------------------- -->
** <!-- STRUCT: ClxPanDisconnectedIndication -->
**
** <!-- DESCRIPTION:-->
**
** ClxPanDisconnectedIndication defines the parameters received when a remote device (PAN User) disconnects.
** Data structure for the indication CLX_PAN_DISCONNECTED_INDICATION
**
**<!-- ----------------------------------------------------------------- -->
*/
typedef struct ClxPanDisconnectedIndicationStruct
{
    ClxDeviceId    remoteDeviceId;              /*!< Remote Bluetooth device (PAN) ID, which connected to the local device. */
} ClxPanDisconnectedIndication;


/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_PAN_CREATE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] stack                   Handle
\param[  in   ] role                    Chosen PAN role (CLX_PAN_GROUP_NETWORK_SERVICE/CLX_PAN_NETWORK_ACCESS_POINT_SERVICE/CLX_PAN_USER_SERVICE)
\param[  in   ] networkAccessInterface  Parameters of the network interface
\param[  in   ] netAccessType           
\param[  in   ] maxNetAccessRate        
\param[  in   ] indicationCallbackFunc  A pointer to the call-back function which will be called upon 
                                        reception of indication events related to this handle.

\return         ClxHandle PAN handle. This handle must be used for any further operations on this instance of PAN profile.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxHandle clxPanCreate(_in_ ClxStack                            stack,
                       _in_ ClxPanRole                          role,
                       _in_ struct ClxNetworkAccessInterface*   networkAccessInterface,
                       _in_ u2                                  netAccessType,
                       _in_ u4                                  maxNetAccessRate,
                       _in_ ClxApplicationCallbackFunc          indicationCallbackFunc);

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_PAN_START_INTERFACE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] pan         Handle
\param[  in   ] configList  List of the configuration parameters set by the application while stack initialization
\param[  in   ] block       Indicates mode of operation:
                            TRUE : Blocking mode
                            FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxPanStartInterface(_in_ ClxHandle                    pan,
                               _user_in_ const ClxConfigList*    configList,
                               _in_ boolean                      block);

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_PAN_CONNECT_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] pan              Handle
\param[  in   ] remoteDeviceId   PAN device to connect to
\param[  in   ] connectToService Chosen PAN service to which the local device to connect to
\param[  in   ] block            Indicates mode of operation:
                                 TRUE : Blocking mode
                                 FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxPanConnect(_in_ ClxHandle      pan,
                        _in_ ClxDeviceId    remoteDeviceId,
                        _in_ u2             connectToService,
                        _in_ boolean        block);

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_PAN_DISCONNECT_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] pan             Handle
\param[  in   ] remoteDeviceId  The connected remote device which required to be disconnected from this PAN user
\param[  in   ] block           Indicates mode of operation:
                                TRUE : Blocking mode
                                FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxPanDisconnect(_in_ ClxHandle    pan,
                           _in_ ClxDeviceId  remoteDeviceId,
                           _in_ boolean      block);

/**

                Blocking mode : This function will not return until the command is complete. The function return value indicates the result of the command

                Non-blocking mode : This function will return immediately with the error code CLX_ERROR_COMPLETION_PENDING.
                When the command is complete, the call-back function will be called with an indication of type #CLX_PAN_STOP_INTERFACE_COMPLETE.
                This indication does not have any parameters.

\param[  in   ] pan    Handle
\param[  in   ] block  Indicates mode of operation:
                       TRUE : Blocking mode
                       FALSE : Non-blocking mode

\return         ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet
                - CLX_ERROR_INVALID_HANDLE: invalid handle

*/
ClxResult clxPanStopInterface(_in_ ClxHandle    pan,
                              _in_ boolean      block);

#ifdef __cplusplus
}
#endif

#endif  // __Pan_Api_h__

