#ifndef _PCE_API_H_
#define _PCE_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Pce.Api.h
* Description         Declares ClarinoxBlue Phone Book Access Profile Client API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Refer to #clxPceConnect function.
*/
#define CLX_PCE_CONNECT_COMPLETE                            0x4A00

/**
Refer to #clxPceSetServerDirectory function.
*/
#define CLX_PCE_SET_SERVER_DIRECTORY_COMPLETE               0x4A01

/**
Refer to #clxPceDownloadPhonebookObject function.
*/
#define CLX_PCE_DOWNLOAD_PHONEBOOK_OBJECT_COMPLETE          0x4A02

/**
Refer to #clxPceDownloadVCardEntry function.
*/
#define CLX_PCE_DOWNLOAD_VCARD_ENTRY_COMPLETE               0x4A03

/**
Refer to #clxPceDownloadVCardListingObject function.
*/
#define CLX_PCE_DOWNLOAD_VCARD_LISTING_OBJECT_COMPLETE      0x4A04

/**
Refer to #clxPceDisconnect function.
*/
#define CLX_PCE_DISCONNECT_COMPLETE                         0x4A05

/**
Refer to #clxPceAbort function.
*/
#define CLX_PCE_ABORT_COMPLETE                              0x4A06

/**
Received when there is any data received from the remote PBAP server.
The parameter of this indication is of the type #ClxPceObjectDataReceptionIndication.

IMPORTANT : The parameter of this indication contains a data buffer which has been allocated by the stack.
When the application is done with this buffer, it shall call #clxPceFreeDataBuffer in order to free this buffer.
Otherwise, a memory leak will occur.
*/
#define CLX_PCE_OBJECT_DATA_RECEPTION_INDICATION            0x8A00

/**
Received when a connection to a remote PBAP server fails. This indication is NOT received
when the local client has initiated the disconnection procedure.
The indication does not have any parameters.
*/
#define CLX_PCE_DISCONNECTED_INDICATION                     0x8A01

/**
Phone Book Access Profile (PBAP) allows exchange of Phone Book Objects between devices.
PCE API implements the Client Equipment role.
*/

/**
Phone Book path enumerations. Telecom addresses the entries on the phone whereas SIM1 addresses the entries on the SIM card.
- pb:  phone book
- ich: incoming calls
- och: outgoing calls
- mch: missed calls
- cch: incoming/outgoing/missed calls
*/
enum ClxPbapPath 
{
    clxPbapRoot, 
    clxPbapTelecom, 
    clxPbapTelecom_pb, 
    clxPbapTelecom_ich, 
    clxPbapTelecom_och, 
    clxPbapTelecom_mch,
    clxPbapTelecom_cch,
    clxPbapSIM1, 
    clxPbapSIM1_telecom, 
    clxPbapSIM1_telecom_pb, 
    clxPbapSIM1_telecom_ich,
    clxPbapSIM1_telecom_och, 
    clxPbapSIM1_telecom_mch, 
    clxPbapSIM1_telecom_cch
};

/**
Phone Book retrieve sorting options
*/
enum ClxPbapListingOrder 
{
    ClxPbapIndexed, 
    ClxPbapAlphabetical, 
    ClxPbapPhonectical
};

/**
Phone Book retrieve attribute options
*/
enum ClxPbapListingSearchAttr 
{
    ClxPbapName, 
    ClxPbapNumber, 
    ClxPbapSound,
    ClxPBapNullAttr
};

/**
Phone Book object types.
*/
enum ClxPbapObject
{
    ClxPbapPhoneBook     = 0,
    ClxPbapIncomingCalls = 1,
    ClxPbapOutgoingCalls = 2,
    ClxPbapMissedCalls   = 3,
    ClxPbapCombination   = 4        /*!< combination of ClxPbapIncomingCalls, ClxPbapOutgoingCalls, and ClxPbapMissedCalls */
};

/**
Phone Book source object source (phone or SIM card)
*/
enum ClxPbapObjectSource
{
    ClxPbapSIM,
    ClxPbapPhone
};

/**
Phone Book filter type bit assignments
*/
#define CLX_PCE_FILTER_VERSION                  0x00000001 /*!< vCard Version */
#define CLX_PCE_FILTER_FN                       0x00000002 /*!< Formatted Name */
#define CLX_PCE_FILTER_N                        0x00000004 /*!< Structured Presentation of Name */
#define CLX_PCE_FILTER_PHOTO                    0x00000008 /*!< Associated Image or Photo */
#define CLX_PCE_FILTER_BDAY                     0x00000010 /*!< Birthday */
#define CLX_PCE_FILTER_ADR                      0x00000020 /*!< Delivery Address */
#define CLX_PCE_FILTER_LABEL                    0x00000040 /*!< Delivery */
#define CLX_PCE_FILTER_TEL                      0x00000080 /*!< Telephone Number */
#define CLX_PCE_FILTER_EMAIL                    0x00000100 /*!< Electronic Mail Address */
#define CLX_PCE_FILTER_MAILER                   0x00000200 /*!< Electronic Mail */
#define CLX_PCE_FILTER_TZ                       0x00000400 /*!< Time Zone */
#define CLX_PCE_FILTER_GEO                      0x00000800 /*!< Geographic Position */
#define CLX_PCE_FILTER_TITLE                    0x00001000 /*!< Job */
#define CLX_PCE_FILTER_ROLE                     0x00002000 /*!< Role within the Organization */
#define CLX_PCE_FILTER_LOGO                     0x00004000 /*!< Organization Logo */
#define CLX_PCE_FILTER_AGENT                    0x00008000 /*!< vCard of Person Representing */
#define CLX_PCE_FILTER_ORG                      0x00010000 /*!< Name of Organization */
#define CLX_PCE_FILTER_NOTE                     0x00020000 /*!< Comments */
#define CLX_PCE_FILTER_REV                      0x00040000 /*!< Revision */
#define CLX_PCE_FILTER_SOUND                    0x00080000 /*!< Pronunciation of Name */
#define CLX_PCE_FILTER_URL                      0x00100000 /*!< Uniform Resource Locator */
#define CLX_PCE_FILTER_UID                      0x00200000 /*!< Unique ID */
#define CLX_PCE_FILTER_KEY                      0x00400000 /*!< Public Encryption Key */
#define CLX_PCE_FILTER_NICKNAME                 0x00800000 /*!< Nickname */
#define CLX_PCE_FILTER_CATEGORIES               0x01000000 /*!< Categories */
#define CLX_PCE_FILTER_PROID                    0x02000000 /*!< Product ID */
#define CLX_PCE_FILTER_CLASS                    0x04000000 /*!< Class information */
#define CLX_PCE_FILTER_SORT_STRING              0x08000000 /*!<  String used for sorting operations */
#define CLX_PCE_FILTER_X_IRMC_CALL_DATETIME     0x10000000 /*!< X-IRMC-CALL-DATETIME Time stamp */
//bit29~38 Reserved for future use
//bit39    Proprietary Filter Indicates the usage of a proprietary filter
//bit40 ~ 63 Reserved for proprietary filter usage

/**
Phone Book download complete indication structure
*/
struct ClxPceDownloadComplete
{
    _user_out_ u2* numberOfEntries; 
    _user_out_ u1* newMissedCalls;
};


#define clxPceDownloadPhoneBookObjectCommandComplete       ClxPceDownloadComplete
#define clxPceDownloadVcardListingObjectCommandComplete    ClxPceDownloadComplete

/**
Phone Book object download indication structure
*/
struct ClxPceObjectDataReceptionIndication
{
    u1* data;               /*!< A stack-allocated buffer which contains the received data. 
                                 Shall be passed to #clxPceFreeDataBuffer() to be freed */
    u4  dataLength;         /*!< The length of data received, in bytes */
};

/**
Create PBAP Client Equipment (PCE) role service to be able to use the PBAP profile as a client.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\return ClxHandle PBAP PCE profile handle. This handle must be used for any further operations on this instance of the PCE service.
*/
ClxHandle clxPceCreate(_in_ ClxStack stack,
                       _in_ ClxApplicationCallbackFunc callbackFunc);

/**
Connect PBAP Client Equipment (PCE) to a PBAP server profile. Pairing with the remote server may be completed before connecting to the server. 
If there are no pairing at the time of this API, the authentication process will be executed during this API. See GAP API for the details of pairing.
\param[ in ]  pce PBAP PCE profile handle.
\param[ in ]  remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation (valid only in blocking mode)
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INTERNAL_ERROR: a stack internal error occurred during connection
*/
ClxError clxPceConnect(_in_ ClxHandle pce,
                       _in_ ClxDeviceId remoteDeviceId, 
                       _in_ boolean block);

/**
Download selected phone book object(s) from the connected PBAP server equipment. This operation can download multiple vCard object at once.
There must be an existing successful connection to the PSE before this operation.
\param[ in ]  pce PBAP PCE profile handle.
\param[ in ]  object PBAP object type to download
\param[ in ]  source PBAP source type to download
\param[ in ]  filter PBAP filter type to download
\param[ in ]  maxNumberOfEntries maximum number of entries to download with this operation
\param[ in ]  startOffset start offset of the object from the beginning (0: means start from the first object)
\param[ in ]  vcardVersion vCard version 
                - 2: vCard version 2.1
                - 3: vCard version 3.0 
\param[ out ] numberOfEntries number of entries downloaded after this operation (valid only in blocking mode)
\param[ out ] newMissedCalls number of new missed calls since the last API call. This is only valid for the missed calls folder (valid only in blocking mode)
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another object transfer is already running
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no PBAP connection
                - CLX_ERROR_INTERNAL_ERROR: a stack internal error occurred during connection
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before it is complete
*/
ClxError clxPceDownloadPhonebookObject(_in_ ClxHandle pce,
                                       _in_ ClxPbapObject object,
                                       _in_ ClxPbapObjectSource source,
                                       _in_ const u1* filter,
                                       _in_ u2 maxNumberOfEntries,
                                       _in_ u2 startOffset,
                                       _in_ u1 vcardVersion,
                                       _user_out_ u2* numberOfEntries,
                                       _user_out_ u1* newMissedCalls,
                                       _in_ boolean block);

/**
Set the server object path for the following operations. Note that clxPceDownloadPhonebookObject API does not require setting the path.
There must be an existing successful connection to the PSE before this operation.
\param[ in ]  pce PBAP PCE profile handle.
\param[ in ]  serverPath PBAP server path for downloading vCard objects
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another object transfer is already running
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no PBAP connection
                - CLX_ERROR_INTERNAL_ERROR: a stack internal error occurred during connection
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before it is complete 
                  (the current path of the server will be unknown until the next time this command is issued).
*/
ClxError clxPceSetServerDirectory(_in_ ClxHandle pce,
                                  _in_ ClxPbapPath serverPath,
                                  _in_ boolean block);

/**
Download a single vCard phone book object from the connected PBAP server equipment. This operation requires a server path to be set beforehand.
There must be an existing successful connection to the PSE before this operation.
\param[ in ]  pce PBAP PCE profile handle.
\param[ in ]  entryIndex index of the object from the beginning (0: means start from the first object)
\param[ in ]  filter PBAP filter type to download
\param[ in ]  vcardVersion vCard version 
                - 2: vCard version 2.1
                - 3: vCard version 3.0 
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another object transfer is already running
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no PBAP connection
                - CLX_ERROR_INTERNAL_ERROR: a stack internal error occurred during connection
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before the entire object is downloaded
*/
ClxError clxPceDownloadVCardEntry(_in_ ClxHandle pce,
                                  _in_ u4 entryIndex,
                                  _in_ const u1* filter,
                                  _in_ u1 vcardVersion,
                                  _in_ boolean block);

/**
Download the list of vCard phone book objects from the connected PBAP server equipment. This operation requires a server path to be set beforehand.
There must be an existing successful connection to the PSE before this operation.
In both blocking and non-blocking modes, the downloaded phone book data will cause an indication call-back with the following arguments;
- messageID: CLX_PCE_OBJECT_DATA_RECEPTION_INDICATION 
- params: pointer to structure of type ClxPceObjectDataReceptionIndication
- errorCode: CLX_SUCCESS
In non-blocking mode, the response will have the following arguments;
- messageID: CLX_PCE_DOWNLOAD_VCARD_LISTING_OBJECT_COMPLETE 
- params: pointer to structure of type clxPceDownloadVCardListingObjectComplete
- errorCode: as described below

\param[ in ]  pce PBAP PCE profile handle.
\param[ in ]  order ordering of the downloaded objects
                - ClxPbapIndexed sorted in increasing handle order.
                - ClxPbapAlphabetical order in alphabetical order.
                - ClxPbapPhonectical order in based on sound attribute.

\param[ in ]  searchAttribute listing search attribute
                - ClxPbapName search based on name.
                - ClxPbapNumber search based on number.
                - ClxPbapSound search based on sound.
                - ClxPBapNullAttr all headers shall be returned.
\param[ in ]  searchValue search value 
\param[ in ]  searchValueLength length of search value (in bytes). If 0 is specified, all vCards shall be returned.
\param[ in ]  maxNumberOfEntries maximum number of entries to download with this operation
\param[ in ]  startOffset start offset of the object from the beginning (0: means start from the first object)
\param[ out ] numberOfEntries number of entries downloaded after this operation (valid only in blocking mode)
\param[ out ] newMissedCalls number of new missed calls since the last API call. This is only valid for the missed calls folder (valid only in blocking mode)
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_OBEX_OBJECT_TRANSFER_IN_PROGRESS: another object transfer is already running
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no PBAP connection
                - CLX_ERROR_INTERNAL_ERROR: a stack internal error occurred during connection
                - CLX_ERROR_OBEX_OPERATION_ABORTED: the operation was aborted before the entire object is downloaded
*/
ClxError clxPceDownloadVCardListingObject(_in_ ClxHandle pce, 
                                          _in_ ClxPbapListingOrder order,
                                          _in_ ClxPbapListingSearchAttr    searchAttribute,
                                          _in_ const u1* searchValue,
                                          _in_ u1 searchValueLength,
                                          _in_ u2 maxNumberOfEntries,
                                          _in_ u2 startOffset,
                                          _user_out_ u2* numberOfEntries,
                                          _user_out_ u1* newMissedCalls,
                                          _in_ boolean block);


/**
Disconnect PBAP Client Equipment (PCE) from the PBAP server. 
There must be an existing successful connection to the PSE before this operation.
In non-blocking mode, the call-back function will have the following arguments;
- messageID: CLX_PCE_DISCONNECT_COMPLETE 
- params: NULL
- errorCode: as described below

\param[ in ]  pce PBAP PCE profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_COMMAND_NOT_COMPLETE the operation is cancelled by the user
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
*/
ClxError clxPceDisconnect(_in_ ClxHandle pce,
                          _in_ boolean block);


/**
Aborts an ongoing download or set-path operation. If there is an any ongoing operation, this
command will abort it. Therefore, the respective command for the aborted operation (#clxPceSetServerDirectory, #clxPceDownloadPhonebookObject,
#clxPceDownloadVCardEntry, or #clxPceDownloadVCardListingObject) will be immediately complete with the error CLX_ERROR_OBEX_OPERATION_ABORTED.

NOTE : This command CANNOT be used to abort #clxPceConnect or #clxPceDisconnect command.

There must be an existing successful connection to the PSE before this operation.
In non-blocking mode, the call-back function will have the following arguments;
- messageID: CLX_PCE_ABORT_COMPLETE 
- params: NULL
- errorCode: as described below

\param[ in ]  pce PBAP PCE profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_COMMAND_NOT_COMPLETE the operation is cancelled by the user
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful (the ongoing operation has been aborted)
                - CLX_ERROR_COMMAND_UNSUCCESSFUL : there is no ongoing operation to be aborted
*/
ClxError clxPceAbort(_in_ ClxHandle pce,
                     _in_ boolean block);


/**
Frees the buffer containing the received data, which has been delivered to the application with an indication of type CLX_PCE_OBJECT_DATA_RECEPTION_INDICATION.

This function MUST be called for EACH indication of type CLX_PCE_OBJECT_DATA_RECEPTION_INDICATION, after the application is done with the buffer. Otherwise, memory leak will occur.

This function may be called inside the indication call-back function, or in another context.

\param[ in ]  pce PBAP PCE profile handle.
\param[ in ]  arg The data buffer which has been delivered to the application via an indication of CLX_PCE_OBJECT_DATA_RECEPTION_INDICATION.
                  this shall be set to ClxPceObjectDataReceptionIndication.data member of the CLX_PCE_OBJECT_DATA_RECEPTION_INDICATION indication argument (which is of type ClxPceObjectDataReceptionIndication).
*/
void clxPceFreeDataBuffer(_in_ ClxHandle pce,
                          _in_ u1* buffer);


#ifdef __cplusplus
}
#endif

#endif // _PCE_API_H_
