#ifndef _SDAP_API_H_
#define _SDAP_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Sdap.Api.h
* Description         Declares ClarinoxBlue Service Discovery Application Profile API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
Refer to #ClxSdapDiscoverRemoteServicesComplete
*/
#define CLX_SDAP_DISCOVER_REMOTE_SERVICES_COMPLETE            0x4600

/**
Determines the type of a service for which information has been provided.
*/
enum ClxBlueServiceType
{
    ClxHspHs = 0,                   /*!< Headset Profile (HSP) Headset role */
    ClxHspAg = 1,                   /*!< Headset Profile (HSP) AudioGateway role */
    ClxHfpHf = 2,                   /*!< Hands-Free Profile (HFP) Hands-Free role */
    ClxHfpAg = 3,                   /*!< Hands-Free Profile (HFP) AudioGateway role */
    ClxSpp = 4,                     /*!< Serial Port Profile (SPP) */
    ClxPce = 5,                     /*!< Phonebook Access Profile (PBAP) Client role */
    ClxPse = 6,                     /*!< Phonebook Access Profile (PBAP) Server role */
    ClxOpp = 7,                     /*!< Object Push Profile (OPP) */
    ClxMns = 8,                     /*!< Message Notification Server (MNS) of Message Access Profile (MAP) */
    ClxMas = 9,                     /*!< Message Access Server (MAS) of Message Access Profile (MAP) */
    ClxSap = 10,                    /*!< SIM Access Profile (SAP) Server role */
    ClxFtpServer = 11,              /*!< File Transfer Profile (FTP) Server Role */
    ClxAvrcpTarget = 12,            /*!< Audio-Video Remote Control Profile (AVRCP) Target role */
    ClxAvrcpController = 13,        /*!< Audio-Video Remote Control Profile (AVRCP) Controller role */
    ClxA2dpSink = 14,               /*!< Audio Advanced Distribution Profile (A2DP) Sink role */
    ClxA2dpSource = 15,             /*!< Audio Advanced Distribution Profile (A2DP) Source role */
    ClxDun = 16,                    /*!< Dialup Networking Profile (DUN) */
    ClxPan = 17,                    /*!< Personal Area Networking Profile (PAN) */
    ClxHid = 18,                    /*!< Human Interface Device Profile (HID) */
    ClxVideoSink = 19,              /*!< Video Distribution Profile (VDP) Sink role */
    ClxVideoSource = 20,            /*!< Video Distribution Profile (VDP) Source role */
    ClxHdpSink = 21,                /*!< Health Device Profile (HDP) Sink role */
    ClxHdpSource = 22,              /*!< Health Device Profile (HDP) Source role */
    ClxSync = 23,                   /*!< Synchronization Profile (SYNC) */
    ClxDeviceID = 24                /*!< DeviceID Profile (DI) */
};

#define CLX_SDAP_FILTER_HSP_HS              0x00000001
#define CLX_SDAP_FILTER_HSP_AG              0x00000002
#define CLX_SDAP_FILTER_HFP_HF              0x00000004
#define CLX_SDAP_FILTER_HFP_AG              0x00000008
#define CLX_SDAP_FILTER_SPP                 0x00000010
#define CLX_SDAP_FILTER_PCE                 0x00000020
#define CLX_SDAP_FILTER_PSE                 0x00000040
#define CLX_SDAP_FILTER_OPP                 0x00000080
#define CLX_SDAP_FILTER_MCE                 0x00000100
#define CLX_SDAP_FILTER_MSE                 0x00000200
#define CLX_SDAP_FILTER_SAP                 0x00000400
#define CLX_SDAP_FILTER_FTP_SERVER          0x00000800
#define CLX_SDAP_FILTER_AVRCP_TARGET        0x00001000
#define CLX_SDAP_FILTER_AVRCP_CONTROLLER    0x00002000
#define CLX_SDAP_FILTER_A2DP_SINK           0x00004000
#define CLX_SDAP_FILTER_A2DP_SOURCE         0x00008000
#define CLX_SDAP_FILTER_DUN                 0x00010000
#define CLX_SDAP_FILTER_PAN                 0x00020000
#define CLX_SDAP_FILTER_HID                 0x00040000
#define CLX_SDAP_FILTER_VIDEO_SINK          0x00080000
#define CLX_SDAP_FILTER_VIDEO_SOURCE        0x00100000
#define CLX_SDAP_FILTER_HDP_SINK            0x00200000
#define CLX_SDAP_FILTER_HDP_SOURCE          0x00400000
#define CLX_SDAP_FILTER_SYNC                0x00800000
#define CLX_SDAP_FILTER_DI                  0x01000000

#define CLX_SDAP_FILTER_ALL                 0x01FFFFFF

/**
Contains information on a single service which has been discovered on a remote device.
*/
struct ClxBlueServiceInfo
{
    enum ClxBlueServiceType       type;                 /*!< The type of the service for which the following information has been provided */
    u1                            serverChannelId;      /*!< The RFCOMM server channel ID (only valid for RFCOMM-based services) */
    u2                            version;              /*!< The version of the service implemented on the remote device */
    s1                            name[33];             /*!< The name of the service as a UTF-8 null-terminated string */
    u1                            availability;         /*!< A value of 0 indicates the service will not accept new clients */
};

/**
The parameter type of the indication #CLX_SDAP_DISCOVER_REMOTE_SERVICES_COMPLETE.
*/
struct ClxSdapDiscoverRemoteServicesComplete
{
    _user_out_ struct ClxBlueServiceInfo*   serviceInfoList;            /*!< Pointer to the user allocated array of discovered services on the remote device */
    _user_out_ ClxSize*                     numberOfDiscoveredServices; /*!< The user provided pointer, which holds the number of discovered services */
};

/**
Creates an instance of SDAP profile.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\return A handle to the SDAP service. This handle must be used for any further operations on this instance of SDAP profile.
*/
ClxHandle clxSdapCreate(_in_ ClxStack                   stack, 
                        _in_ ClxApplicationCallbackFunc callbackFunc);

/**
Performs a service discovery procedure.

Non-blocking mode: When the command is complete, the call-back function will be called with an indication of type #CLX_SDAP_DISCOVER_REMOTE_SERVICES_COMPLETE.
The indication parameter will be of type #ClxSdapDiscoverRemoteServicesComplete.

\param[ in ]  sdap the handle to SDAP service.
\param[ in ]  remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ out ] serviceInfoList A caller-allocated buffer for SDAP to write the details of discovered services. The size of the array is determined by
              serviceInfoListSize argument. Note that regardless of the mode of operation (blocking or non-blocking), this buffer MUST be provided
              by the user. Stack will never provide this buffer. This argument CANNOT be NULL.
\param[ in ]  serviceInfoListSize the size of the array provided by the user in serviceInfoList argument. This argument CANNOT be 0.
\param[ out ] numberOfDiscoveredServices A caller allocated buffer for storing the actual number of discovered services regardless of the more of 
              the operation (blocking or non-blocking).
\param[ in ]  serviceTypeFilter A bit-wise flag containing the list of services the user is interested in and is to be searched for in the remote device.
              This flag CANNOT be 0. Use the special value CLX_SDAP_FILTER_ALL to search for all services.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid.
                - CLX_ERROR_SERVICE_DISCOVERY_FAILED: the service discovery procedure failed
                - CLX_ERROR_SERVICE_DISCOVERY_IN_PROGRESS: The service discovery procedure is currently in progress with the same remote device.
                - CLX_ERROR_INVALID_COMMAND_ARGUMENT: either serviceInfoList is NULL or serviceInfoListSize is 0.
*/
ClxError clxSdapDiscoverRemoteServices(_in_         ClxHandle                   sdap,
                                       _in_         ClxDeviceId                 remoteDeviceId,
                                       _user_out_   struct ClxBlueServiceInfo*  serviceInfoList,
                                       _in_         ClxSize                     serviceInfoListSize,
                                       _user_out_   ClxSize*                    numberOfDiscoveredServices,
                                       _in_         u4                          serviceTypeFilter,
                                       _in_         boolean                     block);

#ifdef __cplusplus
}
#endif


#endif // _SDAP_API_H_

