#ifndef _SPP_API_H_
#define _SPP_API_H_

/*******************************************************************************
*
* Project             ClarinoxBlue
* File                Spp.Api.h
* Description         Declares ClarinoxBlue Serial Port Profile API
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*
Opcodes for Command Complete confirmations:
*/

/**
Refer to #clxSppConnect function.
*/
#define CLX_SPP_CONNECT_COMPLETE                            0x4700

/**
Refer to #clxSppDiscoverRemoteComPorts function.
*/
#define CLX_SPP_DISCOVER_REMOTE_COM_PORTS_COMPLETE          0x4701

/**
Refer to #clxSppDisconnect function.
*/
#define CLX_SPP_DISCONNECT_COMPLETE                         0x4702

/**
Refer to #clxSppSend function.
*/
#define CLX_SPP_SEND_COMPLETE                               0x4703

/**
Refer to #clxSppReceive function.
*/
#define CLX_SPP_RECEIVE_COMPLETE                            0x4704

/**
Refer to #clxSppNegotiatePortParameters function.
*/
#define CLX_SPP_NEGOTIATE_PORT_PARAMETERS_COMPLETE          0x4705

/**
Refer to #clxSppAcceptRemotePortNegotiation function.
*/
#define CLX_SPP_ACCEPT_REMOTE_PORT_NEGOTIATION_COMPLETE     0x4706

/**
Refer to #clxSppRejectRemotePortNegotiation function.
*/
#define CLX_SPP_REJECT_REMOTE_PORT_NEGOTIATION_COMPLETE     0x4707

/**
Refer to #clxSppReportLineStatus function.
*/
#define CLX_SPP_REPORT_LINE_STATUS_COMPLETE                 0x4708

/**
Refer to #clxSppSendTestMessage function.
*/
#define CLX_SPP_SEND_TEST_MESSAGE_COMPLETE                  0x4709


/*
Opcodes for indications:
*/

/**
This indication is received when a connection is made to a remote SPP device.
This indication is only received when the connection has been initiated by the
remote device. The parameter of this indication (params argument of the call-back
function) will be of type #ClxSppConnectionIndication.
*/
#define CLX_SPP_CONNECTION_INDICATION                       0x8700

/**
This indication is received when a disconnection is made to a remote SPP device.
This indication is only received when the disconnection has been initiated by the
remote device. The parameter of this indication (params argument of the call-back
function) will be of type #ClxSppDisconnectionIndication.
*/
#define CLX_SPP_DISCONNECTION_INDICATION                    0x8701

#define CLX_SPP_REMOTE_PORT_NEGOTIATION_INDICATION          0x8702
#define CLX_SPP_LINE_STATUS_INDICATION                      0x8703

#define CLX_SPP_PORT_NAME_MAX_LENGTH                        40

struct ClxSppConnectionIndication
{
    ClxDeviceId deviceId;                                            /*!< The device ID of the remote device to which the connection has been established */
};

struct ClxSppDisconnectionIndication
{
    ClxDeviceId deviceId;                                            /*!< The device ID of the remote device which is disconnected */
};


enum ClxComPortBitRate
{
    ClxBitRate_2400         = 0x00,                                  /*!< 2400 bits per second */
    ClxBitRate_4800         = 0x01,                                  /*!< 4800 bits per second */
    ClxBitRate_7200         = 0x02,                                  /*!< 7200 bits per second */
    ClxBitRate_9600         = 0x03,                                  /*!< 9600 bits per second */
    ClxBitRate_19200        = 0x04,                                  /*!< 19200 bits per second */
    ClxBitRate_38400        = 0x05,                                  /*!< 38400 bits per second */
    ClxBitRate_57600        = 0x06,                                  /*!< 57600 bits per second */
    ClxBitRate_115200       = 0x07,                                  /*!< 115200 bits per second */
    ClxBitRate_230400       = 0x08                                   /*!< 230400 bits per second */
};


enum ClxComPortDataBits
{
    ClxDataBits_5           = 0x00,                                  /*!< 5 bits data words */
    ClxDataBits_6           = 0x02,                                  /*!< 6 bits data words */
    ClxDataBits_7           = 0x01,                                  /*!< 7 bits data words */
    ClxDataBits_8           = 0x03                                   /*!< 8 bits data words (default) */
};

enum ClxComPortStopBits
{
    ClxStopBits_1           = 0x00,                                  /*!< 1 stop bit (default) */
    ClxStopBits_1_5         = 0x01                                   /*!< 1.5 stop bits */    
};

enum ClxComPortParity
{
    ClxParityDisabled       = 0x00,                                  /*!< parity is not used (default) */
    ClxParityEnabled        = 0x01                                   /*!< parity is used */
};


enum ClxComPortParityType
{
    ClxOddParity            = 0x00,                                  /*!< odd parity is to be used */
    ClxEvenParity           = 0x02,                                  /*!< event parity is to be used */
    ClxMarkParity           = 0x01,                                  /*!< mark parity is to be used */
    ClxSpaceParity          = 0x03                                   /*!< space parity is to be used */
};


struct ClxComPortParameters
{
    enum ClxComPortBitRate     bitRate;
    enum ClxComPortDataBits    dataBits;
    enum ClxComPortStopBits    stopBit;
    enum ClxComPortParity      parity;
    enum ClxComPortParityType  parityType;

    boolean                    XON_XOFF_Input;
    boolean                    XON_XOFF_Output;
    boolean                    RTR_Input;
    boolean                    RTR_Output;
    boolean                    RTC_Input;
    boolean                    RTC_Output;

    s1                         XON_Character;
    s1                         XOFF_Character; 
};


struct ClxComPortParametersMask
{
    boolean bitRate;
    boolean dataBits;
    boolean stopBit;
    boolean parity;
    boolean parityType;

    boolean XON_XOFF_Input;
    boolean XON_XOFF_Output;
    boolean RTR_Input;
    boolean RTR_Output;
    boolean RTC_Input;
    boolean RTC_Output;

    boolean xOnCharacter;
    boolean xOffCharacter; 
};


enum ClxComPortLineStatus
{
    ClxComPortNoError      = 0x00,                                   /*!< No error has happened */
    ClxComPortOverrunError = 0x03,                                   /*!< Received character overwrote an unread character */
    ClxComPortParityError  = 0x05,                                   /*!< Received characters parity was incorrect */
    ClxComPortFramingError = 0x09                                    /*!< A character did not terminate with a stop bit */
};


struct clxSppRemotePortNegotiationIndication
{
    struct ClxComPortParameters params;                              /*!< COM port parameters suggested by the remote device which need to be negotiated */ 
    struct ClxComPortParametersMask validParams;                     /*!< Indicates which parameters inside param argument have valid values and need to be negotiated */
};


#define ClxSppNegotiatePortParametersComplete  ClxComPortParametersMask
#define ClxSppLineStatusIndication             ClxComPortLineStatus


struct ClxRemoteComPort
{
    ClxDeviceId   deviceId;                                          /*!< remote com port's deviceId */
    u1            portChannelId;                                     /*!< remote com port's channelId; required during connecting to remote SPP service */
    s1            portName[CLX_SPP_PORT_NAME_MAX_LENGTH + 1];        /*!< remote com port's name as a null-terminated UTF8 string */
};


struct ClxSppDiscoverRemoteComPortsComplete
{
    _user_out_ struct ClxRemoteComPort* comPortsList;                /*!< Pointer to the user-allocated array of type ClxRemoteComPort, which contains the details of 0 or more discovered COMM ports on the remote machine */
    _user_out_ u4*                      numberOfDiscoveredCommPorts; /*!< Pointer to a user-provided variable containing the total number of discovered COM ports on the remote machine. This value could be larger than the maximum size #comPortsList */
};

struct ClxSppReceiveComplete
{
    _user_out_ u1*      buffer;                                      /*!< A pointer to the user-allocated buffer to hold the received data */
    _user_out_ ClxSize* receivedDataLength;                          /*!< Pointer to a user-provided variable containing the length of the data received */
};



/**
Creates a serial port service on the local machine.
\param[ in ] stack Bluetooth stack handle. A stack object must be created before any other profiles are created.
\param[ in ] serviceUUID universal unique Identifier 128bit. This argument may be NULL. If so, the standard Bluetooth SPP identifier
will be used.
\param[ in ] serviceName name of the service. Name can be either in the form of "name" or "name:description". In the latter form the description
will be presented as the service description part of the SPP profile's SDP record. The name will form the service name element of the SDP service record.
Total length of the name + ':' description must be less than or equal to 41 characters (including null termination and :). 
Alternatively if only name is used, then the "name" can be as long as 40 characters plus null termination (total of 41 bytes). API will limit the total to
40 characters if user provides a larger buffer.
\param[ in ] serviceType type of service. client(Dev A) or server(Dev B)
\param[ in ] callbackFunc A pointer to the call-back function which will be called upon reception of indication events related to this handle.
\param[ in ] securityRequirement level of security needed for authentication.
\return ClxHandle serial port profile handle. This handle must be used for any further operations on this instance of SPP profile.
*/
ClxHandle clxSppCreate(_in_ ClxStack stack, 
                       /*@null@*/ _in_ const u1* serviceUUID,
                       _in_ const s1* serviceName,
                       _in_ enum ClxServiceType serviceType,
                       _in_ ClxApplicationCallbackFunc callbackFunc,
                       _in_ enum ClxSecurityRequirement securityRequirement);    // security level needed for authentication.

/**
Discovers a list of SPP ports on the remote device. This command is available only if the local service is a client (Dev A).

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_SPP_DISCOVER_REMOTE_COM_PORTS_COMPLETE.
The indication parameter is of type #ClxSppDiscoverRemoteComPortsComplete.

\param[ in ]  spp  serial port profile handle.
\param[ in ]  uuid  customer UUID for searching on the remote device. If this is 0, then the default Bluetooth SIG serial port profile will be searched
\param[ in ]  remoteDeviceId  remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ]  comPortsListSize The number of elements of the caller-defined COM port list array. This value CANNOT be 0.
\param[ out ]  comPortsList A caller-allocated array of type #ClxRemoteComPort and the size 'comPortsListSize', which on the completion of the command will contain
the list of details of discovered COM ports. If the number of discovered COM ports is more than 'comPortsListSize', then only the details of the first 'comPortsListSize' discovered COM ports
will be returned. The argument CANNOT be NULL.
\param[ out ] numberOfDiscoveredCommPorts A pointer to a caller-provided variable which on the completion of the command will contain the total number of discovered COM ports on the remote 
device. If this value is larger than 'comPortsListSize', then only the details of the first 'comPortsListSize' discovered COM ports have been returned in 'comPortsList'. This argument CANNOT be NULL.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                In case of blocking mode the following error codes may also be returned.
                - CLX_SUCCESS: operation is successful
*/
ClxError clxSppDiscoverRemoteComPorts(_in_ ClxHandle spp,
                                      /*@null@*/ _in_ const u1* uuid,
                                      _in_ ClxDeviceId remoteDeviceId,
                                      _in_ u4 comPortsListSize,
                                      _user_out_ struct ClxRemoteComPort* comPortsList,
                                      _user_out_ u4* numberOfDiscoveredCommPorts,
                                      _in_ boolean block);

/**
Establish an instance of virtual serial connection to remote SPP device. Pairing with the remote server may be completed before connecting to the server(Dev B). 
If there are no pairing at the time of this API, the authentication process will be executed during this API. See GAP API for the details of pairing.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_SPP_CONNECT_COMPLETE.
The response does not have any parameters. If multiple SPP services are used simultaneously, having a separate call-back function for each of
these connections is necessary to be able to identify connection/disconnection of each SPP entity.

The following example code shows how to connect to a remote device;

-	First create a client handle;
\code
    ClxHandle sppHandle = clxSppCreate(stack, NULL, "Port1", ClxClient, stackMessageHandler, ClxMediumSecurity);
\endcode

-	Then discover remote SPP ports
\code
    // Discover remote comm ports via SDP operation, if a suitable SPP port is found then connect to that port by using the parameters retrieved via SDP operation
    // This operation is only applicable to client side, that's why you need a client SPP handle (ClxClient)

    ClxRemoteComPort ports[12];
    u4 numberOfDiscoveredPorts = 0;

    ret = clxSppDiscoverRemoteComPorts(sppHandle, 
                                        NULL, 
                                        connectedDeviceId, 
                                        sizeof(ports)/sizeof(ClxRemoteComPort),
                                        ports, 
                                        &numberOfDiscoveredPorts,
                                        TRUE);
        
    if (ret != CLX_SUCCESS)
    {
        ClxConsoleUIEngine::text ("\nError : clxSppDiscoverRemoteComPorts failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
    }
\endcode

-	If there are any SPP ports discovered on the remote device, then connect to the one that you like (or connect to the first one as in the example below);
\code
    if (numberOfDiscoveredPorts)
    {
        ClxConsoleUIEngine::text ("Connecting to COM port \"%s\" on the remote device . . .\n", ports[0].portName);
        // the SPP connection based on a RFCOMM channel will be made by the following code. If the SDP is disconnected and ACL connection is no longer available
        // then a new ACL connection will be made automatically
        ret = clxSppConnect(sppHandle, connectedDeviceId, ports[0].portChannelId, TRUE);
        if (ret != CLX_SUCCESS)
        {
            ClxConsoleUIEngine::text("SPP connection failed\n");
            ClxConsoleUIEngine::text ("\nError : clxSppConnect failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
        }
        else
        {
            // Now there is a connection, hence you can do whatever you want, maybe stop being discoverable and connectable, so that
            // The data rate will be much higher. You could start asynchronous receive operation as well
            clxGapSetConnectability(stack, FALSE, FALSE);
            clxGapSetDiscoverability(stack, FALSE, FALSE);
            ret = clxSppReceive(spp_, RX_BUFFER_SIZE, sppReceive, &sppReceiveDataSize, FALSE);
            if (ret != CLX_SUCCESS && ret != CLX_ERROR_COMPLETION_PENDING)
            {
                ClxConsoleUIEngine::text("SPP buffer receive failed\n");
                ClxConsoleUIEngine::text ("\nError : clxSppReceive failed with error: %d (%s)\n", ret, clxGetErrorCodeText(ret));
            }
        }
    }
    else
    {
        // No SPP ports found on the remote device
    }
\endcode

\param[ in ] spp SPP profile handle.
\param[ in ] remoteDeviceId remote Bluetooth device id. This id must be obtained via GAP APIs.
\param[ in ] portChannelId port channel identifier. This identifier is obtained from function #clxSppDiscoverRemoteComPorts().
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_UNSUCCESSFUL: operation is unsuccessful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_EXISTS: there is a connection to the same profile of the same device
                - CLX_ERROR_INVALID_DEVICE_ID: the provided device ID is invalid
*/
ClxError clxSppConnect(_in_ ClxHandle spp,
                       _in_ ClxDeviceId remoteDeviceId,
                       _in_ u1 portChannelId,
                       _in_ boolean block);

/**
Disconnects from the remote virtual serial connection.

Non-blocking mode : When the command is complete, the call-back function will be called with a response of type #CLX_SPP_DISCONNECT_COMPLETE.
The response does not have any parameters. If multiple SPP services are used simultaneously, having a separate call-back function for each of
these connections is necessary to be able to identify connection/disconnection of each SPP entity.

\param[ in ]  spp SPP profile handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote SPP service.
*/
ClxError clxSppDisconnect(_in_ ClxHandle spp,
                          _in_ boolean block);


/**
Start sending data to the remote server.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_SPP_SEND_COMPLETE.
The indication does not have any parameters. Until receipt of this event, the data buffer must not be deleted or used for other purposes.

\param[ in ] spp SPP profile handle.
\param[ in ] data pointer to the data buffer.
\param[ in ] dataLength size of the send/transmitting data buffer.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote SPP service.
*/
ClxError clxSppSend(_in_ ClxHandle spp,
                    _in_ const u1* data,
                    _in_ ClxSize dataLength,
                    _in_ boolean block);

/**
Start receiving data from the remote client.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_SPP_RECEIVE_COMPLETE.
The indication has parameters of type ClxSppReceiveComplete.

\param[ in ]  spp SPP profile handle.
\param[ in ]  bufferLength size of the user allocated data buffer.
\param[ out ] buffer user allocated buffer. The user may not delete or access this buffer until this command is complete. This argument CANNOT be NULL.
\param[ out ] receivedDataLength A pointer to a caller-provided variable which on the completion of the command will contain the length of data received and stored in "buffer". 
The value may range between 0 (in case of an error) and bufferLength. This argument CANNOT be NULL.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful, or disconnection has occurred (in the case of the latter, the buffer length will be 0)
                - CLX_ERROR_CONNECTION_ESTABLISHMENT_IN_PROGRESS: another connection is already being established
                - CLX_ERROR_DISCONNECTION_IN_PROGRESS: the local service is disconnecting from the remote service. 
                - CLX_ERROR_CONNECTION_NOT_EXIST: there is no connection to a remote SPP service.
*/
ClxError clxSppReceive(_in_ ClxHandle spp,
                       _in_ ClxSize bufferLength,
                       _user_out_ u1* buffer,
                       _user_out_ ClxSize* receivedDataLength,
                       _in_ boolean block);



ClxError clxSppNegotiatePortParameters(_in_ ClxHandle spp,
                                       _in_ const struct ClxComPortParameters* params,
                                       _in_ const struct ClxComPortParametersMask* validParams,
                                       _user_out_ struct ClxComPortParametersMask* acceptedParams,    
                                       _in_ boolean block);


ClxError clxSppAcceptRemotePortNegotiation(_in_ ClxHandle spp,
                                           _in_ boolean block);

ClxError clxSppRejectRemotePortNegotiation(_in_ ClxHandle spp,
                                           _in_ const struct ClxComPortParametersMask* acceptedParams,
                                           _in_ boolean block);

ClxError clxSppReportLineStatus(_in_ ClxHandle spp,
                                _in_ enum ClxComPortLineStatus lineStatus,
                                _in_ boolean block);

ClxError clxSppSendTestMessage(_in_ ClxHandle spp,
                               _in_ const s1* textMessage,
                               _in_ u4 timeout,                // The time period allowed for clxSppSendTestMessage() to wait before the response to the test message is received.     
                               _in_ boolean block);



#ifdef __cplusplus
}
#endif


#endif // _SPP_API_H_
