#ifndef _WLAN_API_H_
#define _WLAN_API_H_

/*******************************************************************************
*
* Project             ClarinoxWLAN
* File                Wlan.Api.h
* Description         ClarinoxWlan API functions
*
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif


/* Opcodes for Command Complete confirmations: */

/**
Please refer to #clxWlanStartInterface function.
*/
#define CLX_WLAN_START_INTERFACE_COMPLETE                     0x7800

/**
Please refer to #clxWlanStopInterface function.
*/
#define CLX_WLAN_STOP_INTERFACE_COMPLETE                      0x7801

/**
Please refer to #clxWlanScan function.
*/
#define CLX_WLAN_SCAN_COMPLETE                                0x7802

/**
Please refer to #clxWlanGetAvailableBssList function.
*/
#define CLX_WLAN_GET_AVAILABLE_BSSLIST_COMPLETE               0x7803

/**
Please refer to #clxWlanConnect function.
*/
#define CLX_WLAN_CONNECT_COMPLETE                             0x7804

/**
Please refer to #clxWlanDisconnect function.
*/
#define CLX_WLAN_DISCONNECT_COMPLETE                          0x7805

/**
Please refer to #clxWlanInterfaceGetMacAddr function.
*/
#define CLX_WLAN_GET_MAC_ADDR_COMPLETE                        0x7806

/**
Please refer to #clxWlanGetParametersValue function.
*/
#define CLX_WLAN_GET_PARAM_VALUE_COMPLETE                     0x7807

/**
Please refer to #clxWlanSetParametersValue function.
*/
#define CLX_WLAN_SET_PARAM_VALUE_COMPLETE                     0x7808

/**
Please refer to #clxWlanProbe function.
*/
#define CLX_WLAN_PROBE_COMPLETE                               0x7809

/**
Please refer to #clxWlanStartRole function.
*/
#define CLX_WLAN_START_ROLE_COMPLETE                          0x780A

/**
Please refer to #clxWlanStopRole function.
*/
#define CLX_WLAN_STOP_ROLE_COMPLETE                           0x780B

/**
Please refer to #clxWlanP2PEnterPincode function.
*/
#define CLX_WLAN_WPS_ENTER_PIN_CODE_COMPLETE                  0x780C

/**
Please refer to #clxWlanWpsPushButtonPressed function.
*/
#define CLX_WLAN_WPS_PUSH_BUTTON_PRESSED_COMPLETE             0x780D


/**
This indication is received when the initialization of the stack is
complete either with success or in error.

The argument errorCode of the call-back function will be one of the following:
 
 - CLX_SUCCESS : the stack is successfully initialized and ready to use.
 - CLX_WLAN_FAILED : the stack could not be initialized. Although the
   stack is not initialized, but still the stack object has been created (and received
   as the argument stack of the call-back function). Therefore, the object still needs
   to be destroyed by calling #clxDestroyClarinoxWlan (or the equivalent in the target platform). 
   Please note that, in this case, the function #clxTerminateClarinoxWlan (or the equivalent in the target platform)
   MUST NOT be called since the stack has never been initialized, hence no need for termination.

This indication does not have any parameter (params argument of the call-back function
will be NULL).
*/
#define CLX_WLAN_STACK_INITIALIZATION_COMPLETE_INDICATION     0x7700

/**
This indication is received when the local application has requested for the stack to terminate, and the first phase of termination is complete.
Upon reception of this indication, the stack MUST be destroyed by calling the function provided to destroy the stack (this function may be different
for different platforms). Please note that, the stack CANNOT be destroyed within the context of the call-back function thread.

This indication does not have any parameter (params argument of the call-back function
will be NULL). Also, all other arguments of the call-back function (except for indicationID) will have INVALID values.
*/
#define CLX_WLAN_STACK_TERMINATION_COMPLETE_INDICATION        0x7701

/**
This indication is sent to the application when a station has joined the local virtual AP.
The parameter for this indication is of type #ClxWlanStationJoined.
*/
#define CLX_WLAN_NEW_STATION_JOINED_INDICATION                0xB801

/**
This indication is sent to the application when a station has disconnected from the local virtual AP.
The parameter for this indication is of type #ClxWlanStationDisconnected.
*/
#define CLX_WLAN_STATION_DISCONNECTED_INDICATION              0xB802

/**
This indication is sent to the application when authentication initiated by station.
The parameter for this indication is of type #ClxWlanStationAuthInitiated.
*/
#define CLX_WLAN_STATION_AUTHENTICATION_INITIATED_INDICATION  0xB803

/**
This indication is sent to the application when authentication completed by station.
The parameter for this indication is of type #ClxWlanStationAuthAssociationCompleted.
*/
#define CLX_WLAN_STATION_AUTHENTICATION_COMPLETED_INDICATION  0xB804

/**
This indication is sent to the application when association initiated by station.
The parameter for this indication is of type #ClxWlanStationAssociationInitiated.
*/
#define CLX_WLAN_STATION_ASSOCIATION_INITIATED_INDICATION     0xB805

/**
This indication is sent to the application when association completed by station.
The parameter for this indication is of type #ClxWlanStationAuthAssociationCompleted.
*/
#define CLX_WLAN_STATION_ASSOCIATION_COMPLETED_INDICATION     0xB806

/**
The indication is sent to the application when a fatal hardware error has occurred and the application needs to start recovery procedure.
This indication does not belong to a specific virtual interface, but to all initialized virtual interfaces present in the application.
Therefore, the recovery procedure MUST be performed on all virtual interfaces. The actual reason of this indication is hardware-specific.

Upon reception of this indication, the application MUST stop all virtual interfaces which are open in the current session of the stack. In order
to stop an interface, the function clxWlanStopInterface() must be called. Please note that,
during recovery procedures, the function clxWlanStopInterface() will not try to gracefully terminate the links, and the ongoing tasks in the stack since
the hardware is in a instable or failure state. In recovery mode, all links and tasks are forcefully terminated, without any further communication with the hardware.

IMPORTANT : After stopping all interfaces, the hardware must be power-cycled. This is done either automatically by the driver when the interfaces are re-started or must be done manually 
by the application. Please refer to the documentation of the driver being used for more information on how and when the power-cycling procedure can be carried out.

If this procedure for all virtual interfaces is successful, then these interfaces can be re-started by calling #clxWlanStartInterface. In this case, there is no need to destroy 
the entire stack and re-initialize it.

However, if the procedure mentioned above fails for any of the opened virtual interfaces, the stack instance MUST be terminated, and destroyed.
Upon a successful termination and destroying, the stack may be re-initialized again. 

IMPORTANT : Please note that all open handles to virtual interfaces MUST be closed before the stack can be terminated.

During the recovery procedure, all established links to other wireless stations or networks will be lost, and any configuration parameters set by the application will be dropped. 
Therefore, the application must update its internal states accordingly. In recovery mode, the indications, which are normally sent by the stack to the application, will not be sent. 

Upon the reception of this indication, all non-blocking API function calls will return with an appropriate error code, and subsequent attempts to call any new API
functions (except for #clxWlanStopInterface() and #clxCloseHandle() functions) will fail. This is due to the fact that the stack has entered the recovery mode in order 
to prevent the application from inflicting any further damage. This rule also applies to all BSP interfaces.
*/
#define CLX_WLAN_FATAL_HARDWARE_ERROR_INDICATION              0xB807

/**
This indication is sent to the application when AP disconnects from Station. This is indication is only sent when the local device is in station mode. This indication is only generated
when the access point has initiates the disconnection procedure. It will not be generated when the disconnection procedure has been initiated by the local device.
This indication has no parameters.
*/
#define CLX_WLAN_AP_DISCONNECTED_INDICATION                   0xB808

/**
This indication is sent to the application when the data link (the link over which data frames are sent to the remote machine) has
been suspended since the link to the remote machine seems to be lost (decided based on the recent beacon losses or data transmission failures).
This is a hint that the AccessPoint might be out of range or (possibly) tuned off.
This indication informs the application that the data flow has been suspended (so only management frames can be sent to the remote machine)
until the application takes appropriate action.
Upon reception of this indication, the application MUST start Link Recovery procedures. The actual procedure used is application-specific.
If the link cannot be recovered, the API function #clxWlanDisconnect() MUST be called.

If it is established that the local station needs to connect to a different access point in the same network (a.k.a Roaming), the API function
#clxWlanConnect() can be called to re-associate to the new AP.

This indication is only received when the local machine is in a non-master role (e.g. Station role or P2P Client role), and there is currently a connection
to a wireless network.

NOTE : This indication will NOT be re-generated as long as the data link remains suspended.

NOTE : This indication might be not supported by some drivers/chips.

This indication has no parameters.
*/
#define CLX_WLAN_DATA_LINK_SUSPENDED_DUE_TO_LINK_LOSS_INDICATION                      0xB809


struct ClxWlanNetworkInterface;


struct ClxInterfaceInitializationCompleteIndication
{
    ClxError result;
};

#define CLX_MAX_SSID_LEN                            32  /*!< Max Length of an SSID */
#define CLX_MAC_ADDR_LEN                            6   /*!< Length of the MAC ADDR */
#define CLX_PIN_CODE_LEN                            8   /*!< Length of Pin Code */
#define CLX_MAX_KEY_LEN                             32  /*!< Max Length of a Key */
#define CLX_MAX_PSWD_PHRASE_LEN                     63  /*!< Max Length of a password phrase */
#define CLX_MAX_NUMBER_OF_KEYS                      4   /*!< Key indexes of 0 to 3 are supported by IEEE802.11 */
#define CLX_WLAN_WPS_MAX_PIN_SIZE                   32  /*!< Max Length of WPS pin code */


enum ClxFrequencyBand
{
    ClxFreqBandUnknown,
    ClxFreqBand2GHz,
    ClxFreqBand5GHz
};

/**
The Type of WLAN Interface.
*/
enum ClxInterfaceType
{
    ClxIfTypeAdhoc,                         /*!< Independent BSS member */
    ClxIfTypeAP,                            /*!< Access Point */
    ClxIfTypeSTA,                           /*!< Station */
    ClxIfTypeP2P,                           /*!< A P2P Device. After a P2P connection, the role will be either P2P Group Owner (GO) or P2P Group Client */
    ClxIfTypeP2P_GO                         /*!< A P2P Group Owner. This will start the interface directly as a P2P Group Owner (GO) */
};

/**
The Type of Basic Service Set (BSS).
*/
enum ClxBSSType
{
    ClxBssTypeUnknown,
    ClxBssTypeInfrastructure,               /*!< Infrastructure BSS */
    ClxBssTypeIndependent,                  /*!< Independent BSS */
    ClxBssAnyType                           /*!< Both Infrastructure and Independent BSS */
};

/**
 * The Type of Authentication Mechanism. 
 * In WIFI Alliance terminology, there are two types of Authentication methods :
 * PSK and Enterprise
 */
enum ClxAuthType                            /*! The Authentication types */
{
    ClxAuthTypeOpenSystem    = 0x01,        /*!< Open System Authentication (only used for non-secure networks) */
    ClxAuthTypeLegacy        = 0x02,        /*!< Legacy authentication (only used for WEP. Deprecated). */
    ClxAuthTypePresharedKey  = 0x04,        /*!< Authentication based on pre-shared master key or pass-phrase (Also known as WPA/WPA2 PSK or personal) */
    ClxAuthTypeIEEE802_1x    = 0x08         /*!< IEEE802.1x Port-Based Authentication (Also known as WPA/WPA2 Enterprise. Not supported in this version of ClarinoxWLAN) */
};


/**
 * The WIFI Alliance standard(s) supported by a WLAN device (local or remote).
 * NOTE : The standards of a remote device are detected based on the information elements present in the management frames
 * sent by that device. This does NOT imply a complete compliance with WIFI Alliance qualification programs. 
 */
enum ClxWifiSupportedStandard
{
    ClxWifiSupportedStandard_WPA            = 0x01,        /*!< WPA Authentication (Usually used along with TKIP encryption protocol) */
    ClxWifiSupportedStandard_WPA2           = 0x02,        /*!< WPA2 Authentication (Usually used along with AES-CCMP encryption protocol) */
    ClxWifiSupportedStandard_WPS            = 0x10,        /*!< Wi-Fi Protected Setup */
    ClxWifiSupportedStandard_WMM            = 0x20,        /*!< Wi-Fi MultiMedia */
    ClxWifiSupportedStandard_WIFI_Direct    = 0x40         /*!< Wi-Fi Direct (P2P) */
};


/**
 * The Type of Encryption Protocol. 
 */
enum ClxEncryptionProtocol                  /*! The encryption Protocols */
{
    ClxEncProtoNONE     = 0x00,             /*!< No Encryption protocol to be used (only used with open system authentication) */
    ClxEncProtoWEP40    = 0x01,             /*!< WEP40 (Deprecated) */
    ClxEncProtoWEP104   = 0x02,             /*!< WEP104 (Deprecated) */
    ClxEncProtoTKIP     = 0x04,             /*!< Temporal Key Integrity Protocol (Deprecated as of IEEE802.11n standard) */
    ClxEncProtoAES_CCMP = 0x08              /*!< Advanced Encryption Standard Counter Cipher Mode 
                                                 with Block Chaining Message Authentication Code Protocol */
};

/**
The security credential of a wireless network,
containing all security-related information required to connect
to a BSS.
*/
struct ClxWlanSecurityCredential
{
    enum ClxAuthType                 authTypeToUse;                             /*!< Authentication type of BSS of type  #ClxAuthType (Ignored in this version of ClarinoxWLAN) */
    enum ClxEncryptionProtocol       encProtocolToUse;                          /*!< Encryption algorithm of BSS of type #ClxEncryptionProtocol */
    u1                               keyIndex;                                  /*!< Required for WEP. Must be always set to 0 for WPA/WPA2 */
    u2                               keyLen;                                    /*!< The length of WEP key or WPA/WPA2 master key. Must be set to 0 if a pass-phrase is to be used to 
                                                                                     extract the master key instead (only in case of WPA/WPA2) */
    u1                               key[CLX_MAX_KEY_LEN];                      /*!< Array of either the WEP key(5 or 13 bytes) or WPA/WPA2 master key(always 32bytes). */
    s1                               passwdPhrase[CLX_MAX_PSWD_PHRASE_LEN+1];   /*!< WPA-PSk passPhrase, as null terminated UTF-8 string. Used only if keyLen is 0. Otherwise, it will be ignored */
};


/**
The Service Set Identifier(SSID) structure
*/
struct ClxSSID                        
{
    s1 value[CLX_MAX_SSID_LEN + 1];    /*!< The buffer to hold the SSID, followed by a null character.
                                            In theory (as per the specification), SSID may contain any arbitrary ASCII characters.
                                            However, in practice, SSID is always a human-readable ASCII or UTF-8 string.
                                            In either case, the null-termination character is not part of SSID */ 
    u1 len;                            /*!< The length of the SSID, excluding the null character */
};

/**
List of #ClxSSID
*/
struct ClxSSIDList
{
    struct ClxSSID*      head;       /*!< The head of the list */
    u2                   length;     /*!< The length of the list */
};

/**
Return parameters of the indication #CLX_WLAN_NEW_STATION_JOINED_INDICATION.
*/
struct ClxWlanStationJoined
{
   u1         macAddress[6];         /*!< The MAC address of the station joined or disconnected */
   u1         bssID[6];              /*!< The BSSID to which the station has joined, or from which the station has disconnected */ 
};

/**
Return parameters of the indication #CLX_WLAN_STATION_DISCONNECTED_INDICATION.
*/
struct ClxWlanStationDisconnected
{
   u1         macAddress[6];         /*!< The MAC address of the station joined or disconnected */
   u1         bssID[6];              /*!< The BSSID to which the station has joined, or from which the station has disconnected */ 
   ClxError   reason;
};

/**
Return parameters of the indications #CLX_WLAN_STATION_AUTHENTICATION_INITIATED_INDICATION.
*/
struct ClxWlanStationAuthInitiated
{
   u1   macAddress[6];               /*!< The MAC address of the station initiated the authentication*/
}; 

/**
Return parameters of the indications #CLX_WLAN_STATION_AUTHENTICATION_COMPLETED_INDICATION and, 
CLX_WLAN_STATION_ASSOCIATION_COMPLETED_INDICATION.
*/
struct ClxWlanStationAuthAssociationCompleted
{
   u1        macAddress[6];            /*!< The MAC address of the station completed the authentication or association.*/
   ClxError  statusCode;               /*!< Is the status that AP returns to STA, in the response to the Authentication request or association request
                                            it can be printed with clxGetWlanErrorCodeText() function.*/
};

/**
Return parameters of the indications #CLX_WLAN_STATION_ASSOCIATION_INITIATED_INDICATION.
*/
struct ClxWlanStationAssociationInitiated
{
   u1       macAddress[6];            /*!< The MAC address of the station initiated the association*/
};

/**
Return parameters of the indications #CLX_WLAN_AP_DISCONNECTED_INDICATION.
*/
struct ClxWlanApDisconnected
{
   u1         macAddress[6];         /*!< The MAC address of the disconnected AP */
   ClxError   reason;                 /*!<  The reason why the connection has been terminated */
};

/**
The information related with the Basic Service Set(BSS)
*/
struct ClxBSSInfo
{
    u1                           bssid[CLX_MAC_ADDR_LEN];    /*!< The BSSID (Basic Service Set ID) */
    u2                           channel;                    /*!< The operating frequency channel. In case of IEEE802.11n, this is the primary 20MHz channel */
    enum ClxFrequencyBand        band;                       /*!< The operating frequency band */
    enum ClxBSSType              bssType;                    /*!< Type of the BSS */
    s1                           rssi;                       /*!< Signal power level in dBm */
    u4                           authType;                   /*!< Authentication types supported by the BSS, as a bit-wise combination of values of type #ClxAuthType */
    u4                           encProtocol;                /*!< Encryption protocols supported by the BSS, as a bit-wise combination of values of type #ClxEncryptionProtocol */
    u4                           capabilities;               /*!< Capabilities supported by the BSS, as an bit-wise combination of values defined in Wlan.Config.h */ 
    u2                           supportedNonHTRates;        /*!< The list of non-HT rates supported by the BSS, as a bit-wise combination of rate bits defined in Wlan.Config.h */
    u1                           supportedHTRates[10];       /*!< The list of HT rates supported by the BSS, with each corresponding to one MCS (Modulation and Coding Scheme) index.
                                                                  MCS indexes in each byte are ordered in LSB (lowest MCS index) to MSB (highest MCS indeX). Also, supportedHTRates[0] contains
                                                                  lowest MCS indexes, supportedHTRates[9] the highest MCS indexes.
                                                                  Example : (supportedHTRates[0] & 0x01) corresponds to MCS 0
                                                                            (supportedHTRates[0] & 0x80) corresponds to MCS 7
                                                                            (supportedHTRates[1] & 0x01) corresponds to MCS 8 */
    struct ClxSSID               ssid;                       /*!< The SSID (Service Set ID) */
};

/**
List of information related to BSS. see #ClxBSSInfo
*/
struct ClxBSSInfoList
{
    struct ClxBSSInfo* head;
    u2                 size;
};


enum ClxWlanBssConnectionMode
{
    ClxWlanBssConnectionMode_Associate = 0,                 /*!< Used when there is currently no connection to a BSS (Basic Service Set)*/ 
    ClxWlanBssConnectionMode_Reassociate = 1                /*!< Used when there is a connection to a BSS but the station is trying to re-associate to the same BSS or another BSS
                                                                 in the same ESS (Extended Service Set). The latter procedure is also called Roaming */
};


struct ClxWlanGetAvailableBssListComplete
{
    _user_out_ ClxBSSInfo*    bssInfoList;
    _user_out_ ClxSize*       numberOfAvailableBss;
};


struct ClxWlanScanComplete
{
    _user_out_ u2* numOfFoundBSS;
};


/**
The API function group used to connect to a BSS. There are two types of connection modes (as specified by #bssConnectionMode):

- ClxWlanBssConnectionMode_Associate : When there is currently no connection to a Basic Service Set (BSS), this procedure must be issued to connect to a BSS. This procedure starts with a 
scan procedure to find the information about the BSS to connect to. The BSS is identified by either #bssid or #ssid (but not both at the same time). 

- ClxWlanBssConnectionMode_Reassociate : When there is already a connection to a BSS, this procedure may be issued to re-associate to the same BSS, or another BSS
belonging to the same ESS (Extended Service Set). In this mode, the destination BSS can only be specified by its BSSID (ssid.len SHAL be set to 0).

Also, in this mode, no scan procedure is carried out. The information about the BSS comes from the stack internal cache. If the internal cache does not have any
information about the BSS with the provided BSSID, connection procedure will fail. New entries are added to the internal cache (or its current information is updated) upon calling #clxWlanScan() or
#clxWlanProbe() command.

NOTE : the contents of #securityCredential object is ignored in ClxWlanBssConnectionMode_Reassociate mode as the current key information is used for the next connection.

ClxWlanConnectionProfile has the following members which need detailed introduction (only in connection mode ClxWlanBssConnectionMode_Associate):

- key : The key in binary format. The member keyLen must be set to the length
of the key stored in this member. Ignored in ClxWlanBssConnectionMode_Reassociate mode.

- passwdPhrase : An ASCII(or UTF-8) null-terminated string which stores the
pass phrase. Ignored in ClxWlanBssConnectionMode_Reassociate mode.

The stack first checks "key" and "keyLen" members. If "keyLen" has been set to
the expected key length, "key" will be used as the encryption key (WEP) or
master key (WPA/WPA2). In this case, "passwdPhrase" will be ignored.
For instance, "keyLen" is supposed to be 32 in case of WPA/WPA2.

If "keyLen" is zero, or not the expected value, "passwdPhrase" will be used to
generate the master key, internally.

Please note that the master key MUST be stored in BINARY format in "key". If
the user interface takes the key in HEX format (64 characters), it must be
converted to binary (32 bytes) before it is stored in "key".

The pass-phrase can be 1 to 63 bytes. Therefore, if the value entered by the
user is shorter than 64 bytes, it must be taken as the pass phrase. If the
value entered is exactly 64 bytes, it must be taken as the master key in HEX
format. The user must not be able to enter a value longer than 64 bytes.

NOTE : Although ClarinoxWLAN accepts a pass-phrase of less than 8 characters,
most Access Points reject such a short pass-phrase. If required, This rule can
be enforced by the application code.
*/
struct ClxWlanConnectionProfile
{
    enum ClxWlanBssConnectionMode        bssConnectionMode;                         /*!< The connection mode to be initiated */
    struct ClxSSID                       ssid;                                      /*!< name of BSS of type #ClxSSID. In case of bssConnectionMode being set to ClxWlanBssConnectionMode_Reassociate, this parameters
                                                                                         MUST not be used (ssid.len MUST be set to 0). */
    u1                                   bssid[CLX_MAC_ADDR_LEN];                   /*!< The BSS's MAC address (used only when ssid.len is set to 0. Otherwise ignored) */
    struct ClxWlanSecurityCredential     securityCredential;                        /*!< The security credential of the BSS to connect to (only in ClxWlanBssConnectionMode_Aassociate connection mode) */
};


/**
Creates a WLAN virtual Interface on the WLAN physical adaptor, and associates a handle to it. The handle is passed
to any subsequent API call. After successful creation of a new virtual interface, the interface MUST be started (by calling the #clxWlanStartInterface() function), before
any other API functions, which take the handle of the interface as an input argument, may be called.

\param[ in ]  stack    Wi-Fi stack handle. A stack object must be created before any other operation.
\param[ in ]  intfName the name of the interface that will be opened, it should be a null-terminated string with the maximum length of 32 bytes, including the null-termination
\param[ out ] handle   A pointer which will receive the handle for the virtual interface, on return. handle will be set to NULL if the return value of the function is not CLX_SUCCESS.
\return       ClxError error code for the operation.
                         - CLX_SUCCESS if the creation of the handle has been successful, or a driver-specific error code describing the reason that the operation has been failed.
*/
ClxError clxWlanCreateInterface(_in_  ClxStack     stack, 
                                _in_  const s1*    intfName,
                                _out_ ClxHandle*   handle);


/**
Gets the MAC address of the given interface on the local machine. This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

\param[ in ]  interfaceHandle the WLAN interface handle. A WLAN interface should be opened first with the clxWlanInterfaceOpen function
\param[ out ] macAddrBuffer   the buffer where the MAC address will be returned. The buffer must be pre allocated by the user and must have enough space to hold the MAC address 
                              (which is an array of u1[6])
\param[ in ]  block           type of the operation.
                                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return       ClxError        error code for the operation.
*/
ClxError clxWlanInterfaceGetMacAddr(_in_       ClxHandle    interfaceHandle, 
                                    _user_out_ u1*          macAddrBuffer,
                                    _in_       boolean      block);



/**
Retrieves the value of one or more driver parameters requested by the user. The set of parameters that can be interrogated is driver-specific. Please refer to the documentation of the 
driver used for the list of parameters. This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

\param[ in ]  interfaceHandle the WLAN interface handle. A WLAN interface should be opened first with the clxWlanInterfaceOpen function
\param[ out ] configList      The list of parameters to be interrogated, as a set of name/value pairs. The list object, 
                              and all its child objects, must not be modified or deleted until this command is complete. 
                              These objects are all allocated by the caller.
\param[ in ] block            type of the operation.
                                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return      ClxError         error code for the operation.
*/
ClxError clxWlanGetParametersValue(_in_       ClxHandle          interfaceHandle,
                                   _user_out_ ClxConfigList*     configList,
                                   _in_       boolean            block);


/**
Sets the value of one or more driver parameters requested by the user. The set of parameters that can be configured is driver-specific. Please refer to the documentation of the 
driver used for the list of parameters. This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

\param[ in ]  interfaceHandle the WLAN interface handle. A WLAN interface should be opened first with the clxWlanInterfaceOpen function
\param[ out ] configList      The list of parameters to be configured, as a set of name/value pairs. 
                              The list object, and all its child objects, must not be modified or deleted until this command is complete. 
                              These objects are all allocated by the caller. 
                              The driver will only perform a READ-ONLY access on these parameters.
\param[ in ] block            type of the operation.
                                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError              error code for the operation.
*/
ClxError clxWlanSetParametersValue(_in_      ClxHandle          interfaceHandle,
                                   _user_in_ ClxConfigList*     configList,
                                   _in_      boolean            block);


/**
Starts the virtual interface operation which will provide service using the given WLAN Interface. A valid ClxHandle (of the interface) must be provided and 
the interface should be of type #ClxIfTypeAP.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_START_INTERFACE_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle. A WLAN interface of type ClxIfTypeAP should be opened first.
\param[ in ] interfaceType the type of the configuration for the virtual interface. Access point or Station modes. 
                           Note: When the virtual interface has been started as STA, but there is no connection to a network or no RF activity.
\param[ in ] networkStackInterface Implementation of the network stack as a caller-allocated #ClxWlanNetworkInterface object. This is the interface between ClarinoxWLAN stack 
and the external Network (e.g. TCP/IP) stack. This parameter CANNOT be NULL. The object is allocated by the caller,
and CANNOT be deleted or modified until the virtual interface is stopped by calling clxWlanStopInterface() function (or clxWlanStartInterface() returns an error). 
\param[ in ] configList List of the initial configuration parameters. The stack will only perform a READ-ONLY access on these parameters.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle. 

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_INTERFACE_NOT_STARTED: Interface failed to start due to one of the reasons such as signal strength, not powered etc..

\remark The function networkStackInterface->init (of the network interface) will be called before this function returns successfully (If the creation of the new virtual interface fails for any reason,
networkStackInterface->init will not be called). the handle object (of type ClxHandle) which is passed to networkStackInterface->init is the same as the handle object returned by #clxWlanCreateInterface().
*/
ClxError clxWlanStartInterface(_in_      ClxHandle                          interfaceHandle,
                               _in_      enum ClxInterfaceType              interfaceType,
                               _in_      ClxWlanNetworkInterface*           networkStackInterface,
                               _user_in_ ClxConfigList*                     configList,
                               _in_      boolean                            block);


/**
Stops the virtual interface operation.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_STOP_INTERFACE_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ]  block          type of the operation.
                                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return       ClxError       error code for the operation.
                                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
*/
ClxError clxWlanStopInterface(_in_ ClxHandle          interfaceHandle,
                              _in_ boolean            block);


/**
Starts the WLAN role which is associated to this interface. This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

This function can ONLY be used when the local interface has been configured as an AccessPoint. With Station and ADHOC mode, role start is automatically
performed during the connection procedure.

Normally, This procedure involves the following steps:

- The chip RF transceiver is powered on. Thus, thus chip will be able to send and receive packets. 
- The stack informs the network interface that the link is now up, and packets can be sent over the air.

The first step might not be implemented by all drivers. In this case, the RF transceiver is on as soon as clxWlanStartInterface() returns with success.
However, the second step is always performed. Therefore, it is MANDATORY to call this function (in AccessPoint mode) after the successful start of the interface in order to make sure that
(at least) the network interface will operate properly. 

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_START_ROLE_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ]  block          type of the operation.
                                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return       ClxError       error code for the operation.
                                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
*/
ClxError clxWlanStartRole(_in_ ClxHandle        interfaceHandle,
                          _in_ boolean          block);


/**
Stops the WLAN role which is associated to this interface. This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

This function can ONLY be used when the local interface has been configured as an AccessPoint. With Station and ADHOC mode, role start is automatically
performed during the disconnection procedure.

Normally, This procedure involves the following steps:

- All client stations (station currently connected to the local AccessPoint, or about to be connected) are removed. If necessary, a deassociation or de-authentication
frame is sent to the remote stations in order to inform them of the connection loss.
- The chip RF transceiver is powered off. Thus, thus chip will not be able to send and received packets. 
- The stack informs the network interface that the link is now down.

The second step might not be implemented by all drivers. In this case, the RF transceiver is powered off only when the interface is stopped.
However, the first and third steps are always performed.

NOTE : Although it is not mandatory to call this function before stopping the interface, it is recommended that this be done since clxWlanStopRole() provides a 
peaceful termination of connections by sending appropriate disconnection frames to the connected stations. clxWlanStopInterface() carries out the same steps (if not yet carried out),
except for peaceful termination of the connections.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_STOP_ROLE_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ]  block          type of the operation.
                                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return       ClxError       error code for the operation.
                                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
*/
ClxError clxWlanStopRole(_in_ ClxHandle        interfaceHandle,
                         _in_ boolean          block);



/**
Performs A WLAN Scan. This scans all the available BSSs. This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

NOTE : This operation may not have been implemented for some interface types or roles. Please refer to the relevant driver documentation for more information.

Blocking mode : When the command completes successfully, this API function returns numOfFoundBSS containing number of SSID found during the scan operation.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_SCAN_COMPLETE, which has configuration
parameter of type #ClxWlanScanComplete.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ] ssidToScan of type #ClxSSID, the SSID to search for. If ssidToScan is null than all the available BSSs will be searched.
\param[ in ] scanTimeout Maximum time in seconds to return, in case scan fails.
\param[ out ] numOfFoundBSS A caller-provided variable which on a successful return will contain the number of SSIDs that are found as a result of the scan.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_WLAN_ERROR_SCAN_FAILED: Scan operation failed.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_CANCELLED: the interface was stopped, or the connection failed before the command has completed.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanScan(_in_  ClxHandle            interfaceHandle,
                     _in_  struct ClxSSID*      ssidToScan,
                     _in_  u1                   scanTimeout,
                     _user_out_ u2*             numOfFoundBSS,
                     _in_  boolean              block);



/**
Probes a station in the currently connected BSS/IBSS by sending probe requests and waiting for the probe responses. This function can only be used when there is an active connection to a BSS or IBSSS.
This function cannot be used when the local virtual interface is in AccessPoint mode. Also, this function cannot be used to probe a station not connected to the same network.

In station mode, this function can only be used to probe the access point to which the station is connected. It may also be used to probe another access point of the same network (same BSS) as long as
this access point operates at the same channel as the connected access point.

In ADHOC mode, this function can be used to probe any other station joined to the same network. But the remote station will not respond to the probe request if it is currently sleeping.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_PROBE_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ] destinationMacAddr The MAC address of the remote station to which the probe requests are to be sent. This argument is optional. If set to NULL, the broadcast address (FF:FF:FF:FF:FF:FF) is to
be used to broadcast the probe request to all the stations of the same network in the area.
\param[ in ] maxNumberOfProbeRequests Maximum number of probe requests to be sent to the remote machine until a response to the probe is received, or the function times out. The minimum value is 1.
\param[ in ] probeRequestInterval The interval between two consecutive probe requests, in units of Beacon Interval. The Beacon Interval is determined by the connected BSS/IBSS.
Assume the Beacon Interval is specified by the variable "bi". Then, the interval between two consecutive probe requests, in seconds, is probeRequestInterval*bi. Therefore, the maximum time this function may
block before timing out is maxNumberOfProbeRequests*probeRequestInterval*bi.
The minimum value is 1.
\param[ in ] bssInfo A caller-allocated object of type ClxBSSInfo which, on a successful return of this function, will contain the details of the probe response. All members of the structure (including rssi)
will have valid values. The object must not be modified or deleted until this command is complete either with success or in error. If the return value of this function is not CLX_SUCCESS, then this structure
will NOT contain valid values.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_WLAN_ERROR_SCAN_FAILED: Scan operation failed.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_COMMAND_CANCELLED: the interface was stopped, or the connection failed before the command has completed.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: The same or another conflicting task is currently in progress. 
                - CLX_ERROR_TIMEOUT_OCCURRED: Timeout has occurred and no probe response has been received from the remote machine.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanProbe(_in_       ClxHandle      interfaceHandle,
                      _in_       const u1*      destinationMacAddr,
                      _in_       u4             maxNumberOfProbeRequests,
                      _in_       u4             probeRequestInterval,
                      _user_out_ ClxBSSInfo*    bssInfo,
                      _in_       boolean        block);


/**
Gets the available BSS List after performing a scan. Calling #clxWlanScan() is pre-requisite for this function. This function MUST be called only if the interface is successfully 
started (by a call to the #clxWlanStartInterface() function).

NOTE : This operation may not have been implemented for some interface types or roles. Please refer to the relevant driver documentation for more information.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_GET_AVAILABLE_BSSLIST_COMPLETE.
The indication parameter is of type ClxWlanGetAvailableBssListComplete.

\param[ in ]  interfaceHandle      the WLAN interface handle
\param[ out ] bssInfoList          the List of available BSS. Note that the availableBss memory buffer should be allocated by the caller.
                                   It is advised that firstly a scan is performed so that the number of SSID found can be successfully returned by this function.
\param[ in ]  sizeOfBssInfoList    Size of availableBss array.
\param[ out ] numberOfAvailableBss Upon return of the function, the number of available networks will be written in this caller-provided variable.
\param[ in ]  block                type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanGetAvailableBssList(_in_       ClxHandle      interfaceHandle,
                                    _user_out_ ClxBSSInfo*    bssInfoList,
                                    _in_       ClxSize        sizeOfBssInfoList,
                                    _user_out_ ClxSize*       numberOfAvailableBss,
                                    _in_       boolean        block);


/**
Makes a connection using the provided connection profile. By this method the authentication and association operations are
handled. 

When there is no connection to a BSS, this API is called to connect the station to a BSS.
If there is already a connection to a BSS, this API may be called to re-associate to the same AP or a new AP in the same BSS (e.g. with the same BSSID).

This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

NOTE : This operation may not have been implemented for some interface types or roles. Please refer to the relevant driver documentation for more information.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_CONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ] stack       the WLAN stack handle
\param[ in ] connProfile A user-allocated structure of type #ClxWlanConnectionProfile which contains the ssid or the BssID to connect to, 
                         the authentication type, the encryption protocol and key information. See #ClxWlanConnectionProfile for further details.
             NOTE : the object must not be deleted or modified until this command is complete.
\param[ in ] block      type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.
                - CLX_WLAN_ERROR_ASSOCIATION_FAILED: Association procedure failed
                - CLX_WLAN_ERROR_REQUESTED_AUTH_NOT_SUPPORTED_BY_BSS: Authentication type filled in connProfile is not supported by BSS.
                - CLX_WLAN_ERROR_REQUESTED_ENC_NOT_SUPPORTED_BY_BSS: Mentioned encryption type in connProfile not supported by BSS.
                - - CLX_WLAN_ERROR_USER_TIMEOUT_OCCURRED: Scan did not complete in 5secs.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanConnect(_in_      ClxStack                           stack,
                        _user_in_ const ClxWlanConnectionProfile*    connProfile,
                        _in_      boolean                            block);

/**
Disconnects a previously connected connection. This API is called to disconnect the station from network. 
Upon the successful operation, The connection to the BSS will be terminated.
This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

NOTE : This operation may not have been implemented for some interface types or roles. Please refer to the relevant driver documentation for more information.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_DISCONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ]  block          type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanDisconnect(_in_ ClxHandle    interfaceHandle,
                           _in_ boolean      block);


/**
Passes the Wireless Protected Setup (WPS) pin code, as entered by the user, to the stack. This function must ONLY be called when
an indication of type CLX_WLAN_WPS_PIN_CODE_REQUIRED_INDICATION has been received.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_WPS_ENTER_PIN_CODE_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle.
\param[ in ] pincode A caller-allocated buffer containing the pin code as provided by the user. The pin code does not need to be null-terminated. The buffer
must not be modified or deleted until this command is complete.
\param[ in ] pincodeLength The length of the pin code. This must NOT include the null-terminated character (if there is any).
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PEnterPincode(_in_ ClxHandle      interfaceHandle,
                                _user_in_ const u1* pincode, 
                                _in_ ClxSize        pincodeLength,
                                _in_ boolean        block);


/**
Informs the stack that the Wireless Protected Setup (WPS) push button has been pressed by the user. This function must ONLY be called when
an indication of type CLX_WLAN_WPS_PUSH_BUTTON_REQUIRED_INDICATION has been received.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_WPS_PUSH_BUTTON_PRESSED_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle.
\param[ in ]  block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanWpsPushButtonPressed(_in_ ClxHandle      interfaceHandle,
                                     _in_ boolean        block);



/**
Returns a text string containing the name of the error code. This function only returns the string in debug mode.
In release mode, the return string is always an empty string.

\param[ in ] errorCode The error code for which a string representation must be returned.

\return A stack-allocated string containing the string representation of the error code. This string MUST not be modified or deleted.
*/
const s1* clxGetErrorCodeText(ClxError errorCode);

/**
Returns the ClarinoxWLan version.
\return A function-allocated buffer containing the version as a null-terminated string. The buffer
must NOT be modified by the caller.
*/
const s1* clxGetClarinoxWlanVersion(void);

/**
Returns the ClarinoxWLan repository revision.
\return the repository revision as an unsigned integer number. 
*/
u4 clxGetClarinoxWlanRevision(void);


#ifdef __cplusplus
}
#endif


#endif // _WLAN_API_H_
