#ifndef _WLAN_WIFI_DIRECT_API_H_
#define _WLAN_WIFI_DIRECT_API_H_

/*******************************************************************************
*
* Project             ClarinoxWLAN
* File                Wlan.WifiDirect.Api.h
* Description         ClarinoxWlan WIFI Direct API functions
*
*
* This file contains proprietary information of 
* Clarinox Technologies Proprietary Limited. 
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif


/**
Please refer to #clxWlanP2PConnect function.
*/
#define CLX_WLAN_P2P_CONNECT_COMPLETE                               0x7850

/**
Please refer to #clxWlanP2PCancelConnectionRequest function.
*/
#define CLX_WLAN_P2P_CANCEL_CONNECTION_REQUEST_COMPLETE             0x7851
    
/**
Please refer to #clxWlanP2PDisconnect function.
*/
#define CLX_WLAN_P2P_DISCONNECT_COMPLETE                            0x7852

/**
Please refer to #clxWlanP2PAcceptConnectionRequest function.
*/
#define CLX_WLAN_P2P_ACCEPT_CONNECTION_REQUEST_COMPLETE             0x7853

/**
Please refer to #clxWlanP2PRejectConnectionRequest function.
*/
#define CLX_WLAN_P2P_REJECT_CONNECTION_REQUEST_COMPLETE             0x7854

/**
Please refer to #clxWlanP2PStartDeviceDiscovery function.
*/
#define CLX_WLAN_P2P_START_DISCOVERY_COMPLETE                       0x7855

/**
Please refer to #clxWlanP2PStopDeviceDiscovery function.
*/
#define CLX_WLAN_P2P_STOP_DISCOVERY_COMPLETE                        0x7856

/**
Please refer to #clxWlanP2PGetGroupInfo function.
*/
#define CLX_WLAN_P2P_GET_GROUP_INFO_COMPLETE                        0x7857

/**
Please refer to #clxWlanP2PGetCurrentGroup function.
*/
#define CLX_WLAN_P2P_GET_CURRENT_GROUP_COMPLETE                     0x7858

/**
Please refer to #clxWlanP2PGetLocalDevice function.
*/
#define CLX_WLAN_P2P_GET_LOCAL_DEVICE_COMPLETE                      0x7859

/**
Please refer to #clxWlanP2PGetDiscoveredPeersList function.
*/
#define CLX_WLAN_P2P_GET_DISCOVERED_PEERS_LIST_COMPLETE             0x7860



/**
This indication is received when a connection to a peer device has been successful.
The connection could have been initiated by the local device or the peer device.
The parameter for this indication is of type #ClxWlanP2PDevice.
*/
#define CLX_WLAN_P2P_CONNECTION_INDICATION                          0xB850

/**
This indication is received when a new peer device has been discovered.
The parameter for this indication is of type #ClxWlanP2PDevice.
*/
#define CLX_WLAN_P2P_DEVICE_DISCOVERED_INDICATION                   0xB851


/**
This indication is received when the status of the local device is changed.
The parameter for this indication is of type #ClxWlanP2PDeviceStatus which specifies
the new status of the local machine.
*/
#define CLX_WLAN_P2P_LOCAL_DEVICE_STATUS_CHANGED_INDICATION         0xB852

/**
This indication is received when a new peer device joins the WIFI Direct group to which the local device has joined.
The parameter for this indication is of type #ClxWlanP2PDevice.
*/
#define CLX_WLAN_P2P_PEER_DEVICE_JOINED_GROUP_INDICATION            0xB853

/**
This indication is received when a peer device leaves the WIFI Direct group to which the local device has joined.
The parameter for this indication is of type #ClxWlanP2PDevice.
*/
#define CLX_WLAN_P2P_PEER_DEVICE_DISCONNECTED_FROM_GROUP_INDICATION 0xB854


/**
This indication is received when a connection to the P2P group is disrupted.
The disconnection procedure could have been initiated by the local device or the peer device.
The indication does not have any parameter.
*/
#define CLX_WLAN_P2P_DISCONNECTION_INDICATION                       0xB855


/**
This indication is received when a connection request is received from a remote P2P device.
The application may prompt the user about the request and allows them to decide whether or not
to accept the request, or it may decide automatically on behalf of the user. 
If the request is accepted, the application must call #clxWlanP2PAcceptConnectionRequest() API
function to allow the stack to start the connection procedure. If the request is rejected, the application
must call #clxWlanP2PRejectConnectionRequest API function. This indication may be received
when the local device is in Idle mode (not connected to any P2P group), or it is already the group owner
of a P2P group.
The parameter for this indication is of type #ClxWlanP2PConnectionRequestIndication (details of the requester P2P device).

NOTE : Upon reception of this indication, either #clxWlanP2PAcceptConnectionRequest or #clxWlanP2PRejectConnectionRequest
MUST be called in a timely manner.
*/
#define CLX_WLAN_P2P_CONNECTION_REQUEST_INDICATION                  0xB856

/**
This indication is received during a P2P connection procedure when the local device has to
prompt the user to enter a PIN code. The pin code is being shown on the remote device display (this
is the device to which the local device is in connection procedure). 
Upon reception of this indication, and if the request is accepted, the local application must prompt the user and ask them to enter
the pin code of the remote device. After the pin code is obtained from the local user, the function
#clxWlanP2PAcceptConnectionRequest must be called and the entered PIN code must be passed to this function. If the request is rejected, 
the application must call #clxWlanP2PRejectConnectionRequest API function. This indication may be received
when the local device is in Idle mode (not connected to any P2P group), or it is already the group owner of a P2P group.
The parameter for this indication is of type #ClxWlanP2PConnectionRequestIndication (details of the requester P2P device).

NOTE : Upon reception of this indication, either #clxWlanP2PAcceptConnectionRequest or #clxWlanP2PRejectConnectionRequest
MUST be called in a timely manner.
*/
#define CLX_WLAN_P2P_CONNECTION_REQUEST_KEYPAD_METHOD_INDICATION    0xB857


/**
This indication is received during a P2P connection procedure when the local device has to
display a randomly selected pin code on its display. The user of the remote device will have to enter this pin code
in the remote device.
If the request is accepted, the application must display the PIN code on the screen, and then call #clxWlanP2PAcceptConnectionRequest. 
The pin code must be displayed on the screen until the connection procedure is complete (either in error or with success).
If the request is rejected, the application must call #clxWlanP2PRejectConnectionRequest API function. This indication may be received
when the local device is in Idle mode (not connected to any P2P group), or it is already the group owner of a P2P group.
The parameter for this indication is of type #ClxWlanP2PConnectionRequestIndication (details of the requester P2P device).

NOTE : Upon reception of this indication, either #clxWlanP2PAcceptConnectionRequest or #clxWlanP2PRejectConnectionRequest
MUST be called in a timely manner.
*/
#define CLX_WLAN_P2P_CONNECTION_REQUEST_DISPLAY_METHOD_INDICATION   0xB858




#define CLX_MAX_P2P_DEVICE_NAME_LEN      32         /*!< Max Length of a P2P device name */
#define CLX_MAX_P2P_PINCODE_LEN          8          /*!< Max Length of a P2P pin code used during connection for authentication purposes*/



/**
P2P Device capabilities bitmap:
*/
enum ClxWlanP2PDeviceCapabilityBitmap
{
    ClxWlanP2PDeviceCabaility_ServiceDiscovery          = 0x01,             /*!< P2P Device supports Service Discovery */
    ClxWlanP2PDeviceCabaility_ClientDiscoverablity      = 0x02,             /*!< P2P Device supports P2P Client Discoverability */
    ClxWlanP2PDeviceCabaility_ConcurrentOperation       = 0x04,             /*!< P2P Device supports Concurrent Operation with WLAN */
    ClxWlanP2PDeviceCabaility_InfrastructurManaged      = 0x08,             /*!< P2P interface of the P2P Device is capable of being managed by 
                                                                                the WLAN (infrastructure network) based on P2P Coexistence Parameters */
    ClxWlanP2PDeviceCabaility_DeviceLimit               = 0x10,             /*!< P2P Device Limit field shall be set to 1 when the P2P Device 
                                                                                is unable to participate in additional P2P Groups */
    ClxWlanP2PDeviceCabaility_InvitationProcedure       = 0x20              /*!< P2P Device is capable of processing P2P Invitation Procedure signalling */
};


/**
P2P Group capabilities bitmap:
*/
enum ClxWlanP2PGroupCapabilityBitmap
{
    ClxWlanP2PGroupCabailityBitmap_GroupOwner           = 0x01,             /*!< P2P Device is operating as a Group Owner */
    ClxWlanP2PGroupCabailityBitmap_PersistentGroup      = 0x02,             /*!< P2P Device is hosting, or intends to host, a persistent P2P Group */
    ClxWlanP2PGroupCabailityBitmap_GroupLimit           = 0x04,             /*!< P2P Group Owner is unable to add additional Clients to its P2P Group */
    ClxWlanP2PGroupCabailityBitmap_IntraBSSDistribution = 0x08,             /*!< P2P Device is hosting, or intends to host, a P2P Group that provides a data 
                                                                               distribution service between Clients in the P2P Group */
    ClxWlanP2PGroupCabailityBitmap_CrossConnection      = 0x10,             /*!< P2P Device is hosting, or intends to host, a P2P Group that provides cross connection 
                                                                               between the P2P Group and a WLAN */
    ClxWlanP2PGroupCabailityBitmap_PersistentReconnect  = 0x20,             /*!< P2P Device is hosting, or intends to host, a persistent P2P Group that allows reconnection 
                                                                               without user intervention */
    ClxWlanP2PGroupCabailityBitmap_GroupFormation       = 0x40              /*!< P2P Device is operating as a Group Owner in the Provisioning phase of Group Formation */
};


/**
Authentication action which must be performed during the connection procedure.
*/
enum ClxP2PAuthenticationAction
{
    ClxP2PAuthenticationMethod_DisplayPinCode,                              /*!< Display a random PIN code on the screen */
    ClxP2PAuthenticationMethod_EnterPinCode,                                /*!< Prompt the user to enter the PIN code which is showing in the screen of the peer device */
    ClxP2PAuthenticationMethod_PressPushButton,                             /*!< Perform authentication procedures without a need for a PIN code */
    ClxP2PAuthenticationMethod_AutoSelect                                   /*!< Let the stack to choose the best action based on the WPS capabilities of both the local and remote devices */
};


/**
The status of a remote P2P device.
*/
enum ClxWlanP2PDeviceStatus
{
    ClxWlanP2PDeviceStatus_Unknown,                                         /*!< The status of the P2P device is currently unknown */
    ClxWlanP2PDeviceStatus_Idle,                                            /*!< P2P device is not connected to any P2P group */
    ClxWlanP2PDeviceStatus_FormingGroup,                                    /*!< P2P device is in the process of forming a P2P group either as the P2P Group Owner */
    ClxWlanP2PDeviceStatus_Connecting,                                      /*!< P2P device is in the process of joining a P2P group as a P2P client */ 
    ClxWlanP2PDeviceStatus_GroupOwner,                                      /*!< P2P device is currently operating as a Group Owner (GO) of a P2P group */
    ClxWlanP2PDeviceStatus_Connected                                        /*!< P2P device has joined a group as a P2P group client */ 
};


/**
The details of a P2P device, or a non-P2P legacy client (only when the local device is the P2P Group Owner, and it supports connection of non-P2P legacy devices).
*/
struct ClxWlanP2PDevice
{
    ClxConfigParam           paramInfo;                                     /*!< For internal use. Not to be set or modified */

    boolean                  nonP2PDevice;                                  /*!< If TRUE, this object holds information of a non-P2P device. In this case, only the following parameters contain valid values:
                                                                                 deviceAddress
                                                                                 currentStatus */
    u1                        deviceAddress[CLX_MAC_ADDR_LEN];              /*!< The MAC address which uniquely identifies the P2P device */
    s1                        deviceName[CLX_MAX_P2P_DEVICE_NAME_LEN + 1];  /*!< The friendly name of the P2P device as a null terminated UTF-8 string */
    u4                        primaryDeviceType;                            /*!< The primary type of the P2P device, as defined in #ClxWpsDeviceType enumerator */
    u4                        secondaryDeviceType;                          /*!< The secondary type of the P2P device (if available), as defined in #ClxWpsDeviceType enumerator.
                                                                                 It will be set to 0 if the P2P device does not have any secondary type information (ClarinoxWLAN supports only
                                                                                 one secondary device type per P2P device) */
    ClxWlanP2PDeviceStatus    currentStatus;                                /*!< The current status of the P2P device */
    u4                        wpsCapabilities;                              /*!< The WPS methods supported by the P2P device, as a bit-wise combination of values of type #ClxWpsSupportedMethod */
    u4                        deviceCapabilities;                           /*!< Indicates the device capabilities as a bit-wise combination of values of type #ClxWlanP2PDeviceCapabilityBitmap */
};


CLX_DEFINE_CONFIG_TYPE(ClxWlanP2PDevice);


/**
The details of a P2P group.
*/
struct ClxWlanP2PGroup
{
    u1                                   groupID[CLX_MAC_ADDR_LEN];         /*!< The ID which uniquely identifies this group (the same as the device address of the Group Owner).
                                                                                 May be used to identify a group at any time even after re-formation of a persistent group */
    struct ClxBSSInfo                    bssInfo;                           /*!< Low level information about the group as an infrastructure IEEE802.11 network. The information may be used
                                                                                 for debugging/testing purposes. */
    u4                                   groupCapabilities;                 /*!< Indicates the group capabilities as a bit-wise combination of values of type #ClxWlanP2PGroupCapabilityBitmap */
    u4                                   numberOfClients;                   /*!< The current number of clients which have successfully joined this group */
    boolean                              localDeviceGroupOwner;             /*!< TRUE if the local device is this group owner. FALSE otherwise */
};


/**
Parameter of the indications of type #CLX_WLAN_P2P_CONNECTION_REQUEST_INDICATION,
#CLX_WLAN_P2P_CONNECTION_REQUEST_KEYPAD_METHOD_INDICATION, and #CLX_WLAN_P2P_CONNECTION_REQUEST_DISPLAY_METHOD_INDICATION.
*/
struct ClxWlanP2PConnectionRequestIndication
{
    struct ClxWlanP2PDevice    remoteDevice;                                /*!< The remote device with which the local device is in connection procedure. */
    s1                         pincode[CLX_MAX_P2P_PINCODE_LEN + 1];        /*!< A random pin code, as null terminated string, which is selected by the stack and must be displayed on the local device screen. */
};


/**
Initiates a connection to a peer WIFI Direct device.

NOTE : This operation is only possible if the local device is in the status of #ClxWlanP2PDeviceStatus_Idle.

A connection to a remote device is possible if the remote device is in one of the following statuses:
- #ClxWlanP2PDeviceStatus_Idle : Group Owner negotiation is conducted to determine which side will the Group Owner (GO). Also, Authentication (setting up security features) is also conducted.
- #ClxWlanP2PDeviceStatus_GroupOwner : Authentication is conducted, and then the local device (as the group client) will join the group, of which the remote device is the group owner.

It is not possible to connect to a remote device which is in a status other than those mentioned above.

NOTE : If only the information about a P2P group is available (as an object of type #ClxWlanP2PGroup), it is possible to join the group by passing the GroupID of the P2P group to this function
as the argument remoteDeviceAddress. This is due to that fact that the GroupID of a P2P group is the same as the device address of the group owner of the group.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_CONNECT_COMPLETE.
The indication does not have any parameters.

NOTE : This command is not complete until the connection procedure is complete, either in error or with success. Upon a successful connection, an indication of type
#CLX_WLAN_P2P_CONNECTION_INDICATION will also be received by the application. The parameter of this indication will have all the information on the formed or joined P2P
group.

NOTE : The command uses an internal timeout value of 120 seconds which should be enough for any real-time situation. The command will return with the error CLX_ERROR_TIMEOUT_OCCURRED if the connection
cannot be established after 120 seconds.
Furthermore, the connection request may be cancelled at any time by calling #clxWlanP2PCancelConnectionRequest function.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ] remoteDeviceAddress The device Address of the peer device to which the connection request is to made. The device address is 6 bytes.
The address of this device must have been obtained from the latest discovery procedure (by a call to #clxWlanP2PStartDeviceDiscovery). This parameter cannot be NULL.
\param[ in ] goIntent A value between 0 to 15 which indicates the local device desire to be the Group Owner (GO). A value of 0 for this parameter means the local device has no desire. 
A value of 15 means the local device has an absolute desire to be the Group Owner.
the local device has an absolute desire to be the Group Owner.
\param[ in ] authenticationAction The authentication action which must be conducted by the REMOTE machine (e.g. a value of ClxP2PAuthenticationMethod_EnterPinCode means that the user of remote device must enter the
local device pin code on the remote device while the local device will show a random pin code on its display).
The authentication action must be selected based on the WPS capabilities of the both local device and remote device. Otherwise, authentication procedure may fail.
Alternatively, this parameter can be set to #ClxP2PAuthenticationMethod_AutoSelect which forces the stack to select the best action based on the WPS capabilities of both parties (recommended).
\param[ in ] block      type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PConnect(_in_ ClxHandle                        interfaceHandle,
                           _in_ const u1*                        remoteDeviceAddress,
                           _in_ u4                               goIntent,
                           _in_ ClxP2PAuthenticationAction       authenticationAction,
                           _in_ boolean                          block);




/**
Cancels the ongoing connection procedure. The procedure may have been initiated by the local device (via a previous call to #clxWlanP2PConnect), 
or initiated by a remote device and accepted by the local device via a previous call to #clxWlanP2PAcceptConnectionRequest.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_CANCEL_CONNECTION_REQUEST_COMPLETE.
The indication does not have any parameters.

NOTE : Upon completion of this command, the connection has been cancelled and a new connection may be initiated or a new connection request from a remote P2P-enabled device may be accepted.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ] block      type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PCancelConnectionRequest(_in_ ClxHandle    interfaceHandle,
                                           _in_ boolean      block);



/**
Disconnects the local device from the WIFI Direct group to which the local device has joined.
This function MUST be called only if the interface is successfully started (by a call to the #clxWlanStartInterface() function).

NOTE : This operation may not have been implemented for some interface types or roles. Please refer to the relevant driver documentation for more information.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_DISCONNECT_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ] block      type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PDisconnect(_in_ ClxHandle            interfaceHandle,
                              _in_ boolean              block);

    

/**
Accepts a connection request from a remote device. The local device might be in Idle mode (not connected to any group), or it is currently the group owner
of a group. In the former case, Group Owner (GO) negotiation will be conducted to determine which party will be the group owner.

This function MUST be called only after receiving an indication of type #CLX_WLAN_P2P_CONNECTION_REQUEST_INDICATION, #CLX_WLAN_P2P_CONNECTION_REQUEST_KEYPAD_METHOD_INDICATION,
or #CLX_WLAN_P2P_CONNECTION_REQUEST_DISPLAY_METHOD_INDICATION.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_ACCEPT_CONNECTION_REQUEST_COMPLETE.
The indication does not have any parameters.

NOTE : This command is complete immediately without waiting for the connection procedure to complete. Please note that the another connection request cannot be initiated
(either by the local device or a remote device) until the current connection procedure is successfully complete. The connection procedure may be cancelled at any time by a call to
#clxWlanP2PCancelConnectionRequest() API function.

NOTE : In case of the local device being the Group Owner (GO), while a P2P connection is ongoing, legacy devices can freely join the group. Also, the current P2P-enabled clients may be disconnected and
connected again with no issue (as long as they use the security key they have obtained during the original connection procedure to re-connect to the group). But new P2P-enabled clients (which need to obtain
the security key) are not allowed to initiate the connection until the ongoing connection procedure is complete with success or cancelled by a call to #clxWlanP2PCancelConnectionRequest() API function.

\param[ in ] interfaceHandle the WLAN interface handle.
\param[ in ] goIntent A value between 0 to 15 which indicates the local device desire to be the Group Owner (GO). This parameter is only valid when
the local device is in Idle mode. If the local device is already a group owner, this parameter will be ignored.
A value of 0 for this parameter means the local device has no desire. A value of 15 means the local device has an absolute desire to be the Group Owner.
\param[ in ] userPincode The PIN code which has been entered by the user of the local device and it to be used to authenticate the remote device.
The PIN code must be provided when this is a response to a previously received indication of type #CLX_WLAN_P2P_CONNECTION_REQUEST_KEYPAD_METHOD_INDICATION or #CLX_WLAN_P2P_CONNECTION_REQUEST_DISPLAY_METHOD_INDICATION.
In the former case, this PIN has been obtained from the keypad (as entered by the user). In the latter case, This is generally the PIN code which has been randomly generated by the stack and passed as the parameter
pincode of the #ClxWlanP2PConnectionRequestIndication parameter of the indication.
NOTE : It is possible to overwrite this randomly generated PIN code with a application-defined code. But, this is not recommended.
NOTE : In all cases, the PIN code provided must be 8 digits, made up of ASCII '0' to '9' characters. Please note that the PIN code must be ASCII and not binary. The PIN code must also be null-terminated.
In other cases, this argument is ignored and must be set to NULL.
\param[ in ] block      type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PAcceptConnectionRequest(_in_ ClxHandle    interfaceHandle,
                                           _in_ u4           goIntent,
                                           _in_ const s1*    userPincode,
                                           _in_ boolean      block);



/**
Rejects a connection request from a remote device. The local device might be in Idle mode (not connected to any group), or it is currently the group owner
of a group.

This function MUST be called only after receiving an indication of type #CLX_WLAN_P2P_CONNECTION_REQUEST_INDICATION, #CLX_WLAN_P2P_CONNECTION_REQUEST_KEYPAD_METHOD_INDICATION,
or #CLX_WLAN_P2P_CONNECTION_REQUEST_DISPLAY_METHOD_INDICATION.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_REJECT_CONNECTION_REQUEST_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle.
\param[ in ] block      type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PRejectConnectionRequest(_in_ ClxHandle    interfaceHandle,
                                           _in_ boolean      block);


/**
Initiates Device Discovery procedure. This procedure involves discovering peer WIFI-Direct-enabled devices in the area. The following P2P devices may be discovered:

- Idle devices : devices which have not joined or formed a P2P group yet.

- Group owners : devices which act as a group owner.

- Client devices : devices which are already a member of a P2P group. It is not possible to directly connect to a P2P client. A connection request to the group owner
should be initiated instead. The function #clxWlanP2PGetGroupInfo may be called to extract information about the group owner of P2P group, from the device address of any
P2P client in the group.

NOTE : The status(es) in which the local device is able to start a discovery depends on the driver and hardware. 
Please refer to the relevant driver documentation for more information.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_START_DISCOVERY_COMPLETE.
The indication does not have any parameters.

NOTE : This command does not wait until peer devices are discovered. It only starts the discovery procedure. Upon discovery of any new peer device, an indication of type
#CLX_WLAN_P2P_DEVICE_DISCOVERED_INDICATION will be received for each discovered device. The procedure will remain active until #clxWlanP2PStopDeviceDiscovery,
#clxWlanP2PConnect, or #clxWlanP2PAcceptConnectionRequest is called.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ] block      type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PStartDeviceDiscovery (_in_ ClxHandle            interfaceHandle,
                                         _in_ boolean              block);



/**
Stops the Device Discovery procedure which has been started by a call to #clxWlanP2PStartDeviceDiscovery.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_STOP_DISCOVERY_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ] block      type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_ERROR_ANOTHER_COMMAND_IN_PROGRESS: another command is in progress.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PStopDeviceDiscovery (_in_ ClxHandle            interfaceHandle,
                                        _in_ boolean              block);


/**
Obtains the list of P2P peer devices which have been discovered during the discovery procedure. This is the collective list of the same P2P devices for which indications
of type #CLX_WLAN_P2P_DEVICE_DISCOVERED_INDICATION have already been sent to the application.

NOTE : The list of discovered P2P devices is cleared every time #clxWlanP2PStartDeviceDiscovery is called. After the discovery procedure has been stopped, the list will
remain in the stack until the next time #clxWlanP2PStartDeviceDiscovery is called.

NOTE : This command may be issued when there is an ongoing discovery procedure.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_GET_DISCOVERED_PEERS_LIST_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ out ] peersList A caller-provided array of type #ClxWlanP2PDevice, and size peersListSize which, on a successful command completion, will contain the list of
the discovered devices. The provided array must NOT be removed or modified until this command is complete. This parameter may be NULL if the caller only intends to obtain the
number of discovered devices (as returned by currentNumberOfClients argument).
\param[ in ] peersListSize The number of #ClxWlanP2PDevice objects which can be stored in peersList. This parameter MUST be set to 0 if peersListSize is set to NULL.
\param[ out ] currentNumberOfDiscoveredPeers A caller-provided variable which, on a successful command completion, will contain the number of P2P devices which have been discovered.
If this value is smaller than (or equal to) peersListSize, then this value specifies the number of P2P device objects which have been returned in peersList.
If this value is larger than peersListSize, then only the first peersListSize P2P device objects have been returned in peersList.
The variable must not be deleted or modified until this command is complete. This parameter cannot be NULL.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PGetDiscoveredPeersList (_in_ ClxHandle                        interfaceHandle,
                                           _user_out_ struct ClxWlanP2PDevice*   peersList,
                                           _in_ ClxSize                          peersListSize,
                                           _user_out_ ClxSize*                   currentNumberOfDiscoveredPeers,
                                           _in_ boolean                          block);



/**
Extracts information about a P2P group from the device address of the Group Owner (GO) or any group client which has currently joined the group.
The device must be either in status of #ClxWlanP2PDeviceStatus_GroupOwner, or #ClxWlanP2PDeviceStatus_Connected. Also, the device must have been
discovered in the latest discovery procedure.

NOTE : This command may be issued when there is an ongoing discovery procedure.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_GET_GROUP_INFO_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ in ] deviceAddress The device address of a device which is in the status of either #ClxWlanP2PDeviceStatus_GroupOwner or #ClxWlanP2PDeviceStatus_Connected.
This command will return the information of the P2P group in which this device is a member, or the P2P group for which this device is the Group Owner (GO). 
The device address is 6 bytes. The address of this device must have been obtained from the latest discovery procedure (by a call to #clxWlanP2PStartDeviceDiscovery). 
This parameter cannot be NULL.
\param[ out ] group A caller-provided object of type #ClxWlanP2PGroup which, on successful command completion, will contain the details of the P2P group. 
The provided object must NOT be removed or modified until this command is complete. This parameter cannot be NULL.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PGetGroupInfo(_in_ ClxHandle                        interfaceHandle,
                                _in_ const u1*                        deviceAddress,
                                _user_out_ struct ClxWlanP2PGroup*    group,
                                _in_ boolean                          block);


/**
Obtains information about the current P2P group. This is the group in which the local device is currently a member, or the group for which the local device
is the group owner. The local device must be either in status of #ClxWlanP2PDeviceStatus_GroupOwner, or #ClxWlanP2PDeviceStatus_Connected.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_GET_CURRENT_GROUP_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ out ] group A caller-provided object of type #ClxWlanP2PGroup which, on successful command completion, will contain the details of the current P2P group. 
The provided object must NOT be removed or modified until this command is complete. This parameter cannot be NULL.
\param[ out ] device An optional caller-provided object of type #ClxWlanP2PDevice which, on successful command completion, will contain the details of the Group Owner (GO). 
The provided object must NOT be removed or modified until this command is complete. This parameter may be set to NULL if this information is not required.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PGetCurrentGroup(_in_ ClxHandle                        interfaceHandle,
                                    _user_out_ struct ClxWlanP2PGroup*    group,
                                    _user_out_ struct ClxWlanP2PDevice*    device,
                                    _in_ boolean                        block);


/**
Obtains information about the local device as a P2P device, in the form of an object of type #ClxWlanP2PDevice.

Non-blocking mode : When the command is complete, the call-back function will be called with an indication of type #CLX_WLAN_P2P_GET_LOCAL_DEVICE_COMPLETE.
The indication does not have any parameters.

\param[ in ] interfaceHandle the WLAN interface handle
\param[ out ] device A caller-provided object of type #ClxWlanP2PDevice which, on successful command completion, will contain the P2P details of the local device. 
The provided object must NOT be removed or modified until this command is complete.
\param[ in ] block type of the operation.
                - TRUE:  API will be blocked until this command is completed (successfully or failed).
                - FALSE: API will return immediately and then a corresponding API completion event will be raised upon command completion.
\return ClxError error code for the operation.
                - CLX_ERROR_COMPLETION_PENDING: operation will be completed asynchronously (only in non-blocking mode)
                - CLX_ERROR_COMMAND_CANCELLED: the handle was closed before the command has completed (only in blocking mode)
                - CLX_ERROR_COMMAND_NOT_COMPLETE: A previous same command has been issued which has not completed yet.
                - CLX_ERROR_INVALID_HANDLE: invalid handle.

In case of blocking mode the following error codes may also be returned. In case of non-blocking mode the following error codes may be obtained
from the errorCode argument of the call-back function.
                - CLX_SUCCESS: operation is successful.
                - CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE : This operation is not supported by this interface current role.
*/
ClxError clxWlanP2PGetLocalDevice (_in_ ClxHandle                        interfaceHandle,
                                   _user_out_ struct ClxWlanP2PDevice*   device,
                                   _in_ boolean                          block);



/**
Initializes a member of parentList with the given object of type #ClxWlanP2PDevice.
*/
void clxWlanConfigInitP2PDevice (ClxWlanP2PDevice* object,
                                 const s1* paramName,
                                 const s1* deviceName,
                                 u4 primaryDeviceType,
                                 u4 secondaryDeviceType,
                                 u4 wpsCapabilities,
                                 u4 deviceCapabilities,
                                 ClxConfigList* parentList);

#ifdef __cplusplus
}
#endif


#endif // _WLAN_WIFI_DIRECT_API_H_
