#ifndef _WlanBufferManagerInterface_
#define _WlanBufferManagerInterface_

/******************************************************************************
 *
 * Project             ClarinoxWLAN
 * File                WlanBufferManagerInterface.h
 * Description         Clarinox WLAN Buffer Manager Interface
 *
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/
 
#ifdef __cplusplus
extern "C" {
#endif


    enum ClxWlanBufferType
    {
        ClxWlanBufferType_PhysicalContiguousPages   = 0x01,          /* The buffer is one or more contiguous physical memory pages */                                                                            
        ClxWlanBufferType_VirtualContiguousPages    = 0x02,          /* The buffer is one or more contiguous virtual memory pages */
                                                                     /* NOTE : Contiguous pages in the virtual memory space of the stack are not necessarily contiguous in physical memory space */    
        ClxWlanBufferType_PhysicalMemory            = 0x04,          /* The buffer is not guaranteed to be aligned with any memory page. The buffer address is the physical memory address */
        ClxWlanBufferType_VirtualMemory             = 0x08           /* The buffer is not guaranteed to be aligned with any memory page. The buffer address is the virtual memory address */
    };


    enum ClxWlanBufferPath
    {
        ClxWlanBufferPath_TX,
        ClxWlanBufferPath_RX,
        ClxWlanBufferPath_Bidirectional
    };

    typedef void* ClxWlanBufferID;


    typedef struct ClxWlanBufferStruct
    {
        void*                       platformSpecificData;
        enum ClxWlanBufferType      type;
        enum ClxWlanBufferPath      path;
        u1*                         address;
        ClxSize                     length;
        ClxSize                     alignment;
    } ClxWlanBuffer;


#ifdef __cplusplus
}
#endif

#endif  // _WlanBufferManagerInterface_

