#ifndef _WlanErrors_
#define _WlanErrors_

/******************************************************************************
 *
 * Project             ClarinoxWLAN
 * File                WlanErrors.h
 * Description         Declares Clarinox WLAN common errors
 *
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#include "ClarinoxErrorCodes.h"

/** 
All errors are of type ClxResult. These error codes are positive numbers, so they can be used as both
internal errors, or API errors (the error codes which are reported to the API user).
Error codes between 0-20 are common errors and have been declared in ClarinoxErrorCodes.h file,
API error codes 21 through 999 has been reserved for ClarinoxBlue. 
Error codes 1021 through 2000 may be used for ClarinoxWLAN.
*/

#define CLX_WLAN_FAILED                                                 1021
#define CLX_WLAN_TERMINATED                                             1022
#define CLX_WLAN_REQUEST_PENDING                                        1023
#define CLX_WLAN_ERROR_UNKNOWN_VIRTUAL_INTERFACE                        1024
#define CLX_WLAN_ERROR_NOT_ENOUGH_RESOURCES                             1025
#define CLX_WLAN_ERROR_INVALID_PARAMETERS                               1026
#define CLX_WLAN_ERROR_DRIVER_INIT_FAILED                               1027
#define CLX_WLAN_ERROR_MULTIPLE_REQUESTS_NOT_ALLOWED                    1028
#define CLX_WLAN_ERROR_HARD_RESET_FAILED                                1029
#define CLX_WLAN_ERROR_SOFT_RESET_FAILED                                1030
#define CLX_WLAN_ERROR_NO_FIRMWARE_FOUND                                1031
#define CLX_WLAN_ERROR_BAD_FIRMWARE_FORMAT                              1032
#define CLX_WLAN_ERROR_INSUFFICIENT_MEMORY                              1033
#define CLX_WLAN_ERROR_PLATFORM_NOT_SUPPORTED                           1034
#define CLX_WLAN_ERROR_NO_NVS_FOUND                                     1035
#define CLX_WLAN_ERROR_BAD_NVS_FORMAT                                   1036
#define CLX_WLAN_ERROR_CHIPSET_NOT_SUPPORTED                            1037
#define CLX_WLAN_ERROR_BOOT_PROCESS_FAILED                              1038
#define CLX_WLAN_ERROR_ANOTHER_COMMAND_PENDING                          1039
#define CLX_WLAN_ERROR_USER_TIMEOUT_OCCURRED                            1040
#define CLX_WLAN_ERROR_SCAN_FAILED                                      1041
#define CLX_WLAN_ERROR_NOT_IMPLEMENTED                                  1042
#define CLX_WLAN_ERROR_BASIC_RATE_SET_NOT_SUPPORTED                     1043
#define CLX_WLAN_ERROR_NOT_ENOUGH_MEMORY                                1044
#define CLX_WLAN_ERROR_SEGMENTED_RX_PDU_PENDING                         1045
#define CLX_WLAN_ERROR_FRAME_PARSING_ERROR                              1046
#define CLX_WLAN_ERROR_STACK_TERMINATING                                1047
#define CLX_WLAN_ERROR_TOO_MANY_REQUESTS_PENDING                        1048
#define CLX_WLAN_ERROR_AUTHENTICATION_FAILED                            1049
#define CLX_WLAN_ERROR_ASSOCIATION_FAILED                               1050
#define CLX_WLAN_ERROR_INTERFACE_IN_WRONG_STATE                         1051
#define CLX_WLAN_ERROR_BSS_INFO_NOT_AVAILABLE                           1052
#define CLX_WLAN_ERROR_NOT_SUPPORTED_BY_VIRTUAL_INTERFACE               1053
#define CLX_WLAN_ERROR_VIRTUAL_INTERFACE_TYPE_INCORRECT                 1054
#define CLX_WLAN_ERROR_COMMAND_CANCELLED                                1055
#define CLX_WLAN_ERROR_INTERFACE_NOT_STARTED                            1056
#define CLX_WLAN_ERROR_NOT_SUPPORTED_BY_ROLE                            1057 
#define CLX_WLAN_ERROR_ACKNOWLEDGEMENT_FAILED                           1058
#define CLX_WLAN_ERROR_HARDWARE_ERROR_OCCURRED                          1059
#define CLX_WLAN_ERROR_LEGACY_AUTHENTICATION_NOT_SUPPORTED              1060
#define CLX_WLAN_ERROR_KEY_INSTALLATION_FAILED                          1061
#define CLX_WLAN_ERROR_REQUESTED_ENC_NOT_SUPPORTED_BY_DRIVER            1062        /*!< Requested encryption protocol is not supported by the driver */
#define CLX_WLAN_ERROR_REQUESTED_AUTH_NOT_SUPPORTED_BY_DRIVER           1063        /*!< Requested authentication mechanism is not supported by the driver */
#define CLX_WLAN_ERROR_REQUESTED_ENC_NOT_SUPPORTED_BY_BSS               1064        /*!< Requested encryption protocol is not supported by the RemoteAccessPoint */
#define CLX_WLAN_ERROR_REQUESTED_AUTH_NOT_SUPPORTED_BY_BSS              1065        /*!< Requested authentication mechanism is not supported by the RemoteAccessPoint */
#define CLX_WLAN_ERROR_CONNECTION_TO_NETWORK_STACK_FAILED               1066
#define CLX_WLAN_ERROR_INTERFACE_STARTED                                1067
#define CLX_WLAN_ERROR_ASSOCIATION_REQUESTED_BEFORE_AUTHENTICATION      1068
#define CLX_WLAN_ERROR_LEGACY_AUTHENTICATION_CHALLENGE_FAILED           1069
#define CLX_WLAN_ERROR_LEGACY_AUTHENTICATION_TIMEOUT                    1070
#define CLX_WLAN_ERROR_REASSOCIATION_DENIED                             1071
#define CLX_WLAN_ERROR_ASSOCIATION_DENIED_IEEE802_11b_CONFIG_MISMATCH   1072
#define CLX_WLAN_ERROR_ASSOCIATION_DENIED_IEEE802_11g_CONFIG_MISMATCH   1073
#define CLX_WLAN_ERROR_ASSOCIATION_DENIED_IEEE802_11h_CONFIG_MISMATCH   1074
#define CLX_WLAN_ERROR_ASSOCIATION_DENIED                               1075
#define CLX_WLAN_ERROR_UNKNOWN_IEEE802_11_ERROR                         1076
#define CLX_WLAN_ERROR_INVALID_LEGACY_AUTHENTICATION_ALGORITHM          1077
#define CLX_WLAN_ERROR_UNEXPECTED_SEQUENCE_NUMBER                       1078
#define CLX_WLAN_ERROR_UNSPECIFIED_FAILURE                              1079
#define CLX_WLAN_ERROR_4WAY_KEY_HANDSHAKE_TIMEOUT                       1080
#define CLX_WLAN_ERROR_GROUPWISE_KEY_HANDSHAKE_TIMEOUT                  1081
#define CLX_WLAN_ERROR_HANDSHAKE_INFO_MISMATCH                          1082
#define CLX_WLAN_ERROR_INVALID_SECURITY_INFORMATION_ELEMENT             1083
#define CLX_WLAN_ERROR_INVALID_GROUPWISE_CIPHER                         1084
#define CLX_WLAN_ERROR_INVALID_PAIRWISE_CIPHER                          1085
#define CLX_WLAN_ERROR_INVALID_AUTHENTICATION_PROTOCOL                  1086
#define CLX_WLAN_ERROR_UNSUPPORTED_RSN_IE_VERSION                       1087
#define CLX_WLAN_ERROR_INVALID_CAPABILITIES_IN_RSN                      1088
#define CLX_WLAN_ERROR_IEEE802_1X_AUTHENTICATION_FAILED                 1089
#define CLX_WLAN_ERROR_DEAUTHENTICATED_STATION_LEFT_BSS                 1090
#define CLX_WLAN_ERROR_DEASSOCIATED_STATION_LEFT_BSS                    1091
#define CLX_WLAN_ERROR_STATION_INACTIVITY_TIMER_EXPIRED                 1092
#define CLX_WLAN_ERROR_INCORRECT_FRAME_TYPE_FROM_UNAUTHENTICATED_STA    1093
#define CLX_WLAN_ERROR_INCORRECT_FRAME_TYPE_FROM_UNASSOCIATED_STA       1094
#define CLX_WLAN_ERROR_ROLE_NOT_STARTED                                 1095
#define CLX_WLAN_ERROR_ROLE_HAS_STARTED                                 1096
#define CLX_WLAN_ERROR_CONNECTION_ATTEMPT_TIMEOUT                       1097
#define CLX_WLAN_ERROR_LINK_NOT_SECURE_FOR_GROUPKEY_TRANSFER            1098
#define CLX_WLAN_ERROR_HT_WITH_TKIP_ONLY_IS_ILLEGAL                     1099
#define CLX_WLAN_ERROR_ILLEGAL_FREQUENCY_CHANNEL                        1100
#define CLX_WLAN_ERROR_BSS_TYPE_DOES_NOT_MATCH_INTERFACE_TYPE           1101
#define CLX_WLAN_ERROR_WPS_SESSION_TERMINATED_SUCCESSFULLY              1102
#define CLX_WLAN_ERROR_WPS_SESSION_FAILED                               1103
#define CLX_WLAN_ERROR_WPS_OOB_INTERFACE_READ_ERROR                     1104
#define CLX_WLAN_ERROR_WPS_DYCRYPTION_CRC_ERROR                         1105
#define CLX_WLAN_ERROR_WPS_2_4_GHZ_CHANNEL_NOT_SUPPORTED                1106
#define CLX_WLAN_ERROR_WPS_5_GHZ_CHANNEL_NOT_SUPPORTED                  1107
#define CLX_WLAN_ERROR_WPS_EXTERNAL_REGISTRAR_NOT_REACHABLE             1108
#define CLX_WLAN_ERROR_WPS_MULTIPLE_PBC_SESSION_DETECTED                1109
#define CLX_WLAN_ERROR_WPS_ROGUE_ACTIVITY_DETECTED                      1110
#define CLX_WLAN_ERROR_WPS_DEVICE_BUSY                                  1111
#define CLX_WLAN_ERROR_WPS_SETUP_LOCKED                                 1112
#define CLX_WLAN_ERROR_WPS_DEVICE_PASSWORD_AUTHENTICATION_FAILED        1113
#define CLX_WLAN_ERROR_WPS_REGISTRAR_ALREADY_ACTIVE                     1114
#define CLX_WLAN_ERROR_WPS_REGISTRAR_NOT_ACTIVE                         1115
#define CLX_WLAN_ERROR_WPS_REGISTRAR_SETUP_LOCKED                       1116
#define CLX_WLAN_ERROR_WPS_REGISTRAR_METHOD_NOT_SUPPORTED               1117
#define CLX_WLAN_ERROR_P2P_NO_PENDING_CONNECTION                        1118
#define CLX_WLAN_ERROR_P2P_CONNECTION_CANCELLED                         1119
#define CLX_WLAN_ERROR_LINK_LOST                                        1120
#define CLX_WLAN_ERROR_ROAMING_ATTEMPT_FAILED                           1121


#endif  // _WlanErrors_

