#ifndef _WlanStackPacketCaptureInterface_h_
#define _WlanStackPacketCaptureInterface_h_

/*******************************************************************************
*
* Project             ClarinoxWLAN
* File                WlanStackPacketCaptureInterface.h
* Description         Clarinox WLAN stack Packet Capture interface
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif


/**
Frame types as flag bits.
*/
#define CLX_WLAN_FRAME_TYPE_UNKNOWN                 0x00001
#define CLX_WLAN_FRAME_TYPE_CONTROL_FRAME           0x00002
#define CLX_WLAN_FRAME_TYPE_BEACON                  0x00004
#define CLX_WLAN_FRAME_TYPE_ASSOC_REQUEST           0x00008
#define CLX_WLAN_FRAME_TYPE_ASSOC_RESPONSE          0x00010
#define CLX_WLAN_FRAME_TYPE_REASSOC_REQUEST         0x00020
#define CLX_WLAN_FRAME_TYPE_REASSOC_RESPONSE        0x00040
#define CLX_WLAN_FRAME_TYPE_PROBE_REQUEST           0x00080
#define CLX_WLAN_FRAME_TYPE_PROBE_RESPONSE          0x00100
#define CLX_WLAN_FRAME_TYPE_ATIM                    0x00200
#define CLX_WLAN_FRAME_TYPE_DEASSOCIATION           0x00400
#define CLX_WLAN_FRAME_TYPE_AUTHENTICATION          0x00800
#define CLX_WLAN_FRAME_TYPE_DEAUTHENTICATION        0x01000
#define CLX_WLAN_FRAME_TYPE_ACTION                  0x02000
#define CLX_WLAN_FRAME_TYPE_DATA_EAPOL              0x04000
#define CLX_WLAN_FRAME_TYPE_DATA_ARP                0x08000
#define CLX_WLAN_FRAME_TYPE_DATA_IPv4               0x10000
#define CLX_WLAN_FRAME_TYPE_DATA_IPv6               0x20000
#define CLX_WLAN_FRAME_TYPE_DATA_UNKNOWN            0x40000


typedef enum ClxWlanCapturedPacketDirectionEnum
{
    ClxWlanCapturedPacketDirection_Tx,
    ClxWlanCapturedPacketDirection_Rx
} ClxWlanCapturedPacketDirection; 


typedef struct ClxWlanCapturedPacketInfoStruct
{
    ClxWlanPacketStatus                status;
    u4                                 type;
    ClxWlanCapturedPacketDirection     direction;
    const s1*                          typeString;
    u4                                 timeStamp_s; 
    u4                                 timeStamp_us;
    u4                                 totalLength;
    u4                                 linkID;
}ClxWlanCapturedPacketInfo;




typedef void (*ClxWlanPacketCaptured) (ClxHandle interfaceHandle,                       /* Handle to the interface via which this packet has been sent. */
                                       void* interfaceObject,                           /* Pointer to the driver interface object (of type ClarinoxWLAN::VirtualInterface). 
                                                                                           To be used by the driver only (if this function is implemented by the driver). */
                                       const ClxWlanCapturedPacketInfo* frameInfo,      /* Information about the captured packet. */
                                       const ClxCapturedPacketData* data);              /* Array of type ClxCapturedPacketData, containing the chunks of the packet data.
                                                                                           The last element in the array is set to NULL, specifying the end of the array. */

extern ClxWlanPacketCaptured clxWlanPacketCaptured;


#ifdef __cplusplus
}
#endif


#endif    // _WlanStackPacketCaptureInterface_h_


