#ifndef _ClarinoxConst_h_
#define _ClarinoxConst_h_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClarinoxConst.h
* Description         Declares Clarinox Stack constants and types
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/** A handle represents a service, or an instance of a service */
typedef void*   ClxHandle;
typedef void*   ClxStack;

/** Represents an API error */
typedef s4      ClxError;

#if defined(_in_)
#   undef _in_
#endif
/** parameters with an attribute _in_ are input-only variables which are provided by the user. 
The memory may be destroyed after the API function returns (in either blocking or non-blocking mode).*/
#define _in_

#if defined(_out_)
#   undef _out_
#endif
/** 
Parameters with an attribute _out_ point to a buffer which must be provided by the user in blocking mode API function calls. 
These parameters are not used in non-blocking mode and may be set to NULL. In non-blocking mode, the same data elements are
accessible in a temporary buffer provided and owned by the stack.
*/
#define _out_

#if defined(_user_in_)
#   undef _user_in_
#endif
/** 
Parameters with an attribute _user_in_ are input-only pointers which point to a memory buffer provided by the user. 
The buffer must NOT be touched or destroyed until the API command is complete (in either blocking or non-blocking mode).
The buffer may only be destroyed when the command is complete.
*/
#define _user_in_

#if defined(_user_out_)
#   undef _user_out_
#endif
/** 
Parameters with an attribute _user_out_ point to a buffer provided by the user.
The buffer may NOT be modified or destroyed until the command is complete (in either blocking or non-blocking mode).
*/
#define _user_out_

#if defined(_inout_)
#   undef _inout_
#endif

/**
Parameters with an attribute _inout_ are served as both input and output. These parameters are always defined as a pointer. 
This attribute is never used with a buffer. It is only used along with numeric parameters.
*/
#define _inout_

/** Callback function prototype is used by application to handle stack callback function calls */
typedef boolean (*ClxApplicationCallbackFunc) (ClxStack stack, ClxHandle serviceHandle, u4 indicationID, const void* params, ClxError errorCode); 
/** 
Exception function prototype is used by an application to handle stack exception. The handler provides moduleId and lineNumber to the call-back function 
and the application must log the moduleId and lineNumber for informing Clarinox to find out to determine a possible stack issue. 
Then the application must restart the stack and application upon an exception for proper device operation. 
*/
typedef void (*ClxExceptionCallback) ( s4 moduleId, s4 lineNumber );


#ifdef __cplusplus
}
#endif


#endif // _ClarinoxConst_h_

