#ifndef _ClxTime_
#define _ClxTime_

/******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxTime.h
* Description         Declares ClxTime related
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
A structure to hold time of day details
*/
struct ClxFullTime {
    u1 sec;
    u1 min;
    u1 hour;
    u1 day;
    u1 month;
    u2 year;
    u1 dayOfWeekSinceSun;
};

/**
A structure to provide the Time value in seconds and microseconds
*/
struct ClxTimeval {
    u4 tv_sec;    // time value seconds
     u4 tv_usec;    // time value micro seconds
};

/**
Sleeps for the given amount of time.
\param[ in ] milliseconds is the amount of time to sleep in milliseconds.
*/
extern  void         clxSleep                    ( u4 milliseconds );

/**
Reads the tick time in milliseconds
\return the number of seconds since the OS was started.
*/
extern  u4           clxTickTime                 ( void );

/**
Reads the tick time in milliseconds.
Clarinox stack does not utilize this function, hence it is provided for application portability.
\param[ in ] previousTick if set to 0 then the internal difference counter is used, if set to a non-zero value, then user supplied difference counter will be used.
\return the number of seconds since the OS was started (or previousTick value) in ASCII format.
*/
extern  const char*  clxTickTimeFormatted        ( u4* previousTick = 0 );

/**
Reads the high resolution tick time in microseconds (some platforms do not provide the high resolution time).
Clarinox stack does not utilize this function, hence it is provided for application portability.
\return the high resolution tick time in microseconds
*/
extern  u4           clxHighResolutionTime       ( void );

/**
Reads the current time into an ASCII string
Clarinox stack does not utilize this function, hence it is provided for application portability.
\param[ out ] t[ 18 ] is a buffer for the date-time as string
(yyyymmdd hh:mm:ss) to be written into.
*/
extern  void         clxGetCurrentTimeFormatted  ( s1 t[ 18 ] );

/**
Reads the current time into the ClxTimeval structure.
Clarinox stack does not utilize this function, hence it is provided for application portability.
\param[ out ] time is a ClxTimeval structure to read the current time. \sa ClxTimeval
*/
extern  void         clxGetCurrentTime           ( struct ClxTimeval* time );

/**
Reads the current date of time from the Operating System.
Clarinox stack does not utilize this function, hence it is provided for application portability.
\return the current date as an unsigned 4 byte value (yyyymmdd)
*/
extern  u4           clxGetCurrentDate           ();

// _____________________________________________________________________________
//
// ClxTime
// _____________________________________________________________________________
//

/**
    Time functionality. Tick time is used in various time stamp and time measurement functions within the Clarinox libraries.
    Tick time is in millisecond resolution and targeting soft real-time systems.
    There is a high resolution (nanoseconds) tick time function prototype is provided for convenience,
    but this function is never used in Clarinox libraries.
    In fact only clxTickTime and clxSleep functions are used by Clarinox libraries.
*/

#ifdef __cplusplus
}
#endif


#endif    // _ClxTime_

