#ifndef _ClxTypes_
#define _ClxTypes_

/*******************************************************************************
*
* Project             ClarinoxSoftFrame
* File                ClxTypes.h
* Description         Clarinox Type definitions
*
* This file contains proprietary information of
* Clarinox Technologies Proprietary Limited.
* Copying or reproduction without prior written approval is prohibited.
*
* All rights reserved (C) Copyright Clarinox Technologies Pty Ltd 2001-2015
*
*******************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif


// _____________________________________________________________________________
//
// ClxTypes
// _____________________________________________________________________________
//

#if defined( CLX_GNU ) || defined( CLX_MICROSOFT ) || defined( CLX_MULTI ) || defined( CLX_IAR ) || defined( CLX_KEIL ) || defined( CLX_CGT_C6000 ) || defined( CLX_VXWORKS_7 )

/**
One byte signed:            s1
*/
typedef          char       s1;

/**
One byte unsigned:          u1
*/
typedef unsigned char       u1;

/**
Two bytes signed:           s2
*/
typedef          short      s2;

/**
Two bytes unsigned:         u2
*/
typedef unsigned short      u2;

/**
Four bytes signed:          s4
*/
typedef          int        s4;

/**
Four bytes unsigned:        u4
*/
typedef unsigned int        u4;
#if defined(CLX_FLOATING_POINT_SUPPORTED)
typedef          float      r4;
typedef          double     r8;
#endif

/**
a single byte:              byte
*/
typedef unsigned char       byte;

/**
Boolean:                    boolean
*/
typedef unsigned char       boolean;

/**
Size (of buffers, data structures . . .)
*/
typedef unsigned int        ClxSize;

/**
Result of an operation (API call, ...)
*/
typedef s4                  ClxResult;


#elif defined( CLX_CODE_COMPOSER )
typedef                     char       s1; // There is no single byte type support for TI 55x processors
typedef unsigned            char       u1;
typedef                     short      s2;
typedef unsigned            short      u2;
typedef                     long       s4;
typedef unsigned            long       u4;
typedef          long       long       s8;
typedef unsigned long       long       u8;


#if defined(CLX_FLOATING_POINT_SUPPORTED)
typedef                     float      r4;
typedef                     double     r8;
#endif

typedef unsigned            char       byte;
typedef unsigned            char       boolean;

#else
#   error "Define your compiler specific types here"
#endif

#if defined( CLX_GNU )
//typedef          uint64_t   u8;
//typedef          int64_t    s8;
#elif defined( CLX_MICROSOFT )
typedef unsigned __int64    u8;
typedef          __int64    s8;
#else
// 64 bit signed and unsigned types are only used for the tools and utilities but not the core and the wireless stack development
#endif

#if defined( CLX_MICROSOFT ) 
#   define ATTRIBUTE_ALIGNED
    typedef void*           HANDLE;
#   define CLX_HANDLE       HANDLE
#elif defined( CLX_GNU )
#   if !defined( CLX_UNALIGNED_ACCESS_ALLOWED )
#       define ATTRIBUTE_ALIGNED __attribute__ ((aligned (4)))
#   else
#       define ATTRIBUTE_ALIGNED
#   endif
#   define LPVOID           void*
#   define UINT             u4
#   define CLX_HANDLE       void*
#elif defined( CLX_MULTI )
#   define ATTRIBUTE_ALIGNED __attribute__ ((aligned (4)))
#   define LPVOID           void*
#   define UINT             u4
#   define CLX_HANDLE       void*
#elif defined( CLX_CODE_COMPOSER ) || defined( CLX_CGT_C6000 )
#   define ATTRIBUTE_ALIGNED
#   define LPVOID           void*
#   define UINT             u4
#   define CLX_HANDLE       void*
#   elif defined ( CLX_IAR )
#   define LPVOID           void*
#   define UINT             u4
#   define CLX_HANDLE       void*    
#   elif defined ( CLX_KEIL )
#   define LPVOID           void*
#   define UINT             u4
#   define CLX_HANDLE       void*
#   elif defined ( CLX_DIAB )
#   define LPVOID           void*
#   define UINT             u4
#   define CLX_HANDLE       void* 
#else
#   error "Define your OS/Compiler specific definitions here"
#endif // defined( CLX_WINDOWS )

#ifndef TRUE
#    define TRUE 1
#endif
#ifndef FALSE
#    define FALSE 0
#endif
#ifndef NULL
#    define NULL 0
#endif


typedef struct ClxUInteger64Struct
{
    u4 msb;
    u4 lsb;
} ClxUInteger64;

typedef struct ClxUInteger128Struct
{
    ClxUInteger64 msb;
    ClxUInteger64 lsb;
} ClxUInteger128;

typedef struct ClxSInteger64Struct
{
    u4 msb;
    u4 lsb;
} ClxSInteger64;

typedef struct ClxSInteger128Struct
{
    ClxSInteger64 msb;
    ClxSInteger64 lsb;
} ClxSInteger128;



#if !defined( CHECK_IF_ARG_NULL_IN_CREATE )
#define CHECK_IF_ARG_NULL_IN_CREATE(value)                                      \
	if(value == 0)                                                              \
    {                                                                           \
        return 0;                                                               \
    } 
#endif

#if !defined( CHECK_IF_ARG_NULL )
#define CHECK_IF_ARG_NULL(value)                                                \
	if(value == 0)                                                              \
    {                                                                           \
        return CLX_ERROR_INVALID_COMMAND_PARAMETER;                             \
    } 
#endif

#if !defined( CLX_UNKNOWN_ITEM )
#define CLX_UNKNOWN_ITEM -1
#endif


#ifdef __cplusplus
}
#endif


/** \file ClxTypes.h
    \brief ClxTypes.h contains the Clarinox type information
*/

#endif    // _ClxTypes_
